// A variation on TableSorter.java that incorporates some additional features needed by RAVE.
// Original version downloaded from http://download.oracle.com/javase/tutorial/uiswing/examples/components/TableSorterDemoProject/src/components/TableSorter.java
/*
 * Copyright (c) 1995, 2008, Oracle and/or its affiliates. All rights reserved.
 *
 * Redistribution and use in source and binary forms, with or without
 * modification, are permitted provided that the following conditions
 * are met:
 *
 *   - Redistributions of source code must retain the above copyright
 *     notice, this list of conditions and the following disclaimer.
 *
 *   - Redistributions in binary form must reproduce the above copyright
 *     notice, this list of conditions and the following disclaimer in the
 *     documentation and/or other materials provided with the distribution.
 *
 *   - Neither the name of Oracle or the names of its
 *     contributors may be used to endorse or promote products derived
 *     from this software without specific prior written permission.
 *
 * THIS SOFTWARE IS PROVIDED BY THE COPYRIGHT HOLDERS AND CONTRIBUTORS "AS
 * IS" AND ANY EXPRESS OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO,
 * THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR
 * PURPOSE ARE DISCLAIMED.  IN NO EVENT SHALL THE COPYRIGHT OWNER OR
 * CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL,
 * EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO,
 * PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR
 * PROFITS; OR BUSINESS INTERRUPTION) HOWEVER CAUSED AND ON ANY THEORY OF
 * LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING
 * NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS
 * SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.
 */

import java.awt.*;
import java.awt.event.*;
import java.util.*;
import java.util.List;

import javax.swing.*;
import javax.swing.event.TableModelEvent;
import javax.swing.event.TableModelListener;
import javax.swing.table.*;

public class RaveTableSorter extends DefaultTableModel {
    protected DefaultTableModel tableModel;
    
    public static final int DESCENDING = -1;
    public static final int NOT_SORTED = 0;
    public static final int ASCENDING = 1;
    
    private static Directive EMPTY_DIRECTIVE = new Directive(-1, NOT_SORTED);
    
    public static final Comparator COMPARABLE_COMAPRATOR = new Comparator() {
        public int compare(Object o1, Object o2) {
            return ((Comparable) o1).compareTo(o2);
        }
    };
    
    public static final Comparator LEXICAL_COMPARATOR = new Comparator() {
        public int compare(Object o1, Object o2) {
            return o1.toString().compareTo(o2.toString());
        }
    };
    
    private Row[] viewToModel;
    private int[] modelToView;
    private JTableHeader tableHeader;
    //  private MouseListener mouseListener;
    private TableModelListener tableModelListener;
    private Map columnComparators = new HashMap();
    private List sortingColumns = new ArrayList();
    
    public boolean[] isFunction;
    public boolean[] isNoise;
    public boolean[] isConstant;
	public boolean[] isVirtual;
    public boolean[] isRowShaded;
    public boolean[] isRowFiltered;
    public boolean[][] isDataValid;
	
    public int[] rowColors;
    public Color[] rowColorSpecs = new Color[10];
    
    
    public RaveTableSorter() {
        //   this.mouseListener = new TableMouseHandler();
        this.tableModelListener = new TableModelHandler();
    }
    
    public RaveTableSorter(DefaultTableModel tableModel) {
        this();
        setTableModel(tableModel);
        boolean[] isFunction = new boolean[tableModel.getColumnCount()];
        boolean[] isNoise = new boolean[tableModel.getColumnCount()];
        boolean[] isConstant = new boolean[tableModel.getColumnCount()];
		boolean[] isVirtual = new boolean[tableModel.getColumnCount()];
        boolean[] isRowShaded = new boolean[tableModel.getRowCount()];
        boolean[] isRowFiltered = new boolean[tableModel.getRowCount()];
        int[] rowColors = new int[tableModel.getRowCount()];
        boolean[][] isDataValid = new boolean[tableModel.getRowCount()][tableModel.getColumnCount()];
        
        this.isFunction = isFunction;
        this.isNoise = isNoise;
        this.isConstant = isConstant;
		this.isVirtual = isVirtual;
        this.isRowShaded = isRowShaded;
        this.isRowFiltered = isRowFiltered;
        this.rowColors = rowColors;
        this.isDataValid = isDataValid;
        
        for (int i = 0; i < tableModel.getRowCount(); i++)
            this.rowColors[i] = 0;
        
        for (int i = 0; i < 10; i++)
            this.rowColorSpecs[i] = Color.red;
        
        /*for (int i = 0; i < tableModel.getRowCount(); i++) {
            for (int j = 0; j < tableModel.getColumnCount(); j++) {
                this.isDataValid[i][j] = true;
            }
        }
         */
    }
    
    public RaveTableSorter(DefaultTableModel tableModel, JTableHeader tableHeader) {
        this();
        setTableHeader(tableHeader);
        setTableModel(tableModel);
    }
    
    private void clearSortingState() {
        viewToModel = null;
        modelToView = null;
    }
    
    public TableModel getTableModel() {
        return tableModel;
    }
    
    //All of these methods take in Object arrays and then cast them to the
    //desired type. This is because MATLAB sends everything as Object, so keeping them
    //that way avoids a VERY slow conversion in matlab.
    
    /*public void setIsFunction(Object[] newFunction) {
        for (int i = 0; i < newFunction.length; i++)
            this.isFunction[i] = (Boolean)newFunction[i];
    }
     */
    
    public void setIsFunction(boolean[] newFunction) {
        this.isFunction = newFunction;
    }
    
    
   /* public void setIsNoise(Object[] newNoise) {
        for (int i = 0; i < newNoise.length; i++)
            this.isNoise[i] = (Boolean)newNoise[i];
    }*/
    
    public void setIsNoise(boolean[] newNoise) {
        this.isNoise = newNoise;
    }
    
    
    /*public void setIsConstant(Object[] newConstant) {
        for (int i = 0; i < newConstant.length; i++)
            this.isConstant[i] = (Boolean)newConstant[i];
    }*/
    
    public void setIsConstant(boolean[] newConstant) {
        this.isConstant = newConstant;
    }
    
     public void setIsVirtual(boolean[] newVirtual) {
        this.isVirtual = newVirtual;
    }   
    
   /* public void setIsRowShaded(Object[] newRowShaded) {
        for (int i = 0; i < newRowShaded.length; i++)
            this.isRowShaded[i] = (Boolean)newRowShaded[i];
    }*/
    
    
    public void setIsRowShaded(boolean[] newRowShaded) {
        this.isRowShaded = newRowShaded;
    }
    
   /* public void setIsRowFiltered(Object[] newRowFiltered) {
        for (int i = 0; i < newRowFiltered.length; i++)
            this.isRowFiltered[i] = (Boolean)newRowFiltered[i];
    }*/
    
    public void setIsRowFiltered(boolean[] newRowFiltered) {
        this.isRowFiltered = newRowFiltered;
    }
    
    
    public void setRowColors(Object[] newRowColors) {
        for (int i = 0; i < newRowColors.length; i++)
            this.rowColors[i] = (Integer)newRowColors[i];
    }
    
    /*public void setIsDataValid(Object[][] newDataValid) {
        for (int i = 0; i < newDataValid.length; i++)
            for (int j = 0; j < newDataValid[i].length ; j++)
                this.isDataValid[i][j] = (Boolean)newDataValid[i][j];
    }
     */
    
    /*public void setIsDataValid(boolean[][] newDataValid) {
        for (int i = 0; i < newDataValid.length; i++)
            for (int j = 0; j < newDataValid[i].length ; j++)
                this.isDataValid[i][j] = newDataValid[i][j];
    }
     */
    
    
    public void setIsDataValid(boolean[][] newDataValid) {
        //for (int i = 0; i < newDataValid.length; i++)
        //    for (int j = 0; j < newDataValid[i].length ; j++)
        this.isDataValid = newDataValid;
    }
    
    
    
    public void setIsDataValidAt(int row, int col, Boolean newDataValid) {
        this.isDataValid[row][col] = newDataValid.booleanValue();
    }
    
    public void setRowColorSpec(int index, Color newRowColor) {
        this.rowColorSpecs[index] = newRowColor;
    }
    
    public boolean[] getIsRowShaded() {
        return isRowShaded;
    }
    
    public boolean[] getIsRowFiltered() {
        return isRowFiltered;
    }
    
    public boolean[] getIsFunction() {
        return isFunction;
    }
    
    public boolean[] getIsNoise() {
        return isNoise;
    }
    
    public boolean[] getIsConstant() {
        return isConstant;
    }
	
	public boolean[] getIsVirtual() {
        return isVirtual;
    }
	
    
    public int[] getRowColors() {
        return rowColors;
    }
    
    public boolean[][] getIsDataValid() {
        return isDataValid;
    }
    
    public int getRowColor(int row) {
        return rowColors[row];
    }
    
    public Color[] getRowColorSpecs() {
        return rowColorSpecs;
    }
    
    public void setTableModel(DefaultTableModel tableModel) {
        if (this.tableModel != null) {
            this.tableModel.removeTableModelListener(tableModelListener);
        }
        
        this.tableModel = tableModel;
        if (this.tableModel != null) {
            this.tableModel.addTableModelListener(tableModelListener);
        }
        
        clearSortingState();
        fireTableStructureChanged();
    }
    
    public JTableHeader getTableHeader() {
        return tableHeader;
    }
    
    public void setTableHeader(JTableHeader tableHeader) {
        if (this.tableHeader != null) {
            //  this.tableHeader.removeMouseListener(mouseListener);
            TableCellRenderer defaultRenderer = this.tableHeader.getDefaultRenderer();
            if (defaultRenderer instanceof SortableHeaderRenderer) {
                this.tableHeader.setDefaultRenderer(((SortableHeaderRenderer) defaultRenderer).tableCellRenderer);
            }
        }
        this.tableHeader = tableHeader;
        if (this.tableHeader != null) {
            //  this.tableHeader.addMouseListener(mouseListener);
            this.tableHeader.setDefaultRenderer(
            new SortableHeaderRenderer(this.tableHeader.getDefaultRenderer()));
        }
    }
    
    public boolean isSorting() {
        return sortingColumns.size() != 0;
    }
    
    private Directive getDirective(int column) {
        for (int i = 0; i < sortingColumns.size(); i++) {
            Directive directive = (Directive)sortingColumns.get(i);
            if (directive.column == column) {
                return directive;
            }
        }
        return EMPTY_DIRECTIVE;
    }
    
    public int getSortingStatus(int column) {
        return getDirective(column).direction;
    }
    
    private void sortingStatusChanged() {
        clearSortingState();
        fireTableDataChanged();
        if (tableHeader != null) {
            tableHeader.repaint();
        }
    }
    
    public void setSortingStatus(int column, int status) {
        Directive directive = getDirective(column);
        if (directive != EMPTY_DIRECTIVE) {
            sortingColumns.remove(directive);
        }
        if (status != NOT_SORTED) {
            sortingColumns.add(new Directive(column, status));
        }
        sortingStatusChanged();
    }
    
    protected ImageIcon sUp   = new ImageIcon(getClass().getResource("/com/mathworks/mwswing/resources/sort_arrow_up_xp.gif"));
    protected ImageIcon sDown = new ImageIcon(getClass().getResource("/com/mathworks/mwswing/resources/sort_arrow_down_xp.gif"));
    
    protected Icon getHeaderRendererIcon(int column, int size) {
        Directive directive = getDirective(column);
        if (directive == EMPTY_DIRECTIVE) {
            return null;
        }
        // Yair 10/1/2007: use the default (system) arrow icon for thetop-priority sorting column
        if (sortingColumns.indexOf(directive) == 0)
            return (directive.direction == DESCENDING) ? sDown : sUp;
        else
            // Yair 10/1/2007: only render arrow icons programatically for lower-priority sort columns
            return new Arrow(directive.direction == DESCENDING, size, sortingColumns.indexOf(directive));
    }
    
    private void cancelSorting() {
        sortingColumns.clear();
        sortingStatusChanged();
    }
    
    public void setColumnComparator(Class type, Comparator comparator) {
        if (comparator == null) {
            columnComparators.remove(type);
        } else {
            columnComparators.put(type, comparator);
        }
    }
    
    protected Comparator getComparator(int column) {
        //Class columnType = tableModel.getColumnClass(column);  // Yair 11/1/2007: see getColumnClass() comment below
        Class columnType = getColumnClass(column);               // Yair 11/1/2007: see getColumnClass() comment below
        Comparator comparator = (Comparator) columnComparators.get(columnType);
        if (comparator != null) {
            return comparator;
        }
        if (Comparable.class.isAssignableFrom(columnType)) {
            return COMPARABLE_COMAPRATOR;
        }
        return LEXICAL_COMPARATOR;
    }
    
    private Row[] getViewToModel() {
        if (viewToModel == null) {
            int tableModelRowCount = getRowCount();  // Yair 11/1/2007 null fix
            viewToModel = new Row[tableModelRowCount];
            for (int row = 0; row < tableModelRowCount; row++) {
                viewToModel[row] = new Row(row);
            }
            
            if (isSorting()) {
                Arrays.sort(viewToModel);
            }
        }
        return viewToModel;
    }
    
    public int modelIndex(int viewIndex) {
        return getViewToModel()[viewIndex].modelIndex;
    }
    
    private int[] getModelToView() {
        if (modelToView == null) {
            int n = getViewToModel().length;
            modelToView = new int[n];
            for (int i = 0; i < n; i++) {
                modelToView[modelIndex(i)] = i;
            }
        }
        return modelToView;
    }
    
    // Yair 14/1/2007: New public method
    public int viewIndex(int viewIndex) {
        return getModelToView()[viewIndex];
    }
    
    // TableModel interface methods
    
    public int getRowCount() {
        return (tableModel == null) ? super.getRowCount() : tableModel.getRowCount();
    }
    
    public int getColumnCount() {
        return (tableModel == null) ? super.getColumnCount() : tableModel.getColumnCount();
    }
    
    public String getColumnName(int column) {
        return (tableModel == null) ? super.getColumnName(column) : tableModel.getColumnName(column);  // fix for null case
    }
    
    // Yair 11/1/2007: several additional required interfaces
    
    public void addRow(Vector vec) {
        if (tableModel != null)
            tableModel.addRow(vec);
        else
            super.addRow(vec);
    }
    public void addRow(Object[] vec) {
        try {
            if (tableModel != null)
                tableModel.addRow(vec);
            else
                super.addRow(vec);
        } catch (Exception e) {
            // do nothing
        }
    }
    
    public void addColumn(Object name) {
        if (tableModel != null)
            tableModel.addColumn(name);
        else
            super.addColumn(name);
    }
    public void addColumn(Object name, Vector data) {
        if (tableModel != null)
            tableModel.addColumn(name,data);
        else
            super.addColumn(name,data);
    }
    public void addColumn(Object name, Object[] data) {
        if (tableModel != null)
            tableModel.addColumn(name,data);
        else
            super.addColumn(name,data);
    }
    
    public void insertRow(int beforeRow, Vector data) {
        if (tableModel != null)
            tableModel.insertRow(beforeRow, data);
        else
            super.insertRow(beforeRow, data);
    }
    public void insertRow(int beforeRow, Object[] data) {
        if (tableModel != null)
            tableModel.insertRow(beforeRow, data);
        else
            super.insertRow(beforeRow, data);
    }
    
    public void removeRow(int rowIdx) {
        if (tableModel != null)
            tableModel.removeRow(rowIdx);
        else
            super.removeRow(rowIdx);
    }
    
    public void moveRow(int a, int b, int c) {
        if (tableModel != null)
            tableModel.moveRow(a,b,c);
        else
            super.moveRow(a,b,c);
    }
    
    public void setNumRows(int numRows) {
        if (tableModel != null)
            tableModel.setNumRows(numRows);
        else
            super.setNumRows(numRows);
    }
    public void setRowCount(int numRows) {
        try {
            if (tableModel != null)
                tableModel.setRowCount(numRows);
            else
                super.setRowCount(numRows);
        } catch (Exception e) {
            // do nothing
        }
    }
    public void setColumnCount(int numCols) {
        if (tableModel != null)
            tableModel.setColumnCount(numCols);
        else
            super.setColumnCount(numCols);
    }
    
    public void setColumnIdentifiers(Vector colNames) {
        if (tableModel != null)
            tableModel.setColumnIdentifiers(colNames);
        else
            super.setColumnIdentifiers(colNames);
    }
    public void setColumnIdentifiers(Object[] colNames) {
        try {
            if (tableModel != null)
                tableModel.setColumnIdentifiers(colNames);
            else
                super.setColumnIdentifiers(colNames);
        } catch (Exception e) {
            // do nothing
        }
    }
    
    public Vector getDataVector() {
        return (tableModel == null) ? super.getDataVector() : tableModel.getDataVector();
    }
    public void setDataVector(Vector data1, Vector data2) {
        if (tableModel != null)
            tableModel.setDataVector(data1, data2);
        else
            super.setDataVector(data1, data2);
    }
    public void setDataVector(Object[][] data1, Object[] data2) {
        if (tableModel != null)
            tableModel.setDataVector(data1, data2);
        else
            super.setDataVector(data1, data2);
    }
    
    public String getColumnModel(int column) {
        return (tableModel == null) ? super.getColumnName(column) : tableModel.getColumnName(column);  // fix for null case
    }
    
    // Yair 11/1/2007: infer the column class from the actual data.
    // Otherwise, LEXICAL_COMPARATOR (not COMPARABLE_COMPARATOR) will be used also for numbers
    public Class getColumnClass(int column) {
        try {
            //return tableModel.getColumnClass(column);    // Yair - old code
            if (getRowCount() > 0) {
                if (isNoise[column]) {
                    Rectangle arrr = new Rectangle(0,0,1,1); // Noise columns return class Rectangle, which here is just a dummy class value in that isnt used by any other renderers. The corresponding renderer is RaveNoiseTableCellRenderer.
                    return arrr.getClass();
                } else {
                    return getValueAt(0, column).getClass();
                }
            } else {
                return (tableModel == null) ? super.getColumnClass(column) : tableModel.getColumnClass(column);  // revert back to model's class if no data; also-nullfix
            }
        }
        catch (Exception e)
        {
            return "".getClass();  // Yair 11/1/2007: null fix
        }
    }
    
    public boolean isCellEditable(int row, int column) {
        // return (tableModel == null) ? super.isCellEditable(modelIndex(row), column) : tableModel.isCellEditable(modelIndex(row), column);  // Yair 11/1/2007: null fix
        return (isFunction == null) ? true : !isFunction[column];
    }
    
    
    
    
    public Object getValueAt(int row, int column) {
        return (tableModel == null) ? super.getValueAt(modelIndex(row), column) : tableModel.getValueAt(modelIndex(row), column);  // Yair 11/1/2007: null fix
    }
    
    public void setValueAt(Object aValue, int row, int column) {
        //System.out.println("TableSorter\\setValueAt("+aValue+","+row+","+column+")");
        if (tableModel != null)  // Yair 11/1/2007: null fix
            tableModel.setValueAt(aValue, modelIndex(row), column);
        else
            super.setValueAt(aValue, modelIndex(row), column);
    }
    
    
    // Helper classes
    private class Row implements Comparable {
        private int modelIndex;
        
        public Row(int index) {
            this.modelIndex = index;
        }
        
        public int compareTo(Object o) {
            int row1 = modelIndex;
            int row2 = ((Row) o).modelIndex;
            int column = 0;
            Object o1 = null;
            Object o2 = null;
            try {
                for (Iterator it = sortingColumns.iterator(); it.hasNext();) {
                    Directive directive = (Directive) it.next();
                    column = directive.column;
                    o1 = tableModel.getValueAt(row1, column);
                    o2 = tableModel.getValueAt(row2, column);
                    
                    int comparison = 0;
                    // Define null less than everything, except null.
                    if (o1 == null && o2 == null) {
                        comparison = 0;
                    } else if (o1 == null) {
                        comparison = -1;
                    } else if (o2 == null) {
                        comparison = 1;
                    } else {
                        try {
                            comparison = getComparator(column).compare(o1, o2);
                        }
                        catch (Exception e) {
                            comparison = LEXICAL_COMPARATOR.compare(o1, o2);
                        }
                    }
                    if (comparison != 0) {
                        return directive.direction == DESCENDING ? -comparison : comparison;
                    }
                }
            }
            catch (Exception e)
            {
                System.out.println("Error comparing ("+row1+","+column+")="+o1+" to ("+row2+","+column+")="+o2+": "+e);
                //throw(e);
            }
            return 0;
        }
    }
    
    private class TableModelHandler implements TableModelListener {
        public void tableChanged(TableModelEvent e) {
            // If we're not sorting by anything, just pass the event along.
            if (!isSorting()) {
                clearSortingState();
                fireTableChanged(e);
                return;
            }
            
            // If the table structure has changed, cancel the sorting; the
            // sorting columns may have been either moved or deleted from
            // the model.
            if (e.getFirstRow() == TableModelEvent.HEADER_ROW) {
                cancelSorting();
                fireTableChanged(e);
                return;
            }
            
            // We can map a cell event through to the view without widening
            // when the following conditions apply:
            //
            // a) all the changes are on one row (e.getFirstRow() == e.getLastRow()) and,
            // b) all the changes are in one column (column != TableModelEvent.ALL_COLUMNS) and,
            // c) we are not sorting on that column (getSortingStatus(column) == NOT_SORTED) and,
            // d) a reverse lookup will not trigger a sort (modelToView != null)
            //
            // Note: INSERT and DELETE events fail this test as they have column == ALL_COLUMNS.
            //
            // The last check, for (modelToView != null) is to see if modelToView
            // is already allocated. If we don't do this check; sorting can become
            // a performance bottleneck for applications where cells
            // change rapidly in different parts of the table. If cells
            // change alternately in the sorting column and then outside of
            // it this class can end up re-sorting on alternate cell updates -
            // which can be a performance problem for large tables. The last
            // clause avoids this problem.
            int column = e.getColumn();
            if (e.getFirstRow() == e.getLastRow()
            && column != TableModelEvent.ALL_COLUMNS
            && getSortingStatus(column) == NOT_SORTED
            /*&& modelToView != null*/) {  // Yair 14/1/2007: never mind performance, we want the sorted (not original) data...
                int viewIndex = getModelToView()[e.getFirstRow()];
                fireTableChanged(new TableModelEvent(RaveTableSorter.this,
                viewIndex, viewIndex,
                column, e.getType()));
                return;
            }
            
            // Something has happened to the data that may have invalidated the row order.
            clearSortingState();
            //System.out.println("TableSorter: ("+e.getFirstRow()+","+e.getColumn()+"): "+modelToView);
            //fireTableDataChanged();  // Yair 14/1/2007: we need the row/column info within e...
            fireTableChanged(e);       // Yair 14/1/2007: we need the row/column info within e... - however, this returns the unsorted data, which is different from the view...
            return;
        }
    }
    
    
    
    
    
    private static class Arrow implements Icon {
        private boolean descending;
        private int size;
        private int priority;
        
        public Arrow(boolean descending, int size, int priority) {
            this.descending = descending;
            this.size = size;
            this.priority = priority;
        }
        
        public void paintIcon(Component c, Graphics g, int x, int y) {
            //Color color = c == null ? Color.GRAY : c.getBackground();
            Color color = c == null ? new Color(128,128,128) : c.getBackground();  //Yair: new Color(0,0,255);
            // In a compound sort, make each succesive triangle 20%
            // smaller than the previous one.
            int dx = (int)(size/2*Math.pow(0.8, priority-1));
            int dy = descending ? dx : -dx;
            // Align icon (roughly) with font baseline.
            y = y + 5*size/6 + (descending ? -dy : 0);
            int shift = descending ? 1 : -1;
            g.translate(x, y);
            
            // Right diagonal.
            g.setColor(color.darker());
            g.drawLine(dx / 2, dy, 0, 0);
            g.drawLine(dx / 2, dy + shift, 0, shift);
            
            // Left diagonal.
            g.setColor(color.brighter());
            g.drawLine(dx / 2, dy, dx, 0);
            g.drawLine(dx / 2, dy + shift, dx, shift);
            
            // Horizontal line.
            if (descending) {
                g.setColor(color.darker().darker());
            } else {
                g.setColor(color.brighter().brighter());
            }
            g.drawLine(dx, 0, 0, 0);
            
            g.setColor(color);
            g.translate(-x, -y);
        }
        
        public int getIconWidth() {
            return size;
        }
        
        public int getIconHeight() {
            return size;
        }
    }
    
    private class SortableHeaderRenderer implements TableCellRenderer {
        private TableCellRenderer tableCellRenderer;
        
        public SortableHeaderRenderer(TableCellRenderer tableCellRenderer) {
            this.tableCellRenderer = tableCellRenderer;
        }
        
        public Component getTableCellRendererComponent(JTable table,
        Object value,
        boolean isSelected,
        boolean hasFocus,
        int row,
        int column) {
            Component c = tableCellRenderer.getTableCellRendererComponent(table,
            value, isSelected, hasFocus, row, column);
            if (c instanceof JLabel) {
                JLabel l = (JLabel) c;
                l.setHorizontalTextPosition(JLabel.LEFT);
                int modelColumn = table.convertColumnIndexToModel(column);
                l.setIcon(getHeaderRendererIcon(modelColumn, l.getFont().getSize()));
                if (isFunction != null){
                    if (column < isFunction.length){
                        if (isFunction[modelColumn])
                            l.setForeground(Color.BLUE); //Make functions blue
                    }
                }
                
                if (isConstant!= null) {
                    if (column < isConstant.length){
                        if (isConstant[modelColumn])
                            l.setForeground(new Color(128,128,128)); //Make constants gray
                    }
                }
                
                if (isNoise != null) {
                    if (column < isNoise.length){
                        if (isNoise[modelColumn])
                            l.setForeground(new Color(160,0,160)); //Make noise purple
                    }
                }
				
				if (isVirtual != null) {
                    if (column < isVirtual.length){
                        if (isVirtual[modelColumn])
                            l.setForeground(new Color(0,116,107)); //Make virtuals turqoise
                    }
                }
				
				
                
                
            }
            return c;
        }
    }
    
    private static class Directive {
        private int column;
        private int direction;
        
        public Directive(int column, int direction) {
            this.column = column;
            this.direction = direction;
        }
    }
}
