function f=animationsettingsgui(src,ev)
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);
handles.temp=[];
guidata(handles.fig,handles);

%make figure:
f=figure('units','pixels','position',[0 0 460 460],'name',...
    'Optimizer Animation Settings','numbertitle','off','menubar','none','resize','off',...
    'color',handles.colors.frame,'windowstyle','normal','closerequestfcn',@closewindow);
origin=get(handles.fig,'position');
thispos=get(f,'position');
thispos(1)=origin(1)+origin(3)/2-thispos(3)/2;
thispos(2)=thispos(2)+origin(2)+origin(4)/2-thispos(4)/2;
set(f,'position',thispos);

closebutton=uicontrol('style','pushbutton','units','pixels','position',[165 10 140 30],'tag','cancelbutton',...
    'fontsize',16,'string','Done','callback',@closebutton_Callback);

u=uipanel('parent',f,'units','pixels','position',[30 50 400 400],'backgroundcolor',handles.colors.tabbg);

uicontrol('parent',u,'position',[15 280 200 22],'style','checkbox',...
    'string','Save Animation to File',...
    'backgroundcolor',handles.colors.tabbg,'fontsize',10,'hor','left',...
    'tag','anicheck','value',handles.prefs.animation.animation,'fontweight','bold');

uicontrol('parent',u,'position',[35 240 200 22],'style','text',...
    'string','File format:','fontweight','bold',...
    'backgroundcolor',handles.colors.tabbg,'fontsize',10,'hor','left');

formats={'gif';'avi'};
val=find(ismember(formats,handles.prefs.animation.format));
formatmenu=uicontrol('parent',u,'position',[115 245 200 20],'style','popupmenu',...
    'string',{'gif','avi'},'value',val,...
    'tag','formatmenu','backgroundcolor',[1 1 1],'callback',@formatmenu_callback);

uicontrol('parent',u,'position',[35 210 200 22],'style','text',...
    'string','Settings:','fontweight','bold',...
    'backgroundcolor',handles.colors.tabbg,'fontsize',10,'hor','left');

uicontrol('parent',u,'position',[15 370 250 22],'style','checkbox',...
    'string','Show optimizer info on graph',...
    'backgroundcolor',handles.colors.tabbg,'fontsize',10,'hor','left',...
    'tag','infocheck','value',handles.prefs.animation.showinfo,'fontweight','bold');

uicontrol('parent',u,'position',[35 340 200 22],'style','text',...
    'string','Location:',...
    'backgroundcolor',handles.colors.tabbg,'fontsize',10,'hor','left');

uicontrol('parent',u,'position',[35 310 200 22],'style','text',...
    'string','Font:',...
    'backgroundcolor',handles.colors.tabbg,'fontsize',10,'hor','left');

locs={'ne','nw','se','sw'};
val=find(ismember(locs,handles.prefs.animation.infolocation));
uicontrol('parent',u,'position',[95 345 200 20],'style','popupmenu',...
    'string',{'Top Right';'Top Left';'Bottom Right';'Bottom Left'},'value',val,...
    'tag','locationmenu','backgroundcolor',[1 1 1]);

val=find(ismember(handles.fonts.listfonts,handles.prefs.animation.infofont));
if isempty(val);val=1;end
uicontrol('parent',u,'position',[95 315 140 20],'style','popupmenu',...
    'string',handles.fonts.htmllistfonts,'value',val,...
    'tag','fontmenu','backgroundcolor',[1 1 1]);

val=find(ismember({'6';'7';'8';'9';'10';'11';'12';'14';'16';'18';'20';'22';'24';'28';'32';'40'},num2str(handles.prefs.animation.infofontsize)));
if isempty(val);val=7;end
uicontrol('parent',u,'position',[245 315 50 20],'style','popupmenu',...
    'string',{'6';'7';'8';'9';'10';'11';'12';'14';'16';'18';'20';'22';'24';'28';'32';'40'},'value',val,...
    'tag','fontsizemenu','backgroundcolor',[1 1 1]);

uicontrol('parent',u,'position',[15 45 370 22],'style','checkbox',...
    'string','Pause between iterations until mouse/key is pressed',...
    'backgroundcolor',handles.colors.tabbg,'fontsize',10,'hor','left',...
    'tag','pausecheck','value',handles.prefs.animation.pause,'fontweight','bold');

uicontrol('parent',u,'position',[15 25 370 22],'style','checkbox',...
    'string','Update Analysis''s "Current Point" after each iteration',...
    'backgroundcolor',handles.colors.tabbg,'fontsize',10,'hor','left',...
    'tag','updateanalysischeck','value',handles.prefs.animation.updateanalysis,'fontweight','bold');


handles.temp.formatobjects=[];
handles.temp.u=u;
handles.temp.f=f;

guidata(f,handles);

set(0,'userdata',handles.prefs.animation);

formatmenu_callback(formatmenu,[]);

%%
function closebutton_Callback(src,ev)
close(gcf);

function closewindow(src,ev)
try
process_settings(src,ev)
end
delete(src);


function formatmenu_callback(src,ev)
handles=guidata(src);

delete(handles.temp.formatobjects);
handles.temp.formatobjects=[];

val=get(src,'value');

settings=get(0,'userdata');
fps=num2str(settings.fps);
if isnan(fps);fps='10';end

if val==1 % gif
    txt1=uicontrol('position',[50 190 150 22],'parent',handles.temp.u,...
        'style','text','backgroundcolor',[1 1 1],'string','Frame Rate:',...
        'hor','left','fontsize',10,'fontweight','normal');
    radio1=uicontrol('position',[50 170 20 22],'parent',handles.temp.u,...
        'style','radio','backgroundcolor',[1 1 1],'tag','fpsradio',...
        'string','','value',~settings.realtime,'callback',@fpsradio_callback);
    editbox=uicontrol('position',[70 170 60 22],'parent',handles.temp.u,...
        'style','edit','backgroundcolor',[1 1 1],'tag','fpsbox',...
        'string',fps);
    txt2=uicontrol('position',[135 168 150 22],'parent',handles.temp.u,...
        'style','text','backgroundcolor',[1 1 1],'string','Frames per second',...
        'hor','left','fontsize',10,'fontweight','normal');
    radio2=uicontrol('position',[50 140 20 22],'parent',handles.temp.u,...
        'style','radio','backgroundcolor',[1 1 1],'tag','rtradio',...
        'string','','value',settings.realtime,'callback',@rtradio_callback);
    txt3=uicontrol('position',[70 138 250 22],'parent',handles.temp.u,...
        'style','text','backgroundcolor',[1 1 1],'string','Real time (averaged to constant frame rate)',...
        'hor','left','fontsize',10,'fontweight','normal');
    txt4=uicontrol('position',[50 100 150 22],'parent',handles.temp.u,...
        'style','text','backgroundcolor',[1 1 1],'string','Loop:',...
        'hor','left','fontsize',10,'fontweight','normal');
    editbox2=uicontrol('position',[90 102 60 22],'parent',handles.temp.u,...
        'style','edit','backgroundcolor',[1 1 1],'tag','loopbox',...
        'string',num2str(settings.loop));
    txt5=uicontrol('position',[155 100 100 22],'parent',handles.temp.u,...
        'style','text','backgroundcolor',[1 1 1],'string','times',...
        'hor','left','fontsize',10,'fontweight','normal');
    
    handles.temp.formatobjects=[txt1 txt2 editbox radio1 radio2 txt3 txt4 editbox2 txt5];
    
elseif val==2 % avi
    txt1=uicontrol('position',[50 190 150 22],'parent',handles.temp.u,...
        'style','text','backgroundcolor',[1 1 1],'string','Frame Rate:',...
        'hor','left','fontsize',10,'fontweight','normal');
    radio1=uicontrol('position',[50 170 20 22],'parent',handles.temp.u,...
        'style','radio','backgroundcolor',[1 1 1],'tag','fpsradio',...
        'string','','value',~settings.realtime,'callback',@fpsradio_callback);
    editbox=uicontrol('position',[70 170 60 22],'parent',handles.temp.u,...
        'style','edit','backgroundcolor',[1 1 1],'tag','fpsbox',...
        'string',fps);
    txt2=uicontrol('position',[135 168 150 22],'parent',handles.temp.u,...
        'style','text','backgroundcolor',[1 1 1],'string','Frames per second',...
        'hor','left','fontsize',10,'fontweight','normal');
    radio2=uicontrol('position',[50 140 20 22],'parent',handles.temp.u,...
        'style','radio','backgroundcolor',[1 1 1],'tag','rtradio',...
        'string','','value',settings.realtime,'callback',@rtradio_callback);
    txt3=uicontrol('position',[70 138 250 22],'parent',handles.temp.u,...
        'style','text','backgroundcolor',[1 1 1],'string','Real time (averaged to constant frame rate)',...
        'hor','left','fontsize',10,'fontweight','normal');
    txt4=uicontrol('position',[50 100 250 22],'parent',handles.temp.u,...
        'style','text','backgroundcolor',[1 1 1],'string','Quality:                  (0-100)',...
        'hor','left','fontsize',10,'fontweight','normal');
    editbox2=uicontrol('position',[100 102 60 22],'parent',handles.temp.u,...
        'style','edit','backgroundcolor',[1 1 1],'tag','qualitybox',...
        'string',num2str(settings.quality));
    txt5=uicontrol('position',[50 70 100 22],'parent',handles.temp.u,...
        'style','text','backgroundcolor',[1 1 1],'string','Compression:',...
        'hor','left','fontsize',10,'fontweight','normal');
    menu1=uicontrol('position',[140 72 100 22],'parent',handles.temp.u,...
        'style','popup','backgroundcolor',[1 1 1],'string',{'None'},...
        'hor','left','tag','qualitymenu');
    
    handles.temp.formatobjects=[txt1 txt2 editbox radio1 radio2 txt3 txt4 editbox2 txt5 menu1];
    
end

guidata(handles.temp.f,handles)


function fpsradio_callback(src,ev)
rtradio=findobj(gcf,'tag','rtradio');
set(rtradio,'value',0);
set(src,'value',1);

function rtradio_callback(src,ev)
fpsradio=findobj(gcf,'tag','fpsradio');
set(fpsradio,'value',0);
set(src,'value',1);


function process_settings(src,ev)
src=ancestor(src,'figure');
handles=guidata(src);
if isempty(handles.temp.formatobjects);return;end

formatmenu=findobj(src,'tag','formatmenu');
anicheck=findobj(src,'tag','anicheck');
infocheck=findobj(src,'tag','infocheck');
fontmenu=findobj(src,'tag','fontmenu');
fontsizemenu=findobj(src,'tag','fontsizemenu');
locationmenu=findobj(src,'tag','locationmenu');
pausecheck=findobj(src,'tag','pausecheck');
uacheck=findobj(src,'tag','updateanalysischeck');

settings=get(0,'userdata');
settings.animation=get(anicheck,'value');
settings.showinfo=get(infocheck,'value');
settings.pause=get(pausecheck,'value');
settings.updateanalysis=get(uacheck,'value');
fontz=handles.fonts.listfonts;
settings.infofont=fontz(get(fontmenu,'value'));
sizes=get(fontsizemenu,'string');
settings.infofontsize=str2double(sizes{get(fontsizemenu,'value')});
locs={'ne','nw','se','sw'};
settings.infolocation=locs{get(locationmenu,'value')};

fpsradio=findobj('tag','fpsradio');

fpsbox=findobj(src,'tag','fpsbox');
fps=str2double(get(fpsbox,'string'));
if isnan(fps) || fps<=0
    errordlg('You must enter a positive value in the Frames per second box')
    return
end
settings.fps=fps;

if get(fpsradio,'value')
    settings.realtime=0;
else
    settings.realtime=1;
end


if get(formatmenu,'value')==1 %gif
    settings.format='gif';
    
    loopbox=findobj(src,'tag','loopbox');
    loop=str2double(get(loopbox,'string'));
    
    if isnan(loop) || loop<1
        errordlg('You must enter value <=1 in the Loop box')
        return
    end
    settings.loop=loop;
    
elseif get(formatmenu,'value')==2
    settings.format='avi';
    
    qualitybox=findobj(src,'tag','qualitybox');
    quality=str2double(get(qualitybox,'string'));
    
    if isnan(quality) || quality<0 || quality>100
        errordlg('You must enter value between 0 and 100 in the Quality box')
        return
    end
    
    settings.quality=quality;
    
    qualitymenu=findobj(src,'tag','qualitymenu');
    strz=get(qualitymenu,'string');
    strr=strz{get(qualitymenu,'value')};
    settings.compression=strr;
end

set(0,'userdata',settings);

