function ravemouseoverderivativeprofileraxes(src,ev,ai)
%persistent handles udata

%hide mouseover text

handles=guidata(src);
stop(handles.mousetimer);

udata=get(handles.fakeax(ai),'userdata');


d=udata.view.dataset;
a=udata.view.analysis;

xmin=handles.minlimit{d}{a};
xmax=handles.maxlimit{d}{a};

xcol=udata.view.x;
ycol=udata.view.y;

numy=numel(ycol);

%find this plot's location in the matrix:
[junk,index]=find(handles.subax{ai}==gca);

if isempty(index);
    return
end


%If mouseing over a graph other than the one that was last clicked, return:
if gca~=hittest(handles.fig)
    set(udata.explore.mouseoverhandles,'visible','off')
    return
end



%if ~isempty(src) %if this function is called as a callback: (get mouse location and slide bars)
clicked=get(gca,'currentpoint');
val=clicked(1,1);

%move marker within bounds
if val>xmax(xcol(index))
    val=xmax(xcol(index));
elseif val<xmin(xcol(index))
    val=xmin(xcol(index));
end

%if this variable is non-continuous, round val to nearest allowable value:
thistype=handles.datatype{d}{xcol(index)};

if ~strcmp(thistype,'continuous')
    %find nearest neighbor in allowable values:
    allowed=handles.discretevalues{d}{a}{xcol(index)};
    val=allowed(abs(allowed-val)==min(abs(allowed-val)));
end

%if new value is the same as old value (because of discrete vars) don't
%waste time evaluating functions:
xdials=handles.xdials{d}{a};

if xdials(xcol(index))==val;
    return
end

xdials(xcol(index))=val;

%calculate profiler data for each graph:
if udata.view.singleyaxis
    rows=1;
else
    rows=length(udata.view.y);
end

cols=length(udata.view.x);
numpts=udata.view.numpts;

%get type of each variable:
datatypes=handles.datatype{d}(xcol);

[xsteps totalrows]=ravecalculatexsteps(src,d,a,xcol,numpts,1);

rowz=sum(totalrows);
xfixed=repmat(xdials,[rowz,1]);

%convert to cell if strings are present:
if any(strcmp('string',datatypes))
    xfixed=num2cell(xfixed);
    for k=1:cols
        [xfixed{sum(totalrows(1:k-1))+1:sum(totalrows(1:k)),xcol(k)}]=deal(xsteps{k});
    end
else
    for k=1:cols
        xfixed(sum(totalrows(1:k-1))+1:sum(totalrows(1:k)),xcol(k))=xsteps{k};
    end
end

%evaluate the functions:
yvalues=raveevaluatefunction(src,d,ycol,xfixed,handles);

doupdate=0;
for g=1:numy
    if min(yvalues(:,g))<handles.minlimit{d}{a}(ycol(g));handles.minlimit{d}{a}(ycol(g))=min(yvalues(:,g));doupdate=1;end
    if max(yvalues(:,g))>handles.maxlimit{d}{a}(ycol(g));handles.maxlimit{d}{a}(ycol(g))=max(yvalues(:,g));doupdate=1;end
end

if doupdate;
    ax=handles.subax{ai};
    fakeax=handles.subfakeax{ai};
    fakerax=handles.subfakerax{ai};
    ylimmin=handles.minlimit{d}{a}(ycol);
    ylimmax=handles.maxlimit{d}{a}(ycol);
    inset = .06;
    
    %fix y limits
    for r=1:rows,
        if ~udata.view.singleyaxis
            set([ax(r,1) fakeax(r,1) fakerax(r,1)],'ylim',[ylimmin(r) ylimmax(r)])
            dy = diff(get(ax(r,1),'ylim'))*inset;
            set([ax(r,:) fakeax(r,:) fakerax(r,:)],'ylim',[ylimmin(r)-dy ylimmax(r)+dy])
            set(udata.view.xhair(r,:),'ydata',[ylimmin(r)-dy ylimmax(r)+dy])
        else
            set([ax(r,1) fakeax(r,1) fakerax(r,1)],'ylim',[min(ylimmin) max(ylimmax)])
            dy = diff(get(ax(r,1),'ylim'))*inset;
            set([ax(r,:) fakeax(r,:) fakerax(r,:)],'ylim',[min(ylimmin)-dy max(ylimmax)+dy])
            set(udata.view.xhair(r,:),'ydata',[min(ylimmin)-dy max(ylimmax)+dy])
        end

    end
    
    for c=1:cols
        ylimits=get(ax(end,c),'Ylim');
        ypos=ylimits(1)-.02*(ylimits(2)-ylimits(1));
        ticks=findall(ax(end,c),'tag','xticklabel');
        
        posses=get(ticks,'position');
        if iscell(posses);posses=cell2mat(posses);end
        posses(:,2)=ypos;
        posses=mat2cell(posses,ones(size(posses,1),1),3);
        set(ticks,{'position'},posses);
    end
    
    guidata(src,handles)
end

ycurrent=raveevaluatefunction(src,d,ycol,xdials,handles);

%put results of each response in a seperate cell:
yvalues=mat2cell(yvalues,size(yvalues,1),ones(1,numy));

ycurrent=mat2cell(ycurrent,size(ycurrent,1),ones(1,numy));

%and put each x for each resposne in its own cell too:
for zz=1:numy
    yvalues{zz}=mat2cell(yvalues{zz},totalrows,1);
end

%update current-value texts for responses:
for tt=1:rows
    %set y label:
    g=udata.view.ylabels(tt);
    if udata.view.showcurrentpoint && ~udata.view.singleyaxis
%         set(g,'string',['\color[rgb]{',udata.format.linecolorString,'} ',strtrim(sprintf('%5.5g',ycurrent{tt}))]);
    end
    
    for cc=1:cols
        if  ~strcmp(datatypes{cc},'continuous') %if this variable is discrete
            allowed=handles.discretevalues{d}{a}{xcol(cc)};
            endind=numel(allowed);
            
            intsides=repmat(2:(endind-1),3,1);
            ymap=[1 1 intsides(:)' endind endind];
            
            xvals=xsteps{cc}(:); %force to be column.
            xintermediates=xvals+[diff(xvals)./2;0];
            
            xvals=sort([xvals;xintermediates(1:end-1)]);
            
            xmap=repmat(2:2:2*(endind-1),3,1);
            xmap(end,:)=xmap(end,:)+ones(size(xmap(end,:)));
            
            xmap=[1 xmap(:)'];
            
%             set(udata.view.yhair(tt,cc),'ydata',[ycurrent{tt} ycurrent{tt}])
%             set(udata.view.mainhandles(tt,cc),'ydata',yvalues{tt}{cc}(ymap),'xdata',xvals(xmap))
%             if udata.explore.mouseover && ~udata.view.singleyaxis
                set(udata.explore.mouseoverhandles(tt,cc),'ydata',yvalues{tt}{cc}(ymap),'xdata',xvals(xmap),'visible','on')
%             end
        else
            if ~udata.view.singleyaxis
             %   set(udata.view.yhair(tt,cc),'ydata',[ycurrent{tt} ycurrent{tt}])
             %   set(udata.view.mainhandles(tt,cc),'ydata',yvalues{tt}{cc},'xdata',xsteps{cc})
            %    if udata.explore.mouseover && ~udata.view.singleyaxis
                    set(udata.explore.mouseoverhandles(tt,cc),'ydata',yvalues{tt}{cc},'xdata',xsteps{cc},'visible','on')
            %    end
                
%             else
%                 for R=1:numy
%              %       set(udata.view.yhair(R,cc),'ydata',[ycurrent{R} ycurrent{R}])
%                     set(udata.view.mainhandles(R,cc),'ydata',yvalues{R}{cc},'xdata',xsteps{cc})
%                 end
            end
        end
    end
end
