function raveresizeaxesderivativeprofiler(src,eventdata,targetindex)
%RAVE: resize the current target axes to fit inside their bounding box,
%including axes titles/labels.
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

%Do custom resizing since we need to account for x/y labels of subax AND
%subfakeax

handles=guidata(src);
udata=get(handles.fakeax(targetindex),'userdata');
targetaxes=handles.allgraphs(targetindex);
targettype=get(handles.allgraphs(targetindex),'userdata');
targetpos=get(handles.allblockers(targetindex),'position');

%% Here comes the insanity:
axesformat=handles.graphinfo.(targettype).axesformat;

%1) Begin with default graph inset:

inset=[10 10 10 10];
targetpos=targetpos+[inset(1) inset(2) -(inset(1)+inset(3)) -(inset(2)+inset(4))];

%Shift by cvpos:
cvpos = get(handles.currentview,'position');
targetpos = targetpos - [cvpos(1) cvpos(2) 0 0];


%2) If graph has a colorbar, give it 100 pixels:
if handles.graphinfo.(targettype).hascolorbar;
    if udata.format.colorbar && udata.view.c;
        targetpos(3)=targetpos(3)-100;
    end
end

% 3) If graph has a title, allow for it based on font size:
htitle=get(handles.allgraphs(handles.activeindex),'title');
titlestr=get(htitle,'str');
if ~isempty(titlestr)
    ti=get(handles.allgraphs(handles.activeindex),'tightinset');
    targetpos=targetpos+[0 0 0 -ti(4)];
end

% 4) Space for xlabel and ylabel if they are present:

%determine which axes will have labels/ticks that affect placement:
%prefixes: l = left, b = bottom
switch axesformat
    case {'single','3d','3D','image'}
        lax=targetaxes;
        bax=targetaxes;
    case {'matrix','row'}
        lax=handles.subax{targetindex}(:,1);
        bax=handles.subax{targetindex}(end,:);
end

xlab=get(bax,'xlabel');
ylab=get(lax,'ylabel');

% if iscell(xlab);xlab=cell2mat(xlab);end
% if iscell(ylab);ylab=cell2mat(ylab);end

if iscell(xlab);xlab=[xlab{:}];end
if iscell(ylab);ylab=[ylab{:}];end


%get strings:
xstr=get(xlab,'string');
ystr=get(ylab,'string');

if ~iscell(xstr);xstr={xstr};end
if ~iscell(ystr);ystr={ystr};end

%Determine how many lines each label has:
for h=numel(xstr):-1:1
    if iscell(xstr{h})
        xlines(h)=numel(xstr{h});
    elseif ~isempty(xstr{h})
        xlines(h)=1;
    else
        xlines(h)=0;
    end
end

for h=numel(ystr):-1:1
    if iscell(ystr{h})
        ylines(h)=numel(ystr{h});
    elseif ~isempty(ystr{h})
        ylines(h)=1;
    else
        ylines(h)=0;
    end
end

%interpolate to guess how many pixels are needed based on number of lines
%and fontsize:
[junk,labelheight]=ravecalculatestringlengthfast('M',handles.fontmetrics.graph);

%If current values are displayed, allow for them too...
if udata.view.showcurrentpoint
    cvheight=ravecalculatestringlengthfast('M',handles.fontmetrics.tick);
else
    cvheight=0;
end

xaxisallowance=max(xlines)*labelheight+cvheight;
yaxisallowance=max(ylines)*labelheight+cvheight;

%add 5 extra pixels for padding between label and tick labels:
xaxisallowance=xaxisallowance+5;
yaxisallowance=yaxisallowance+5;

if udata.format.xlabelabove && udata.format.xlabelbelow
    targetpos=targetpos+[yaxisallowance xaxisallowance -yaxisallowance -2*xaxisallowance+20]; %not sure why but this seems to add way too much space. The +20 is to help a little
elseif udata.format.xlabelabove
    targetpos=targetpos+[yaxisallowance 0 -yaxisallowance -xaxisallowance];
elseif udata.format.xlabelbelow
    targetpos=targetpos+[yaxisallowance xaxisallowance -yaxisallowance -xaxisallowance];
else %no labels
    targetpos=targetpos+[yaxisallowance 0 -yaxisallowance 0];
end


%5) Calculate space for the tick marks on the left side of the graph
%(yticks for 2D graphs, zticks for 3D graphs)

lticks=findall(lax,'type','text','tag','yticklabel');
if ~isempty(lticks) %get strings from custom ticks:
    lstrings=get(lticks,'string');
else %get strings from axes:
    lstrings=get(lax,'yticklabel');
end


if ~iscell(lstrings)
    lstrings=mat2cell(lstrings,ones(size(lstrings,1),1),size(lstrings,2));
end

pixels=ravecalculatestringlengthfast(lstrings,handles.fontmetrics.tick);


maxpixels=max(pixels);
%allow for 5 pixels for space between ticks and axes: (added by ravemovecustomticks)
maxpixels=maxpixels+5;
targetpos=targetpos+[maxpixels 0 -maxpixels 0];
if targetpos(3)<20;targetpos(3)=20;end %enforce a minimum graph size

%6) If any ylim (zlim for 3d) has order of magnitude >10^4,
%   add 15 pixels at the top of the graph for the
%   exponential notation matlab puts there: (if there is a title,
%   don't bother since there's already room up there.)

llim=get(lax,'ylim');

if iscell(llim);llim=cell2mat(llim);end
if isempty(titlestr) && floor(log10(max(abs(llim(:)))))>3
    targetpos=targetpos+[0 0 0 -15];
end

%7) witdh is now fully calculated. apply positioning to get accurate xaxis scales:
newpos=floor(targetpos);
set(targetaxes,'position',single(newpos))
set(handles.fakeax(targetindex),'position',single(newpos))
set(handles.fakerax(targetindex),'position',single(newpos))

% do layout:
% ravearrangeobjects(targetaxes,[]);

%8) Determine what to do with x tick labels:
%get x tick label strings:
if udata.format.xlabelbelow %if not labeling below, then there are no x ticks...
    for b=1:numel(bax)
        
        xticks=findall(bax(b),'type','text','tag','xticklabel');
        if ~isempty(xticks) %get labels from axis:
            xstrings=get(xticks,'string');
        else
            xstrings=get(bax,'xticklabel');
        end
        
        if ~iscell(xstrings)
            xstrings=mat2cell(xstrings,ones(size(xstrings,1),1),size(xstrings,2));
        end
        
        %if xstrings are cells, make them just be strings (not sure what causes
        %them to be cells, but it breaks betterlinewrap:
        for s=1:numel(xstrings)
            if iscell(xstrings{s})
                xstrings{s}=[xstrings{s}{:}];
            end
        end
        
        %determine number of rows in tick labels:
        if ~isempty(xstrings)
            xlim=get(bax(b),'xlim');
            thispos=get(bax(b),'position');
            
            %determine how many pixels are available for each label:
            %axpos=getpixelposition(src);
            w=thispos(3);
            ticks=get(bax(b),'xtick');
            tickpix=interp1(xlim,[0 w],ticks);
            
            %subtract half of last ticklabel width from axes to allow room
            %for rightmost tick label, but only if this tick is within 10%
            %of max xlim
            if numel(tickpix)>1 && max(ticks)>(0.9*(xlim(2)-xlim(1))+xlim(1));
                lastgap=tickpix(end)-tickpix(end-1);
                wid=ravecalculatestringlengthfast(xstrings{end},handles.fontmetrics.graph);
                thispos=thispos+[0 0 -min([lastgap*0.4 wid 30]) 0];
                w=thispos(3);
                ticks=get(bax(b),'xtick');
                tickpix=interp1(xlim,[0 w],ticks);
            end
            
            if numel(tickpix)>1
                divs=tickpix(1:end-1)+diff(tickpix)/2;
                overshoot=floor(min(20,w/6)); %number of pixels beyond width of axes that tick labels are allowed to extend.
                maxtickwidth=diff([-overshoot divs w+overshoot]);
                maxtickwidth(1)=maxtickwidth(2);
                maxtickwidth(end)=maxtickwidth(end-1);
            else %if only one tick label:
                maxtickwidth=w;
            end
            
            %determine wrapping based on maxtickwidth:
            nrows=1;
            if ismember('alwaysrotatexticks',udata.attributes)
                status=false(1,numel(xticks));
            else
                status=true(1,numel(xticks));
            end
            
            for s=numel(xticks):-1:1;
                if all(status) %if no ticks need to be rotated, keep wrapping (if any are rotated, no need to wrap)
                    %[wrapped,status(s)]=superiorlinewrap(xstrings{s},maxtickwidth(s),handles.fonts.tick,handles.fonts.ticksize,'plain',handles.axiscanvas);
                    [wrapped,status(s)]=superiorlinewrapfast(xstrings{s},maxtickwidth(s),handles.fontmetrics.tick);
                    if iscell(wrapped) && numel(wrapped)>nrows;nrows=numel(wrapped);end
                    set(xticks(s),'string',wrapped,'rotation',0,'hor','center','vert','top');
                end
            end
            
            if any(status==0) %if any are rotated
                set(xticks,'rotation',90,'hor','right','ver','middle');
                for s=1:numel(xticks)
                    set(xticks(s),'string',xstrings{s})
                end
            end
            
        else %no ticks
            status=1; %unrotated
            nrows=0;
        end

        %determine pixels for height:
        if any(status==0) %tick labels are rotated, so base height on string length:
            for g=1:numel(xstrings)
                if iscell(xstrings{g});xstrings{g}=xstrings{g}{:};end
            end
            pixels=ravecalculatestringlengthfast(xstrings,handles.fontmetrics.tick);
            
            maxpixels(b)=max(pixels);
        else %base height on number of rows of text:
            [junk,tickheight]=ravecalculatestringlengthfast('M',handles.fontmetrics.tick); %M is as good a letter as anything...
            if isempty(xticks)
                maxpixels(b)=nrows*tickheight;
            else
                maxpixels(b)=nrows*tickheight;
            end
        end
    end
    
    %5 pixels extra for space between axes on tick labels:
    maxpixels=maxpixels+5;
    maxpixels=max(maxpixels);
    targetpos=targetpos+[0 maxpixels 0 -maxpixels];
end

if targetpos(4)<20;targetpos(4)=20;end

%% Resize graph:
ticklength=handles.prefs.ticklength./max(targetpos(3),targetpos(4));
newpos=floor(targetpos);

set([targetaxes handles.fakeax(targetindex) handles.fakerax(targetindex)],'position',single(newpos),'ticklength',[ticklength ticklength])


%% do layout:
ravearrangeobjects(targetaxes,[]);

%% notify explore methods of resize:
raveupdateexploremethods(handles.fig,handles,targetindex,'resize');

