function raveupdatecontour(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

%this is the buttondownfcn for hyperslice axes, to update the x
%value of the selected graph and redraw all graphs in the new position.
%Also includes buttonmotionfcn version.

handles=guidata(src);
%udata=get(handles.fakeax(handles.activeindex),'userdata');

ai=find(handles.allgraphs==src);

udata=get(handles.fakeax(ai),'userdata');
if ~udata.view.showcurrentpoint
    return
end

raveslidecontour(handles.fig,[],ai);
%make sliding along graph update continuously:
set(handles.fig,'windowbuttonupfcn',{@raveslidedone,ai})
set(handles.fig,'windowbuttonmotionfcn',{@raveslidecontour,ai})

function raveslidecontour(src,ev,ai)
%NOTE: SRC FOR THIS FUNCTION IS EQUAL TO HANDLES.FIG REGARDLESS OF WHAT WAS
%CLICKED.
%persistent handles
handles=guidata(src);
udata=get(handles.fakeax(ai),'userdata');

d=udata.view.dataset;
a=udata.view.analysis;

mins=handles.minlimit{d}{a};
maxs=handles.maxlimit{d}{a};


xcol=udata.view.x;
ycol=udata.view.y;
zcol=udata.view.z;

numy=numel(ycol);

%if ~isempty(src) %if this function is called as a callback: (get mouse location and slide bars)
clicked=get(gca,'currentpoint');
xval=clicked(1,1);
yval=clicked(1,2);

%move marker within bounds
if xval>maxs(xcol)
    xval=maxs(xcol);
elseif xval<mins(xcol)
    xval=mins(xcol);
end

if yval>maxs(ycol)
    yval=maxs(ycol);
elseif yval<mins(ycol)
    yval=mins(ycol);
end


%if this variable is non-continuous, round val to nearest allowable value:
thistype=handles.datatype{d}{xcol};
if ~strcmp(thistype,'continuous')
    %find nearest neighbor in allowable values:
    allowed=handles.discretevalues{d}{a}{xcol};
    xval=allowed(abs(allowed-xval)==min(abs(allowed-xval)));
end

thistype=handles.datatype{d}{ycol};
if ~strcmp(thistype,'continuous')
    %find nearest neighbor in allowable values:
    allowed=handles.discretevalues{d}{a}{ycol};
    yval=allowed(abs(allowed-yval)==min(abs(allowed-yval)));
end

xval=xval(1); %sometimes weird rounding can give two values.
yval=yval(1);

set(udata.view.xhair,'xdata',[xval xval])
set(udata.view.yhair,'ydata',[yval yval])

handles.xdials{d}{a}(xcol)=xval;
handles.xdials{d}{a}(ycol)=yval;

%determine current c value:
thisvalue=raveevaluatefunction(src,udata.view.dataset,udata.view.c,handles.xdials{udata.view.dataset}{udata.view.analysis},handles);
if thisvalue<handles.minlimit{d}{a}(udata.view.z);handles.minlimit{d}{a}(udata.view.z)=thisvalue;end
if thisvalue>handles.maxlimit{d}{a}(udata.view.z);handles.maxlimit{d}{a}(udata.view.z)=thisvalue;end

guidata(handles.fig,handles);

%if colorbar is displayed, show current value:
if udata.format.colorbar
    legdata=get(handles.fakerax(ai),'userdata');
    cbar=findobj(legdata,'tag','Colorbar');
    oldlim=udata.view.clim;
    newlim=[handles.minlimit{d}{a}(udata.view.z) handles.maxlimit{d}{a}(udata.view.z)];
    udata.view.clim=newlim;
    set(handles.fakeax(ai),'userdata',udata);
    ylim=get(cbar,'ylim');
     if ~all(oldlim==newlim)
         %guidata(handles.fig,handles);
         raveupdatecolorbar(handles.fig,[],ai);
         legdata=get(handles.fakerax(ai),'userdata');
         cbar=findobj(legdata,'tag','Colorbar');
     end

    thiscoord=interp1(newlim,ylim,thisvalue);
    %make triangle patch at this value:
    cvpatch=findobj(cbar,'tag','currentvalue');
    set(cvpatch,'ydata',[thiscoord-1.5 thiscoord+1.5 thiscoord]);
    cvtext=findobj(cbar,'tag','currentvaluetxt');
    if ~isempty(cvtext)
        pos=get(cvtext,'pos');
        set(cvtext,'pos',[pos(1) thiscoord],'string',sprintf('%5.5g',thisvalue));
    end
end

%handles.xdials{1}{1}

%update on slide if desired:
if handles.prefs.alwaysupdate && ~handles.prefs.alwaysrasterize
    raveupdateanalysis(src,handles,'xdials',ai,udata.view.dataset,udata.view.analysis);
end

function raveslidedone(src,ev,ai)
handles=guidata(src);
udata=get(handles.fakeax(ai),'userdata');
set(handles.fig,'windowbuttonmotionfcn',@ravedefaultwindowbuttonmotionfcn)
set(handles.fig,'windowbuttonupfcn','')

%update other analyses to reflect new values:
raveupdateanalysis(handles.fig,handles,'xdials',ai,udata.view.dataset,udata.view.analysis);

clear contourplot %clear persistent vars




