function handles=raveupdatesurfaceprofiler(src,ev,handles)
%update the current surface profiler because the xdials have changed.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.



if nargin==2
    handles=guidata(src);
end

udata=get(handles.fakeax(handles.activeindex),'userdata');

d=udata.view.dataset;
a=udata.view.analysis;
%
% prefs=handles.preferences{d};
% targets=handles.targetvalues{d};

xcol=udata.view.x;
ycol=udata.view.y;
zcol=udata.view.z;
ccol=udata.view.c;

xdials=handles.xdials{d}{a};

%get type of each variable:
datatypes=handles.datatype{d}([xcol ycol]);

totalrows=zeros(size(datatypes)); %totalrows lists the total number of rows in the input data that correspond to the graph of this input.
%thus final input data has sum(totalrows) rows.

xmin=handles.minlimit{d}{a};
xmax=handles.maxlimit{d}{a};

numpts=udata.view.numpts;

step=(xmax-xmin)/numpts;
xsteps=cell(1,numel(datatypes));

colz=[xcol ycol];

doupdate=0;

[xsteps totalrows]=ravecalculatexsteps(src,d,a,colz,numpts,0);

%evaluation grid will be numel(xsteps{2}) by numel(xsteps{1}) elements.

yevals=repmat(xsteps{2}(:),[numel(xsteps{1}) 1]);
index=repmat(1:numel(xsteps{1}),[numel(xsteps{2}) 1]);
xevals=xsteps{1}(index(:));

%make whole matrix of poitns to eval. this is fixed array of xdials, with xeval/yeval subbed in:
indata=repmat(xdials,[numel(xevals) 1]);
indata(:,xcol)=xevals;
indata(:,ycol)=yevals;

%eval z at indata:
zvalues=raveevaluatefunction(src,d,zcol,indata,handles);

%update axis limits only if needed:
if min(zvalues)<handles.minlimit{d}{a}(zcol);handles.minlimit{d}{a}(zcol)=min(zvalues);doupdate=1;end
if max(zvalues)>handles.maxlimit{d}{a}(zcol);handles.maxlimit{d}{a}(zcol)=max(zvalues);doupdate=1;end

%reshape to square matrices:
xdata=reshape(xevals,[numel(xsteps{2}) numel(xsteps{1})]);
ydata=reshape(yevals,[numel(xsteps{2}) numel(xsteps{1})]);
zdata=reshape(zvalues,[numel(xsteps{2}) numel(xsteps{1})]);

if ccol~=0
    c=raveevaluatefunction(src,d,ccol,indata,handles);
    if min(c)<handles.minlimit{d}{a}(ccol);handles.minlimit{d}{a}(ccol)=min(c);doupdate=1;end
    if max(c)>handles.maxlimit{d}{a}(ccol);handles.maxlimit{d}{a}(ccol)=max(c);doupdate=1;end
    
    minc=handles.minlimit{d}{a}(ccol);
    maxc=handles.maxlimit{d}{a}(ccol);
    
    %normalize cdata to desired colormap:
    c=(c-minc)/(maxc-minc)*127+128*(udata.format.mcmap-1)+1;
    cdata=reshape(c,[numel(xsteps{2}) numel(xsteps{1})]);
    
else
    cdata=[0 .5 1]; %non-noisy colors cdata=0.5
end

%if either xcol or ycol is discrete, need to fold surface:
main=findobj(handles.activegraph,'tag','main');

%fold if xvar is discrete:
if  ~strcmp(datatypes{1},'continuous') %if this variable is discrete
    allowed=handles.discretevalues{d}{a}{xcol};
    endind=numel(allowed);
    
    xvals=xsteps{1}(:); %force to be column.
    xintermediates=xvals+[diff(xvals)./2;0];
    xvals=sort([xvals;xintermediates(1:end-1)]);
    
    xmap=repmat(2:2:2*(endind-1),2,1);
    xmap=[1 xmap(:)' xmap(end)+1];
    
    zmap=repmat(1:numel(xmap)/2,2,1);
    zmap=zmap(:)';
    
    xdata=repmat(xvals(xmap)',size(xdata,1),1);
    ydata=repmat(ydata(:,1),1,numel(xmap));
    zdata=zdata(:,zmap);
    cdata=cdata(:,zmap);
end

%fold if yvar is discrete:
if  ~strcmp(datatypes{2},'continuous') %if this variable is discrete
    allowed=handles.discretevalues{d}{a}{ycol};
    endind=numel(allowed);
    
    yvals=xsteps{2}(:); %force to be column.
    yintermediates=yvals+[diff(yvals)./2;0];
    yvals=sort([yvals;yintermediates(1:end-1)]);
    
    ymap=repmat(2:2:2*(endind-1),2,1);
    ymap=[1 ymap(:)' ymap(end)+1];
    
    zmap=repmat(1:numel(ymap)/2,2,1);
    zmap=zmap(:)';
    
    ydata=repmat(yvals(ymap),1,size(ydata,2));
    xdata=repmat(xdata(1,:),numel(ymap),1);
    zdata=zdata(zmap,:);
    cdata=cdata(zmap,:);
end

set(main,'xdata',xdata,'ydata',ydata,'zdata',zdata,'cdata',cdata);

if udata.view.showcurrentpoint
    cp=handles.xdials{d}{a};
    cpz=raveevaluatefunction(handles.fig,d,zcol,cp,handles);
    %nudge up 4 pixels so it sits on top of the point instaed of centered
    %on the point:
    vvv=get(handles.activegraph,'view');
    el=vvv(2);
    pos=get(handles.activegraph,'position');
    zpix=pos(4)*cos(el*2*pi/180);
    zrange=diff(get(handles.activegraph,'zlim'));
    
    zshift=6*zrange/zpix;
    set(udata.view.currentpointhandle,'xdata',cp(xcol),'ydata',cp(ycol),'zdata',cpz+zshift);
end

if doupdate;
    targetaxes=get(main,'parent');
    zlims=[handles.minlimit{d}{a}(zcol) handles.maxlimit{d}{a}(zcol)];
    xlims=[handles.minlimit{d}{a}(xcol) handles.maxlimit{d}{a}(xcol)];
    ylims=[handles.minlimit{d}{a}(ycol) handles.maxlimit{d}{a}(ycol)];
    
    try %#ok<TRYNC> %gives error if x/y is constant. just let it use auto lims in that case.
        set(targetaxes,'xlim',xlims,'ylim',ylims,'zlim',zlims);
    end
    
    if nargin==2
        guidata(handles.fig,handles);
    end
    
end

if udata.format.colorbar && udata.view.showcurrentpoint
    ai=handles.activeindex;
    legdata=get(handles.fakerax(ai),'userdata');
    cbar=findobj(legdata,'tag','Colorbar');
    oldlim=udata.view.clim;
    newlim=[handles.minlimit{d}{a}(udata.view.z) handles.maxlimit{d}{a}(udata.view.z)];
    udata.view.clim=newlim;
    set(handles.fakeax(ai),'userdata',udata);
    ylim=get(cbar,'ylim');
    if ~all(oldlim==newlim)
        %guidata(handles.fig,handles);
        raveupdatecolorbar(handles.fig,[],ai);
        legdata=get(handles.fakerax(ai),'userdata');
        cbar=findobj(legdata,'tag','Colorbar');
    end
    
    thiscoord=interp1(newlim,ylim,cpz);
    %make triangle patch at this value:
    cvpatch=findobj(cbar,'tag','currentvalue');
    set(cvpatch,'ydata',[thiscoord-1.5 thiscoord+1.5 thiscoord]);
    cvtext=findobj(cbar,'tag','currentvaluetxt');
    pos=get(cvtext,'pos');
    set(cvtext,'pos',[pos(1) thiscoord],'string',sprintf('%5.5g',cpz));
end


