function filterclick(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%NOTE: all of these functions need to work REGARDLESS OF WHICH GRAPH IS
%SELECTED. so no using activeindex or activegraph!

handles=guidata(src);

uistack(src,'top'); %important: this prevents the filter from getting stuck under the opposing filter.

set(handles.fig,'windowbuttonmotionfcn',{@filterslide,src})
set(handles.fig,'windowbuttonupfcn',{@filterdone,src})

function filterslide(fig,eventdata,src) %#ok<INUSL>
handles=guidata(fig);
ai=find(handles.allgraphs==get(src,'parent'));
udata=get(handles.fakeax(ai),'userdata'); 

d=udata.view.dataset;
a=udata.view.analysis;

thisvar=get(gco,'userdata');
thistype=handles.datatype{udata.view.dataset}{thisvar};
allowablevalues=handles.discretevalues{udata.view.dataset}{udata.view.analysis}{thisvar};

if ~isempty(fig) %if this function is called as a callback: (get mouse location and slide bars)
    clicked=get(get(src,'parent'),'currentpoint');
    val=clicked(1,1);

    %move marker within bounds
    lims=[0.43 0.73];
    switch get(src,'tag')
        case 'low'
            lims(2)=get(findobj(udata.view.highfilterblobs,'userdata',thisvar),'xdata'); 
            idx=udata.view.lowfilterblobs==src;
           % udata.view.lowfilters(thisvar)=get(src,'xdata');
        case 'high'
            lims(1)=get(findobj(udata.view.lowfilterblobs,'userdata',thisvar),'xdata');      
            idx=udata.view.highfilterblobs==src;
           % udata.view.highfilters(thisvar)=get(src,'xdata');
    end
    
    if val>lims(2)
        val=lims(2);
    elseif val<lims(1)
        val=lims(1);
    end
    
    %round to nearest allowable point:
    if ~any(strcmp(thistype,{'continuous','integer'}))
        allowed=linspace(.43,.73,numel(allowablevalues));
        val=allowed(abs(allowed-val)==min(abs(allowed-val)));
        val=val(1); %in case above line has 2 mins.
        newval=find(abs(allowed-val)==min(abs(allowed-val)));
    else
        range=[handles.minlimit{d}{a}(thisvar) handles.maxlimit{d}{a}(thisvar)];
        if range(1)-range(2)==0;return;end %avoids errors with constant variables.
        newval=range(1)+((val-0.43)/0.3)*(range(2)-range(1));
    end
  
    set(gco,'xdata',val)

    %make current value texts:
    if ~any(strcmp(thistype,{'string','logical','discrete','integer'}))
        valtext=strtrim(sprintf(udata.format.formatstring,newval));
    elseif strcmp(thistype,'integer')
        valtext=strtrim(sprintf(udata.format.formatstring,round(newval)));
    elseif strcmp(thistype,'discrete')
        newval=allowablevalues(newval);
        valtext=strtrim(sprintf(udata.format.formatstring,newval));
    elseif strcmp(thistype,'string')
        valtext=handles.discretevalues{d}{a}{thisvar}{newval};
        %valtext=newval;
    else %logical
        if newval
            valtext=handles.prefs.logicaltrue;
        else
            valtext=handles.prefs.logicalfalse;
        end
    end

    xdata=get(udata.view.blobs(idx),'xdata');

    switch get(src,'tag')
        case 'low'
            udata.view.lowfilters(thisvar)=newval;
            set(udata.view.minvaltexts(idx),'string',valtext,'color',[0 0 0]);
            xdata(1)=get(src,'xdata');
        case 'high'
            udata.view.highfilters(thisvar)=newval;
            set(udata.view.maxvaltexts(idx),'string',valtext,'color',[0 0 0]);
            xdata(2)=get(src,'xdata');
    end
end

set(udata.view.blobs(idx),'xdata',xdata);
set(handles.fakeax(ai),'userdata',udata);

%update global filters:
handles.minfilter{d}{a}=udata.view.lowfilters;
handles.maxfilter{d}{a}=udata.view.highfilters;

%update discrete filters:
switch handles.datatype{d}{thisvar}
    case 'string'
        newval=false(size(handles.discretefilter{d}{a}{thisvar}));
        newval(udata.view.lowfilters(thisvar):udata.view.highfilters(thisvar))=true;
        handles.discretefilter{d}{a}{thisvar}=newval(:);      
    case {'discrete','integer','logical'};       
        discretevalues=handles.discretevalues{d}{a}{thisvar};
        newval=discretevalues>=udata.view.lowfilters(thisvar) & discretevalues<=udata.view.highfilters(thisvar);
        handles.discretefilter{d}{a}{thisvar}=newval(:);
end

guidata(handles.fig,handles);

%update on slide if desired:
 if handles.prefs.alwaysupdate && ~handles.prefs.alwaysrasterize
    filterapplyfilter(get(src,'parent'),[]);
 end

function filterdone(fig,eventdata,src) %#ok<INUSL>
handles=guidata(fig);
set(handles.fig,'windowbuttonmotionfcn',@ravedefaultwindowbuttonmotionfcn)
set(handles.fig,'windowbuttonupfcn','')

filterapplyfilter(get(src,'parent'),[]);



