function formattabfilter(src,eventdata)
%rave format tab.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');

%move front tabs to front if necessary
set(handles.fronttabaxes,'position',handles.fronttabposition)
set(handles.backtabaxes,'position',handles.backtabposition)
[x,y,v]=calculatetabprofile(handles.backtabprofile,.1,1,0);
set(handles.backtablines,'xdata',x,'ydata',y);

[x,y,v]=calculatetabprofile(handles.fronttabprofile,.1,3,1);
set(handles.fronttablines,'xdata',x,'ydata',y)

origin=get(handles.tabaxes,'position');


%% clear existing tab
delete(handles.tabobjects)

%% do work
graphtext=text(.15-.10317,.97*1.0317-.0317,'Appearance','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',handles.fonts.titlesize,'fontname',handles.fonts.title,'color',handles.colors.title);

fonttext=text(.15-.10317,.73,'Font','horizontalalignment','left','backgroundcolor',handles.colors.tabbg,...
    'parent',handles.tabaxes,'fontsize',handles.fonts.titlesize,'fontname',handles.fonts.title,'color',handles.colors.title);

bgtext=uicontrol('position',[origin(1)+25 origin(2)+575 180 20],...
    'style','text','string','Background Color:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');
bgidx=udata.format.bg;

% color blobs: (userdata = gridcolor; textcolor; index)
bgwhite=uicontrol('position',[origin(1)+155 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',handles.colors.axes{1}(2,:),'value',bgidx==1,...
    'callback',@bgblob_callback,'userdata',1,'tag','bgblob');
bglight=uicontrol('position',[origin(1)+180 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',handles.colors.axes{2}(2,:),'value',bgidx==2,...
    'callback',@bgblob_callback,'userdata',2,'tag','bgblob');
bgdark=uicontrol('position',[origin(1)+205 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',handles.colors.axes{3}(2,:),'value',bgidx==3,...
    'callback',@bgblob_callback,'userdata',3,'tag','bgblob');
bgblack=uicontrol('position',[origin(1)+230 origin(2)+578 20 20],...
    'style','togglebutton','string','','backgroundcolor',handles.colors.axes{4}(2,:),'value',bgidx==4,...
    'callback',@bgblob_callback,'userdata',4,'tag','bgblob');

colortext=uicontrol('position',[origin(1)+25 origin(2)+543 180 20],...
    'style','text','string','Slider Color:','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','bold');

facecheck=uicontrol('position',[origin(1)+35 origin(2)+500+25 150 20],'style','radiobutton',...
    'string','Single Color','value',~udata.format.usevariablecolors,'backgroundcolor',handles.colors.tabbg,...
    'callback',@facecheck_callback,'tag','facecheck','fontsize',10);

facecolor=uicontrol('position',[origin(1)+130 origin(2)+497+25 25 25],...
    'style','pushbutton','string','','backgroundcolor',udata.format.facecolor,'value',0,...
    'callback',@faceblob_callback);

varcolorscheck=uicontrol('position',[origin(1)+35 origin(2)+542-50 150 20],'style','radiobutton',...
    'string','Use Variable Colors','value',udata.format.usevariablecolors,'backgroundcolor',handles.colors.tabbg,...
    'callback',@varcheck_callback,'tag','varcolors','fontsize',10,'visible','off');

shownamescheck=uicontrol('position',[origin(1)+25 origin(2)+330 150 20],'style','checkbox',...
    'string','Show variable names','value',udata.format.shownames,'backgroundcolor',handles.colors.tabbg,...
    'callback',@shownamescheck_callback,'tag','shownames','fontsize',10);




%% font stuff
fontlist=handles.fonts.listfonts;

sizelist={'6';'7';'8';'9';'10';'11';'12';'14';'16';'18';'20';'22';'24';'28';'32';'36';'42';'48'};

fontvalue=find(strcmpi(fontlist,udata.format.fontname),1);
if isempty(fontvalue)
    fontvalue=1;
end

sizevalue=find(strcmp(sizelist,num2str(udata.format.fontsize)),1);
if isempty(sizevalue)
    sizevalue=1;
end

[style,javastyle,boldvalue,italicvalue]=ravegetfontstyle(udata.format.fontstyle);

%uicontrols
fontpicker=uicontrol('position',[origin(1)+25 origin(2)+44+380 219 22],'style','popupmenu',...
    'string',fontlist,'parent',handles.fig,'max',10,'value',fontvalue,...
    'backgroundcolor',[1 1 1],'tag','fontpicker','callback',@fontpicker_callback);

boldbutton=uicontrol('position',[origin(1)+25 origin(2)+10+380 24 24],'style','togglebutton',...
    'string','','callback','','backgroundcolor',[0 0 0],...
    'fontname','arial narrow','fontsize',12,'parent',handles.fig,'cdata',imread('bold.png'),...
    'tag','boldbutton','callback',@fontpicker_callback,'value',boldvalue);

italicbutton=uicontrol('position',[origin(1)+55 origin(2)+10+380 24 24],'style','togglebutton',...
    'string','','callback','','backgroundcolor',[0 0 0],...
    'fontname','arial narrow','fontsize',12,'parent',handles.fig,'cdata',imread('italic.bmp'),...
    'tag','italicbutton','callback',@fontpicker_callback,'value',italicvalue);

fontdownbutton=uicontrol('position',[origin(1)+145 origin(2)+10+380 24 24],'style','pushbutton',...
    'string','','callback','','backgroundcolor',[0 0 0],...
    'fontname','arial narrow','fontsize',12,'parent',handles.fig,'cdata',imread('fontdown.bmp'),...
    'callback',@fontdown_callback);

fontsizer=uicontrol('position',[origin(1)+176 origin(2)+10+380 40 22],'style','popupmenu',...
    'string',sizelist,...
    'parent',handles.fig,'max',1,'value',sizevalue,...
    'backgroundcolor',[1 1 1],'tag','fontsizer','callback',@fontpicker_callback);

fontupbutton=uicontrol('position',[origin(1)+220 origin(2)+10+380 24 24],'style','pushbutton',...
    'string','','callback','','backgroundcolor',[0 0 0],...
    'fontname','arial narrow','fontsize',12,'parent',handles.fig,'cdata',imread('fontup.bmp'),...
    'callback',@fontup_callback);

fontcolortext=uicontrol('position',[origin(1)+25 origin(2)+355 180 20],...
    'style','text','string','Text Color','backgroundcolor',handles.colors.tabbg,...
    'fontname','arial','fontsize',10,'horizontalalignment','left','fontweight','normal');

textcolor=uicontrol('position',[origin(1)+95 origin(2)+356 25 25],...
    'style','pushbutton','string','','backgroundcolor',udata.format.textcolor,'value',0,...
    'callback',@textblob_callback);



%% group items and finish up
handles.tabobjects=[graphtext varcolorscheck shownamescheck facecheck facecolor...
    bgtext bgwhite bglight bgdark bgblack colortext fontsizer fontupbutton fontdownbutton...
    italicbutton boldbutton fontpicker fonttext fontcolortext textcolor];

handles.activetab='format';

set(handles.infobar,'string','');

guidata(src,handles);


function facecheck_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
varcolors=findobj(handles.tabobjects,'tag','varcolors');
set(varcolors,'value',0);
udata.format.usevariablecolors=false;
set(handles.fakeax(handles.activeindex),'userdata',udata);

%update:
ravecreatefilter(handles.fig,[]);


function varcheck_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
facecheck=findobj(handles.tabobjects,'tag','facecheck');
set(facecheck,'value',0);
udata.format.usevariablecolors=true;
set(handles.fakeax(handles.activeindex),'userdata',udata);

%update:
ravecreatefilter(handles.fig,[]);


function faceblob_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
newcolor=ravecolorchooser(handles.fig,udata.format.facecolor,'Choose a slider color');
set(src,'backgroundcolor',newcolor);
udata.format.facecolor=newcolor;
set(handles.fakeax(handles.activeindex),'userdata',udata);

%update:
ravecreatefilter(handles.fig,[]);

function shownamescheck_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
udata.format.shownames=get(src,'value');
set(handles.fakeax(handles.activeindex),'userdata',udata);

%update:
ravecreatefilter(handles.fig,[]);


function bgblob_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
uicontrol(handles.dummyobj)

bgblobs=findobj(handles.tabobjects,'tag','bgblob');
set(bgblobs,'value',0);
set(src,'value',1);

newidx=get(src,'userdata');
udata.format.bg=newidx;
set(handles.fakeax(handles.activeindex),'userdata',udata);

ravecreatefilter(src,ev);

function fontdown_callback(src,ev)
handles=guidata(src);
fontsizer=findobj(handles.tabobjects,'tag','fontsizer');

value=get(fontsizer,'value');
if value>=2
    value=value-1;
    set(fontsizer,'value',value);
    fontpicker_callback(src,ev);
end

function fontup_callback(src,ev)
handles=guidata(src);
fontsizer=findobj(handles.tabobjects,'tag','fontsizer');
maxvalue=numel(get(fontsizer,'string'));

value=get(fontsizer,'value');
if value<maxvalue
    value=value+1;
    set(fontsizer,'value',value);
    fontpicker_callback(src,ev);
end

function fontpicker_callback(src,ev) %#ok<INUSD>

handles=guidata(src);
getudata;

fontpicker=findobj(handles.tabobjects,'tag','fontpicker');
fontsizer=findobj(handles.tabobjects,'tag','fontsizer');
boldbutton=findobj(handles.tabobjects,'tag','boldbutton');
italicbutton=findobj(handles.tabobjects,'tag','italicbutton');

fontlist=get(fontpicker,'string');
fontname=fontlist(get(fontpicker,'value'));

sizelist=get(fontsizer,'string');
fontsize=str2double(sizelist{get(fontsizer,'value')});

if get(boldbutton,'value') && get(italicbutton,'value')
    fontstyle=3;
elseif get(boldbutton,'value')
    fontstyle=1;
elseif get(italicbutton,'value')
    fontstyle=2;
else
    fontstyle=4;
end

udata.format.fontname=fontname{1};
udata.format.fontsize=fontsize;
udata.format.fontstyle=fontstyle;

set(handles.fakeax(handles.activeindex),'userdata',udata);
guidata(src,handles);

ravecreatefilter(handles.fig,[]);


function textblob_callback(src,ev)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
newcolor=ravecolorchooser(handles.fig,udata.format.textcolor,'Choose a text color');
set(src,'backgroundcolor',newcolor);
udata.format.textcolor=newcolor;
set(handles.fakeax(handles.activeindex),'userdata',udata);

%update:
ravecreatefilter(handles.fig,[]);