function ravecreatediscretecontrols(src,ev)
%create discrete graph controls on workspace.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);
%get format inputs:
udata=get(handles.fakeax(handles.activeindex),'userdata');


targetaxes=handles.activegraph; %=handles.activegraph
xcol=udata.view.x;
a=udata.view.analysis;
d=udata.view.dataset; %data set index

set(targetaxes,'visible','off')
set(handles.fakeax(handles.activeindex),'visible','off')
set(handles.fakerax(handles.activeindex),'visible','off')

origin=get(targetaxes,'position');

%make pixel coordinates:
set(targetaxes,'xlim',[1 origin(3)],'ylim',[1 origin(4)],'xticklabel','','yticklabel','');
text(0,88,'Select Data:','parent',targetaxes,'fontweight','bold');

switch handles.selectionmode
    case 'point'
        vals=[1 0 0];
    case 'lasso'
        vals=[0 0 1];
    case 'box'
        vals=[0 1 0];
end


pointtoggle=drawtogglebutton(handles.wspanel,[origin(1)+10 origin(2)+50 26 26],handles.colors.togglebuttonbg,imread('singlepoint.bmp'),'',@ravesetpointselectionmode,vals(1));
boxtoggle=drawtogglebutton(handles.wspanel,[origin(1)+40 origin(2)+50 26 26],handles.colors.togglebuttonbg,imread('box.bmp'),'',@ravesetboxselectionmode,vals(2));
lassotoggle=drawtogglebutton(handles.wspanel,[origin(1)+70 origin(2)+50 26 26],handles.colors.togglebuttonbg,imread('lasso.bmp'),'',@ravesetlassoselectionmode,vals(3));

set(pointtoggle,'tag','pointtoggle')
set(boxtoggle,'tag','boxtoggle')
set(lassotoggle,'tag','lassotoggle')

text(120,88,'Hide Selected Data:','parent',targetaxes,'fontweight','bold');
hidebutton=drawpushbutton(handles.wspanel,[origin(1)+130 origin(2)+50 26 26],handles.colors.togglebuttonbg,imread('hide.bmp'),'',@hidebutton_callback);


text(0,30,'Recolor Selected Data:','parent',targetaxes,'fontweight','bold');
%make color buttons:
colortestbutton=zeros(1,10);
thecolors=handles.prefs.rowcolors;

for c=1:10
    colortestbutton(c)=drawpushbutton(handles.wspanel,[origin(1)+10+22*(c-1) origin(2)+0 18 18],...
        thecolors(c,:),[],'',{@rowcolorblob_callback,handles.activeindex});
    set(colortestbutton(c),'userdata',c);
end

udata.view.allbuttons=[colortestbutton pointtoggle boxtoggle lassotoggle hidebutton];
udata.view.colorbuttons=colortestbutton;
udata.view.pointbutton=pointtoggle;
udata.view.boxbutton=boxtoggle;
udata.view.lassobutton=lassotoggle;
udata.view.hidebutton=hidebutton;


set(handles.fakeax(handles.activeindex),'userdata',udata);

raveresizeaxes(handles.fig,[],handles.activeindex);
clear ravearrangeobjects


function rowcolorblob_callback(src,ev,ai)
handles=guidata(src);

udata=get(handles.fakeax(ai),'userdata');

d=udata.view.dataset;
a=udata.view.analysis;

thesepoints=handles.isselected{d}{a};
handles.rowcolors{d}{a}(thesepoints)=uint16(get(src,'userdata'));

guidata(handles.fig,handles);
raveupdateanalysis(handles.fig,handles,'rowcolors',[],d,a);


function hidebutton_callback(src,ev,value)
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
handles.isvisible{udata.view.dataset}{udata.view.analysis}(handles.isselected{udata.view.dataset}{udata.view.analysis})=false;
raveupdateanalysis(handles.activegraph,handles,'visiblerows',[],udata.view.dataset,udata.view.analysis);
