function handles=raveupdatedensityisselected(src,ev,handles)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2012 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

%handles=guidata(src);
datasets=raveguidata(handles.fig,'datasets');
udata=get(handles.fakeax(handles.activeindex),'userdata');

targetaxes=handles.activegraph;
xcol=udata.view.x;
ycol=udata.view.y;
ccol=udata.view.c;
scol=udata.view.s;
a=udata.view.analysis;
d=udata.view.dataset; %data set index

%get data
x=datasets.alldata{d}(:,xcol);
y=datasets.alldata{d}(:,ycol);

isselected=handles.isselected{d}{a};
nothidden=handles.isvisible{d}{a};
isfiltered=handles.isfiltered{d}{a};

%Normalize Data:
if isempty(udata.format.xlim);
    minx=min(x);
    maxx=max(x);
else
    minx=udata.format.xlim(1);
    maxx=udata.format.xlim(2);
end

if isempty(udata.format.ylim);
    miny=min(y);
    maxy=max(y);
else
    miny=udata.format.ylim(1);
    maxy=udata.format.ylim(2);
end

rangex=maxx-minx;
rangey=maxy-miny;

%normalize data before using inpolygon:
x=x-minx;
x=x./rangex;

y=y-miny;
y=y./rangey;

%% generate grid:
% For hexagons....
% Generate hexagonal grid

ny=udata.view.ny;
nx=udata.view.nx;

X=repmat(linspace(0,1,nx),ny,1);

Y=linspace(0,1,2*ny);
Y=reshape(Y,2,[])';

if rem(nx,2)==0 %nx is even
    Y=repmat(Y,1,nx/2);
else
    Y=repmat(Y,1,(nx-1)/2+1);
end

rx=diff(X(1,1:2))*(2/3);
ry=diff(Y(1:2,1))/sqrt(3);

X=X(:);
Y=Y(:);

thetas=[0 60 120 180 240 300]*pi/180;

inhere=udata.view.inhere;

inherefiltered=bsxfun(@and,inhere,nothidden);
thismanyfiltered=sum(inherefiltered,1);

inhere=bsxfun(@and,inhere,~isfiltered);
inhere=bsxfun(@and,inhere,nothidden);
%thismany=sum(inhere,1);

inhereselected=bsxfun(@and,inherefiltered,nothidden);
inhereselected=bsxfun(@and,inhereselected,isselected);
thismanyselected=sum(inhereselected,1);


%dont let any tiles get drawn at less than 0.3 scale
tilescale=sqrt(thismanyfiltered)./max((sqrt(thismanyfiltered)));
tilescale(tilescale<0.1)=0.1;
%
% tilescale=sqrt(thismany)./max((sqrt(thismanyfiltered)));
% tilescale(tilescale<0.1)=0.1;


%% SIZE TILES
if scol==-1 %Do density sizing
    sizescale=tilescale;
elseif scol==0 %Do uniform sizing
    sizescale = ones(numel(X),1);
else %size by avg of selected variable.
    s=zeros(numel(X),1);
    for h=1:numel(X)
        s(h)=mean(datasets.alldata{d}(inhere(:,h)',scol));
    end
    sizescale=s-min(s);
    sizescale=sizescale./max(sizescale);
end

%% Draw the patches:
%denormalize data:
if ~isempty(x)   
    sverts=zeros(numel(X)*6,2);

    faces=(1:(6*numel(X)))';

    sisvacant=false(numel(X),1);

    
    for h=1:numel(X)
       
        %selected data:
        if thismanyselected(h)>0 && (~udata.view.preciseoutliers && thismanyfiltered(h)>0) || (udata.view.preciseoutliers && thismanyfiltered(h)>1)
            sxhex=sqrt(thismanyselected(h))/sqrt(thismanyfiltered(h))*sizescale(h)*rx*cos(thetas)+X(h);
            syhex=sqrt(thismanyselected(h))/sqrt(thismanyfiltered(h))*sizescale(h)*ry*sin(thetas)+Y(h);
        elseif thismanyselected(h)>0 && thismanyfiltered(h)==1
            thisone=inpolygon(x,y,rx*cos(thetas)+X(h),ry*sin(thetas)+Y(h));
            sxhex=sqrt(thismanyselected(h))/sqrt(thismanyfiltered(h))*sizescale(h)*rx*cos(thetas)+x(thisone);
            syhex=sqrt(thismanyselected(h))/sqrt(thismanyfiltered(h))*sizescale(h)*ry*sin(thetas)+y(thisone);
        else
            sisvacant(h)=true;
            sxhex=zeros(6,1);
            syhex=zeros(6,1);
        end
        
        sverts((h-1)*6+1:h*6,1)=sxhex(:);
        sverts((h-1)*6+1:h*6,2)=syhex(:);
    end
end

%restore original scale:
sverts(:,1)=sverts(:,1)*rangex+minx;
sverts(:,2)=sverts(:,2)*rangey+miny;

%crop to stay within plot bounds:

sverts(sverts(:,1)<minx,1)=minx;
sverts(sverts(:,1)>maxx,1)=maxx;
sverts(sverts(:,2)<miny,2)=miny;
sverts(sverts(:,2)>maxy,2)=maxy;


%make selected data series:
[suverts,junk,suidx]=unique(sverts,'rows');
spatchdata.vertices=suverts;
sfacemat=reshape(faces(suidx),6,[])';
spatchdata.faces=sfacemat(~sisvacant,:);
set(udata.view.selectedhandles,'faces',spatchdata.faces,'vertices',spatchdata.vertices);




