function handles=raveupdatehistogramdatachanged(src,ev,handles,thesevars)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

ai=handles.activeindex;

%get format inputs:
udata=get(handles.fakeax(handles.activeindex),'userdata');
xcol=udata.view.x;
a=udata.view.analysis;
d=udata.view.dataset; %data set index

%% make the graph.

%how many graphs to draw?
numvars=length(xcol);

[mainrows,selectedrows,filteredrows]=ravemakeplotdataseries(handles,udata);

isnoisy=handles.isnoise{d}(xcol);

ax=handles.subax{ai};
fakeax=handles.subfakeax{ai};
fakerax=handles.subfakerax{ai};

for c=1:numvars
    if isnoisy(c) %nothing to update for random variables
        continue
    end
    
    [maincount,selectedcount,filteredcount,nbars,edges]=...
        ravehistogramcount(handles,d,a,xcol(c),[],mainrows,selectedrows,filteredrows,...
        udata.view.nbars,udata.view.discreteascategory);
    udata.view.edges{c}=edges;
    if  udata.view.normalize || isnoisy(c) %normalize counts (make pdfs)
        nrows=size(handles.isselected{d}{1},1);
        maincount=maincount./nrows;
        selectedcount=selectedcount./nrows;
        filteredcount=filteredcount./nrows;
    end
    
    % MAIN
    main=findobj(get(handles.subax{ai}(c),'children'),'tag','main');
    maincount=maincount(:);
    maindata=[zeros(size(maincount,1),2),maincount,maincount,zeros(size(maincount,1),1)];
    maindata=maindata';
    mh=get(main,'children');
    mh=findobj(mh,'type','patch'); %not sure why, but main includes several rectangle objects as well...
    mv=get(mh,'vertices');
    mv(1:end-1,2)=maindata(:);
    set(mh,'vertices',mv);
    
    %SELECTED
    selected=findobj(get(handles.subax{ai}(c),'children'),'tag','selected');
    selectedcount=selectedcount(:); %ensure it is a column (sometimes it turns into a row, not sure why)
    seldata=[zeros(size(selectedcount,1),2),selectedcount,selectedcount,zeros(size(selectedcount,1),1)];
    seldata=seldata';
    sh=get(selected,'children');
    sv=get(sh,'vertices');
    sv(1:end-1,2)=seldata(:);
    set(sh,'vertices',sv);
    
    %FILTERED
    filtered=findobj(get(handles.subax{ai}(c),'children'),'tag','filtered');
    filteredcount=filteredcount(:); %ensure it is a column (sometimes it turns into a row, not sure why)
    fildata=[zeros(size(filteredcount,1),2),filteredcount,filteredcount,zeros(size(filteredcount,1),1)];
    fildata=fildata';
    sh=get(filtered,'children');
    sv=get(sh,'vertices');
    sv(1:end-1,2)=fildata(:);
    set(sh,'vertices',sv);
    
    if udata.view.showfilter
        lox=findobj(handles.subax{ai}(c),'tag','loxfilter');
        hix=findobj(handles.subax{ai}(c),'tag','hixfilter');
        xlimz=get(handles.subax{ai}(c),'xlim');
        minf=handles.minfilter{d}{a}(xcol(c));
        maxf=handles.maxfilter{d}{a}(xcol(c));
        
        set(lox,'xdata',minf)
        set(hix,'xdata',maxf)
        
        if minf+eps(minf)<xlimz(1)
            set(lox,'visible','off')
        else
            set(lox,'visible','on')
        end
        
        if maxf>xlimz(2)+eps(xlimz(2));
            set(hix,'visible','off')
        else
            set(hix,'visible','on')
        end
    end
    
    
    axis(ax(c),'tight');
    set(ax(c),'ylimmode','auto');
    
    %Make ticks:
%     yticks=get(ax(c),'yticklabel');
%     if ~iscell(yticks);
%         yticks=mat2cell(yticks,ones(size(yticks,1),1),size(yticks,2));
%         yticks=cellfun(@strtrim,yticks,'uniformoutput',0);
%     end
    
    ravemaketextticks(ax(c),[],xcol(c),-1,[],[],[])
    set([fakeax(c) fakerax(c)],'xtick',get(ax(c),'xtick'),'xticklabel','','ytick',get(ax(c),'ytick'),'yticklabel','',...
        'ztick',get(ax(c),'ztick'),'zticklabel','')
    
end


