function ravecreatebarcategorical(src,ev)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);
datasets=raveguidata(handles.fig,'datasets');

udata=get(handles.fakeax(handles.activeindex),'userdata');


%raveresizeaxes(src,[],handles.activeindex) %make appropriate margins for this graph type.
%inputs=varargin{3};

targetaxes=handles.activegraph;
xcol=udata.view.x;
ycol=udata.view.y;
zcol=udata.view.z;


a=udata.view.analysis;
d=udata.view.dataset; %data set index

%clear existing graph
labels=findall(handles.activegraph,'tag','xticklabel','-or','tag','yticklabel');
delete(labels);
leg=get(handles.fakerax(handles.activeindex),'userdata');
if ~isempty(leg)
    delete(leg(ishandle(leg)));
end
% set(targetaxes,'xtickmode','manual','ytickmode','manual','xticklabelmode','manual','yticklabelmode','manual')
cla(targetaxes);
% legend(targetaxes,'off');

set(targetaxes,'nextplot','add','xlimmode','auto')
% set(targetaxes,'xtickmode','manual','ytickmode','manual','xticklabelmode','manual','yticklabelmode','manual')


if xcol==0
    text(0.5,0.5,{'Data set must contain at least';'one discrete or string variable';'in order to create a bar chart.'},'hor','center','ver','mid','parent',handles.activegraph);
    return
end


%get data
if strcmp(handles.datatype{d}{xcol},'string')
    xvalues=1:numel(handles.discretevalues{d}{a}{xcol});
else
    xvalues=handles.discretevalues{d}{a}{xcol}; %discrete
end

if zcol>0
    if strcmp(handles.datatype{d}{zcol},'string')
        zvalues=1:numel(unique(handles.discretevalues{d}{a}{zcol}));
        
    else
        zvalues=unique(handles.discretevalues{d}{a}{zcol});
    end
    
elseif zcol==0 %row color.
    zvalues=1:10;
else
    zvalues=1;
end

if ycol>0
    y=datasets.alldata{d}(:,ycol);
else
    y=ones(size(handles.isselected{d}{1},1),1);
end

x=datasets.alldata{d}(:,xcol);

if zcol>0
    z=datasets.alldata{d}(:,zcol);
end

%make bar data:
bardata=zeros(numel(xvalues),numel(zvalues));

if zcol>0 %stack by variable
    
    for c=1:numel(zvalues)
        for r=1:numel(xvalues)
            idx = z==zvalues(c) & x==xvalues(r);
            if any(idx)
                bardata(r,c)=aggregate(y(idx),udata.view.aggregateby);
            end
        end
    end
elseif zcol==0 %stack by row color
    for c=1:numel(zvalues)
        for r=1:numel(xvalues)
            idx = handles.rowcolors{d}{a}==c & x==xvalues(r);
            if any(idx)
                bardata(r,c)=aggregate(y(idx),udata.view.aggregateby);
            end
        end
    end
    
elseif zcol==-1 %no stacking
    for r=1:numel(xvalues)
        idx = x==xvalues(r);
        if any(idx)
            bardata(r)=aggregate(y(idx),udata.view.aggregateby);
        end
    end
end

bardata(isnan(bardata))=0;

if udata.view.stack
    layoutstring='stacked';
else
    layoutstring='grouped';
end


%Sort data if necessary:
if udata.view.sortby==2
    totalheight = sum(bardata,2);
    [junk,order]=sort(totalheight,'descend');
%     xvalues = xvalues(order);
    bardata = bardata(order,:);
elseif udata.view.sortby==3
    totalheight = sum(bardata,2);
    [junk,order]=sort(totalheight,'ascend');
%     xvalues = xvalues(order);
    bardata = bardata(order,:);    
else
    order = 1:numel(xvalues);    
end
    
%store order to updateticks can easily access it:
udata.view.order = order;
set(handles.fakeax(handles.activeindex),'userdata',udata);
    

if numel(xvalues)==1;
    set(handles.infobar,'string','Sorry, I can''t make a bar graph when the x-axis data is constant');
    return;
    
else %Make bars:
    hh=ravebar(targetaxes,xvalues,bardata,udata.view.barwidth,layoutstring,false,false,handles.ishg2);
    set(hh,'clipping','on')
end

%% Set Colors

nstacks=size(bardata,2);

if zcol~=0
    allcolors=handles.prefs.barcolors;
else
    allcolors=handles.prefs.rowcolors;
end


ncolors=size(allcolors,1);

reps=ceil(nstacks/ncolors);
allcolors=repmat(allcolors,reps,1);
allcolors = allcolors(1:nstacks,:);

if udata.format.drawedge
    edgecolor=udata.format.edgecolor;
else
    edgecolor='none';
end

for n=1:nstacks
    set(hh(n),'facecolor',allcolors(n,:),'edgecolor',edgecolor);
end

%% Do formatting
if strcmp(handles.datatype{d}{xcol},'string')
    set(targetaxes,'xlim',[0 numel(xvalues)+1])
else
    set(targetaxes,'xlimmode','auto')
end

if strcmp(udata.format.orient,'horizontal')
    view(targetaxes,90,90);
    view(handles.fakeax(handles.activeindex),90,90);
    view(handles.fakerax(handles.activeindex),90,90);
end

set(handles.fakerax(handles.activeindex),'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'))
set(handles.fakeax(handles.activeindex),'xlim',get(targetaxes,'xlim'),'ylim',get(targetaxes,'ylim'))

if strcmp(udata.format.orient,'horizontal')
    view(handles.activegraph,90,90);
    view(handles.fakeax(handles.activeindex),90,90);
    view(handles.fakerax(handles.activeindex),90,90);
end

set(targetaxes,'color','none');

%set axes labels
set(get(targetaxes,'XLabel'),'String',handles.allnames{d}{xcol},'fontsize',handles.fonts.graphsize,...
    'fontweight','normal','interpreter','none','fontname',handles.fonts.graph)

if ycol>0 %if plotting 1 response
    set(get(targetaxes,'YLabel'),'String',handles.allnames{d}{ycol},...
        'fontsize',handles.fonts.graphsize,'fontweight','normal','interpreter','none','fontname',handles.fonts.graph)
else
    set(get(targetaxes,'YLabel'),'String','Count',...
        'fontsize',handles.fonts.graphsize,'fontweight','normal','interpreter','none','fontname',handles.fonts.graph)
end


%adjust gridlines to match settings:
if udata.format.grid==1 %no grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')
elseif udata.format.grid==2 %major grid
    set(handles.fakerax(handles.activeindex),'XGrid','on','YGrid','on')
    set(handles.fakerax(handles.activeindex),'XminorGrid','off','YminorGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','off','YGrid','off')
elseif udata.format.grid==3; %minor grid
    set(handles.fakerax(handles.activeindex),'XminorGrid','on','YminorGrid','on')
    set(handles.fakerax(handles.activeindex),'XGrid','off','YGrid','off')
    set(handles.fakeax(handles.activeindex),'XGrid','on','YGrid','on')
end

set(targetaxes,'userdata','barcategorical');

%update da handles
guidata(src,handles)


%make legend
if udata.format.legend
    if zcol>0
        legstrings=handles.discretevalues{d}{a}{zcol};
        if isnumeric(legstrings)
            legstrings=num2cell(legstrings);
            for c=1:numel(legstrings);
                legstrings{c}=num2str(legstrings{c});
            end
        end
        
        if zcol>0
            legtitle=handles.allnames{d}{zcol};
        else
            legtitle='Row Color';
        end
        
        leg1=ravemakelegend(src,[],handles.activeindex,legstrings,hh,[],legtitle);
        set(handles.fakerax(handles.activeindex),'userdata',leg1);
    end
end

set(targetaxes,'xtickmode','manual');

%Put strings in sorted order:
xstrs = handles.discretevalues{d}{a}{udata.view.x}(order);

if ycol>0
    ravemaketextticks(targetaxes,[],xcol,ycol,[],[],xstrs)
else
%     yticks=get(targetaxes,'ytick');
%     yticklabels=num2cell(yticks);
%     yticklabels=cellfun(@num2str,yticklabels,'UniformOutput',false);
    ravemaketextticks(targetaxes,[],xcol,-1,[],[],xstrs)
end



raveresizeaxes(handles.fig,[],handles.activeindex);
clear ravearrangeobjects

raveupdateexploremethods(handles.fig,handles,handles.activeindex,'recreate');


function y=aggregate(yvec,mode)

if mode==1
    y=sum(yvec);
elseif mode==2
    y=mean(yvec);
elseif mode==3
    y=min(yvec);
elseif mode==4
    y=max(yvec);
else
    y=numel(yvec);
end




