function newsubaxeschangelistener(src,ev,axisname,newvalue,handles)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2011 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.


%This function runs any time the axes are resized and xtickmode etc are set to auto, regardless of whether
%the ticks actually change. ALso runs when xticklabelmodes are set to auto,
%even if other things are manual. Note that it usually does not run
%immediately after the xlim/ylim properties are set, but waits until a
%drawnow or a get of the axis properties.

% persistent handles

if isempty(axisname);return;end

ax=double(src);
ai=get(ax,'userdata'); %all matrix or row type graphs must store activeindex in the subaxes' userdata.

if isempty(handles) || handles.fig~=ancestor(ax,'figure') %second case is in case you're running 2 rave's at once
    handles=guidata(ax);
end

[r,c]=find(handles.subax{ai}==ax);

if any(axisname=='x')
    try %gives error when resizing tables because tables store a uipanel in fakerax
        set([handles.subfakeax{ai}(r,c) handles.subfakerax{ai}(r,c)],'xlim',get(ax,'xlim'),...
            'xtick',newvalue{1});
    end
end

if any(axisname=='y')
    try%gives error when resizing tables because tables store a uipanel in fakerax
        set([handles.subfakeax{ai}(r,c) handles.subfakerax{ai}(r,c)],...
            'ylim',get(ax,'ylim'),'ytick',newvalue{2});
    end
end

if any(axisname=='z')
    try%gives error when resizing tables because tables store a uipanel in fakerax
        set([handles.subfakeax{ai}(r,c) handles.subfakerax{ai}(r,c)],...
            'zlim',get(ax,'zlim'),'ztick',newvalue{3});
    end
end


thistype=get(handles.allgraphs(ai),'userdata');
if ~isempty(which(['raveupdateticks',thistype]))
    guidata(handles.fig,handles); %this is important, because updateticks functions need most recent handles
    eval(['raveupdateticks',thistype,'(ax,[],''xyz'');'])
end




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
return












ax=double(src);
ai=get(ax,'userdata'); %all matrix or row type graphs must store activeindex in the subaxes' userdata.

if isempty(handles) || handles.fig~=ancestor(ax,'figure') %second case is in case you're running 2 rave's at once
    handles=guidata(ax);
end

[r,c]=find(handles.subax{ai}==ax);

switch axisname
    case 'XTick'
        try %gives error when resizing tables because tables store a uipanel in fakerax
            set([handles.subfakeax{ai}(r,c) handles.subfakerax{ai}(r,c)],'xlim',get(ax,'xlim'),...
                'xtick',newvalue);
        end
        
        %check if this graph has an associated tick update function:
        thistype=get(handles.allgraphs(ai),'userdata');
        if ~isempty(which(['raveupdateticks',thistype]))
            oldmode=get(ax,'xtickmode');
            set(ax,'xtick',newvalue); %as a consequence, this sets xtickmode to manual, stopping this function from running again.
            set(ax,'xtickmode',oldmode);
            %At this point, the new ticks have been set, but even with
            %xticklabelmode set to auto, the tick labels will not update
            %until axeschangelistener finishes running.
            eval(['raveupdateticks',thistype,'(ax,[],''xyz'');'])
        end
    case 'YTick'
        try%gives error when resizing tables because tables store a uipanel in fakerax
            set([handles.subfakeax{ai}(r,c) handles.subfakerax{ai}(r,c)],...
                'ylim',get(ax,'ylim'),'ytick',newvalue);
        end
        
        %check if this graph has an associated tick update function:
        thistype=get(handles.allgraphs(ai),'userdata');
        if ~isempty(which(['raveupdateticks',thistype]))
            set(ax,'ytick',newvalue); %as a consequence, this sets xtickmode to manual, stopping this function from running again.
            set(ax,'ytickmode','auto');
            %eval(['raveupdateticks',thistype,'(ax,[],''xyz'');'])
            updatefun=str2func(['raveupdateticks',thistype]);
            updatefun(ax,[],'xyz');
        end
        
    case 'ZTick'
        try%gives error when resizing tables because tables store a uipanel in fakerax
            set([handles.subfakeax{ai}(r,c) handles.subfakerax{ai}(r,c)],...
                'zlim',get(ax,'zlim'),'ztick',newvalue);
        end
        
        %check if this graph has an associated tick update function:
        thistype=get(handles.allgraphs(ai),'userdata');
        if ~isempty(which(['raveupdateticks',thistype]))
            set(ax,'ztick',newvalue); %as a consequence, this sets xtickmode to manual, stopping this function from running again.
            set(ax,'ztickmode','auto'); %note: we can safely set this to auto because if it werent already auto this function would not be running.
            eval(['raveupdateticks',thistype,'(ax,[],''xyz'');'])
        end
        
end