function do_optimizer(src,ev,settings)

global funcount
funcount=0;

%runs the optimizer specified by the settings structure
handles=guidata(src);
thismethod=settings.method;

%check for validity:

if strcmp(handles.optimizerinfo.(thismethod).class,'multi')
    if numel(settings.y)<handles.optimizerinfo.(thismethod).maxobjectives(1)
        errordlg(['You have selected only ',num2str(numel(settings.y)),' objective, but this optimizer requires at least ',num2str(handles.optimizerinfo.(thismethod).maxobjectives(1)),' objectives.'])
        return
    end
    
    if numel(settings.y)>handles.optimizerinfo.(thismethod).maxobjectives(2)
        errordlg(['You have selected ',num2str(numel(settings.y)),' objectives, but this optimizer only supports up to ',num2str(handles.optimizerinfo.(thismethod).maxobjectives(2)),' objectives.'])
        return
    end
end

%turn off start button
set(settings.startbutton,'enable','off')

%clear old optimization results:
delete(findobj(settings.plotaxes,'tag','optplots'));

%make stop button:
buttonpos=get(src,'position');
stops=findobj(handles.optimizetabobjects,'tag','optimizestopbutton');
delete(stops);
continuebutton=findobj(handles.optimizetabobjects,'tag','continuebutton');
set(continuebutton,'enable','off');

%origin=get(handles.tabaxes,'position'); %#ok<NASGU>
if ~isempty(settings.stopbuttonpos)
    stopbutton=uicontrol('position',buttonpos,'parent',handles.fig,...
        'style','togglebutton','string','Stop','backgroundcolor',handles.colors.buttonbg,...
        'callback','drawnow','enable','on','tag','optimizestopbutton','foregroundcolor',[.8 0 0],'fontsize',12,'fontweight','bold');
    handles.tabobjects=union(handles.tabobjects,stopbutton);
    guidata(src,handles);
else
    stopbutton=[];
end
settings.stopbutton=stopbutton;
drawnow;

guidata(handles.fig,handles);

%do optimizer
try
    optfun=str2func(['raveopt',thismethod]);
    results=optfun(src,[],settings);
    handles=guidata(handles.fig);
    handles.optimizerresults{end+1}=results;
    guidata(handles.fig,handles);
catch err
    %Restore tab buttons:
    if ishandle(stopbutton)
        handles.tabobjects=setdiff(handles.tabobjects,stopbutton);
        delete(stopbutton);
        guidata(handles.fig,handles);
    end
    set(settings.startbutton,'enable','on')
    
    txtcheck=findobj(handles.tabobjects,'tag','txtcheck');
    if get(txtcheck,'value')
        ravesendtxtmessage(handles.fig,[],'Oh no! Your optimizer has failed!');
    end
    
    if isjava(handles.trayIcon)
        trayIconClasses = handles.trayIcon.getClass.getClasses;
        MessageTypes = trayIconClasses(1).getEnumConstants;
        handles.trayIcon.displayMessage('Rave','Your optimizer has failed :(',MessageTypes(1));
    end
    
    handles.opttemp.batchname='';
    guidata(handles.fig,handles);
    rethrow(err);
    
end

funcount;

%turn on export button:
exportbutton=findobj(handles.tabobjects,'tag','exportbutton');
if isempty(handles.optimizerresults)
    set(exportbutton,'enable','off')
else
    set(exportbutton,'enable','on')
end

%delete stop button:
if ishandle(stopbutton)
    handles.tabobjects=setdiff(handles.tabobjects,stopbutton);
    delete(stopbutton);
end
set(settings.startbutton,'enable','on')

guidata(handles.fig,handles);

try %#ok<TRYNC>
    txtcheck=findobj(handles.tabobjects,'tag','txtcheck');
    if get(txtcheck,'value')
        ravesendtxtmessage(handles.fig,[],'Your optimizer has finished.');
    end
end