function [c,ceq]=ravenbi_constraint_calc(X,handles,settings,eqconstraint,base)
%calculates all regular inequality and equality constraints, and also
%calculates the nbi search direction equality constraint.
% x = point at which to evaluate constraints
% base = location of basepoint in the objective space
% src = a handle to use with guidata
% settings = optimization settings structure
% index = index of columns of x into columns of datasets.alldata
% mins = minimum bound of x variables
% maxs = maximum bound of x variables

%denormalize x:
mins=settings.normalizationfactors(1,1:end-1);
ranges=settings.normalizationfactors(2,1:end-1);

n=size(X,1); %number of points being evaluated

x=X(:,1:end-1);
t=X(:,end);

%denormalize x:
x=bsxfun(@times,x,ranges);
x=bsxfun(@plus,x,mins);

%deal columns of X to correct data set columns:
xdata=zeros(n,size(handles.funnames{settings.d},2));
xdata(:,settings.funvars(1:end-1))=x; %remove the t from the end

%are there any constraints?
if numel(settings.activeconstraints)==1 && settings.activeconstraints==0
    c=zeros(n,0);
    ceq=zeros(n,0);
else
    %Assess constraints:
    iqs=settings.activeconstraints(ismember(handles.constrainttype{settings.d}{settings.a}(settings.activeconstraints),'inequality'));
    eqs=settings.activeconstraints(ismember(handles.constrainttype{settings.d}{settings.a}(settings.activeconstraints),'equality'));
    
    niq=numel(iqs);
    neq=numel(eqs);
    
    c=zeros(n,niq);
    ceq=zeros(n,neq);

    for k=1:niq
        c(:,k)=raveevaluateconstraint(handles,settings.d,settings.a,iqs(k),xdata,false(1,size(xdata,2)));
    end
    
    for k=1:neq
        ceq(:,k)=raveevaluateconstraint(handles,settings.d,settings.a,eqs(k),xdata,false(1,size(xdata,2)));
    end 
end

%lastly, calculate the nbi search direction constraint:
%put the t back on X:
eqconstvector=eqconstraint(xdata,base,t)';
% eqconstvector=bsxfun(@minus,eqconstvector,min(eqconstvector,[],1));
% eqconstvector=bsxfun(@rdivide,eqconstvector,max(abs(eqconstvector),[],1));
% eqconstvector(isnan(eqconstvector))=0;
% eqconstvector(isinf(eqconstvector))=0;


ceq=[ceq,eqconstvector];


