function distance=crowding(I,settings,xxx)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
M=size(I,2); %number of obj functions
l=size(I,1); %number of points on this frontier
distance=zeros(l,1);

I = bsxfun(@minus,I,min(I));
I = bsxfun(@rdivide,I,max(I));

 switch settings.crowdfcn
    case 'original'
        for m=1:M %for each objective
            objrange=max(I(:,m))-min(I(:,m));
            if objrange==0 %all points have a common value for this objective
                continue %go to next objective (crowding distance = 0 for this obj) (prevents dividing by zero later)
            end
            [Isorted,index]=sortrows(I,m); %sort in ascending order according to this objective (descending would be fine too)
            for i=2:l-1
                distance(index(i))=distance(index(i))+(I(index(i+1),m)-I(index(i-1),m))/objrange; %dividing by objrange prevents biasing according to o'mag of objectives.
            end
            distance(index(1))=inf;
            distance(index(end))=inf; %... so that end points are always selected first.
        end
        
    case 'minx'
        %take the x'th smallest distance:
        
        x=3;
        x=min(x,size(I,1));
        
        i=ipdm(I);
        pause(0.01);
        i(1:size(i,1)+1:end)=inf;
        [sorted,b]=sort(i,'ascend');
        distance=sorted(x,:);
        
        for m=1:M %for each objective
            [Isorted,index]=sortrows(I,m); %sort in ascending order according to this objective (descending would be fine too)
            distance(index(1))=2000*max(distance);
            distance(index(end))=2000*max(distance); %... so that end points are always selected first.
        end
        
    case 'quantile'
%         i=ipdm(I);
         i(1:size(i,1)+1:end)=inf;
%         distance=quantile(i,0.0005);
%         for m=1:M %for each objective
%             [Isorted,index]=sortrows(I,m); %sort in ascending order according to this objective (descending would be fine too)
%             distance(index(1))=2*max(distance);
%             distance(index(end))=2*max(distance); %... so that end points are always selected first.
%         end
        
%     case 'test1' %choose points whose NDLCs are closest to summing to 1.
% 
%         %calculate the leave-one-out nondomination levels
%         ranks=zeros(size(I));
%         for i=1:size(I,2)
%             idx=setdiff(1:size(I,2),i);
%             ranks(:,i)=raveparetofront(I(:,idx),true);
%         end
%         ranks=ranks-1;
%         ranks=bsxfun(@rdivide,ranks,max(ranks));
%         
%         i=ipdm(ranks);
%         i(1:size(i,1)+1:end)=inf;
%         
%         distance=quantile(i,0.003);
%         
%         for m=1:M %for each objective
%             [Isorted,index]=sortrows(I,m); %sort in ascending order according to this objective (descending would be fine too)
%             distance(index(1))=2000*max(distance);
%             distance(index(end))=2000*max(distance); %... so that end points are always selected first.
%         end
%         
%         
%         c=sum(ranks,2);
%         c=abs(c-1);
%         
%         
%         %plot progress:
%         if isempty(findobj('tag','experimental'))
%             %make figure
%             experimental=figure('windowstyle','docked');
%             tax=ternaxes;
%             set(tax,'tag','experimental')
%             colormap(hsv(512));
%         end
%         tax=findobj('type','axes','tag','experimental');
%         nranks=bsxfun(@rdivide,ranks,sum(ranks,2));
%         %nranks=ranks;
%         if xxx==1
%             [x,y]=terncoords(nranks(:,1),nranks(:,3),nranks(:,2));
%             delete(findobj(tax,'tag','exp'));
%             p=patch(x,y,c,'markersize',6,'marker','o',...
%                 'markerfacecolor','flat','parent',tax,'tag','exp',...
%                 'cdatamapping','scaled',...
%                 'markeredgecolor','none','normalmode','manual','facecolor','none','edgecolor','none');
%             drawnow
%             colorbar('peer',tax)
%             
%             set(tax,'clim',[0 max(c)]);
%         end
%         % distance=sum(ranks,2);
%         % distance=1./abs(distance-1);
%         
%     case 'test2' %assign crowding based on number of points in each bin in a triangualr grid in NDLC space.
%         
%         thegrid=trigrid(10);
%         
%         %calculate the leave-one-out nondomination levels
%         ranks=zeros(size(I));
%         for i=1:size(I,2)
%             idx=setdiff(1:size(I,2),i);
%             ranks(:,i)=raveparetofront(I(:,idx),true);
%         end
%         ranks=ranks-1;
%         ranks=bsxfun(@rdivide,ranks,max(ranks));
%         
%         d=ipdm(ranks,thegrid);
%         %d=distance from r'th population member to c'th grid point
%         
%         %find which pop members are nearest to which grid points:
%         a=bsxfun(@eq,d,min(d,[],2));
%         [r,c]=find(a);
% 
%         %c is a list of which grid point is closest to the corresponding
%         %pop member. Repeated values in c indicate multiple pop members are
%         %near that grid point.
%         
%         %make sure we end up with length(c)==pop size
%         [junk,b]=unique(r);
%         c=c(b);
% 
%         %Crowding distance values: If only one pop member is near a grid
%         %point, its crowding distanec is simply the distance to the grid
%         %point. if multiple pop members are near a grid point, the nearest
%         %pop members crowding distance is simply the distance. The
%         %remaining pop members crowding distance is a multiple of the
%         %distance. (20x, 30x, etc for how ever many repetitions there are).
%         
%         
%         distance=zeros(size(I,1),1);
%         
%         gridz=unique(c);
%         
%         for j=1:numel(gridz)
%             thesepts=find(c==gridz(j));
%             dists=d(thesepts,gridz(j));
%             incr=0;
%             for g=1:numel(thesepts)
%                 distance(thesepts(g))=dists(g)+incr*g;
%                 incr=incr+10;
%             end
%         end
%         
%         distance=-distance;
%         C=distance;
%         
%         for m=1:M %for each objective
%             [Isorted,index]=sortrows(I,m); %sort in ascending order according to this objective (descending would be fine too)
%             distance(index(1))=2000*max(distance);
%             distance(index(end))=2000*max(distance); %... so that end points are always selected first.
%         end
%         
%         %C=sum(ranks,2);
%         
%         
%        % C=abs(C-1);
%         %plot progress:
%         if isempty(findobj('tag','experimental'))
%             %make figure
%             experimental=figure('windowstyle','docked');
%             tax=ternaxes;
%             set(tax,'tag','experimental')
%             colormap(mantar(128));
%         end
%         tax=findobj('type','axes','tag','experimental');
%         nranks=bsxfun(@rdivide,ranks,sum(ranks,2));
%         %nranks=ranks;
%         if xxx==1
%             [x,y]=terncoords(nranks(:,1),nranks(:,3),nranks(:,2));
%             delete(findobj(tax,'tag','exp'));
%             p=patch(x,y,C,'markersize',6,'marker','o',...
%                 'markerfacecolor','flat','parent',tax,'tag','exp',...
%                 'cdatamapping','scaled',...
%                 'markeredgecolor','none','normalmode','manual','facecolor','none','edgecolor','none');
%             drawnow
%             colorbar('peer',tax)
%             
%             set(tax,'clim',[min(C) max(C)]);
%         end
 end
