function ravensgaii_bounds(src,ev,settings) %#ok<INUSD>
%MODIFIED NSGAII ALGORITHM FOR FINDING DESIGN SPACE BOUNDARY
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');


%existingboundaries=findobj(handles.activegraph,'tag','boundaryexplorer');
% 
% crap=findobj('tag','pareto');
% delete(crap);

% 
% 
% genzdone=findobj('tag','genzdone');
% set(genzdone,'visible','on');
drawnow;



genedit=findobj(handles.tabobjects,'tag','genedit');
popedit=findobj(handles.tabobjects,'tag','popedit');

paretotarget={[-inf -inf];[-inf inf];[ inf inf];[inf -inf]};

d=udata.view.dataset;
x=udata.view.x;
y=udata.view.y;

N=floor(str2double(get(popedit,'string')));
T=floor(str2double(get(genedit,'string')));
M=2; %always 2 objectives for 2-D scatterplot pareto front.


pcross=0.9;
pmutation=0.9;
nbits=8; %number of bits to discretize each input var

plotvars=[udata.view.x udata.view.y];

constraintlist=findobj(handles.tabobjects,'tag','constraintlist');
if ~isempty(constraintlist) %for methods without a list.
handles.temp.activeconstraints=get(constraintlist,'value')-1;
handles.temp.activeconstraints=handles.temp.activeconstraints(logical(handles.temp.activeconstraints)); %remove 0 from list.
else
    handles.temp.activeconstraints=0;
    handles.temp.activeconstraints=handles.temp.activeconstraints(logical(handles.temp.activeconstraints)); %remove 0 from list.
    
end

settings.activeconstraints=handles.temp.activeconstraints;

%get variables for these two functions:
vars=union(handles.funvars{d}{x},handles.funvars{d}{y});

%allnames=handles.allnames{d};
%varnames=allnames(vars);

% %remove spaces from these to match varnames format (ravejmp2m removes
% %spaces in variable names)
% for j=1:length(allnames)
%     allnames{j}=allnames{j}(allnames{j}~=char(32));
% end
% 
% %reorder varnames into order it occurs in table:
% indexx=zeros(1,numel(varnames));
% for q=1:numel(varnames)
%     indexx(q)=find(strcmp(allnames,varnames{q}));
% end
% 
% %get col numbers of input variables that will be changed (sorted)
% indexx=sort(indexx);


nvars=numel(vars);%(number of input vars) not the same as size(d,2)

%ranges=[min(datasets.alldata{d}); max(datasets.alldata{d})];
ranges=[handles.minlimit{settings.d}{settings.a}; handles.maxlimit{settings.d}{settings.a}];
rangestep=(ranges(2,:)-ranges(1,:))/(2^nbits-1);

%need to replace columns for discrete variables in varTable with only discrete values:
%but first need to decide how to determine which variables are discrete.
%let this slide for now.

%generate random initial populations:
P{1}=uint16(round(rand(N,nvars*nbits))); %P is binary representation of population
P{2}=uint16(round(rand(N,nvars*nbits)));
P{3}=uint16(round(rand(N,nvars*nbits)));
P{4}=uint16(round(rand(N,nvars*nbits)));

Plast=P;

objective=cell(4,1);
cviolation=cell(4,1);
F=cell(4,1);
rank=cell(4,1);


for D=1:4

    %% evaluate this population:
    [objective{D} cviolation{D}]=evaluate_population(P{D},M,nbits,nvars,paretotarget{D},ranges,rangestep,vars,handles,udata,settings);


    %% calculate sorting/crowding of initial population:
    [F{D},rank{D}]=fast_non_dominated_sort(objective{D},cviolation{D});

    %% calculate crowding distance: (for all frontiers)
    distance=cell(4,1);
    crowding=cell(4,1);

    for i=1:numel(F{D})
        I=objective{D}(F{D}{i},:); %i think it's ok to keep this in max-the-best format... but make sure thats correct.
        distance{D}{i}=crowding_distance_assignment(I);
        crowding{D}(F{D}{i})=distance{D}{i};
    end

    Plast{D}=P{D};
    P{D}=make_new_population(P{D},crowding{D},rank{D},pcross,pmutation,cviolation{D},handles);

end

crap=[0 0 0 0];
craper=[0 0 0 0];
plotdata=cell(4,1);

tic %used to calculate ETA and total time.

%intialize plot objects:
for D=1:4
    crap(D)=line(0,0,'marker','none','linestyle','-','parent',handles.activegraph,...
        'tag','boundaryexplorer','linewidth',1,'color',[.5 1 0],'visible','off','buttondownfcn',@lineobject_callback);
    craper(D)=line(0,0,'marker','none','linestyle','-','parent',handles.activegraph,...
        'tag','boundaryexplorer','linewidth',1,'color',[.5 1 0],'visible','off','buttondownfcn',@lineobject_callback);
end

newhandles=[crap craper];
userdata.handles=newhandles;
userdata.delete=1;
set(newhandles,'userdata',userdata); %needed for lineobject_callback



%% DO GENERATIONS:
for t=1:T

    objective=cell(4,1);
    cviolation=cell(4,1);
    R=cell(4,1);
    F=cell(4,1);
    rank=cell(4,1);



    for D=1:4

        %% combine new/old population:
        R{D}=[P{D};Plast{D}]; %combine new/old popualtions

        %evaluate population at R:
        [objective{D} cviolation{D}]=evaluate_population(R{D},M,nbits,nvars,paretotarget{D},ranges,rangestep,vars,handles,udata,settings);
        %% calculate sorting/crowding of initial population:
        [F{D},rank{D}]=fast_non_dominated_sort(objective{D},cviolation{D});
        %trim down to N members using rank/crowding sort:

        %% calculate crowding distance: (for all frontiers)
        distance=cell(4,1);
        crowding=cell(4,1);
        for i=1:numel(F{D})
            I=objective{D}(F{D}{i},:); %i think it's ok to keep this in max-the-best format... but make sure thats correct.
            distance{i}=crowding_distance_assignment(I);
            crowding{D}(F{D}{i})=distance{i};
        end

        survivors=[];
        i=1;
        while numel(survivors)<N && i<=N
            if numel(survivors)+numel(F{D}{i})<=N
                survivors=union(survivors,F{D}{i}); %add all members of F to survivors
            else %add only enough members to make numel(survivors)==N
                shortage=N-numel(survivors);

                %add this many elements to survivors:
                cdist=crowding{D}(F{D}{i});
                index=F{D}{i}; %index of cdist points into R
                data=[cdist' index'];
                data=sortrows(data);
                %take the last (shortage) rows of data (these are least crowded)
                survivors=union(survivors,data(end-shortage+1:end,2));

            end
            i=i+1;
        end

        %pull out these rows of R:
        Plast{D}=R{D}(survivors,:);
        P{D}=make_new_population(Plast{D},crowding{D},rank{D},pcross,pmutation,cviolation{D},handles);

        %end

        %for plotting, keep statements below in the loop. If not plotting til end, use end statement above
        %this line.

        %convert final population back to real numbers:
        %convert binary to decimal:
        P3d=uint16(reshape(Plast{D},[size(P{D},1),nbits,nvars]));

        Prowindex=ones(N,size(ranges,2));

        colnums=(size(P3d,2):-1:1)-1;
        colz=uint16(repmat(colnums,[size(P3d,1) 1 size(P3d,3)]));

        prodz=P3d.*colz;
        expoz=2.^prodz.*P3d;
        sumz=sum(expoz,2);
        Prow2D=permute(sumz,[1 3 2]);
        Prowindex(:,vars)=Prow2D;

        Preal=bsxfun(@times,Prowindex,rangestep);
        Preal=bsxfun(@plus,Preal,ranges(1,:)); %Preal is decimal representation of population

        objective{D}=raveevaluatefunction(src,settings.d,[udata.view.x udata.view.y],Preal,handles);
        rank{D}=rank{D}(survivors);

        %delete(crap{D})
        %crap{D}=[];

        plotdata{D}=sortrows(objective{D}(rank{D}==1,:));
        plotdata{D}=flipud(plotdata{D});
        %crap{D}=line(,,'marker','none','linestyle','-','parent',handles.activegraph,...
        %    'tag','pareto','linewidth',1,'color',[.5 1 0],);
        set(crap(D),'xdata',plotdata{D}(:,1),'ydata',plotdata{D}(:,2),'visible','on')
        
        

        %set(genzdone,'string',[num2str(t) ' Generations Completed'])

    end
    

    %draw lines connecting end points of each frontier:
%     delete(craper)
%     craper=[];
% 
%     craper(1)=line(,,'marker','none','linestyle','-','parent',handles.activegraph,...
%         'tag','pareto','linewidth',1,'color',[.5 1 0],'buttondownfcn',@lineobject_callback);
%     craper(2)=line([plotdata{2}(end,1),plotdata{3}(1,1)],[plotdata{2}(end,2),plotdata{3}(1,2)],'marker','none','linestyle','-','parent',handles.activegraph,...
%         'tag','pareto','linewidth',1,'color',[.5 1 0],'buttondownfcn',@lineobject_callback);
%     craper(3)=line([plotdata{3}(end,1),plotdata{4}(end,1)],[plotdata{3}(end,2),plotdata{4}(end,2)],'marker','none','linestyle','-','parent',handles.activegraph,...
%         'tag','pareto','linewidth',1,'color',[.5 1 0],'buttondownfcn',@lineobject_callback);
%     craper(4)=line([plotdata{4}(1,1),plotdata{1}(end,1)],[plotdata{4}(1,2),plotdata{1}(end,2)],'marker','none','linestyle','-','parent',handles.activegraph,...
%         'tag','pareto','linewidth',1,'color',[.5 1 0],'buttondownfcn',@lineobject_callback);
    
    set(craper(1),'xdata',[plotdata{1}(1,1),plotdata{2}(1,1)],'ydata',[plotdata{1}(1,2),plotdata{2}(1,2)],'visible','on');
    set(craper(2),'xdata',[plotdata{2}(end,1),plotdata{3}(1,1)],'ydata',[plotdata{2}(end,2),plotdata{3}(1,2)],'visible','on');
    set(craper(3),'xdata',[plotdata{3}(end,1),plotdata{4}(end,1)],'ydata',[plotdata{3}(end,2),plotdata{4}(end,2)],'visible','on');
    set(craper(4),'xdata',[plotdata{4}(1,1),plotdata{1}(end,1)],'ydata',[plotdata{4}(1,2),plotdata{1}(end,2)],'visible','on');
    
    
%     userdata.handles=craper;
%     set(craper,'userdata',userdata); %for use in lineobject_callback.

    drawnow
    
       
    elapsed=toc;
    timepergen=elapsed/t;
    timetogo=(timepergen*(T-t))/(24*3600); %in fractional days.
    timestr=datestr(timetogo,13); %13 is format for hhmmss.
    set(handles.infobar,'string',['\fontname{monospace} ',sprintf('%4.0f',t),'\fontname{comic sans ms} Generations Completed.    ETA: ' timestr])

         %fix axes limits:
    set([handles.fakeax(handles.activeindex) handles.fakerax(handles.activeindex)],...
        'xlim',get(handles.activegraph,'xlim'),'ylim',get(handles.activegraph,'ylim'))

    
    
end

%allboundaries=findobj(handles.activegraph,'tag','boundaryexplorer');


elapsed=toc/(24*3600);
timestr=datestr(elapsed,13); %13 is format for hhmmss.
set(handles.infobar,'string',['\fontname{monospace} ',sprintf('%4.0f',t),'\fontname{comic sans ms} Generations Completed.    Elapsed Time: ' timestr])

exportbutton=findobj('tag','exportbutton');
set(exportbutton,'enable','on')



if isjava(handles.trayIcon)
    trayIconClasses = handles.trayIcon.getClass.getClasses;
    MessageTypes = trayIconClasses(1).getEnumConstants;
    handles.trayIcon.displayMessage('Rave','Your optimizer is finished.',MessageTypes(3));
end


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%


data1=sortrows(plotdata{1},-1);
data2=sortrows(plotdata{2},1);
data3=sortrows(plotdata{3},1);
data4=sortrows(plotdata{4},-1);
  

% handles.temp.lasttime=elapsed;
% handles.temp.lastgen=t+startgen;
% handles.temp.finalbin=P(rank==1,:);
handles.temp.finalpop=[data1;data4;data3;data2];

% 
% handles.temp.finalresp=objective(rank==1,:);

%set(handles.grabbers{handles.activeindex},'visible',grabbyviz);

guidata(handles.fig,handles);






