function raveanimatemultiobj(src,iter,settings,pop,cviolation,rank) 
%a generic multiobjective optimizer plotter, used for exmaple by nsgaii
%
%settings is structure of optimizer settings
%pop is population to display (only input values are needed.)
%cviolation is vector of constraint violations.
%mov is handle to movie, or empty.
%rank is pareto front rank (for multiobjective methods, empty for single
%obj.)
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);
plotvars=settings.plotvars;

N=size(pop,1); %population size

objective=raveevaluatefunction(src,settings.d,plotvars,pop,handles);
switch settings.method
    case {'nsgaii','nsgaiiwlocalmc','nsgaii_nbi'} %MULTIOBJECTIVE METHODS, USE RANK
        delete(findobj('tag','optplots'))
        
        if settings.showinfo && ~isempty(iter)
           txt=findobj(settings.plotaxes,'tag','optinfotext');
           set(txt,'string',{['Generation: ',num2str(iter)]})  
        end

        switch settings.plottype
            case 'single'
                %NOTE: SHOULD REALLY BE SORTING ACCORDING TO OPTIMIZED
                %VARIABLES, NOT PLOTTED VARS. this method can cause noise
                %when plot vars ~= objectives.
                %for q=max(rank):-1:2
                plotdata=sortrows(objective(rank>1 & cviolation==0,:));
                L=line(plotdata(:,1),plotdata(:,2),'marker','o','linestyle','none','parent',handles.activegraph,...
                    'tag','optplots','linewidth',0.5,'markerfacecolor',handles.colors.optfeasible(1,:),...
                    'markeredgecolor',handles.colors.optfeasible(2,:),'markersize',6,...
                    'color',[.7 .7 .7],'hittest','off');
                
%                 if numel(settings.y)==2
%                     linestyle='-';
%                 else
                    linestyle='none';
%                 end
                
                plotdata=sortrows(objective(rank==1 & cviolation==0,:));
                L=line(plotdata(:,1),plotdata(:,2),100*ones(size(plotdata(:,2))),'marker','o','linestyle',linestyle,'parent',handles.activegraph,...
                    'tag','optplots','linewidth',1,'markerfacecolor',handles.colors.optoptimal(1,:),...
                    'markeredgecolor',handles.colors.optoptimal(2,:),'markersize',6,'hittest','off');
                
                %plot infeasible points:
                plotdata=objective(cviolation~=0,:);
                L=line(plotdata(:,1),plotdata(:,2),'marker','o','linestyle','none','parent',handles.activegraph,...
                    'tag','optplots','markerfacecolor',handles.colors.optinfeasible(1,:),...
                    'markeredgecolor',handles.colors.optinfeasible(2,:),'markersize',6,'hittest','off');
                
            case '3D'
                for q=max(rank):-1:1
                    if numel(settings.y)==2 %draw line instead of points;
                        plotdata=sortrows(objective(rank==q & cviolation==0,:));
                    else
                        plotdata=objective(rank==q & cviolation==0,:);
                    end
                    
                    L=line(plotdata(:,1),plotdata(:,2),plotdata(:,3),'parent',handles.activegraph,'marker','o',...
                        'tag','optplots','markerfacecolor',handles.colors.optfeasible(1,:),...
                        'markeredgecolor',handles.colors.optfeasible(2,:),'markersize',6,'linestyle','none','hittest','off');
                    
                    if q==1
                        set(L,'markerfacecolor',handles.colors.optoptimal(1,:),'markeredgecolor',handles.colors.optoptimal(2,:));
                    end
                end
                
                %plot infeasible points:
                plotdata=objective(cviolation~=0,:);
                L=line(plotdata(:,1),plotdata(:,2),plotdata(:,3),'marker','o','linestyle','none','parent',handles.activegraph,...
                    'tag','optplots','linewidth',1,'color',[0 0 0],'markersize',4,'hittest','off');
                
            case 'parallelcoords'
                udata=get(handles.fakeax(handles.activeindex),'userdata');
                objective=raveevaluatefunction(src,settings.d,udata.view.x,pop,handles);
                pop(:,udata.view.x)=objective;
                
                set(udata.view.mainhandles,'visible','off')
                set(udata.view.hiddenhandles,'visible','off')
                set(udata.view.selectedhandles,'visible','off')
                xcol=udata.view.x;
                d=udata.view.dataset;
                a=udata.view.analysis;
                
                %expand ranges if necessary:
                handles.minlimit{d}{a}=min(handles.minlimit{d}{a},min(pop,[],1));
                handles.maxlimit{d}{a}=max(handles.maxlimit{d}{a},max(pop,[],1));
                
                x=pop(:,xcol);
                data1=bsxfun(@minus,x,handles.minlimit{d}{a}(xcol));
                data2=bsxfun(@rdivide,data1,(handles.maxlimit{d}{a}(xcol)-handles.minlimit{d}{a}(xcol)));
                data2(isnan(data2))=.5;
                x=data2;
                
                xcoords=linspace(0,1,size(x,2));
                
                allcolorz=get(handles.fig,'colormap');
                if udata.view.c~=0
                    c=pop(:,udata.view.c);
                    %normalize cdata to desired colormap:
                    c=(c-min(c))/(max(c)-min(c))*127+128*(udata.format.lcmap-1)+1;
                    if udata.format.reversecolors
                        c=(1-(c-min(c))/(max(c)-min(c)))*127+128*(udata.format.lcmap-1)+1;
                    end
                    
                    %cdata=[linterp(1:size(allcolorz,1),allcolorz(:,1),c)' linterp(1:size(allcolorz,1),allcolorz(:,2),c)' linterp(1:size(allcolorz,1),allcolorz(:,3),c)'];
                    c(isnan(c))=128*(udata.format.lcmap-1)+1;
                    
                    cdata=allcolorz(floor(c),:);
                    
                else %USE ROW COLORS
                    c=zeros(size(pop,1),1);
                    c=1920+handles.prefs.defaultrowcolor+c;
                    cdata=allcolorz(c,:);
                    
                    if udata.format.ncolor %do noisy colors
                        noise=0.3; %noise multiplier
                        cdata=cdata+(rand(size(cdata))-.5)*noise;
                        cdata(cdata<0)=0;
                        cdata(cdata>1)=1;
                    end
                end
                
                delete(findobj(handles.activegraph,'tag','optplots'));
                
                for r=1:size(pop,1);
                    line(xcoords,x(r,:),'parent',handles.activegraph,'tag','optplots',...
                        'marker','none','color',cdata(r,:),'userdata',cdata(r,:),'linewidth',udata.format.linesize/2);
                end
        end
end

drawnow

if settings.animation
    Framez=get(handles.fig,'userdata');
    Framez(end+1) = getscreen(handles.fig,settings.framebounds);
    set(handles.fig,'userdata',Framez);
end

if settings.pause
    waitforbuttonpress;
    if uint8(get(gcf,'currentcharacter'))== 27
        settings.pause=0;
        set(0,'userdata',settings)
    end
    figure(handles.temp.crapwindow);
end


