function [settingnames,settingvalues]=raveparseoptimizerobjectivetab(src,ev)
%Get settings from all objective tab controls except the settings table.

handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata');
datasetmenu=findobj(handles.tabobjects,'tag','datasetmenu');
analysismenu=findobj(handles.tabobjects,'tag','analysismenu');
methodmenu=findobj(handles.tabobjects,'tag','methodmenu');
objectivemenu=findobj(handles.tabobjects,'tag','objectivemenu');
constraintmenu=findobj(handles.tabobjects,'tag','constraintmenu');
anicheck=findobj(handles.tabobjects,'tag','anicheck');

allmethods=get(methodmenu,'userdata');
thismethod=allmethods{get(methodmenu,'value')};

%load default settings:
[displaynames,settingnames,settingvalues,isastring]=raveparseoptimizersettingsdefault(src,[],thismethod);

d=get(datasetmenu,'value');
a=get(analysismenu,'value');

thisvar=get(objectivemenu,'value');

settings.src=handles.fig;
settings.method=thismethod;
settings.d=d;
settings.a=a;
settings.y=thisvar;

settings.plotprogress=get(anicheck,'value'); %draw progress on current graph (scatterplot)


%animate?
if ~isempty(handles.activegraph) && settings.plotprogress
    settings.plottype=handles.graphinfo.(handles.activetype).axesformat;
    settings.plotaxes=handles.activegraph;
    
    switch settings.plottype
        case 'single'
            plotvars=[udata.view.x(:);udata.view.y(:)];
        case '3D'
            plotvars=[udata.view.x(:);udata.view.y(:);udata.view.z(:)];
        case 'matrix'
            plotvars=[udata.view.x(:);udata.view.y(:)];
        otherwise
            plotvars=[];
    end
else
    plotvars=[];
    settings.plottype='';
    settings.plotaxes=[];
end

settings.animation=handles.prefs.animation.animation; %logical 1=export animation;
settings.pause=handles.prefs.animation.pause;%logical 1=pause between iterations (only works if animating)
settings.updateanalysis=handles.prefs.animation.updateanalysis;

if ~isempty(handles.activeindex)
    %can't use getpixelposition since rectangles don't have units...
    wspanelpos=get(handles.wspanel,'position');
    cvpos = get(handles.currentview,'position');
% targetpos = targetpos - [cvpos(1) cvpos(2) 0 0];
    blockpos=get(handles.allblockers(handles.activeindex),'position');
%     fb=wspanelpos(1:2)+blockpos(1:2)+[2 3];
    fb=blockpos(1:2)-cvpos(1:2)+[2 3] + wspanelpos(1:2);
    settings.framebounds=[fb blockpos(3:4)];
else
    settings.animation=0;
end

settings.showinfo=handles.prefs.animation.showinfo;
if ~isempty(handles.activeindex)
    settings.infofont=handles.prefs.animation.infofont;
    settings.infofontsize=handles.prefs.animation.infofontsize;
    settings.infolocation=handles.prefs.animation.infolocation;
else
    settings.showinfo=0;
end

if isempty(plotvars) %cant animate if no plotvars.
    settings.animation=0;
    settings.plotprogress=0;
    settings.plottype='';
    settings.plotaxes=[];
end 
settings.plotvars=plotvars;

settings.targets=handles.targetvalues{d}(thisvar);
settings.activeconstraints=get(constraintmenu,'value')-1;
settings.activeconstraints=settings.activeconstraints(settings.activeconstraints>0);

origin=get(handles.tabaxes,'position');
settings.startbutton=src; %handle to startbutton
settings.stopbuttonpos=[origin(1)+40 origin(2)+114 100 26]; %position to make stop button, or empty to not make stop button
settings.stopbutton=[];

settings.updatexdials=true;

%get variables for these functions:
indexx=ravegetindependentvars(handles,d,a,settings.y,settings.activeconstraints);
   
settings.funvars=indexx;

%make initial point:
mins=handles.minlimit{d}{a}(indexx);
maxs=handles.maxlimit{d}{a}(indexx);
x0=handles.xdials{d}{a}(indexx);
x0=bsxfun(@minus,x0,mins);
x0=bsxfun(@rdivide,x0,maxs-mins);
x0(maxs-mins==0)=0;
settings.x0=x0;
settings.normalizationfactors=[mins;maxs-mins];

% do the same thing for the plot functions:
if ~isempty(plotvars)
    plotindexx=handles.funvars{d}{settings.plotvars(1)};
    for k=2:numel(settings.plotvars)
        plotindexx=union(plotindexx,handles.funvars{d}{settings.plotvars(k)});
    end
    settings.plotinputs=plotindexx;
else
    settings.plotinputs=[];
end

settingnames=fieldnames(settings);
settingvalues=struct2cell(settings);

