function results=raveoptfminbnd(src,ev,settings,overload)
%optimize in rave using the functions from the optimization toolbox.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata'); %#ok<NASGU>

if nargin<4
    overload=struct();
end

if ~isfield(overload,'alreadyanimating') || ~overload.alreadyanimating
    raveinitializeoptimizeranimation;
end


%initial point:
x0=settings.x0;


%% make objective/constraint functions:
if ~isfield(overload,'objectivefcn')
    [functionname,indexx]=ravemakeminimizefunction(src,ev,settings);
    FITNESSFCN=str2func(functionname);
else
    indexx=settings.funvars;
    FITNESSFCN=overload.objectivefcn;
end

if ~isfield(overload,'normalizationfactors')
%    settings.normalizationfactors=[mins;maxs-mins];
else
    settings.normalizationfactors=overload.normalizationfactors;
end


%% make options structure
options=optimset(@fminsearch);

% merge settings with options:
optnames=fieldnames(options);
for o=1:numel(optnames);
    if isfield(settings,optnames{o});
        options.(optnames{o})=settings.(optnames{o});
    end
end

%plot results?
if settings.plotprogress
    options.PlotFcns=str2func(settings.PlotFcns);
    settings.src=src;
    set(0,'userdata',settings);
else
    set(0,'userdata',settings);
    options.PlotFcns=[];
end

options.UseParallel='always';

%% make variables:
nVars=numel(x0); %#ok

%% convert numerical options to numbers:
numericaloptions={'MaxIter';'MaxFunEvals';'TolFun';'TolX'};
for n=1:numel(numericaloptions) 
    if ischar(options.(numericaloptions{n})) && ~isempty(options.(numericaloptions{n}))
        options.(numericaloptions{n})=eval(options.(numericaloptions{n}));
    end
end

numericalsettings={'LowerBound';'UpperBound'};
for n=1:numel(numericalsettings) 
    if ischar(settings.(numericalsettings{n})) && ~isempty(settings.(numericalsettings{n}))
        settings.(numericalsettings{n})=eval(settings.(numericalsettings{n}));
    end
end

if ~isempty(settings.LowerBound)
    LB=settings.LowerBound;
else
    LB=0;
end

if ~isempty(settings.UpperBound)
    UB=settings.UpperBound;
else
    UB=1;
end

%% do optimizer:
tic

%independent variables are normalized as per ravemakeminimizefunction:

handles.temp.crapwindow=nan;
guidata(handles.fig,handles);

%make settings retrievable for plotfcns:
set(0,'userdata',settings);

[X,FVAL,EXITFLAG,OUTPUT] = ravefminbnd(FITNESSFCN,LB,UB,options); %#ok<ASGLU>

settings=get(0,'userdata'); %in case it was changed by raveanimatefminbnd

if settings.plotprogress && settings.pause
    str=get(handles.infobar,'string');
    str=str(1:end-25);
    set(handles.infobar,'string',str);
end

handles=guidata(src);
if settings.plotprogress && ishandle(handles.temp.crapwindow)
    delete(handles.temp.crapwindow);
end

if ~isfield(overload,'alreadyanimating') || ~overload.alreadyanimating
    ravefinishoptimizeranimation;
end

results.finalinputs=X;
results.finalobjective=FVAL;
results.settings=settings;
results.name=['fminbnd, ',handles.allnames{settings.d}{settings.y}];