function results=raveoptfmincon(src,ev,settings,overload)
%optimize in rave using the functions from the optimization toolbox.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata'); %#ok<NASGU>
starttime=0;

if nargin<4
    overload=struct();
end

if isempty(which('fmincon'));
    errordlg('The optimizer you selected requires the MATLAB Optimization Toolbox, which does not appear to be installed.')
    return
end

if ~license('test','optimization_toolbox')
    errordlg('The optimizer you selected requires the MATLAB Optimization Toolbox, which appears to be installed but unlicensed.')
    return
end

if ~license('checkout','optimization_toolbox')
    errordlg('The optimizer you selected requires the MATLAB Optimization Toolbox, but I could not check out a toolbox license.')
    return
end

if ~isfield(overload,'alreadyanimating') || ~overload.alreadyanimating
    raveinitializeoptimizeranimation;
end

d=settings.d;
a=settings.a;


%% make objective/constraint functions:
if ~isfield(overload,'objectivefcn')
    [functionname,indexx]=ravemakeminimizefunction(src,ev,settings);
    FITNESSFCN=str2func(functionname);
else
    indexx=settings.funvars;
    FITNESSFCN=overload.objectivefcn;
end

%Ensure that independent variables required by FITNESSFCN match those in
%settings. A mismatch can occur if this function was called by another
%optimizer whose objective function requires a superset of the independent
%variables needed here.
% 
% matches=ismember(settings.funvars,indexx);
% settings.funvars=settings.funvars(matches);
% settings.x0=settings.x0(matches);
% settings.normalizationfactors=settings.normalizationfactors(:,matches);

%initial point:
x0=settings.x0;

if ~isfield(overload,'mins')
    mins=zeros(size(x0));
else
    mins=overload.mins;
end

if ~isfield(overload,'maxs')
    maxs=ones(size(x0));
else
    maxs=overload.maxs;
end

if ~isfield(overload,'normalizationfactors')
%    settings.normalizationfactors=[mins;maxs-mins];
else
    settings.normalizationfactors=overload.normalizationfactors;
end


%% make options structure
options=optimset(@fmincon);
options.Display= 'final';
options.MaxFunEvals= [];
options.MaxIter= [];
options.TolFun= 1.0000e-006;
options.TolX= [];
options.FunValCheck= 'off';
options.OutputFcn= [];
options.PlotFcns= [];
options.StallIter=500;
options.ActiveConstrTol= [];
options.Algorithm= 'trust-region-reflective';
options.AlwaysHonorConstraints= 'bounds';
options.BranchStrategy= [];
options.DerivativeCheck= 'off';
options.Diagnostics= 'off';
options.DiffMaxChange= 0.1000;
options.DiffMinChange= 1.0000e-008;
options.FinDiffType= 'forward';
options.GoalsExactAchieve= [];
options.GradConstr= 'off';
options.GradObj= 'off';
options.HessFcn= [];
options.Hessian= [];
options.HessMult= [];
options.HessPattern= 'sparse(ones(numberofvariables))';
options.HessUpdate= [];
options.InitialHessType= [];
options.InitialHessMatrix= [];
options.InitBarrierParam= 0.1000;
options.InitTrustRegionRadius= 'sqrt(numberofvariables)';
options.Jacobian=[];
options.JacobMult= [];
options.JacobPattern= [];
options.LargeScale='on';
options.LevenbergMarquardt= [];
options.LineSearchType= [];
options.MaxNodes= [];
options.MaxPCGIter= 'max(1,floor(numberofvariables/2))';
options.MaxProjCGIter= '2*(numberofvariables-numberofequalities)';
options.MaxRLPIter= [];
options.MaxSQPIter= '10*max(numberofvariables,numberofinequalities+numberofbounds)';
options.MaxTime= [];
options.MeritFunction= [];
options.MinAbsMax= [];
options.NodeDisplayInterval= [];
options.NodeSearchStrategy= [];
options.NonlEqnAlgorithm= [];
options.NoStopIfFlatInfeas= 'on';
options.ObjectiveLimit= -1.0000e+020;
options.PhaseOneTotalScaling= 'off';
options.Preconditioner= [];
options.PrecondBandWidth= 0;
options.RelLineSrchBnd= [];
options.RelLineSrchBndDuration= 1;
options.ScaleProblem= 'obj-and-constr';
options.Simplex= [];
options.SubproblemAlgorithm= 'ldl-factorization';
options.TolCon= 1.0000e-006;
options.TolConSQP= 1.0000e-006;
options.TolGradCon= 1.0000e-006;
options.TolPCG= 0.1000;
options.TolProjCG= 0.0100;
options.TolProjCGAbs= 1.0000e-010;
options.TolRLPFun= [];
options.TolXInteger= [];
options.TypicalX= 'x0';
options.UseParallel= 'never';

% merge settings with options:
optnames=fieldnames(options);
for o=1:numel(optnames);
    if isfield(settings,optnames{o});
        options.(optnames{o})=settings.(optnames{o});
    end
end

%plot results?
if settings.plotprogress
    options.PlotFcns=str2func(settings.PlotFcns);
    settings.src=src;
    set(0,'userdata',settings);
else
    set(0,'userdata',settings);
    options.PlotFcns=[];
end

options.Vectorized='on';
% options.UseParallel='always';

%% make variables:
nVars=numel(x0); %#ok
numberofinequalities=nnz(strcmp('inequality',handles.constrainttype{d}{a})); %#ok
numberofequalities=nnz(strcmp('equality',handles.constrainttype{d}{a})); %#ok
numberofbounds=2*numel(x0); %#ok


%% convert numerical options to numbers:
numericaloptions={'MaxTime';'MaxIter';'MaxFunEvals';'StallIter';'TolFun';'ObjectiveLimit';'TolX';'ActiveConstrTol';'DiffMaxChange';'DiffMinChange';...
    'GoalsExactAchieve';'HessPattern';'InitialHessMatrix';'InitBarrierParam';'InitTrustRegionRadius';'JacobPattern';'MaxNodes';...
    'MaxPCGIter';'MaxProjCGIter';'MaxRLPIter';'MaxSQPIter';'MinAbsMax';'NodeDisplayInterval';'Preconditioner';'PrecondBandWidth';...
    'RelLineSrchBnd';'RelLineSrchBndDuration';'TolCon';'TolConSQP';'TolGradCon';'TolPCG';'TolProjCG';'TolProjCGAbs';'TolRLPFun';...
    'TolXInteger';'TypicalX'};
for n=1:numel(numericaloptions) 
    if ischar(options.(numericaloptions{n})) && ~isempty(options.(numericaloptions{n}))
        options.(numericaloptions{n})=eval(options.(numericaloptions{n}));
    end
end

%% make nonlinear constraints:
if ~isfield(overload,'constraintfcn')
    %anonnonlcon = @(x)ravenonlcon(x,handles,settings,indexx,mins,maxs-mins);
    anonnonlcon = @(x)ravenonlcon(x,handles,settings,indexx,settings.normalizationfactors(1,:),settings.normalizationfactors(2,:));
else
    anonnonlcon=overload.constraintfcn;
end

%% do optimizer:
tic

%independent variables are normalized as per ravemakeminimizefunction:

handles.temp.crapwindow=nan;
guidata(handles.fig,handles);

%make settings retrievable for plotfcns:
set(0,'userdata',settings);

if any(settings.activeconstraints) || isfield(overload,'constraintfcn');
    [X,FVAL,EXITFLAG,OUTPUT] = fmincon(FITNESSFCN,x0,[],[],[],[],mins,maxs,anonnonlcon,options); %#ok<ASGLU>
else
    [X,FVAL,EXITFLAG,OUTPUT] = fmincon(FITNESSFCN,x0,[],[],[],[],mins,maxs,[],options); %#ok<ASGLU>
end

%denormalize x:
X=bsxfun(@times,X,settings.normalizationfactors(2,:));
X=bsxfun(@plus,X,settings.normalizationfactors(1,:));

handles=guidata(src);
if settings.plotprogress && ishandle(handles.temp.crapwindow)
    %delete(gcf) %deletes the invisible figure created by the plot function
    delete(handles.temp.crapwindow);
end

% elapsed=toc/(24*3600);
% timestr=datestr(elapsed+starttime,13); %13 is format for hhmmss.

if ~isfield(overload,'alreadyanimating') || ~overload.alreadyanimating
    ravefinishoptimizeranimation;
end

results.finalinputs=X;
results.finalobjective=FVAL;
results.settings=settings;
results.name=['fmincon, ',handles.allnames{d}{settings.y}];


