function results=raveoptga(src,ev,settings,overload)
%optimize in rave using the functions from the gads toolbox.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
handles=guidata(src);
udata=get(handles.fakeax(handles.activeindex),'userdata'); %#ok<NASGU>
starttime=0;

if nargin<4
    overload=struct();
end

if isempty(which('ga'));
    errordlg('The optimizer you selected requires the MATLAB Optimization Toolbox, which does not appear to be installed.')
    return
end

if ~license('test','optimization_toolbox')
    errordlg('The optimizer you selected requires the MATLAB Optimization Toolbox, which appears to be installed but unlicensed.')
    return
end

if ~license('checkout','optimization_toolbox')
    errordlg('The optimizer you selected requires the MATLAB Optimization Toolbox, but I could not check out a toolbox license.')
    return
end

if ~isfield(overload,'alreadyanimating') || ~overload.alreadyanimating
    raveinitializeoptimizeranimation;
end


d=settings.d;
a=settings.a;


%% make objective/constraint functions:
if ~isfield(overload,'objectivefcn')
    [functionname,indexx]=ravemakeminimizefunction(src,ev,settings);
    FITNESSFCN=str2func(functionname);
else
    indexx=settings.funvars;
    FITNESSFCN=overload.objectivefcn;
end

nvars=numel(indexx);

x0=settings.x0;

if ~isfield(overload,'mins')
    mins=zeros(size(x0));
else
    mins=overload.mins;
end

if ~isfield(overload,'maxs')
    maxs=ones(size(x0));
else
    maxs=overload.maxs;
end

if ~isfield(overload,'normalizationfactors')
%    settings.normalizationfactors=[mins;maxs-mins];
else
    settings.normalizationfactors=overload.normalizationfactors;
end


%% make options structure

options=gaoptimset(@ga);
options.PopulationSize=settings.pop;
options.CrossoverFraction=settings.pcross;
options.EliteCount=settings.elitecount;
options.MigrationDirection=settings.migrationdirection;
options.MigrationInterval=settings.migrationinterval;
options.MigrationFraction=settings.migrationfraction;
options.Generations=settings.gens;
options.TimeLimit=settings.timelimit;
options.StallGenLimit=settings.stallgenlimit;
options.TolFun=settings.tolfun;
options.TolCon=settings.tolcon;
options.SelectionFunction=settings.selectionfun;
% options.CrossoverFcn=settings.crossoverfun;
options.MutationFcn=@mutationadaptfeasible;
% options.DistanceMeasureFcn=settings.distancefun;

% merge settings with options:
optnames=fieldnames(options);
for o=1:numel(optnames);
    if isfield(settings,optnames{o});
        options.(optnames{o})=settings.(optnames{o});
    end
end

%plot results?
if settings.plotprogress
    options.PlotFcns=str2func(settings.plotfunction);
    settings.src=src;
    set(0,'userdata',settings);
    trash=findobj(settings.plotaxes,'tag','optinfeasible','-or','tag','optfeasible','-or','tag','optcurrent','-or','tag','opthistory');
    delete(trash);
else
    set(0,'userdata',settings);
    options.PlotFcns=[];
end

options.Vectorized='on';
options.UseParallel='never';

%% make variables:
nVars=numel(x0); %#ok
numberofinequalities=nnz(strcmp('inequality',handles.constrainttype{d}{a})); %#ok
numberofequalities=nnz(strcmp('equality',handles.constrainttype{d}{a})); %#ok
numberofbounds=2*numel(x0);

% options=gaoptimset(@ga);
% options.PopulationSize=settings.pop;
% options.CrossoverFraction=settings.pcross;
% options.EliteCount=settings.elitecount;
% options.MigrationDirection=settings.migrationdirection;
% options.MigrationInterval=settings.migrationinterval;
% options.MigrationFraction=settings.migrationfraction;
% options.Generations=settings.gens;
% options.TimeLimit=settings.timelimit;
% options.StallGenLimit=settings.stallgenlimit;
% options.TolFun=settings.tolfun;
% options.TolCon=settings.tolcon;
% options.SelectionFunction=settings.selectionfun;

%% convert numerical options to numbers:
numericaloptions={'PopulationSize';'CrossoverFraction';'EliteCount';'MigrationInterval';'MigrationFraction';...
    'Generations';'TimeLimit';'StallGenLimit';'TolFun';'TolCon'};
for n=1:numel(numericaloptions) 
    if ischar(options.(numericaloptions{n})) && ~isempty(options.(numericaloptions{n}))
        options.(numericaloptions{n})=eval(options.(numericaloptions{n}));
    end
end



%% make nonlinear constraints:
if ~isfield(overload,'constraintfcn')
    %anonnonlcon = @(x)ravenonlcon(x,handles,settings,indexx,mins,maxs-mins);
    anonnonlcon = @(x)ravenonlcon(x,handles,settings,indexx,settings.normalizationfactors(1,:),settings.normalizationfactors(2,:));
else
    anonnonlcon=overload.constraintfcn;
end

%% do optimizer:
if any(settings.activeconstraints) || isfield(overload,'constraintfcn');
    [X,FVAL,EXITFLAG,OUTPUT,population] = ga(FITNESSFCN,nvars,[],[],[],[],mins,maxs,anonnonlcon,options); %#ok<ASGLU>
else
    [X,FVAL,EXITFLAG,OUTPUT,population] = ga(FITNESSFCN,nvars,[],[],[],[],mins,maxs,[],options); %#ok<ASGLU>
end

%denormalize x:
X=bsxfun(@times,X,settings.normalizationfactors(2,:));
X=bsxfun(@plus,X,settings.normalizationfactors(1,:));

population=bsxfun(@times,population,settings.normalizationfactors(2,:));
population=bsxfun(@plus,population,settings.normalizationfactors(1,:));

handles=guidata(src);

if settings.plotprogress && ishandle(handles.temp.crapwindow)
    %delete(gcf) %deletes the invisible figure created by the plot function
    delete(handles.temp.crapwindow);
end

%distribute population to correct columns:
% indexx=settings.funvars;

% finalpop=repmat(handles.minlimit{d}{a},size(population,1),1);
% finalpop(:,indexx)=population;

%evaluate this for ALL variables:
% respvars=find(handles.isfunction{settings.d});
% objective=raveevaluatefunction(src,settings.d,respvars,finalpop);
%finalpop(:,respvars)=objective;
%guidata(handles.fig,handles);

%elapsed=toc/(24*3600);
%timestr=datestr(elapsed+starttime,13); %13 is format for hhmmss.
%set(handles.infobar,'string',['\fontname{monospace} ',sprintf('%4.0f',settings.gens),'\fontname{comic sans ms} Generations Completed.    Elapsed Time: ' timestr])

if ~isfield(overload,'alreadyanimating') || ~overload.alreadyanimating
    ravefinishoptimizeranimation;
end
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

%generate name:
objs=handles.allnames{settings.d}{settings.y};
results.name=['GA ',objs,num2str(settings.pop),'p ',num2str(settings.gens),'g   (',handles.datasetnames{settings.d},')'];

% [sortedpop,index]=sortrows(finalpop,settings.y);
results.finalinputs=X;
results.finalobjective=FVAL;
results.settings=settings;

