%Colormaps
function raveeditprefscolormaps(src,ev)

% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2013 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);

txt1=uicontrol('style','text','string','Select a colormap to edit:',...
    'position',[230 428 240 20],'backgroundcolor',handles.colors.frame,'fontsize',10,'hor','left',...
    'parent',src);

menu1=uicontrol('style','popup','string',{'Custom Colormap 1';'Custom Colormap 2';'Custom Colormap 3'},...
    'position',[230 408 200 22],'backgroundcolor',[1 1 1],'hor','left','callback',@menu_Callback,...
    'tag','edit');


txt2=uicontrol('style','text','string','Type or paste a list of color values into the table to the right. You must have at least 2 colors, up to a maximum of 128. If you enter fewer than 128 colors, Rave will interpolate them to create 128 colors. Colors must be entered as decimal RGB values (the normal MATLAB colorspec format).',...
    'position',[230 128 200 220],'backgroundcolor',handles.colors.frame,'fontsize',10,'hor','left',...
    'parent',src);

txt3=uicontrol('style','text','string','The top row of the table corresponds to the lowest numerical value, and the bottom row corresponds to the highest numerical value.',...
    'position',[230 040 200 120],'backgroundcolor',handles.colors.frame,'fontsize',10,'hor','left',...
    'parent',src);

dat=cell(128,3);
n=size(handles.prefs.colormap1,1);
dat(1:n,:)=num2cell(handles.prefs.colormap1);

jtable=javax.swing.JTable(dat,{'R','G','B'}); %initialize a table
javaObjectEDT(jtable);
j=javax.swing.JScrollPane(jtable); %initialize a scrollpane that contains the table
javaObjectEDT(j);
jtable.setFillsViewportHeight(true); %make table fill the scrollpane
jtable.putClientProperty('terminateEditOnFocusLost',true); %doesnt seem to work...
pause(0.01)
[jscrollpane,mtable]=javacomponent(j,[500 120 200 308],src); 
pause(0.01)

set(mtable,'tag','mtable','userdata',jtable);

set(src,'colormap',handles.prefs.colormap1)

unax=axes('parent',src,'visible','off','units','pixels','position',[650 120 100 300]);
C=colorbar('peer',unax);

set(C,'units','pixels','position',[720 120 30 280],'box','on')
set(get(C,'ylabel'),'string','Preview','fontsize',12)

handles.temp.tabobjects=[txt1 menu1 txt2 mtable unax txt3];
guidata(src,handles);

menu_Callback(menu1,[]);

function menu_Callback(src,ev)
handles=guidata(src);
f=ancestor(src,'figure');

i=get(src,'value'); %which colormap to display?

switch i
    case 1
        cmap=handles.prefs.colormap1;
    case 2
        cmap=handles.prefs.colormap2;
    case 3
        cmap=handles.prefs.colormap3;
end

n=size(cmap,1);
dat=cell(128,3);
dat(1:n,:)=num2cell(cmap);


idx2 = linspace(0,1,128);
idx1 = linspace(0,1,n);
cmap=interp1(idx1,cmap,idx2);

set(f,'colormap',cmap);

mtable=findobj(f,'tag','mtable');
mod=javax.swing.table.DefaultTableModel(dat,{'R','G','B'});
jtable=get(mtable,'userdata');
jtable.setModel(mod);




