%Optimizer Animation
function raveeditprefsoptanimation(src,ev)
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2015 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);

%Marker appearance:
feasibletxt=uicontrol('style','text','string','Feasible Points:',...
    'position',[230 428 200 20],'backgroundcolor',handles.colors.frame,'fontsize',10,'hor','left',...
    'parent',src,'fontweight','bold');
[feasiblemarkerstyleaxes,feasiblemarkersizeaxes,feasiblefacecolor,feasibleedgecolor]=makeMarkerControls(src,'feasible',[240 410],handles.prefs.optfeasiblemarker,handles.prefs.optfeasiblemarkersize,...
    handles.colors.optfeasible(1,:),handles.colors.optfeasible(2,:));

infeasibletxt=uicontrol('style','text','string','Infeasible Points:',...
    'position',[230 328 200 20],'backgroundcolor',handles.colors.frame,'fontsize',10,'hor','left',...
    'parent',src,'fontweight','bold');
[infeasiblemarkerstyleaxes,infeasiblemarkersizeaxes,infeasiblefacecolor,infeasibleedgecolor]=makeMarkerControls(src,'infeasible',[240 310],handles.prefs.optinfeasiblemarker,handles.prefs.optinfeasiblemarkersize,...
    handles.colors.optinfeasible(1,:),handles.colors.optinfeasible(2,:));

optimaltxt=uicontrol('style','text','string','Optimal Points:',...
    'position',[230 228 200 20],'backgroundcolor',handles.colors.frame,'fontsize',10,'hor','left',...
    'parent',src,'fontweight','bold');
[optimalmarkerstyleaxes,optimalmarkersizeaxes,optimalfacecolor,optimaledgecolor]=makeMarkerControls(src,'optimal',[240 210],handles.prefs.optoptimalmarker,handles.prefs.optoptimalmarkersize,...
    handles.colors.optoptimal(1,:),handles.colors.optoptimal(2,:));


historytxt=uicontrol('style','text','string','History:',...
    'position',[230 128 100 20],'backgroundcolor',handles.colors.frame,'fontsize',10,'hor','left',...
    'parent',src,'fontweight','bold');
[historymarkerstyleaxes,historymarkersizeaxes,historyfacecolor,historyedgecolor]=makeMarkerControls(src,'history',[240 110],handles.prefs.opthistorymarker,handles.prefs.opthistorymarkersize,...
    handles.colors.opthistory(1,:),handles.colors.opthistory(2,:));

shift=[450 30 0 0];

savecheck=uicontrol('parent',src,'position',[15 280 200 22]+shift,'style','checkbox',...
    'string','Save Animation to File',...
    'backgroundcolor',handles.colors.frame,'fontsize',10,'hor','left',...
    'tag','anicheck','value',handles.prefs.animation.animation,'fontweight','bold',...
    'callback',@savecheck_callback);

formattxt=uicontrol('parent',src,'position',[35 240 200 22]+shift,'style','text',...
    'string','File format:','fontweight','normal',...
    'backgroundcolor',handles.colors.frame,'fontsize',10,'hor','left');

formats={'gif';'avi'};
val=find(ismember(formats,handles.prefs.animation.format));
formatmenu=uicontrol('parent',src,'position',[115 245 200 20]+shift,'style','popupmenu',...
    'string',{'gif','avi'},'value',val,...
    'tag','formatmenu','backgroundcolor',[1 1 1],'callback',@formatmenu_callback);

settingstxt=uicontrol('parent',src,'position',[35 210 200 22]+shift,'style','text',...
    'string','Settings:','fontweight','normal',...
    'backgroundcolor',handles.colors.frame,'fontsize',10,'hor','left');

showinfocheck=uicontrol('parent',src,'position',[15 370+30 250 22]+shift,'style','checkbox',...
    'string','Show optimizer info on graph',...
    'backgroundcolor',handles.colors.frame,'fontsize',10,'hor','left',...
    'tag','infocheck','value',handles.prefs.animation.showinfo,'fontweight','bold',...
    'callback',@showinfocheck_callback);

locationtxt=uicontrol('parent',src,'position',[35 340+30 200 22]+shift,'style','text',...
    'string','Location:',...
    'backgroundcolor',handles.colors.frame,'fontsize',10,'hor','left');

locs={'ne','nw','se','sw'};
val=find(ismember(locs,handles.prefs.animation.infolocation));
locmenu=uicontrol('parent',src,'position',[95 345+30 200 20]+shift,'style','popupmenu',...
    'string',{'Top Right';'Top Left';'Bottom Right';'Bottom Left'},'value',val,...
    'tag','locationmenu','backgroundcolor',[1 1 1],'callback',@locmenu_callback);

fonttxt=uicontrol('parent',src,'position',[35 310+30 200 22]+shift,'style','text',...
    'string','Font:',...
    'backgroundcolor',handles.colors.frame,'fontsize',10,'hor','left');

val=find(ismember(handles.fonts.listfonts,handles.prefs.animation.infofont));
if isempty(val);val=1;end
fontmenu=uicontrol('parent',src,'position',[95 315+30 140 20]+shift,'style','popupmenu',...
    'string',handles.fonts.htmllistfonts,'value',val,...
    'tag','fontmenu','backgroundcolor',[1 1 1],'callback',@fontmenu_callback);

val=find(ismember({'6';'7';'8';'9';'10';'11';'12';'14';'16';'18';'20';'22';'24';'28';'32';'40'},num2str(handles.prefs.animation.infofontsize)));
if isempty(val);val=7;end
fontsizemenu=uicontrol('parent',src,'position',[245 315+30 50 20]+shift,'style','popupmenu',...
    'string',{'6';'7';'8';'9';'10';'11';'12';'14';'16';'18';'20';'22';'24';'28';'32';'40'},'value',val,...
    'tag','fontsizemenu','backgroundcolor',[1 1 1],'callback',@fontsizemenu_callback);

backgroundtxt=uicontrol('parent',src,'position',[35 310 200 22]+shift,'style','text',...
    'string','Background:',...
    'backgroundcolor',handles.colors.frame,'fontsize',10,'hor','left');

locs={'None','Rectangle','Outline'};
val=find(ismember(locs,handles.prefs.animation.background));
backgroundmenu=uicontrol('parent',src,'position',[125 315 170 20]+shift,'style','popupmenu',...
    'string',{'None','Rectangle','Outline'},'value',val,...
    'tag','locationmenu','backgroundcolor',[1 1 1],'callback',@backgroundmenu_callback);

hb1=makehelpblob(src,445,76,'helproot?title=Optimization_animation_preferences#Pause_between_iterations',handles.colors.frame);

pausecheck=uicontrol('parent',src,'position',[15 45 370 22]+shift,'style','checkbox',...
    'string','Pause between iterations until mouse/key is pressed',...
    'backgroundcolor',handles.colors.frame,'fontsize',10,'hor','left',...
    'tag','pausecheck','value',handles.prefs.animation.pause,'fontweight','normal',...
    'callback',@pausecheck_callback);

updatecheck=uicontrol('parent',src,'position',[15 25 370 22]+shift,'style','checkbox',...
    'string','Update Analysis''s "Current Point" after each iteration',...
    'backgroundcolor',handles.colors.frame,'fontsize',10,'hor','left',...
    'tag','updateanalysischeck','value',handles.prefs.animation.updateanalysis,'fontweight','normal',...
    'callback',@updatecheck_callback);

handles.temp.formatobjects=[];

handles.temp.tabobjects=[feasibletxt savecheck formattxt formatmenu settingstxt showinfocheck locationtxt fonttxt locmenu...
    fontmenu fontsizemenu pausecheck updatecheck backgroundtxt backgroundmenu feasiblemarkerstyleaxes feasiblemarkersizeaxes...
    feasiblefacecolor feasibleedgecolor infeasiblemarkerstyleaxes,infeasiblemarkersizeaxes,infeasiblefacecolor,infeasibleedgecolor...
    infeasibletxt optimaltxt optimalmarkerstyleaxes,optimalmarkersizeaxes,optimalfacecolor,optimaledgecolor...
    historytxt historymarkerstyleaxes,historymarkersizeaxes,historyfacecolor,historyedgecolor hb1];
guidata(src,handles);

formatmenu_callback(formatmenu,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function formatmenu_callback(src,ev)

handles=guidata(src);

delete(handles.temp.formatobjects);
handles.temp.formatobjects=[];

val=get(src,'value');


src=ancestor(src,'figure');

fps=num2str(handles.prefs.animation.fps);
if isnan(fps);fps='10';end

shift=[450 30 0 0];

if val==1 % gif
    
    handles.prefs.animation.format='gif';
    
    txt1=uicontrol('position',[50 190 150 22]+shift,'parent',src,...
        'style','text','backgroundcolor',handles.colors.frame,'string','Frame Rate:',...
        'hor','left','fontsize',10,'fontweight','normal');
    radio1=uicontrol('position',[50 170 20 22]+shift,'parent',src,...
        'style','radio','backgroundcolor',handles.colors.frame,'tag','fpsradio',...
        'string','','value',~handles.prefs.animation.realtime,'callback',@fpsradio_callback);
    editbox=uicontrol('position',[70 170 60 22]+shift,'parent',src,...
        'style','edit','backgroundcolor',[1 1 1],'tag','fpsbox',...
        'string',fps,'callback',@fpsedit_callback);
    txt2=uicontrol('position',[135 168 150 22]+shift,'parent',src,...
        'style','text','backgroundcolor',handles.colors.frame,'string','Frames per second',...
        'hor','left','fontsize',10,'fontweight','normal');
    radio2=uicontrol('position',[50 140 20 22]+shift,'parent',src,...
        'style','radio','backgroundcolor',handles.colors.frame,'tag','rtradio',...
        'string','','value',handles.prefs.animation.realtime,'callback',@rtradio_callback);
    txt3=uicontrol('position',[70 138 300 22]+shift,'parent',src,...
        'style','text','backgroundcolor',handles.colors.frame,'string','Real time (averaged to constant frame rate)',...
        'hor','left','fontsize',10,'fontweight','normal');
    txt4=uicontrol('position',[50 100 150 22]+shift,'parent',src,...
        'style','text','backgroundcolor',handles.colors.frame,'string','Loop:',...
        'hor','left','fontsize',10,'fontweight','normal');
    editbox2=uicontrol('position',[90 102 60 22]+shift,'parent',src,...
        'style','edit','backgroundcolor',[1 1 1],'tag','loopbox',...
        'string',num2str(handles.prefs.animation.loop),...
        'callback',@loopedit_callback);
    txt5=uicontrol('position',[155 100 100 22]+shift,'parent',src,...
        'style','text','backgroundcolor',handles.colors.frame,'string','times',...
        'hor','left','fontsize',10,'fontweight','normal');
    
    handles.temp.formatobjects=[txt1 txt2 editbox radio1 radio2 txt3 txt4 editbox2 txt5];
    
elseif val==2 % avi
    
    handles.prefs.animation.format='avi';
    
    txt1=uicontrol('position',[50 190 150 22]+shift,'parent',src,...
        'style','text','backgroundcolor',handles.colors.frame,'string','Frame Rate:',...
        'hor','left','fontsize',10,'fontweight','normal');
    radio1=uicontrol('position',[50 170 20 22]+shift,'parent',src,...
        'style','radio','backgroundcolor',handles.colors.frame,'tag','fpsradio',...
        'string','','value',~handles.prefs.animation.realtime,'callback',@fpsradio_callback);
    editbox=uicontrol('position',[70 170 60 22]+shift,'parent',src,...
        'style','edit','backgroundcolor',[1 1 1],'tag','fpsbox',...
        'string',fps,'callback',@fpsedit_callback);
    txt2=uicontrol('position',[135 168 150 22]+shift,'parent',src,...
        'style','text','backgroundcolor',handles.colors.frame,'string','Frames per second',...
        'hor','left','fontsize',10,'fontweight','normal');
    radio2=uicontrol('position',[50 140 20 22]+shift,'parent',src,...
        'style','radio','backgroundcolor',handles.colors.frame,'tag','rtradio',...
        'string','','value',handles.prefs.animation.realtime,'callback',@rtradio_callback);
    txt3=uicontrol('position',[70 138 300 22]+shift,'parent',src,...
        'style','text','backgroundcolor',handles.colors.frame,'string','Real time (averaged to constant frame rate)',...
        'hor','left','fontsize',10,'fontweight','normal');
    txt4=uicontrol('position',[50 100 250 22]+shift,'parent',src,...
        'style','text','backgroundcolor',handles.colors.frame,'string','Quality:                  (0-100)',...
        'hor','left','fontsize',10,'fontweight','normal');
    editbox2=uicontrol('position',[100 102 60 22]+shift,'parent',src,...
        'style','edit','backgroundcolor',[1 1 1],'tag','qualitybox',...
        'string',num2str(handles.prefs.animation.quality),'callback',@quality_callback);
    txt5=uicontrol('position',[50 70 100 22]+shift,'parent',src,...
        'style','text','backgroundcolor',handles.colors.frame,'string','Compression:',...
        'hor','left','fontsize',10,'fontweight','normal');
    menu1=uicontrol('position',[140 72 100 22]+shift,'parent',src,...
        'style','popup','backgroundcolor',[1 1 1],'string',{'None'},...
        'hor','left','tag','qualitymenu','enable','off');
    
    handles.temp.formatobjects=[txt1 txt2 editbox radio1 radio2 txt3 txt4 editbox2 txt5 menu1];
end

handles.temp.tabobjects=[handles.temp.tabobjects handles.temp.formatobjects];
handles.temp.tabobjects=handles.temp.tabobjects(ishandle(handles.temp.tabobjects));
guidata(src,handles)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function colorblob_Callback(src,ev)
handles=guidata(src);
udat=get(src,'userdata');
newcolor=ravecolorchooser(src,handles.colors.axes{udat(2)}(udat(1),:),'Choose a color');
set(src,'backgroundcolor',newcolor);
handles.colors.axes{udat(2)}(udat(1),:)=newcolor;
guidata(src,handles);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function fpsradio_callback(src,ev)
rtradio=findobj(gcf,'tag','rtradio');
set(rtradio,'value',0);
set(src,'value',1);

handles=guidata(src);
handles.prefs.animation.realtime=0;
guidata(src,handles);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function rtradio_callback(src,ev)
fpsradio=findobj(gcf,'tag','fpsradio');
set(fpsradio,'value',0);
set(src,'value',1);

handles=guidata(src);
handles.prefs.animation.realtime=1;
guidata(src,handles);

function savecheck_callback(src,ev)
handles=guidata(src);
handles.prefs.animation.animation=get(src,'value');
guidata(src,handles);

function showinfocheck_callback(src,ev)
handles=guidata(src);
handles.prefs.animation.showinfo=get(src,'value');
guidata(src,handles);

function locmenu_callback(src,ev)
handles=guidata(src);
locs={'ne','nw','se','sw'};
handles.prefs.animation.infolocation=locs{get(src,'value')};
guidata(src,handles);

function fontmenu_callback(src,ev)
handles=guidata(src);
fontz=handles.fonts.listfonts;
handles.prefs.animation.infofont=fontz{get(src,'value')};
guidata(src,handles);

function fontsizemenu_callback(src,ev)
handles=guidata(src);
sizes=get(src,'string');
handles.prefs.animation.infofontsize=str2double(sizes{get(src,'value')});
guidata(src,handles);

function backgroundmenu_callback(src,ev)
handles=guidata(src);
opts=get(src,'string');
handles.prefs.animation.background=opts{get(src,'value')};
guidata(src,handles);

function loopedit_callback(src,ev)
handles=guidata(src);
loop=str2double(get(src,'string'));
if isnan(loop) || loop<1
    errordlg('You must enter value <=1 in the Loop box')
    return
end
handles.prefs.animation.loop=loop;
guidata(src,handles);

function fpsedit_callback(src,ev)
handles=guidata(src);
fps=str2double(get(src,'string'));
if isnan(fps) || fps<=0
    errordlg('You must enter a positive value in the Frames per second box')
    return
end
handles.prefs.animation.fps=fps;
guidata(src,handles);

function pausecheck_callback(src,ev)
handles=guidata(src);
handles.prefs.animation.pause=get(src,'value');
guidata(src,handles);

function quality_callback(src,ev)
handles=guidata(src);
qualitybox=findobj(src,'tag','qualitybox');
quality=str2double(get(qualitybox,'string'));
if isnan(quality) || quality<0 || quality>100
    errordlg('You must enter value between 0 and 100 in the Quality box')
    return
end
handles.prefs.animation.quality=quality;
guidata(src,handles);

function qualitymenu_callback(src,ev)
handles=guidata(src);
qualitymenu=findobj(src,'tag','qualitymenu');
strz=get(qualitymenu,'string');
strr=strz{get(qualitymenu,'value')};
handles.prefs.animation.compression=strr;
guidata(src,handles);

function updatecheck_callback(src,ev)
handles=guidata(src);
handles.prefs.animation.updateanalysis=get(src,'value');
guidata(src,handles);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function [markerstyleaxes markersizeaxes facecolor edgecolor]=makeMarkerControls(src,label,origin,marker,markersize,facecolor,edgecolor)

handles=guidata(src);

markerstyleaxes=axes('units','pixels','position',[origin(1) origin(2) 144 20],...
    'xlimmode','manual','ylimmode','manual','visible','off','nextplot','add','xcolor',[1 1 1],...
    'ycolor',[1 1 1],'zcolor',[1 1 1],'parent',src,'userdata',label);
gap=.9/8;
%markerstyleoptions:
offcolor=[.95 .95 .95];
oncolor=handles.colors.text;

line(.05,.5,'marker','s','parent',markerstyleaxes,'markersize',8,'markerfacecolor',offcolor,'markeredgecolor',[.5 .5 .5],'buttondownfcn',@styleblob_callback,'tag','styleblob')
line(.05+gap,.5,'marker','o','parent',markerstyleaxes,'markersize',8,'markerfacecolor',offcolor,'markeredgecolor',[.5 .5 .5],'buttondownfcn',@styleblob_callback,'tag','styleblob')
line(.05+2*gap,.5,'marker','d','parent',markerstyleaxes,'markersize',8,'markerfacecolor',offcolor,'markeredgecolor',[.5 .5 .5],'buttondownfcn',@styleblob_callback,'tag','styleblob')
line(.05+3*gap,.5,'marker','^','parent',markerstyleaxes,'markersize',8,'markerfacecolor',offcolor,'markeredgecolor',[.5 .5 .5],'buttondownfcn',@styleblob_callback,'tag','styleblob')
line(.05+4*gap,.5,'marker','h','parent',markerstyleaxes,'markersize',12,'markerfacecolor',offcolor,'markeredgecolor',[.5 .5 .5],'buttondownfcn',@styleblob_callback,'tag','styleblob')
line(.05+5*gap,.5,'marker','p','parent',markerstyleaxes,'markersize',12,'markerfacecolor',offcolor,'markeredgecolor',[.5 .5 .5],'buttondownfcn',@styleblob_callback,'tag','styleblob')
line(.05+6*gap,.5,'marker','*','parent',markerstyleaxes,'markersize',8,'markerfacecolor',offcolor,'markeredgecolor',[.5 .5 .5],'buttondownfcn',@styleblob_callback,'tag','styleblob')
line(.05+7*gap,.5,'marker','+','parent',markerstyleaxes,'markersize',8,'markerfacecolor',offcolor,'markeredgecolor',[.5 .5 .5],'buttondownfcn',@styleblob_callback,'tag','styleblob')
line(.95,.5,'marker','x','parent',markerstyleaxes,'markersize',8,'markerfacecolor',offcolor,'markeredgecolor',[.5 .5 .5],'buttondownfcn',@styleblob_callback,'tag','styleblob')

%handles.styleblobs=get(markerstyleaxes,'children');
%color selected style:

hhh=findobj(markerstyleaxes,'marker',marker);
set(hhh,'markerfacecolor',oncolor,'markeredgecolor',oncolor/3);

markersizeaxes=axes('units','pixels','position',[origin(1) origin(2)-25 144 20],...
    'xlimmode','manual','ylimmode','manual','visible','off','nextplot','add','xcolor',[1 1 1],...
    'ycolor',[1 1 1],'zcolor',[1 1 1],'parent',src,'userdata',label,'tag','markersizeaxes');
%gap=.9/7;
%markersizeoptions:
line(.05,.5,'marker',marker,'parent',markersizeaxes,'markersize',1,'markerfacecolor',offcolor,'markeredgecolor',[.5 .5 .5],'buttondownfcn',@sizeblob_callback,'tag','sizeblob')
line(.12,.5,'marker',marker,'parent',markersizeaxes,'markersize',4,'markerfacecolor',offcolor,'markeredgecolor',[.5 .5 .5],'buttondownfcn',@sizeblob_callback,'tag','sizeblob')
line(.22,.5,'marker',marker,'parent',markersizeaxes,'markersize',6,'markerfacecolor',offcolor,'markeredgecolor',[.5 .5 .5],'buttondownfcn',@sizeblob_callback,'tag','sizeblob')
line(.33,.5,'marker',marker,'parent',markersizeaxes,'markersize',8,'markerfacecolor',offcolor,'markeredgecolor',[.5 .5 .5],'buttondownfcn',@sizeblob_callback,'tag','sizeblob')
line(.45,.5,'marker',marker,'parent',markersizeaxes,'markersize',10,'markerfacecolor',offcolor,'markeredgecolor',[.5 .5 .5],'buttondownfcn',@sizeblob_callback,'tag','sizeblob')
line(.59,.5,'marker',marker,'parent',markersizeaxes,'markersize',12,'markerfacecolor',offcolor,'markeredgecolor',[.5 .5 .5],'buttondownfcn',@sizeblob_callback,'tag','sizeblob')
line(.75,.5,'marker',marker,'parent',markersizeaxes,'markersize',14,'markerfacecolor',offcolor,'markeredgecolor',[.5 .5 .5],'buttondownfcn',@sizeblob_callback,'tag','sizeblob')
line(.95,.5,'marker',marker,'parent',markersizeaxes,'markersize',18,'markerfacecolor',offcolor,'markeredgecolor',[.5 .5 .5],'buttondownfcn',@sizeblob_callback,'tag','sizeblob')

%handles.sizeblobs=get(markersizeaxes,'children');

%make size blobs the right style

%color the selected blob
hhh=findobj(markersizeaxes,'markersize',markersize);
set(hhh,'markerfacecolor',oncolor,'markeredgecolor',oncolor/3);

facecolor=uicontrol('position',[origin(1) origin(2)-55 25 25],...
    'style','pushbutton','string','','backgroundcolor',facecolor,...
    'callback',@faceblob_callback,'parent',src,'userdata',label);

edgecolor=uicontrol('position',[origin(1)+30 origin(2)-55 25 25],...
    'style','pushbutton','string','','backgroundcolor',edgecolor,...
    'callback',@edgeblob_callback,'parent',src,'userdata',label);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function styleblob_callback(src,ev)
handles=guidata(src);
type = get(ancestor(src,'axes'),'userdata');

offcolor=[.95 .95 .95];
styleblobs=get(ancestor(src,'axes'),'children');

set(styleblobs,'markerfacecolor',offcolor,'markeredgecolor',[.5 .5 .5])
set(src,'markerfacecolor',handles.colors.text,'markeredgecolor',handles.colors.text/3)

switch type
    case 'feasible'
        handles.prefs.optfeasiblemarker=get(src,'marker');
    case 'infeasible'
        handles.prefs.optinfeasiblemarker=get(src,'marker');
    case 'optimal'
        handles.prefs.optoptimalmarker=get(src,'marker');
    case 'history'
        handles.prefs.opthistorymarker=get(src,'marker');
end

%update sizeblobs
sizeaxes = findobj(handles.temp.tabobjects,'tag','markersizeaxes','userdata',type);
sizeblobs=get(sizeaxes,'children');
set(sizeblobs,'marker',get(src,'marker'));

guidata(src,handles);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function sizeblob_callback(src,ev)
handles=guidata(src);

offcolor=[.95 .95 .95];
oncolor=handles.colors.text;

sizeblobs=get(ancestor(src,'axes'),'children');

%get new size:
newsize=get(src,'markersize');

type=get(ancestor(src,'axes'),'userdata');
switch type
    case 'feasible'
        handles.prefs.optfeasiblemarkersize=get(src,'markersize');
    case 'infeasible'
        handles.prefs.optinfeasiblemarkersize=get(src,'markersize');
    case 'optimal'
        handles.prefs.optoptimalmarkersize=get(src,'markersize');
    case 'history'
        handles.prefs.opthistorymarkersize=get(src,'markersize');
end

set(sizeblobs,'markerfacecolor',offcolor,'markeredgecolor',[.5 .5 .5])
set(src,'markerfacecolor',oncolor,'markeredgecolor',oncolor/3)

guidata(src,handles);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function faceblob_callback(src,ev)
handles=guidata(src);

newcolor=ravecolorchooser(src,get(src,'backgroundcolor'),'Choose a marker fill color');
set(src,'backgroundcolor',newcolor);

type=get(src,'userdata');
switch type
    case 'feasible'
        handles.colors.optfeasible(1,:)=newcolor;
    case 'infeasible'
        handles.colors.optinfeasible(1,:)=newcolor;
    case 'optimal'
        handles.colors.optoptimal(1,:)=newcolor;
    case 'history'
        handles.colors.opthistory(1,:)=newcolor;
end

guidata(src,handles);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function edgeblob_callback(src,ev)
handles=guidata(src);

newcolor=ravecolorchooser(src,get(src,'backgroundcolor'),'Choose a marker edge color');
set(src,'backgroundcolor',newcolor);

type=get(src,'userdata');
switch type
    case 'feasible'
        handles.colors.optfeasible(2,:)=newcolor;
    case 'infeasible'
        handles.colors.optinfeasible(2,:)=newcolor;
    case 'optimal'
        handles.colors.optoptimal(2,:)=newcolor;
    case 'history'
        handles.colors.opthistory(2,:)=newcolor;
end
guidata(src,handles);



