%Workspace & Grid
function raveeditprefsworkspace(src,ev)

% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2013 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);

txt1=uicontrol('style','text','string','Workspace dimensions:',...
    'position',[230 428 640 20],'backgroundcolor',handles.colors.frame,'fontsize',10,'hor','left',...
    'parent',src,'fontweight','bold');

txt2=uicontrol('style','text','string','Width:',...
    'position',[240 403 640 20],'backgroundcolor',handles.colors.frame,'fontsize',10,'hor','left',...
    'parent',src,'fontweight','normal');
txt3=uicontrol('style','text','string','Height:',...
    'position',[240 378 640 20],'backgroundcolor',handles.colors.frame,'fontsize',10,'hor','left',...
    'parent',src,'fontweight','normal');

e1=uicontrol('style','edit','string',num2str(handles.prefs.wswidth),...
    'position',[300 403 80 20],'backgroundcolor',[1 1 1],'fontsize',10,...
    'parent',src,'fontweight','normal','enable','off');
e2=uicontrol('style','edit','string',num2str(handles.prefs.wsheight),...
    'position',[300 378 80 20],'backgroundcolor',[1 1 1],'fontsize',10,...
    'parent',src,'fontweight','normal','enable','off');

txt4=uicontrol('style','text','string','Grid:',...
    'position',[230 340 640 20],'backgroundcolor',handles.colors.frame,'fontsize',10,'hor','left',...
    'parent',src,'fontweight','bold');
txt5=uicontrol('style','text','string','Major grid size:',...
    'position',[240 315 640 20],'backgroundcolor',handles.colors.frame,'fontsize',10,'hor','left',...
    'parent',src,'fontweight','normal');
e3=uicontrol('style','edit','string',num2str(handles.prefs.gridsize),...
    'position',[340 315 40 20],'backgroundcolor',[1 1 1],'fontsize',10,...
    'parent',src,'fontweight','normal','enable','off');
c1=uicontrol('style','checkbox','string','Snap workspace objects to grid',...
    'position',[240 285 640 20],'backgroundcolor',handles.colors.frame,'fontsize',10,'hor','left',...
    'parent',src,'fontweight','normal','value',handles.prefs.snaptogrid);

[jslider,hslider]=javacomponent('javax.swing.JSlider',[250 200 110 40],src);
jslider.setPaintTicks(1)
jslider.setSnapToTicks(1)
jslider.setMinimum(1)
jslider.setMaximum(3)
jslider.setMajorTickSpacing(1);
jslider.setValue(handles.prefs.gridmode);
set(jslider,'background',java.awt.Color(handles.colors.frame(1),handles.colors.frame(2),handles.colors.frame(3)))
set(jslider,'foreground',java.awt.Color.black)
set(jslider,'StateChangedCallback',{@jslider_Callback,src});


txt6=uicontrol('style','text','string','(All dimensions are in pixels)',...
    'position',[240 80 640 20],'backgroundcolor',handles.colors.frame,'fontsize',10,'hor','left',...
    'parent',src,'fontweight','normal');

txt7=uicontrol('style','text','string','Major          1/2           1/4',...
    'position',[242 185 640 20],'backgroundcolor',handles.colors.frame,'fontsize',10,'hor','left',...
    'parent',src,'fontweight','normal','fontsize',8);

txt8=uicontrol('style','text','string','Display grid refinement:',...
    'position',[240 240 640 20],'backgroundcolor',handles.colors.frame,'fontsize',10,'hor','left',...
    'parent',src,'fontweight','normal');

txt9=uicontrol('style','text','string','Workspace Color:',...
    'position',[500 403 110 20],'backgroundcolor',handles.colors.frame,'fontsize',10,'hor','right',...
    'parent',src,'fontweight','normal');
b1=uicontrol('style','pushbutton','position',[620 403 24 24],'backgroundcolor',handles.colors.wsbg,'parent',src,...
    'callback',@changewscolor);

txt10=uicontrol('style','text','string','Grid Color:',...
    'position',[500 315 110 20],'backgroundcolor',handles.colors.frame,'fontsize',10,'hor','right',...
    'parent',src,'fontweight','normal');
b2=uicontrol('style','pushbutton','position',[620 315 24 24],'backgroundcolor',handles.colors.wsgrid,'parent',src,...
    'callback',@changegridcolor);

c2=uicontrol('style','checkbox','string','Show grid',...
    'position',[570 285 640 20],'backgroundcolor',handles.colors.frame,'fontsize',10,'hor','left',...
    'parent',src,'fontweight','normal','value',handles.prefs.showgrid,'callback',@gridcheck_callback);

handles.temp.tabobjects=[txt1 txt2 txt3 e1 e2 e3 txt4 txt5 txt6 c1 hslider txt7 txt8 txt9 b1 txt10 b2 c2];

guidata(src,handles);


function jslider_Callback(src,eventdata,fighandle) %#ok<INUSL>
moving = get(src,'ValueIsAdjusting');
if strcmp(moving,'off')
    return
end

handles=guidata(fighandle);

handles.prefs.gridmode=src.getValue;

if get(handles.showgridbutton,'value')
    if handles.prefs.gridmode==1
        set(handles.lgrid,'visible','on')
        set(handles.mgrid,'visible','off')
        set(handles.sgrid,'visible','off')
        handles.snapsize=handles.prefs.gridsize;
    elseif handles.prefs.gridmode==2
        set(handles.lgrid,'visible','on')
        set(handles.mgrid,'visible','on')
        set(handles.sgrid,'visible','off')
        handles.snapsize=handles.prefs.gridsize/2;
    else
        set(handles.lgrid,'visible','on')
        set(handles.mgrid,'visible','on')
        set(handles.sgrid,'visible','on')
        handles.snapsize=handles.prefs.gridsize/4;
    end
end

if ~handles.prefs.snaptogrid
    handles.snapsize=1;
end

guidata(fighandle,handles)



function changewscolor(src,ev)
handles=guidata(src);

%also change color of blockers that havent been assigned a different color:
theseblockers=findobj(handles.allblockers(ishandle(handles.allblockers)),'facecolor',handles.colors.wsbg);

%udata.format.linecolor=uisetcolor(udata.format.linecolor);
handles.colors.wsbg=ravecolorchooser(handles.fig,handles.colors.wsbg,'Pick a Color');
set(theseblockers,'facecolor',handles.colors.wsbg);
set(handles.mainaxes,'color',handles.colors.wsbg,'xcolor',handles.colors.wsbg,'ycolor',handles.colors.wsbg,'zcolor',handles.colors.wsbg);
set(handles.explorer,'color',handles.colors.wsbg);
set(src,'backgroundcolor',handles.colors.wsbg);

mgrid=findobj(handles.mainaxes,'tag','mgrid');
sgrid=findobj(handles.mainaxes,'tag','sgrid');

set(mgrid,'color',(handles.colors.wsgrid+handles.colors.wsbg)/2)
set(sgrid,'color',(handles.colors.wsgrid+handles.colors.wsbg)/2)

handles.colors.wsbg=handles.colors.wsbg;

guidata(src,handles);

%update all images in case they use a transparent background:
othergraphs=[];
originalindex=handles.activeindex;
for g=1:numel(handles.allgraphs)
    if ishandle(handles.allgraphs(g))
        if strcmp(get(handles.allgraphs(g),'userdata'),'image')
            othergraphs(end+1)=g;
        end
    end
    
    for ai=othergraphs
        handles.activeindex=ai;
        
        othergraph=handles.allgraphs(ai);
        thistype=get(othergraph,'userdata'); %type of other graph in this analysis
        handles.activetype=thistype;
        handles.activegraph=othergraph;
        guidata(handles.fig,handles);
        eval(['ravecreate',lower(thistype),'(handles.fig,[]);']);
        handles=guidata(handles.fig); %refresh handles;
        
    end
    
end

othergraph=handles.allgraphs(originalindex);
thistype=get(othergraph,'userdata'); %type of other graph in this analysis

handles.activeindex=originalindex;
handles.activetype=thistype;
handles.activegraph=othergraph;
guidata(handles.fig,handles);


function changegridcolor(src,ev)
handles=guidata(src);

%also change color of blockers that havent been assigned a different color:
lgrid1=findobj(handles.mainaxes,'tag','lgrid');
lgrid2=findobj(handles.explorer,'tag','lgrid');
mgrid=findobj(handles.mainaxes,'tag','mgrid');
sgrid=findobj(handles.mainaxes,'tag','sgrid');

%udata.format.linecolor=uisetcolor(udata.format.linecolor);
handles.colors.wsgrid=ravecolorchooser(handles.fig,handles.colors.wsgrid,'Pick a Color');

set(lgrid1,'color',handles.colors.wsgrid);
set(lgrid2,'color',(handles.colors.wsgrid+handles.colors.wsbg)/2);
set(mgrid,'color',(handles.colors.wsgrid+handles.colors.wsbg)/2);
set(sgrid,'color',(handles.colors.wsgrid+handles.colors.wsbg)/2);
set(findobj(handles.mainaxes,'tag','wspanelborder'),'color',handles.colors.wsgrid)

set(src,'backgroundcolor',handles.colors.wsgrid);

idx=ishandle(handles.allblockers);
idx(handles.activeindex)=false;

set(handles.allblockers(idx),'edgecolor',handles.colors.wsgrid);

handles.colors.wsgrid=handles.colors.wsgrid; 
guidata(src,handles);


function gridcheck_callback(hObject,eventdata)
handles=guidata(hObject);

%do not alter the currently selected graph, or any annotation class graph.
changethese=false(size(handles.allgraphs));

for c=1:numel(changethese)
    if ishandle(handles.allgraphs(c)) && handles.allgraphs(c)~=handles.activegraph && ~strcmp(handles.graphinfo.(get(handles.allgraphs(c),'userdata')).class,'annotation')
        changethese(c)=true;
    end
end

if get(hObject,'value')
    if handles.prefs.gridmode==1
        set(handles.lgrid,'visible','on')
    elseif handles.prefs.gridmode==2
        set(handles.lgrid,'visible','on')
        set(handles.mgrid,'visible','on')
    else
        set(handles.lgrid,'visible','on')
        set(handles.mgrid,'visible','on')
        set(handles.sgrid,'visible','on')
    end
    
    set(handles.allblockers(changethese),'edgecolor',handles.colors.wsgrid)
    
else
    set(handles.sgrid,'visible','off')
    set(handles.mgrid,'visible','off')
    set(handles.lgrid,'visible','off')
    set(handles.allblockers(changethese),'edgecolor','none')
    
end

