function ravearrangeobjects(targetaxes,ev)
%
% First input must be a handle in handles.allgraphs.
%
% Arranges all axes, text, controls, etc related to a particular graph to make them
% fit within the bounds defined by the handles.allgraphs axes.
% Used by raveselectgraph (to move without resizing) and by ravemovegraph (to resize).
%should be called after calling raveresizeaxes, which makes handles.allgraphs axes
%fit in the bounds defined by handles.allblockers.
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu


persistent cbar

handles=guidata(targetaxes);
ai=find(handles.allgraphs==targetaxes);
udata=get(handles.fakeax(ai),'userdata');
activetype=get(targetaxes,'userdata');


%do type-specific stuff:
switch handles.graphinfo.(activetype).axesformat
    case 'matrix'
        numy=size(handles.subax{ai},1);
        numx=size(handles.subax{ai},2);
        pos = floor(get(targetaxes,'position'));
        
        vspace=handles.prefs.matrixspacing;
        hspace=handles.prefs.matrixspacing;
        
        if strcmp(activetype,'derivativeprofiler')
            %pos=pos;
            vspace=handles.prefs.profilervspacing;
            hspace=handles.prefs.profilerhspacing;
        end
        
        width = floor((pos(3)-numx*hspace)/numx);
        height = floor((pos(4)-numy*vspace)/numy);
        % pos(1:2) = pos(1:2) + [hspace*width vspace*height];
        
        ax=handles.subax{ai};
        fakeax=handles.subfakeax{ai};
        fakerax=handles.subfakerax{ai};
        
        ticklength=handles.prefs.ticklength./max(width,height);
        
        %set up grid of axes.
        for r=numy:-1:1,
            for c=numx:-1:1,
                
                
                oldxtick = get(ax(r,c),'xtick');
                oldytick = get(ax(r,c),'ytick');
                oldztick = get(ax(r,c),'ztick');
                
                axPos = uint32(floor([pos(1)+(c-1)*(width+hspace) pos(2)+(numy-r)*(height+vspace) ...
                    width height]));
                set([ax(r,c) fakerax(r,c) fakeax(r,c)],'Position',uint32(axPos),'ticklength',[ticklength ticklength]);
                
                newxtick = get(ax(r,c),'xtick');
                newytick = get(ax(r,c),'ytick');
                newztick = get(ax(r,c),'ztick');
                
                changedticks='';
                newticks={};
                
                if ~isequal(oldxtick,newxtick)
                    changedticks(end+1)='x';
                    newticks{1}=newxtick;
                end
                if ~isequal(oldytick,newytick)
                    changedticks(end+1)='y';
                    newticks{2}=newytick;
                end
                if ~isequal(oldztick,newztick)
                    changedticks(end+1)='z';
                    newticks{3}=newztick;
                end
                
                if ~isempty(changedticks)
                    newsubaxeschangelistener(ax(r,c),[],changedticks,newticks,handles);
                end
                
                
                
            end
        end
        
        %for scattermatrix, resize axis labels to fit on diagonals:
        labelz=findobj(handles.subax{ai},'tag','axeslabel');
        udata=get(handles.fakeax(ai),'userdata');
        for g=1:numel(labelz)
            %[wrapped]=superiorlinewrap(handles.allnames{udata.view.dataset}{udata.view.x(g)},width,handles.fonts.graph,handles.fonts.graphsize,'plain',handles.axiscanvas,handles.fontmetrics.graph);
            [wrapped]=superiorlinewrapfast(handles.allnames{udata.view.dataset}{udata.view.x(g)},width,handles.fontmetrics.graph);
            set(labelz(g),'string',wrapped);            
        end
        
        %If layout is square, recenter axis labels on diagonal:
        if isfield(udata.view,'layout') && strcmp(udata.view.layout,'square')
           for g = 1:numel(labelz)
              xlimz=get(ax(g,g),'xlim');
              ylimz=get(ax(g,g),'ylim');
              set(labelz(g),'position',[mean(xlimz) mean(ylimz)]);
           end
        end
            
            
        
        
        
    case 'row' %need to move subax
        udata=get(handles.fakeax(ai),'userdata');
        numvars=numel(handles.subax{ai});
        ax=handles.subax{ai};
        fakeax=handles.subfakeax{ai};
        fakerax=handles.subfakerax{ai};
        
        margin=0;
        
        pos=getpixelposition(targetaxes);
        
        %determine space for y labels:
        alllabs=[get(ax,'ylabel'),get(fakeax,'ylabel'),get(fakerax,'ylabel')];
        if iscell(alllabs);alllabs=[alllabs{:}];end
        strz=get(alllabs,'string');
        nrows=cellfun(@numel,strz);
        for g=1:numel(strz);
            if ~iscell(strz{g}) && ~isempty(strz{g})
                nrows(g)=1;
            end
        end
        
        if max(nrows)>0
            ylabelspace=32*max(reshape(nrows,size(alllabs)),[],2)+10;
            ylabelspace=max(ylabelspace);
        else
            ylabelspace=0;
        end
        
        
        %determine space for x labels:
        alllabs=[get(ax,'xlabel'),get(fakeax,'xlabel'),get(fakerax,'xlabel')];
        if iscell(alllabs);alllabs=[alllabs{:}];end
        strz=get(alllabs,'string');
        nrows=cellfun(@numel,strz);
        for g=1:numel(strz);
            if ~iscell(strz{g}) && ~isempty(strz{g})
                nrows(g)=1;
            end
        end
        xlabelspace=20*max(reshape(nrows,size(alllabs)),[],2);
        
        xlabelspace=max(xlabelspace);
        ytickspace=32;
        xtickspace=16;
        
        %for altered orientation, swap definition of x/y
        if isfield(udata.format,'orient')
            if strcmp(udata.format.orient,'horizontal');
                temp=[xlabelspace xtickspace];
                ylabelspace=temp(1);
                ytickspace=temp(2);
            end
        end
        
        
        if  true;  %pos(3) > pos(4) %&&  ~udata.view.stack  %horizontal stacking
            %             width = pos(3)/numvars-ylabelspace+min(5,ylabelspace/numvars);
            %             height = pos(4)-xlabelspace-margin;
            %             space = .03; % 3 percent space between subaxes
            %             pos(1:2) = pos(1:2) + space*[width height];% + [0 xlabelspace];
            %
            width = (pos(3)-(ylabelspace+ytickspace)*(numvars))/numvars;
            height = pos(4);%-xlabelspace-margin;
            space = .00; % 3 percent space between subaxes
            pos(1:2) = pos(1:2) + space*[width height];% + [0 xlabelspace];
            
            
        else %vertical stacking
            width=pos(3)-ylabelspace-margin;
            height=pos(4)/numvars-xlabelspace;
            space = .03; % 3 percent space between subaxes
            pos(1:2) = pos(1:2) + space*[width height] + [ylabelspace 0];
        end
        
        if width<3;width=3;end
        if height<3;height=3;end
        
        ticklength=handles.prefs.ticklength./max(width,height);
        
        
        for c=1:numvars
            
            
            oldxtick = get(ax(c),'xtick');
            oldytick = get(ax(c),'ytick');
            oldztick = get(ax(c),'ztick');
           
            if true %pos(3) > pos(4) %HORIZONTAL STACKING
                axPos = [pos(1)+(c-1)*(width+ylabelspace+ytickspace)+ylabelspace+ytickspace pos(2)+margin ...
                    width-ylabelspace height]; %subtract margin to make space for statistics
            else
                axPos = [pos(1) pos(2)+(c-1)*(height+xlabelspace)+xlabelspace ...
                    width-ylabelspace height]; %subtract margin to make space for statistics
            end
            
            axPos(axPos<3) = 3; %avoid nonpositive sizes.
            
            set([ax(c) fakerax(c) fakeax(c)],'units','pixels','position',uint64(axPos),'ticklength',[ticklength ticklength])
            
            newxtick = get(ax(c),'xtick');
            newytick = get(ax(c),'ytick');
            newztick = get(ax(c),'ztick');

            changedticks='';
            newticks={};
            
            if ~isequal(oldxtick,newxtick)
                changedticks(end+1)='x';
                newticks{1}=newxtick;
            end
            if ~isequal(oldytick,newytick)
                changedticks(end+1)='y';
                newticks{2}=newytick;
            end
            if ~isequal(oldztick,newztick)
                changedticks(end+1)='z';
                newticks{3}=newztick;
            end
            
            if ~isempty(changedticks)
                newsubaxeschangelistener(ax(c),[],changedticks,newticks,handles);
            end
            
            set([fakerax(c) fakeax(c)],'xlim',get(ax(c),'xlim'),'ylim',get(ax(c),'ylim'),'zlim',get(ax(c),'zlim'));
%             set([fakerax(c) fakeax(c)],'xtick',get(ax(c),'xtick'),'ytick',get(ax(c),'ytick'),'ztick',get(ax(c),'ztick'))
        end
    case '3D'
        ravesynchaxes;
end

if ~strcmp(handles.graphinfo.(activetype).axesformat,'custom');
    %move the colorbar too:
    if ~strcmp(handles.graphinfo.(activetype).axesformat,'custom') && handles.graphinfo.(activetype).hascolorbar
        %newpos=get(handles.allblockers(handles.activeindex),'position');
        if udata.format.colorbar
            if isempty(cbar) || any(~ishandle(cbar))
                legdata=get(handles.fakerax(ai),'userdata');
                if ~isempty(legdata)
                    cbar=findobj(legdata(ishandle(legdata)),'tag','Colorbar');
                else
                    cbar=[];
                end
            end
            
            if strcmpi(handles.graphinfo.(activetype).axesformat,'3d')
                offset3d=20;
            else
                offset3d=0;
            end
            
            axpos=get(targetaxes,'position');
            
            coffset=0;
            for c=1:numel(cbar)
                set(cbar(c),'pos',[axpos(1)+axpos(3)+20+offset3d+coffset axpos(2) 27 axpos(4)]);
                coffset=coffset+100;
            end
        end
    end
    
    %move tick labels:
    if ismember('alwaysrotatexticks',udata.attributes)
        ar=true;
    else
        ar=false;
    end
    
    switch handles.graphinfo.(activetype).axesformat
        case {'single','3d','3D'}
            ravemovecustomticks(targetaxes,[],ar);
        case {'matrix'}
            ravemovecustomticksmatrix(targetaxes,[],ar);
        case {'row'}
            %First do normal ravemovecustomticks for all axes, then to
            %ravemovecustomticksmatrix to fix the ticks that need to align:
            for r=1:numel(handles.subax{ai})
                ravemovecustomticks(handles.subax{ai}(r),[],ar);
            end
            ravemovecustomticksmatrix(targetaxes,[],ar);
    end
end

%run custom arrange objects function if it exists:
if ~isempty(which(['ravearrangeobjects',activetype]))
    eval(['ravearrangeobjects',activetype,'(targetaxes,[])']);
end



