
function obj = ravebar(ax,centers,heights,width,style,flipvertical,flipaxes,ishg2)
%rave-specific implementation of the "bar" function that is drawn with patches.
%ax - handles of axes
%centers - xdata values of center of each bar
%ydata - height of each bar (length == length(centers))
%width - values from 0-1 indicating relative bar width
%style - 'stacked' or 'grouped' for stacked or side-by-side bars
%flipvertical - true to flip bars vertically (reverse stacking order)
%flipaxes - true to swap x and y data, so that bars extend horizontally
%This function only makes the bars... it does not handle coloring.
%Output is an hggroup object whose children are patches representing each bar.




obj=[];

if nargin<8
    ishg2=false;
end

if nargin<7
    flipaxes=false;
end

if nargin<6
    flipvertical=false;
end

if nargin<5
    style='stacked';
end
style=lower(style);
if nargin<4
    width=1;
end

%make sure centers and heights have the right shape
centers = centers(:)'; %a row vector

if ~any(size(heights)==numel(centers))
    error('Heights and centers must be the same size');
elseif size(heights,2)~=size(centers,2)
    heights=heights';
end

if ~ismember(style,{'stacked','grouped'})
    error('Ravebar style must be ''stacked'' or ''grouped''');
end


%make xdata and ydata from centers and heights
if numel(centers)>1
    dx = min(diff(centers)); %theoretically all dx should be identical, but that isnt strictly enforced
else
    dx=1; %if only drawing 1 bar, give it a width of "1"
end
    
if strcmp(style,'stacked') %no need to distribute xdata
    xmin = repmat(centers-(0.5*width*dx),size(heights,1),1);
    xmax = repmat(centers+(0.5*width*dx),size(heights,1),1);
    
    if flipvertical
        ally = [flipud(cumsum(flipud(heights),1));zeros(1,size(heights,2))];
    else
        ally=[zeros(1,size(heights,2));cumsum(heights,1)];
    end
    
    ymin = ally(2:end,:);
    ymax = ally(1:end-1,:);
    
    %turn into patch data:
    
    for i=size(xmin,1):-1:1
        xdata = [xmin(i,:); xmin(i,:);xmax(i,:);xmax(i,:)];
        ydata = [ymin(i,:);ymax(i,:);ymax(i,:);ymin(i,:)];
        if ~flipaxes
            obj(i) =  patch(xdata,ydata,[.5 .5 .5],'parent',ax);
        else
            obj(i) =  patch(ydata,xdata,[.5 .5 .5],'parent',ax);
        end
    end
    

else %distribute x space amoung side-by-side bars
    if flipvertical
        offsets = linspace(1,-1,size(heights,1)+2)*dx/2;
    else
        offsets = linspace(-1,1,size(heights,1)+2)*dx/2;
    end
    
    offsets = offsets(2:end-1); %remove two spacing bars between series.

    newcenters = bsxfun(@plus,centers,offsets(:));
    
    dx = dx./(size(heights,1)+1); %size to allow space for 1 additional (empty) bar between each group
    
    xmin = newcenters - dx/2;
    xmax = newcenters + dx/2;

    ymin = zeros(size(heights));
    ymax = heights;
    
    %turn into patch data:
    
    for i=size(xmin,1):-1:1
        xdata = [xmin(i,:); xmin(i,:);xmax(i,:);xmax(i,:)];
        ydata = [ymin(i,:);ymax(i,:);ymax(i,:);ymin(i,:)];
        if ~flipaxes
            obj(i) =  patch(xdata,ydata,[.5 .5 .5],'parent',ax);
        else
            obj(i) =  patch(ydata,xdata,[.5 .5 .5],'parent',ax);
        end
    end
end

if ishg2
   set(obj,'alignvertexcenters','on') 
end

