function f=ravechangerangesgui(d,a,handles)
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

%make figure:
f=figure('units','pixels','position',[0 0 651 660],'name',...
    'Define Roles and Data Types','numbertitle','off','menubar','none','resize','off',...
    'windowstyle','modal');
handles.temp.gui=f;
origin=get(0,'userdata');
thispos=get(f,'position');
thispos(1)=origin(1)+origin(3)/2-thispos(3)/2;
thispos(2)=thispos(2)+origin(2)+origin(4)/2-thispos(4)/2;
set(f,'position',thispos);

handles.temp.d=d;
handles.temp.a=a;

%get theme
set(f,'color',handles.colors.frame)


%make initial uicontrols:
handles.temp.background=axes('units','pixels','position',[88 55 506 570],'visible','off','xlimmode','manual','ylimmode','manual');

handles.temp.tabaxes=axes('units','pixels','position',[58 125 30 500],'parent',handles.temp.gui,'visible','off','tag','tabaxes','ylimmode','manual','xlimmode','manual');
handles.temp.titleaxes=axes('units','pixels','position',[11 630 485 41],'parent',handles.temp.gui,'visible','off','tag','titleaxes');

[x,y,v]=calculatetabprofile([1.2 .8 1 .8 .85 0.55],.1,1,1);
handles.temp.tabbg=patch(x,y,handles.colors.tabbg,'parent',handles.temp.tabaxes,'edgecolor','none');
handles.temp.bg=patch([0 1 1 0],[0 0 1 1],handles.colors.tabbg,'parent',handles.temp.background,'edgecolor','none');

handles.temp.tablines=line(x,y,'parent',handles.temp.tabaxes,'color',[.5 .5 .5]);

handles.temp.continuous=text(.2,v(1),' Continuous','parent',handles.temp.tabaxes,'rotation',90,'fontname',handles.fonts.tab,'fontsize',0.8*handles.fonts.tabsize,...
    'color',handles.colors.text,'vert','cap','buttondownfcn',@continuous_tab);
handles.temp.integer=text(.2,v(2),' Integer','parent',handles.temp.tabaxes,'rotation',90,'fontname',handles.fonts.tab,'fontsize',0.8*handles.fonts.tabsize,...
    'color',handles.colors.text,'vert','cap','buttondownfcn',@integer_tab);
handles.temp.discrete=text(.2,v(3),' Discrete','parent',handles.temp.tabaxes,'rotation',90,'fontname',handles.fonts.tab,'fontsize',0.8*handles.fonts.tabsize,...
    'color',handles.colors.text,'vert','cap','buttondownfcn',@discrete_tab);
handles.temp.string=text(.2,v(4),' String','parent',handles.temp.tabaxes,'rotation',90,'fontname',handles.fonts.tab,'fontsize',0.8*handles.fonts.tabsize,...
    'color',handles.colors.text,'vert','cap','buttondownfcn',@string_tab);
handles.temp.logical=text(.2,v(5),' Logical','parent',handles.temp.tabaxes,'rotation',90,'fontname',handles.fonts.tab,'fontsize',0.8*handles.fonts.tabsize,...
    'color',handles.colors.text,'vert','cap','buttondownfcn',@logical_tab);
handles.temp.list=text(.2,v(6),' List','parent',handles.temp.tabaxes,'rotation',90,'fontname',handles.fonts.tab,'fontsize',0.8*handles.fonts.tabsize,...
    'color',handles.colors.text,'vert','cap','buttondownfcn',@list_tab);

handles.temp.tabs=[handles.temp.continuous handles.temp.integer handles.temp.discrete handles.temp.string handles.temp.logical handles.temp.list];

line([0 0 1 1 0],[.125 0 0 1 1],'color',handles.colors.tabfg,'parent',handles.temp.background);


handles.temp.donebutton=uicontrol('style','pushbutton','units','pixels','position',[265 11 156 35],'tag','donebutton',...
    'fontsize',14,'string','Ok','callback',@donebutton_Callback);
handles.temp.cancelbutton=uicontrol('style','pushbutton','units','pixels','position',[438 11 156 35],'tag','donebutton',...
    'fontsize',14,'string','Cancel','callback',@cancelbutton_Callback);

%make titles
set(handles.temp.titleaxes,'color','none','xcolor',handles.colors.frame,...
    'ycolor',handles.colors.frame,'zcolor',handles.colors.frame);
text(0,.2,'Change feasible values of input variables','parent',handles.temp.titleaxes,'fontname',handles.fonts.title,'fontsize',handles.fonts.titlesize,...
    'color',handles.colors.title,'horizontalalignment','left','verticalalignment','baseline');

%get data from managedatagui
handles.temp.indata=guidata(gcbo); %gcbo = "define roles" button in managedatagui

handles.temp.savebutton=uicontrol('style','pushbutton','units','pix','parent',handles.temp.gui,'position',[320 65 120 26],...
    'string','Save Settings','callback',@savesettings_callback,'enable','off','visible','off');
handles.temp.loadbutton=uicontrol('style','pushbutton','units','pix','parent',handles.temp.gui,'position',[460 65 120 26],...
    'string','Load Settings','callback',@loadsettings_callback,'enable','off','visible','off');

% Choose default command line output
handles.temp.output = f;
handles.temp.tabobjects=[];

% Update handles structure
guidata(f, handles);

%make table:
dblArray=java.lang.Object; %the table data
dmodel=javax.swing.table.DefaultTableModel(dblArray,{' '});
javaObjectEDT(dmodel);
jtable=javax.swing.JTable(dmodel);
javaObjectEDT(jtable);
jtable.setAutoResizeMode(jtable.AUTO_RESIZE_OFF);
jtable.putClientProperty('terminateEditOnFocusLost',true);
jscroll=javax.swing.JScrollPane(jtable);
javaObjectEDT(jscroll);
[jhandle mhandle]=javacomponent(jscroll,[120 100 440 460],handles.temp.gui);

handles.temp.blankmodel=dmodel;
handles.temp.jtable=jtable;

guidata(f, handles);

%show first tab:
continuous_tab(handles.temp.continuous,[]);


function continuous_tab(src,ev)
%display uicontrols for defining column roles (constant, continuous, discrete)
handles=guidata(src);

d=handles.temp.d;
a=handles.temp.a;
a=a(1); %in case multiple are selected

[x,y]=calculatetabprofile([1.2 .8 1 .8 .85 0.55],.1,1,1);
set(handles.temp.tablines,'xdata',x,'ydata',y);
%set(handles.tabs,'color',handles.prefs.colors.tabfg);
set(src,'color',handles.colors.text);

varnamez=handles.allnames{d};
thistype=strcmp('continuous',handles.datatype{d});
varnamez=varnamez(thistype);
data=[handles.minlimit{d}{a}(thistype);handles.maxlimit{d}{a}(thistype)];

if isempty(varnamez)
    tableData={java.lang.String('No Continuous Variables Found'),java.lang.Double(0),java.lang.Double(0)};
    dmodel=javax.swing.table.DefaultTableModel(tableData,{});
else
    tableData=javaArray('java.lang.Object',numel(varnamez),3);
    for r=1:numel(varnamez)
        tableData(r,1)=java.lang.String(varnamez{r});
        tableData(r,2)=java.lang.Double(data(1,r));
        tableData(r,3)=java.lang.Double(data(2,r));
    end
    dmodel=javax.swing.table.DefaultTableModel(tableData,{'Variable Name','Min Value','Max Value'});
end

handles.temp.jtable.setModel(dmodel);
set(handle(dmodel,'callbackproperties'),'TableChangedCallback',{@continuouschanged_callback,handles.temp.gui})

if ~isempty(varnamez)
    handles.temp.jtable.getTableHeader().setReorderingAllowed(false) %turn off draggable columns
    handles.temp.jtable.setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_OFF)%turn off auto resizing
    colwidths=[240 100 99];
    for c=1:3 %set the widths:
        handles.temp.jtable.getColumnModel().getColumn(c-1).setPreferredWidth(colwidths(c))
    end
end

guidata(src,handles);


function integer_tab(src,ev)
%display uicontrols for defining column roles (constant, continuous, discrete)
handles=guidata(src);

d=handles.temp.d;
a=handles.temp.a;
a=a(1); %in case multiple are selected

[x,y]=calculatetabprofile([1.2 .8 1 .8 .85 0.55],.1,2,1);
set(handles.temp.tablines,'xdata',x,'ydata',y);
%set(handles.tabs,'color',handles.prefs.colors.tabfg);
set(src,'color',handles.colors.text);

%colformat={'char',{'constant','continuous','integer','discrete','string','logical'}};
varnamez=handles.allnames{d};
thistype=strcmp('integer',handles.datatype{d});
varnamez=varnamez(thistype);
data=[handles.minlimit{d}{a}(thistype);handles.maxlimit{d}{a}(thistype)];

if isempty(varnamez)
    handles.temp.jtable.setModel(handles.temp.blankmodel);
    tableData={java.lang.String('No Integer Variables Found'),java.lang.Double(0),java.lang.Double(0)};
    dmodel=javax.swing.table.DefaultTableModel(tableData,{});
else
    tableData=javaArray('java.lang.Object',numel(varnamez),3);
    for r=1:numel(varnamez)
        tableData(r,1)=java.lang.String(varnamez{r});
        tableData(r,2)=java.lang.Integer(data(1,r));
        tableData(r,3)=java.lang.Integer(data(2,r));
    end
    dmodel=javax.swing.table.DefaultTableModel(tableData,{'Variable Name','Min Value','Max Value'});
    
end

handles.temp.jtable.setModel(dmodel);
set(handle(dmodel,'callbackproperties'),'TableChangedCallback',{@integerchanged_callback,handles.temp.gui})

if ~isempty(varnamez)
    handles.temp.jtable.getTableHeader().setReorderingAllowed(false) %turn off draggable columns
    handles.temp.jtable.setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_OFF)%turn off auto resizing
    colwidths=[240 100 99];
    for c=1:3 %set the widths:
        handles.temp.jtable.getColumnModel().getColumn(c-1).setPreferredWidth(colwidths(c))
    end
end

guidata(src,handles);


function discrete_tab(src,ev)
%display uicontrols for defining column roles (constant, continuous, discrete)
handles=guidata(src);

d=handles.temp.d;
a=handles.temp.a;
a=a(1); %in case multiple are selected

[x,y]=calculatetabprofile([1.2 .8 1 .8 .85 0.55],.1,3,1);
set(handles.temp.tablines,'xdata',x,'ydata',y);
%set(handles.tabs,'color',handles.prefs.colors.tabfg);
set(src,'color',handles.colors.text);

%colformat={'char',{'constant','continuous','integer','discrete','string','logical'}};
varnamez=handles.allnames{d};

thistype=strcmp('discrete',handles.datatype{d});

varnamez=varnamez(thistype);
tempdata=handles.discretevalues{d}{a}(thistype);

datasizes=cellfun(@size,tempdata','UniformOutput',0);

datasizes=cell2mat(datasizes);
if any(thistype)
    data=cell(max(datasizes(:,1)),numel(varnamez));
    for v=1:numel(varnamez)
        data(1:datasizes(v,1),v)=num2cell(tempdata{v});
    end
else
    data=cell(0);
end

if isempty(varnamez)
    tableData={};
    dmodel=javax.swing.table.DefaultTableModel(tableData,varnamez);
else
    tableData=javaArray('java.lang.Object',size(data,1)+10,numel(varnamez));
    for c=1:numel(varnamez)
        for r=1:size(data,1);
            if ~isempty(data{r,c})
                tableData(r,c)=java.lang.Double(data{r,c});
            end
        end
    end
    dmodel=javax.swing.table.DefaultTableModel(tableData,varnamez);
end

handles.temp.jtable.setModel(dmodel);
set(handle(dmodel,'callbackproperties'),'TableChangedCallback',{@discretechanged_callback,handles.temp.gui})

if ~isempty(varnamez)
    handles.temp.jtable.getTableHeader().setReorderingAllowed(false) %turn off draggable columns
    handles.temp.jtable.setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_OFF)%turn off auto resizing
    colwidths=140;
    for c=1:numel(varnamez) %set the widths:
        handles.temp.jtable.getColumnModel().getColumn(c-1).setPreferredWidth(colwidths)
    end
end

guidata(src,handles);


function string_tab(src,ev)
%display uicontrols for defining column roles (constant, continuous, discrete)
handles=guidata(src);

d=handles.temp.d;
a=handles.temp.a;
a=a(1); %in case multiple are selected

[x,y]=calculatetabprofile([1.2 .8 1 .8 .85 0.55],.1,4,1);
set(handles.temp.tablines,'xdata',x,'ydata',y);

set(src,'color',handles.colors.text);

varnamez=handles.allnames{d};

thistype=strcmp('string',handles.datatype{d});

varnamez=varnamez(thistype);
tempdata=handles.discretevalues{d}{a}(thistype);
datasizes=cellfun(@size,tempdata','UniformOutput',0);
datasizes=cell2mat(datasizes);

if any(thistype)
    data=cell(max(datasizes(:,1)),numel(varnamez));
    for v=1:numel(varnamez)
        data(1:datasizes(v,1),v)=tempdata{v};
    end
else
    data=cell(0);
end

if isempty(varnamez)
    tableData={java.lang.String('No String Variables Found')};
else
    tableData=javaArray('java.lang.Object',size(data,1)+10,3);
    for c=1:numel(varnamez)
        for r=1:size(data,1);
            if ~isempty(data{r,c})
                tableData(r,c)=java.lang.String(data{r,c});
            end
        end
    end
end

dmodel=javax.swing.table.DefaultTableModel(tableData,varnamez);
handles.temp.jtable.setModel(dmodel);
set(handle(dmodel,'callbackproperties'),'TableChangedCallback',{@stringchanged_callback,handles.temp.gui})

if ~isempty(varnamez)
    handles.temp.jtable.getTableHeader().setReorderingAllowed(false) %turn off draggable columns
    handles.temp.jtable.setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_OFF)%turn off auto resizing
    colwidths=140;
    for c=1:numel(varnamez) %set the widths:
        handles.temp.jtable.getColumnModel().getColumn(c-1).setPreferredWidth(colwidths)
    end
end

guidata(src,handles);

function logical_tab(src,ev)
%display uicontrols for defining column roles (constant, continuous, discrete)
handles=guidata(src);
d=handles.temp.d;
a=handles.temp.a;
a=a(1); %in case multiple are selected


[x,y]=calculatetabprofile([1.2 .8 1 .8 .85 0.55],.1,5,1);
set(handles.temp.tablines,'xdata',x,'ydata',y);
%set(handles.tabs,'color',handles.prefs.colors.tabfg);
set(src,'color',handles.colors.text);

%colformat={'char',{'constant','continuous','integer','discrete','string','logical'}};
varnamez=handles.allnames{d};
thistype=strcmp('logical',handles.datatype{d});
varnamez=varnamez(thistype);
falsedata=handles.minlimit{d}{a}(thistype);
falsedata=~falsedata;
truedata=logical(handles.maxlimit{d}{a}(thistype));

data=[falsedata;truedata];

if isempty(varnamez)
    tableData={java.lang.String('No Logical Variables Found')};
    dmodel=javax.swing.table.DefaultTableModel(tableData,{});
else
    tableData=javaArray('java.lang.Object',numel(varnamez),3);
    for r=1:numel(varnamez)
        tableData(r,1)=java.lang.String(varnamez{r});
        tableData(r,2)=java.lang.Boolean(data(1,r));
        tableData(r,3)=java.lang.Boolean(data(2,r));
    end
    dmodel=javax.swing.table.DefaultTableModel(tableData,{'Variable Name','False','True'});
end

handles.temp.jtable.setModel(dmodel);
set(handle(dmodel,'callbackproperties'),'TableChangedCallback',{@logicalchanged_callback,handles.temp.gui})

if ~isempty(varnamez)
    handles.temp.jtable.getTableHeader().setReorderingAllowed(false) %turn off draggable columns
    handles.temp.jtable.setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_OFF)%turn off auto resizing
    colwidths=[300 70 69];
    for c=1:3 %set the widths:
        handles.temp.jtable.getColumnModel().getColumn(c-1).setPreferredWidth(colwidths(c))
    end
end

guidata(src,handles);


function list_tab(src,ev)
%display uicontrols for defining column roles (constant, continuous, discrete)
handles=guidata(src);

d=handles.temp.d;
a=handles.temp.a;
a=a(1); %in case multiple are selected

[x,y]=calculatetabprofile([1.2 .8 1 .8 .85 0.55],.1,6,1);
set(handles.temp.tablines,'xdata',x,'ydata',y);

set(src,'color',handles.colors.text);

varnamez=handles.allnames{d};

thistype=strcmp('list',handles.datatype{d});

varnamez=varnamez(thistype);
tempdata=handles.discretevalues{d}{a}(thistype);
datasizes=cellfun(@size,tempdata','UniformOutput',0);
datasizes=cell2mat(datasizes);

if any(thistype)
    data=cell(max(datasizes(:,1)),numel(varnamez));
    for v=1:numel(varnamez)
        data(1:datasizes(v,1),v)=tempdata{v};
    end
else
    data=cell(0);
end

if isempty(varnamez)
    tableData={java.lang.String('No String Variables Found')};
else
    tableData=javaArray('java.lang.Object',size(data,1)+10,3);
    for c=1:numel(varnamez)
        for r=1:size(data,1);
            if ~isempty(data{r,c})
                tableData(r,c)=java.lang.String(data{r,c});
            end
        end
    end
end

dmodel=javax.swing.table.DefaultTableModel(tableData,varnamez);
handles.temp.jtable.setModel(dmodel);
set(handle(dmodel,'callbackproperties'),'TableChangedCallback',{@stringchanged_callback,handles.temp.gui})

if ~isempty(varnamez)
    handles.temp.jtable.getTableHeader().setReorderingAllowed(false) %turn off draggable columns
    handles.temp.jtable.setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_OFF)%turn off auto resizing
    colwidths=140;
    for c=1:numel(varnamez) %set the widths:
        handles.temp.jtable.getColumnModel().getColumn(c-1).setPreferredWidth(colwidths)
    end
end

guidata(src,handles);

function donebutton_Callback(src, eventdata)
handles=guidata(src);

%apply edits to table:
if handles.temp.jtable.isEditing
    handles.temp.jtable.getCellEditor().stopCellEditing();
    drawnow;
    handles=guidata(src);
end

set(0,'userdata',handles);
close(handles.temp.gui);

function cancelbutton_Callback(src,ev)
close(gcbf);

function continuouschanged_callback(jsrc,jev,src)

handles=guidata(src);

%determine which cell is selected:
eventDetails = jev;

r=eventDetails.getFirstRow;

newmin = handles.temp.jtable.getValueAt(r,1);
newmax = handles.temp.jtable.getValueAt(r,2);

if ischar(newmin);newmin=str2double(newmin);end
if ischar(newmax);newmax=str2double(newmax);end

if newmin>newmax
    errordlg('Maximum value must be greater than or equal to minimum value','You can''t do that');
    return
end

d=handles.temp.d;
a=handles.temp.a;
datatypes=handles.datatype{d};
thistype=find(strcmp('continuous',datatypes));

index=thistype(r+1);

[handles.minlimit{d}{a}(index)]=deal(newmin);
[handles.maxlimit{d}{a}(index)]=deal(newmax);

guidata(src,handles);


function integerchanged_callback(jsrc,jev,src)

handles=guidata(src);

eventDetails = jev;
r=eventDetails.getFirstRow;

newmin = handles.temp.jtable.getValueAt(r,1);
newmax = handles.temp.jtable.getValueAt(r,2);

if ischar(newmin);newmin=str2double(newmin);end
if ischar(newmax);newmax=str2double(newmax);end

if newmin>newmax
    errordlg('Minimum value must be less than or equal to maximum value','You can''t do that');
    return
end

d=handles.temp.d;
a=handles.temp.a;
datatypes=handles.datatype{d};
thistype=find(strcmp('continuous',datatypes));

index=thistype(r+1);

[handles.minlimit{d}{a}(index)]=deal(round(newmin));
[handles.maxlimit{d}{a}(index)]=deal(round(newmax));

guidata(src,handles);


function discretechanged_callback(jsrc,jev,src)
handles=guidata(src);

%determine which cell is selected:
eventDetails = jev;
c=eventDetails.getColumn;
R=handles.temp.jtable.getRowCount;

newdata=cell(R,1);
for r=1:numel(newdata)
    data=handles.temp.jtable.getValueAt(r-1,c);
    
    if ischar(data)
        try
            newdata{r}=str2double(data);
        catch %#ok<CTCH>
            newdata{r}=[]; %if they didnt enter a number
        end
    else
        newdata{r}=data;
    end
end

newdata=cell2mat(newdata);
newdata=unique(newdata); %sort and remove repeats

d=handles.temp.d;
a=handles.temp.a;
datatypes=handles.datatype{d};
thistype=find(strcmp('discrete',datatypes));

index=thistype(c+1);

[handles.minlimit{d}{a}(index)]=deal(min(newdata));
[handles.maxlimit{d}{a}(index)]=deal(max(newdata));
[handles.discretevalues{d}{a}{index}]=deal(newdata);

guidata(src,handles);


function stringchanged_callback(jsrc,jev,src)
handles=guidata(src);
datasets=raveguidata(handles.fig,'datasets');

eventDetails = jev;

c=eventDetails.getColumn;
R=handles.temp.jtable.getRowCount;

newdata=cell(R,1);
for r=1:numel(newdata)
    newdata{r}=handles.temp.jtable.getValueAt(r-1,c);
end

charz=~cellfun(@isempty,newdata);

%make charz true for all values that appear in alldata, even if their
%string is now empty:

d=handles.temp.d;
a=handles.temp.a;
datatypes=handles.datatype{d};
thistype=find(strcmp('string',datatypes));
index=thistype(c+1);

truevalues=unique(datasets.alldata{d}(:,index));
charz(truevalues)=true;

newdata=newdata(charz);

%updata alltext to reflect new names:

%[handles.oldhandles.minlimit{d}{a}(index)]=deal(min(newdata)); %minlimit is always 1 for strings
[handles.maxlimit{d}{a}(index)]=deal(numel(newdata));
[handles.discretevalues{d}{a}{index}]=deal(newdata);

guidata(src,handles);


function logicalchanged_callback(jsrc,jev,src)

handles=guidata(src);

eventDetails = jev;
r=eventDetails.getFirstRow;

newmin = handles.temp.jtable.getValueAt(r,1);
newmax = handles.temp.jtable.getValueAt(r,2);

if ischar(newmin);newmin=logical(eval(newmin));end
if ischar(newmax);newmin=logical(eval(newmax));end

if ~newmin && ~newmax
    errordlg('You must enable at least one option for each variable','You can''t do that');
    return
end

d=handles.temp.d;
a=handles.temp.a;
datatypes=handles.datatype{d};
thistype=find(strcmp('logical',datatypes));

index=thistype(r+1);

if newmin
    [handles.minlimit{d}{a}(index)]=deal(0);
else
    [handles.minlimit{d}{a}(index)]=deal(1);
end

if ~newmax
    [handles.maxlimit{d}{a}(index)]=deal(0);
else
    [handles.maxlimit{d}{a}(index)]=deal(1);
end

guidata(src,handles);




