function f = ravedefinetargetsgui(src,d)
% RAVEDEFINETARGETSGUI M-file for ravedefinetargetsgui.fig
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);

handles.temp.d=d;
f=figure('units','pixels','position',[0 0 560 420],'menubar','none','toolbar','none','windowstyle','modal','numbertitle','off',...
    'color',handles.colors.frame,'name','Define targets');
axes1=axes('units','pixels','position',[20 369 297 41],'parent',f,...
    'visible','off');
uicontrol('units','pixels','style','text','position',[362 250 152 100],'parent',f,...
    'string',{'Enter the target (optimal) value in the second column of the table. Enter inf to maximize, enter -inf to minimize.'},...
    'hor','left','backgroundcolor',handles.colors.frame)
uicontrol('units','pixels','position',[369 30 175 51],'string','Done','parent',f,...
    'callback',@donebutton_Callback,'fontsize',14);

fig=ancestor(src,'figure');
thispos=get(f,'position');
screensize=get(fig,'position');
set(f,'units','pixels','position',[screensize(1)+screensize(3)/2-thispos(3)/2 screensize(2)+screensize(4)/2-thispos(4)/2 thispos(3) thispos(4)])

%make title
text(0,.75,'Define Target Values','parent',axes1,'fontname',handles.fonts.title,'fontsize',handles.fonts.titlesize,'color',handles.colors.title)

%get data from calling figure:
callershandles=guidata(gcbo); %gcbo = button in either managedatagui or handles.fig


names=handles.allnames{d};
handles.temp.names=names; %make accessible to the done button
targets=num2cell(handles.targetvalues{d});

dat=javaArray('java.lang.Object',numel(names),2);
for g=1:numel(names)
    dat(g,1)=java.lang.String(names{g});
    dat(g,2)=java.lang.Double(targets{g});
end

jtable=javax.swing.JTable(dat,{'Variable Name','Target Value'}); %initialize a table
javaObjectEDT(jtable);
j=javax.swing.JScrollPane(jtable); %initialize a scrollpane that contains the table
javaObjectEDT(j);
jtable.setFillsViewportHeight(true); %make table fill the scrollpane
jtable.putClientProperty('terminateEditOnFocusLost',true); %doesnt seem to work...
pause(0.1)
[jscrollpane mtable]=javacomponent(j,[20 30 328 348],f); 
pause(0.1)
handles.temp.table=jtable;

% Update handles structure
guidata(f, handles);
set(f,'closerequestfcn',@donebutton_Callback);


function donebutton_Callback(src,eventdata)
handles=guidata(src);

%apply edits to table:
if handles.temp.table.isEditing
    handles.temp.table.getCellEditor().stopCellEditing();
    drawnow;
end

for r=1:handles.temp.table.getRowCount;
    thisvalue=handles.temp.table.getValueAt(r-1,1);
    if ischar(thisvalue)
        thisvalue=strrep(thisvalue,'Infinity','inf');
        
        try
            targets{r}=eval(thisvalue);
        catch
            errordlg('All values must be numeric.','You can''t do that.');
            return
        end
    else
        targets{r}=thisvalue;
    end
end

targets=cell2mat(targets);
if any(isnan(targets))
    errordlg('All values must be numeric.','You can''t do that.');
else
    handles.targetvalues{handles.temp.d}=targets;
    guidata(handles.fig,handles);
    delete(ancestor(src,'figure'))
end


