function f=ravedefinevariablecolorsgui(src,d)
%make a dialog box for the user to specify a fixed color for each variable in a data set. 
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu


%note, initial colors are set by managedatagui to hsv(numvars).

handles=guidata(src);

handles.temp.d=d;
handles.temp.thisvar=1;
origin=get(handles.fig,'position');


f=figure('menubar','none','units','pixels','position',[origin(1)+origin(3)/2-240 origin(2)+origin(4)/2-300 600 475],'color',handles.colors.frame,...
    'name','Change the default line colors for each variable','numbertitle','off','resize','off','windowstyle','modal');
handles.temp.f=f;


titleaxes=axes('units','pixels','position',[20 425 400 40],'parent',f,'visible','off');
text(0,.4,'Define Variable Colors','parent',titleaxes,'fontname',handles.fonts.title,'fontsize',handles.fonts.titlesize,...
    'color',handles.colors.title,'horizontalalignment','left','verticalalignment','baseline');

uicontrol('style','text','parent',f,'units','pixels','position',[35 405 400 22],'string','Click a variable in the list below to change its color.',...
    'fontsize',10,'backgroundcolor',handles.colors.frame,'horizontalalignment','left','fontweight','bold')

varnamez=handles.allnames{d};
numvarz=numel(varnamez);

variablecolors=handles.variablecolors{d};

%convert decimal colors to HTML strings:
variablecolors=round(variablecolors*255);
R=dec2hex(variablecolors(:,1));
G=dec2hex(variablecolors(:,2));
B=dec2hex(variablecolors(:,3));

htmlz=[R G B];

%construct monster html formated listbox string:
strng=cell(numvarz,1);
for g=1:numvarz
    strng{g}=['<HTML><B><FONT COLOR="#',htmlz(g,:),'">',varnamez{g},'</FONT></B></HTML>'];    
end

%make java listbox:
handles.temp.jlist=javax.swing.JList(strng);
handles.temp.jlist.setSelectionBackground(java.awt.Color.gray);
handles.temp.jlist.setBackground(java.awt.Color.gray);
handles.temp.jlist.setDoubleBuffered(1);

set(handle(handles.temp.jlist,'callbackproperties'),'MouseClickedCallback',{@listclicked_callback,f});
handles.temp.jscroll=javax.swing.JScrollPane(handles.temp.jlist);
[handles.temp.mlist,handles.temp.hglist]=javacomponent(handles.temp.jscroll,[50 105 200 290]);


%make background color buttons:

% color blobs: (userdata = gridcolor; textcolor; index)
bgwhite=uicontrol('position',[155 80 20 20],...
    'style','togglebutton','string','','backgroundcolor',[1 1 1],'value',0,...
    'callback',@bgblob_callback,'userdata',[.6 .6 .6;0 0 0;1 0 0]);
bglight=uicontrol('position',[180 80 20 20],...
    'style','togglebutton','string','','backgroundcolor',[.753 .753 .753],'value',0,...
    'callback',@bgblob_callback,'userdata',[1 1 1;0 0 0;2 0 0]);
bgdark=uicontrol('position',[205 80 20 20],...
    'style','togglebutton','string','','backgroundcolor',[.502 .502 .502],'value',1,...
    'callback',@bgblob_callback,'userdata',[.6 .6 .6;1 1 1;3 0 0]);
bgblack=uicontrol('position',[230 80 20 20],...
    'style','togglebutton','string','','backgroundcolor',[0 0 0],'value',0,...
    'callback',@bgblob_callback,'userdata',[.3 .3 .3;1 1 1;4 0 0]);

handles.temp.bgblobs=[bgwhite bglight bgdark bgblack];


% make variable info:
if handles.isfunction{d}(1)
    thisvarmodel='Yes';
else
    thisvarmodel='No';
end


uicontrol('style','text','position',[290 350 200 22],'string','Variable Name:','fontname','arial','fontweight','bold','fontsize',10,...
    'horizontalalignment','left','backgroundcolor',handles.colors.frame)
uicontrol('style','text','position',[290 320 200 22],'string','Role:','fontname','arial','fontweight','bold','fontsize',10,...
    'horizontalalignment','left','backgroundcolor',handles.colors.frame)
uicontrol('style','text','position',[290 290 200 22],'string','Modeled:','fontname','arial','fontweight','bold','fontsize',10,...
    'horizontalalignment','left','backgroundcolor',handles.colors.frame)
uicontrol('style','text','position',[290 260 200 22],'string','Range:','fontname','arial','fontweight','bold','fontsize',10,...
    'horizontalalignment','left','backgroundcolor',handles.colors.frame)

handles.temp.thisvarname=uicontrol('style','text','position',[400 350 200 22],'string',varnamez{1},'fontname','arial','fontweight','normal','fontsize',10,...
    'horizontalalignment','left','backgroundcolor',handles.colors.frame);


uicontrol('style','text','position',[290 230 200 22],'string','Color (click to change):','fontname','arial','fontweight','bold','fontsize',10,...
    'horizontalalignment','left','backgroundcolor',handles.colors.frame)

handles.temp.colorbutton=uicontrol('units','pixels','style','pushbutton','position',[445 235 100 20],'backgroundcolor',handles.variablecolors{d}(1,:),...
    'callback',@colorbutton_callback,'tag','colorbutton');

%done button:
uicontrol('units','pixels','style','pushbutton','position',[300 19 133 40],...
    'callback',@donebutton_callback,'string','Done','fontsize',16);


uicontrol('units','pixels','style','pushbutton','position',[450 19 133 40],...
    'callback',@cancelbutton_callback,'string','Cancel','fontsize',16);


guidata(f,handles);

set(0,'userdata',[]);


function bgblob_callback(src,ev)
%make listbox background this color:
handles=guidata(src);
set(handles.temp.bgblobs,'value',0);
set(src,'value',1);
newcolor=get(src,'backgroundcolor');
handles.temp.jlist.setBackground(java.awt.Color(newcolor(1),newcolor(2),newcolor(3)));
handles.temp.jlist.setSelectionBackground(java.awt.Color(newcolor(1),newcolor(2),newcolor(3)));

function listclicked_callback(src,ev,hObject) %#ok<INUSL>
handles=guidata(hObject);
d=handles.temp.d;

thisvar=handles.temp.jlist.getSelectedIndex+1;
handles.temp.thisvar=thisvar;
set(handles.temp.colorbutton,'backgroundcolor',handles.variablecolors{d}(thisvar,:));

set(handles.temp.thisvarname,'string',handles.allnames{d}{thisvar});

guidata(hObject,handles)

function colorbutton_callback(src,ev)
handles=guidata(src);
d=handles.temp.d;

col=ravecolorchooser(handles.fig,handles.variablecolors{handles.temp.d}(handles.temp.thisvar,:),'Pick a Color');

drawnow; %MUST have this to avoid java crash. 

if numel(col)==1
    return    
end


varnamez=handles.allnames{d};
numvarz=numel(varnamez);

set(src,'backgroundcolor',col)
handles.variablecolors{handles.temp.d}(handles.temp.thisvar,:)=col;

%update jlist:
variablecolors=handles.variablecolors{d};
variablecolors=round(variablecolors*255);
R=dec2hex(variablecolors(:,1));
G=dec2hex(variablecolors(:,2));
B=dec2hex(variablecolors(:,3));
htmlz=[R G B];

%construct monster html formated listbox string:
strng=cell(numvarz,1);
for g=1:numvarz
    strng{g}=['<HTML><B><FONT COLOR="#',htmlz(g,:),'">',varnamez{g},'</FONT></B></HTML>'];    
end

%update jlist:
handles.temp.jlist.setListData(strng);

delete(handles.temp.hglist);

%make java listbox:
handles.temp.jlist=javax.swing.JList(strng);
handles.temp.jlist.setSelectionBackground(java.awt.Color.gray);
handles.temp.jlist.setBackground(java.awt.Color.gray);
handles.temp.jlist.setDoubleBuffered(1);
set(handle(handles.temp.jlist,'callbackproperties'),'MouseClickedCallback',{@listclicked_callback,handles.fig});
handles.jscroll=javax.swing.JScrollPane(handles.temp.jlist);
[handles.temp.mlist,handles.temp.hglist]=javacomponent(handles.temp.jscroll,[50 105 200 290]);

%refresh to avoid draw errors:
drawnow;

guidata(src,handles);



function donebutton_callback(src,ev)
handles=guidata(src);
guidata(handles.fig,handles);

close(ancestor(src,'figure'));


function cancelbutton_callback(src,ev)
answer=questdlg('This will cancel all changes. Are you sure you want to do that?','Cancel changes');
if strcmpi(answer,'yes')
    handles=guidata(src);
    set(0,'userdata',[]);
    close(handles.temp.f);
end


