function ravedisplayworkspace(src,ev)
%this function saves all changes made in the table to the correct
%dataset/analysis. It is run when (before) switching to workspace view.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2011 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.

handles=guidata(src);
datasets=raveguidata(handles.fig,'datasets');

%send table selection to workspace:
selectedrows=handles.maintable.jtable.getSelectedRows();

for g=1:numel(selectedrows)
    selectedrows(g)=handles.maintable.models{handles.maintable.d}.modelIndex(selectedrows(g));
end

selectedrows=selectedrows+1;

try %#ok<TRYNC> %gives error if no data set loaded.
    handles.isselected{handles.maintable.d}{handles.maintable.a}(:)=false;
    handles.isselected{handles.maintable.d}{handles.maintable.a}(selectedrows)=true;
end

guidata(src,handles);

%Look for invalid data:
for d = 1:numel(handles.datasetnames)
    invalidcols = any(~datasets.isvalid{d},1);
    invalidrows = any(~datasets.isvalid{d},2);
    
    if any(invalidcols) %update this data set:
        ycol = find(invalidcols);
        data = datasets.alldata{d}(invalidrows,:);
        yout = raveevaluatefunction(src,d,ycol,data,handles,[]);
        datasets.alldata{d}(:,ycol)=yout;
        datasets.isvalid{d}(:,:)=true; %set all data as valid.
        raveguidata(handles.fig,'datasets',datasets);
        handles.datasetupdatespending{d}={'replaceall'}; %override all updates with a replaceall since it covers everything.
    end
end
guidata(handles.fig,handles);


%update data set if needed:
for d=1:numel(handles.datasetnames)
    updates=unique(handles.datasetupdatespending{d});
    %IMPORTANT: this does not actually check the update type since at the
    %moment the only updates you can make from the table are equivalent
    %to updating the rows. might need to fix this in the future.
    if ~isempty(updates)
        for g=1:numel(updates)
            handles=raveupdatedataset(handles.fig,handles,updates{g},[],d);
        end
    end
    
    handles.datasetupdatespending{d}=cell(0);
    
    %then update analyses:
    for a=1:numel(handles.analysisnames)
        
        %always update selection. not gonna bother to check if it's changed.
        handles=raveupdateanalysis(handles.fig,handles,'isselected',[],d,a);
        
        if handles.prefs.setxdialswhensingleselected && sum(handles.isselected{d}{a})==1
            handles.xdials{d}{a}=datasets.alldata{d}(handles.isselected{d}{a},:);
            handles=raveupdateanalysis(handles.fig,handles,'xdials',[],d,a);
        end
        
        if ismember('visiblerows',handles.analysisupdatespending{d}{a})
            handles=raveupdateanalysis(handles.fig,handles,'visiblerows',[],d,a);
        end
        
        if ismember('rowcolors',handles.analysisupdatespending{d}{a})
            handles=raveupdateanalysis(handles.fig,handles,'rowcolors',[],d,a);
        end
    end
end

guidata(handles.fig,handles);



