function ravedragview(src,ev)
%drag around workspace.
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

%NOTE: this function depends on using varargin instead of specifying an exact number of inputs,
%so don't change that in the future.

%ravemoveview(handles.fig,[],clicked);
%clicked is optional

handles=guidata(src);


switch get(handles.fig,'selectiontype')
    case 'alt' %display context menu
        drawnow;
        pos=get(handles.fig,'currentpoint');
        set(handles.contextmenus.workspace,'position',pos,'visible','on')
        
    otherwise %scroll workspace
        %get existing position and click location
        handles.temp.firstclick=get(handles.mainaxes,'currentpoint');
        handles.temp.oldviewpos=get(handles.currentview,'position');
        
        %allow dragging
        guidata(src,handles);
        
        set(handles.fig,'WindowButtonMotionFcn',@ravedragviewmove);
        set(handles.fig,'WindowButtonUpFcn',@movedone);
        
        setptr(handles.fig,'closedhand')
        
end



function ravedragviewmove(src,ev)
handles=guidata(src);
newclick=get(handles.mainaxes,'currentpoint');

try
    shiftx=handles.temp.firstclick(1,1)-newclick(1,1);
    shifty=handles.temp.firstclick(1,2)-newclick(1,2);
catch
    return
end

handles.temp.firstclick=newclick;


%newpos=[shiftx+handles.temp.oldviewpos(3)/2 shifty+handles.temp.oldviewpos(4)/2 0 0]+handles.temp.oldviewpos;

minh=get(handles.hscroll,'min');
maxh=get(handles.hscroll,'max');

oldhvalue=get(handles.hscroll,'value');
newhvalue=oldhvalue+shiftx;

if newhvalue>minh && newhvalue<maxh
    set(handles.hscroll,'value',oldhvalue+shiftx)
elseif newhvalue<=minh
    set(handles.hscroll,'value',minh)
else
    set(handles.hscroll,'value',maxh)
end

minv=get(handles.vscroll,'min');
maxv=get(handles.vscroll,'max');

oldvvalue=get(handles.vscroll,'value');
newvvalue=oldvvalue+shifty;

if newvvalue>minv && newvvalue<maxv
    set(handles.vscroll,'value',oldvvalue+shifty)
elseif newvvalue<=minv
    set(handles.vscroll,'value',minv)
else
    set(handles.vscroll,'value',maxv)
end

xclick=get(handles.hscroll,'value');
if isempty(xclick);xclick=0;end
yclick=get(handles.vscroll,'value');
if isempty(yclick);yclick=0;end

clicked=[xclick yclick];

%center box on click location
clicked(1,1,1)=clicked(1,1,1)-handles.temp.oldviewpos(3)/2;
clicked(1,2,1)=clicked(1,2,1)-handles.temp.oldviewpos(4)/2;

%round to nearest integers:
clicked=floor(clicked);

%move to new location
set(handles.currentview,'position',[clicked(1,1,1) clicked(1,2,1) handles.temp.oldviewpos(3) handles.temp.oldviewpos(4)]);

%Update view:
raveupdateview(src,[],handles);




function movedone(src,eventdata)
%load handles
handles=guidata(src);

%stop dragging
set(handles.fig,'WindowButtonMotionFcn',@ravedefaultwindowbuttonmotionfcn);
set(handles.fig,'windowbuttonupfcn',[]);
set(handles.fig,'pointer','arrow');


