function index=ravefindclosestpoint2D(xclick,yclick,datasource,ax)
%returns a logical vector into xdata/ydata of datasource of the closest point (on screen) to location in xclick/yclick.
%datasource is either tha handle to a graphics object with xdata/ydata properties, or is an nx2 matrix of x/y data points.
%ax is the axes whose xlim/ylim and position should be used to convert
%pixels to graph units. If datasource is a handle, its parent is used for ax.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

if ishghandle(datasource(1)) && strcmp(get(datasource(1),'type'),'axes'); %need a second check cuz 0 appears to be a handle, as do integers if other figures exist
    xdata=get(datasource,'xdata');
    ydata=get(datasource,'ydata');
    activegraph=get(datasource,'parent');
else
    xdata=datasource(:,1);
    ydata=datasource(:,2);
    if nargin>3
        activegraph=ax;
    else
        activegraph=gca;
    end
end



%pos=get(activegraph,'pos');
pos=getpixelposition(activegraph);
xlim=get(activegraph,'xlim');
ylim=get(activegraph,'ylim');

%make conversion factors, units to pixels:
xconvert=(xlim(2)-xlim(1))/pos(3);
yconvert=(ylim(2)-ylim(1))/pos(4);

Xclick=(xclick-xlim(1))/xconvert;
Yclick=(yclick-ylim(1))/yconvert;

Xdata=(xdata-xlim(1))/xconvert;
Ydata=(ydata-ylim(1))/yconvert;

Xdiff=Xdata-Xclick;
Ydiff=Ydata-Yclick;

distnce=sqrt(Xdiff.^2+Ydiff.^2);

index=distnce==min(distnce);

index=index(:); %make sure it's a column.

if sum(index)>1
    thispoint=find(distnce==min(distnce),1);
    index=false(size(distnce));
    index(thispoint)=true;
end



