function [handles,success]=raveloadmetadata(handles,d,path)
%NOTE: THIS funciton takes in handles directly instead of retrieving it
%from guidata (to avoid problems with managedatagui.
%path is path of file to be read.
%results is a structure whose fields depend on the filetype of the file in "path".
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

%Disclaimer: there is 0 cleverness in the text parsing that happens in this
%file. Don't judge.

datasets=raveguidata(handles.fig,'datasets');
% textdata=raveguidata(handles.fig,'textdata');
vectordata=raveguidata(handles.fig,'vectordata');

success=true;

if isempty(path);return;end

%read in file:
fid=fopen(path,'rt');

linez={};
tline=fgetl(fid);
while ischar(tline)
    if ~isempty(strtrim(tline));
        linez{end+1}=strtrim(tline); %#ok<AGROW>
    end
    tline=fgetl(fid);
end
fclose(fid);

%find seperators:

sections=[];
for r=1:numel(linez)
    if length(linez{r})>1 && strcmp(linez{r}(1:2),'%#')
        sections(end+1)=r; %#ok<AGROW>
    end
end

sections(end+1)=numel(linez)+1;


sectiontypes=cell(numel(sections)-1,1);
sectiona=zeros(numel(sections),1);

for s=1:numel(sections)-1
    sectiontypes{s}=upper(linez{sections(s)}(isstrprop(linez{sections(s)},'alpha')));
    if any(linez{sections(s)}==':')
        sectiona(s)=str2double(linez{sections(s)}(isstrprop(linez{sections(s)},'digit')));
    end
end


%Sort sectiontypes in the order in which they need to be evaluated:
evalorder={'DISTRIBUTIONS';'RANDOMVARS';'FUNCTIONS';'STATISTICS';'RANGES';'CONSTRAINTS';'CURRENTPOINT';'TARGETSANDPREFERENCES';'COLORS';'ROWSTATE';'ROWNOTES'};

[junk,idx]=ismember(sectiontypes,evalorder);


evalthese=[];

for s=1:numel(sectiontypes)
    contents=linez(sections(s)+1:sections(s+1)-1);
    if isempty(contents);continue;end
    
    switch sectiontypes{s}
        case 'DISTRIBUTIONS'
            firsttime=true(numel(handles.basenames{d}),1);
            
            %Lines containing variable names are the only ones without an
            %equals sign. Use this to find breaks:
            breaks=[find(cellfun(@isempty,strfind(contents,'='))) numel(contents)+1];
            
            for c=1:numel(breaks)-1
                varname=strtrim(contents{breaks(c)});
                fieldz=contents(breaks(c)+1:breaks(c+1)-1);
                thisindex=find(strcmp(handles.basenames{d},varname));
                if isempty(thisindex)
                    warning('Metadata file contained a distribution for a non-existent variable.');
                    continue
                end
                
                if numel(thisindex)>1
                    warning('Could not load distribution info because variable names are not unique');
                    continue
                end
                
                %Make distribution sturcture:
                dist=struct();
                isactive=0;
                for f=1:numel(fieldz)
                    if ~isempty(strfind(fieldz{f},'isactive'))
                        isactive=eval(fieldz{f}(end));
                    elseif ~isempty(strfind(fieldz{f},'serialnumber'))
                        eval(['dist.',fieldz{f},';']);
                    else
                        eqz=strfind(fieldz{f},'=');
                        rhs=['''',strtrim(fieldz{f}(eqz+1:end)),''''];
                        eval(['dist.',fieldz{f}(1:eqz),rhs,';']);
                    end
                end
                if firsttime(thisindex)
                    firsttime(thisindex)=false;
                    handles.distributions{d}{thisindex}{1}=dist;
                else
                    handles.distributions{d}{thisindex}{end+1}=dist;
                end
                
                if isactive
                    handles.activedistribution{d}(thisindex)=numel(handles.distributions{d}{thisindex});
                end
            end
            
        case 'RANDOMVARS'
            nameline=contents{1};
            varnames=eval(['{''',strrep(nameline,char(9),''';'''),'''}']);
            
            isnoise=contents{2};
            isnoise=eval(['[',strrep(isnoise,char(9),','),']']);
            
            for c=1:numel(varnames);
                thisindex=find(strcmp(handles.basenames{d},varnames{c}));
                handles.isnoise{d}(thisindex)=isnoise(c); %#ok<FNDSB>
            end
            
            
        case 'FUNCTIONS'
            evalthese=[];
            %nvars=size(datasets.alldata{d}); %initial number of variables;
            for c=1:numel(contents)
                infoline=contents{c};
                infoz=regexp(infoline,'\t+','split');
                
                thisvarname{c}=infoz{1};
                if thisvarname{c}(1)=='*';
                    thisvarname{c}=thisvarname{c}(2:end);
                    isobjective(c)=true;
                else
                    isobjective(c)=false;
                end
                thisfunpath=infoz{2};
                thisfunclass{c}=infoz{3};
                thisnargout{c}=infoz{4};
                varz=infoz(5:end);
                
                warningz{c}='';
                isok(c)=true;
                
                [path,name,ext]=fileparts(thisfunpath);
                name=[name,ext];
                
                if ~exist(thisfunpath,'file')
                    if isempty(which(name))
                        warningz{c}=['No file named ''',name,''' could be found on your path.'];
                        isok(c)=false;
                    else
                        thisfunpath=which(name);
                        [path,name,ext]=fileparts(thisfunpath);
%                         warningz{c}=['The file ''',name,''' was not in the folder listed in the .rvm file, but I found a copy in ',path];
                        isok(c)=true;
                    end
                end
                [path,name,ext]=fileparts(thisfunpath);
                if ~isempty(which(name)) && ~strcmp(which(name),thisfunpath)
                    if isdir(path)
                        addpath(path);
                        warningz{c}=['There was another file named ''',name,''' higher on your search path. I have reordered your path to use the version specified in the .rvm file.'];
                        isok(c)=true;
                    end
                end
                
                funpath{c}=thisfunpath;
                
                if ~strcmp(ext,'.m')
                    addpath(path);
                    warningz{c}=['I couldn''t load the file ''',name,''' because only .m files can be loaded from a .rvm file.'];
                    isok(c)=false;
                end
                
                [thispath,namez{c},junk]=fileparts(funpath{c});
                addpath(thispath);
                
                %thisnargout{c}=contents{c}(tabz(3)+1:tabz(4)-1);
                
                for t=1:numel(varz)
                    invars(t)=find(strcmp(handles.basenames{d},varz{t})); %#ok<AGROW>
                end
                thisinvars{c}=sort(invars);
                %load all this info, but dont evaluate functions yet. first
                %check for rveopt file in case any of these are objectives.
            end
            
            
            if any(~isok)
                errordlg([{'I could not load the functions in your .rvm file because of the following errors. I loaded the data set with no functions, and you can use the model tab to load these functions later.'},{''},unique(warningz)],'Error Loading Metadata')
                continue
            else
                idx=~cellfun(@isempty,warningz);
                if any(idx)
                    warningz=warningz(idx);
                    warningz=unique(warningz);
                    warndlg([{'I made the following changes to the function information in your .rvm file:'},{''},warningz],'Metadata Changes')
                end
            end
            
            for c=1:numel(contents)
                %NOTE: handles.allnames is set at the end of the loop
                handles.basenames{d}{end+1}=thisvarname{c};
%                 handles.datasetlinks{d}{end+1}=[];
%                 handles.datasetlinkinfo{d}{end+1}={};
                handles.isfunction{d}(end+1)=1;
                handles.allunits{d}{end+1}=''; %havent implemented this yet
                handles.isincluded{d}(end+1)=1;
                handles.isconstant{d}(end+1)=0;
                handles.funclass{d}{end+1}=thisfunclass{c};
                handles.funattributes{d}{end+1} = raveparsefunctionattributes(funpath{c});
                handles.funnames{d}{end+1}=namez{c};
                uinvars=unique(thisinvars{c});
                handles.funvars{d}{end+1}=uinvars;
                handles.depvars{d}{end+1}=[];
                for i=1:numel(uinvars)
                    handles.depvars{d}{uinvars(i)}(end+1)=numel(handles.isfunction{d});
                end
                
                handles.nargout{d}(end+1)=str2double(thisnargout{c});
                
                %This must be defined before we can fill in the data:
                datasets.datatype{d}{end+1}='continuous';

                handles.datatype{d}{end+1}='continuous';
                handles.isnoise{d}(end+1)=false;
                handles.preferences{d}(end+1)=0;
                handles.targetvalues{d}(end+1)=-inf; %load prefs/targs here in case rveopt file is not found.
                handles.isobjective{d}(end+1)=isobjective(c);
                handles.variablecolors{d}(end+1,:)=rand(1,3);
                handles.maintable.colwidths{d}(end+1)=handles.prefs.colwidth;
                handles.distributions{d}{end+1}{1}=struct();
                handles.aggregatingstatistic{d}{end+1}=handles.prefs.defaultstatistic;
                handles.activedistribution{d}(end+1)=0;
                
                for a=1:numel(handles.analysisnames)
                    handles.xdials{d}{a}(end+1)=0;
                    handles.minlimit{d}{a}(end+1)=0;
                    handles.maxlimit{d}{a}(end+1)=0;
                    handles.discretevalues{d}{a}{end+1}=[];
                    handles.minfilter{d}{a}(end+1)=inf;
                    handles.maxfilter{d}{a}(end+1)=-inf;
                end
                evalthese(end+1)=numel(handles.basenames{d}); %#ok<AGROW>
                handles.allnames{d}(end+1)=ravemakevariablename(handles,d,numel(handles.basenames{d}));
            end
            
            optpath=[path(1:end-3) 'opt'];
            if exist(optpath,'file')
                handles=raveloadmetadata(handles,d,optpath);
            end

        case 'STATISTICS'
            for c=1:numel(contents)
                cellz=regexp(contents{c},'\t','split');
                varname=cellz{1};
                stat=cellz(2:end);
                for q=1:numel(stat)
                    stat{q}=eval([stat{q},';']);
                end
                
                thisindex=find(strcmp(handles.basenames{d},varname));
                if isempty(thisindex)
                    warning('Metadata file contained a statistic for a non-existent variable.');
                    continue
                end
                
                if numel(thisindex)>1
                    warning('Could not load statistic info because variable names are not unique');
                    continue
                end
                handles.aggregatingstatistic{d}{thisindex}=stat;
                handles.allnames{d}(thisindex)=ravemakevariablename(handles,d,thisindex);
            end
            
        case 'RANGES'
            for j=1:numel(contents)
                tline=contents{j};
                
                tabz=find(tline==char(9));
                lastindex=find(isstrprop(tline,'alphanum'),1,'last');
                tabz=[tabz(tabz<lastindex) lastindex+1];
                thisvarname=tline(1:tabz(1)-1);
                thistype=lower(tline(tabz(1)+1:tabz(2)-1));
                thisvar=find(strcmp(handles.basenames{d},thisvarname));
                
                if numel(thisvar)>1
                    warning('Rave:Metadata:nonuniquename',['I could not load the variable range information for ',thisvarname,' because you have more than one variable with that name and I don''t know which one you were referring to.'])
                    
                    continue
                end
                
                if isempty(thisvar)
                    clear values
                    warning('Rave:Metadata:noname',['I could not load the variable range information for ',thisvarname,' because there is no variable in your data set with that name.'])
                    continue
                end
                if ~ismember(thistype,{'continuous','discrete','integer','string','list'})
                    thistype='continuous';
                end
                
                %store type:
                datasets.datatype{d}{thisvar}=thistype;
                handles.datatype{d}{thisvar}=thistype;
                
                %store values:
                if ~strcmp(thistype,'string')
                    for t=2:numel(tabz)-1
                        values(t-1)=str2double(tline(tabz(t)+1:tabz(t+1)-1)); %#ok<AGROW>
                    end
                    values=unique(values);
                else
                    for t=2:numel(tabz)-1
                        values{t-1}=tline(tabz(t)+1:tabz(t+1)-1);
                    end
                    values=unique(values);
                end
                
                a=sectiona(s);
                
                switch lower(thistype)
                    case 'constant' %KEEP for backward compatibility
                        handles.minlimit{d}{a}(thisvar)=values(1);
                        handles.maxlimit{d}{a}(thisvar)=values(1);
                    case {'continuous','integer','logical'}
                        handles.minlimit{d}{a}(thisvar)=min(values);
                        handles.maxlimit{d}{a}(thisvar)=max(values);
                    case 'discrete'
                        handles.minlimit{d}{a}(thisvar)=min(values);
                        handles.maxlimit{d}{a}(thisvar)=max(values);
                        handles.discretevalues{d}{a}{thisvar}=values';
                    case {'string','list'}
                        handles.minlimit{d}{a}(thisvar)=1;
                        handles.maxlimit{d}{a}(thisvar)=numel(values);
                        handles.discretevalues{d}{a}{thisvar}=values';
                end
                clear values
            end
            
        case 'CONSTRAINTS'
            for c=1:numel(contents)
                thisstr=contents{c};
                thisstr=thisstr(thisstr~=char(9));
                %determine color:
                brack=find(thisstr=='[' | thisstr==']');
                if numel(brack==2)
                    color=eval(thisstr(brack(1):brack(2)));
                    thisstr=thisstr(1:(brack(1)-1));
                else
                    color=handles.colors.constraint;
                end
                
                %determine type of constraint:
                if ~isempty(strfind(thisstr,'<='))
                    thisequality='<=';
                    k=strfind(thisstr,'<=');
                    constraintstring=thisstr(1:k-1);
                    constraintvalue=eval(thisstr(k+2:end));
                elseif ~isempty(strfind(thisstr,'>='))
                    thisequality='>=';
                    k=strfind(thisstr,'>=');
                    constraintstring=thisstr(1:k-1);
                    constraintvalue=eval(thisstr(k+2:end));
                elseif ~isempty(strfind(thisstr,'='))
                    thisequality='=';
                    k=strfind(thisstr,'=');
                    constraintstring=thisstr(1:k-1);
                    constraintvalue=eval(thisstr(k+1:end));
                elseif ~isempty(strfind(thisstr,'<'))
                    thisequality='<';
                    k=strfind(thisstr,'<');
                    constraintstring=thisstr(1:k-1);
                    constraintvalue=eval(thisstr(k+1:end));
                    
                elseif ~isempty(strfind(thisstr,'>'))
                    thisequality='>';
                    k=strfind(thisstr,'>');
                    constraintstring=thisstr(1:k-1);
                    constraintvalue=eval(thisstr(k+1:end));
                    
                else
                    continue %invalid constraint
                end
                
                [constraintfunctionstring,constrainttype,constraintvars]=raveprocessconstraint(handles,d,constraintstring,thisequality,constraintvalue);
                
                %store this function annonymously in handles.
                try
                    newhandle=eval(['@(data) ',constraintfunctionstring,';']);
                catch %#ok<CTCH>
                    warning('Rave:Metadata:badconstraint','Something went wrong while creating a constraint. Check to see if you spelled everything right and used correct MATLAB syntax.')
                    continue
                end
                
                for a=1:numel(handles.analysisnames)
                    handles.constrainthandles{d}{a}{end+1}=newhandle;
                    handles.constrainttext{d}{a}{end+1}=strtrim([constraintstring,thisequality,' ',num2str(constraintvalue)]);
                    handles.constrainttype{d}{a}{end+1}=constrainttype;
                    handles.constraintvars{d}{a}{end+1}=constraintvars;
                    handles.constraintcolors{d}{a}{end+1}=color;
                end
                
            end
            
            % NOTE: constraints get evaluated at the end of this function, after all data has been loaded.
            
            
        case 'CURRENTPOINT'
            % contents=importdata(path,'\t');
            
            nameline=contents{1};
            varnames=eval(['{''',strrep(nameline,char(9),''';'''),'''}']);
            
            values=contents{2};
            values=eval(['[',strrep(values,char(9),','),']']);
            
            a=sectiona(s);
            
            for c=1:numel(varnames);
                thisindex=find(strcmp(handles.basenames{d},varnames{c}));
                handles.xdials{d}{a}(thisindex)=values(c); %#ok<FNDSB>
            end
            
        case 'TARGETSANDPREFERENCES'
            nameline=contents{1};
            varnames=eval(['{''',strrep(nameline,char(9),''';'''),'''}']);
            
            targs=contents{2};
            targs=eval(['[',strrep(targs,char(9),','),']']);
            
            prefs=contents{3};
            prefs=eval(['[',strrep(prefs,char(9),','),']']);
            
            for c=1:numel(varnames);
                thisindex=find(strcmp(handles.basenames{d},varnames{c}));
                handles.preferences{d}(thisindex)=prefs(c);
                handles.targetvalues{d}(thisindex)=targs(c);
            end
            
            
        case 'COLORS'
            varnames=contents{1};
            colors=contents{2};
            
            nametabz=find(varnames==char(9));
            lastindex=find(isstrprop(varnames,'alphanum'),1,'last');
            nametabz=[0 nametabz(nametabz<lastindex) lastindex+1];
            colortabz=find(colors==char(9));
            lastindex=find(colors==']',1,'last');
            colortabz=[0 colortabz(colortabz<lastindex) lastindex+1];
            
            for c=1:min(numel(nametabz,colortabz))-1
                thisindex=find(strcmp(handles.basenames{d},varnames(nametabz(c)+1:nametabz(c+1)-1)));
                
                if numel(thisindex)>1
                    warning('Rave:Metadata:nonuniquename',['I could not load the variable color information for ',varnames(nametabz(c)+1:nametabz(c+1)-1),' because you have more than one variable with that name and I don''t know which one you were referring to.'])
                    
                    continue
                end
                
                if isempty(thisindex)
                    clear values
                    warning('Rave:Metadata:noname',['I could not load the variable color information for ',varnames(nametabz(c)+1:nametabz(c+1)-1),' because there is no variable in your data set with that name.'])
                    continue
                end
                
                if ~isempty(thisindex)
                    handles.variablecolors{d}(thisindex,:)=eval(colors(colortabz(c)+1:colortabz(c+1)-1));
                end
            end
            
        case 'ROWSTATE'
            for c=1:numel(contents)
                contents{c}=eval(['[',contents{c},']']);
            end
            contents=cell2mat(contents(:));
            
            if size(contents,1)~=size(handles.isselected{d}{1},1)
                warning('Rave:Metadata:inconsistentrows',['I could not load the row state metadata because the metadata contains a different number of rows than the data set.'])
            else
                a=sectiona(s);
                handles.rowcolors{d}{a}(1:size(contents,1))=contents(:,1);
                handles.isvisible{d}{a}(1:size(contents,1))=contents(:,2);
                handles.isselected{d}{a}(1:size(contents,1))=contents(:,3);
            end
            
        case 'ROWNOTES'
            try
                eval(contents{1});
            catch
               warning('Rave:Metadata:badnotes',['I could not load the row notes; check the rvm file to make sure it evaluates to a cell array named "notes".']) 
            end
            
            datanotes=raveguidata(handles.fig,'datanotes');
            datanotes{d}=notes;
            raveguidata(handles.fig,'datanotes',datanotes);

        case 'POINTSOFINTEREST'
            %not yet implemented
    end
end

guidata(handles.fig,handles);


%evaluate functions and fill in alldata:
if ~isempty(evalthese)
    [y]=raveevaluatefunction(handles.fig,d,evalthese,datasets.alldata{d},handles);

    datasets.alldata{d}(:,evalthese)=y;
%     textdata.alltext{d}=[textdata.alltext{d},cell(size(y))];
    vectordata.vectordata{d}=[vectordata.vectordata{d},cell(size(y))];
    datasets.isvalid{d}(:,evalthese)=true(size(y));
    
    for a=1:numel(handles.analysisnames)
        limitz=[min(datasets.alldata{d}(:,evalthese),[],1);max(datasets.alldata{d}(:,evalthese),[],1)];
        limitz(1,isnan(limitz(1,:)))=0;
        limitz(2,isnan(limitz(2,:)))=1;
        
        handles.minlimit{d}{a}(evalthese)=limitz(1,:);
        handles.maxlimit{d}{a}(evalthese)=limitz(2,:);
        handles.minfilter{d}{a}(evalthese)=limitz(1,:);
        handles.maxfilter{d}{a}(evalthese)=limitz(2,:);
    end
    
    handles.isconstant{d}(evalthese)=limitz(1,:)==limitz(2,:);
end

%Evaluate constraints to get constraint normalization info:

%evaluate this constraint on current data set, and store results in handles.constraintvalues:
guidata(handles.fig,handles);
for a=1:numel(handles.analysisnames)
    for c=1:numel(handles.constrainthandles{d}{a})
        thisindex=c;

        %evaluate constraint:
        handles.constraintvalues{d}{a}{end+1}=evaluateconstraint(handles,d,thisindex,datasets.alldata{d});
        %%%%%%%%%%%%%%%%%%%%%%%%%%%
        
        %evaluate constraint at end points to get normalization parameters:
        mins=handles.minlimit{d}{a};
        maxs=handles.maxlimit{d}{a};
        
        minvalue=evaluateconstraint(handles,d,thisindex,mins,0);
        maxvalue=evaluateconstraint(handles,d,thisindex,maxs,0);
        meanvalue=evaluateconstraint(handles,d,thisindex,(mins+maxs)./2,0);
        
        sample=[minvalue maxvalue meanvalue];
        
        sample(isnan(sample))=1; %not really sure what to do if nan pops up (most likely from 0/0 expression)
        
        %offset=mean(sample);
        range=max(sample)-min(sample);
        
        if range==0
            range=1;
        end
        
        %handles.constraintadders{udata.view.dataset}{udata.view.analysis}{end+1}=offset;
        handles.constraintscalers{d}{a}(end+1)=range;
        guidata(handles.fig,handles);
    end
end

raveguidata(handles.fig,'datasets',datasets);
% raveguidata(handles.fig,textdata);
raveguidata(handles.fig,'vectordata',vectordata);

function g=evaluateconstraint(handles,dataset,thisconstraint,indata,ignore)
%evaluates the regression with index stored in thisconstraint using the data stored in data.
%thisconstraint is an index into:
%handles.constrainthandles{udata.view.dataset}{udata.view.analysis}{thisconstraint}
%ignore is a logical vector of length=size(indata,2) (numvars) where 1 means do not evaluate this
%regression (if that varaible isfunction). use this to do things like sweeps through regressed
%variables. or just make ignore==0, to  evaluate all constraints.


%{

%% determine which variables appear in this constraint:
thisstring=char(handles.constrainthandles{dataset}{a}{thisconstraint});
thisstring=thisstring(8:end);

varnumbers=handles.constraintvars{dataset}{a}{thisconstraint};

varnumbers=setdiff(varnumbers,find(ignore)); %dont evaluate regressions of ignored vars

if ~isempty(varnumbers)
    %evaluate all regressions necessary for this constraint:
    respdata=raveevaluatefunction(handles.fig,udata.view.dataset,varnumbers,indata,handles);
    indata(:,varnumbers)=respdata;
end
%}
%solve evaluate this constraint for y variable:

thishandle=handles.constrainthandles{dataset}{1}{thisconstraint};
g=thishandle(indata);


