function  handles=ravemakegraph(src,ai,type,sheet,position,udata)
%Creates (or recreates) a new workspace object with active index=ai of the
%input type, on the input sheet, in the input position, using the input udata
%structure.
%
%If the created object needs to be "put away", this should be done by
%calling raveputawaygraph after calling this function. 

if nargin<6
   udata=[];
end

handles=guidata(src);

%Check if "allblockers" needs to be created. Depending on which function
%called this one, they may have already been made:

if numel(handles.allblockers)>=ai && ishandle(handles.allblockers(ai))
    %do nothing, all blockers already updated.
else
    if strcmp(handles.graphinfo.(type).class,'control');
        edgecolor=[0 0.5 1];
    elseif strcmp(handles.graphinfo.(type).class,'annotation');
        edgecolor=[0.7 0 1];
    elseif strcmp(handles.graphinfo.(type).class,'table');
        edgecolor=[0.7 0.9 0.1];
    else
        edgecolor=[1 0.5 0];
    end
    
    handles.allblockers(ai)=rectangle('position',position,'linewidth',2,'edgecolor',edgecolor,...
    'parent',handles.mainaxes,'hittest','on','buttondownfcn','','facecolor',handles.colors.wsbg);
    handles.expblockers(ai)=rectangle('position',position,'linewidth',1,'edgecolor',edgecolor,...
    'parent',handles.explorer,'hittest','on','buttondownfcn','');
end

%Ensure that this graph type exists:
if isempty(which(['raveinitialize',type]))
    warning(['The requested workspace object, "',type,'" could not be found in your Rave installation. Ensure that file exists and is on path and try again.'],'File Not Found');   
    delete([handles.allblockers(ai) handles.expblockers(ai)]);
    return
end


%make graph in this position
axescolors=handles.colors.axes{1};
fgcolor=axescolors(1,:);
bgcolor=axescolors(2,:);
lightgrid=axescolors(3,:);
stronggrid=axescolors(4,:);

cvpos = get(handles.currentview,'position');
axpos = position-[cvpos(1) cvpos(2) 0 0];


fakeraxes=axes('parent',handles.wspanel,'units','pixels','box','on','position',axpos,...
    'userdata',[],'hittest','off','xcolor',lightgrid,'ycolor',lightgrid,...
    'fontname',handles.fonts.graph,'fontsize',handles.fonts.graphsize,...
    'zcolor',lightgrid,'xticklabel','','yticklabel','',...
    'gridlinestyle','-','userdata',[],'minorgridlinestyle','-','color',bgcolor,...
    'linewidth',handles.prefs.axeslinewidth);

fakeaxes=axes('parent',handles.wspanel,'units','pixels','box','on','position',axpos,...
    'userdata',[],'hittest','off','xcolor',stronggrid,'ycolor',stronggrid,...
    'fontname',handles.fonts.graph,'fontsize',handles.fonts.graphsize,...
    'zcolor',stronggrid,'xticklabel','','yticklabel','',...
    'gridlinestyle','-','color','none','userdata',[],'linewidth',handles.prefs.axeslinewidth);

newaxes=axes('parent',handles.wspanel,'units','pixels','box','on','position',axpos,...
    'fontname',handles.fonts.graph,'fontsize',handles.fonts.graphsize,...
    'userdata',type,'hittest','off','color','none','buttondownfcn',@raveselectgraph,...
    'xcolor',fgcolor,'ycolor',fgcolor,'zcolor',fgcolor,'linewidth',handles.prefs.axeslinewidth);

titel=get(newaxes,'title');
set(titel,'fontname',handles.fonts.graphtitle,'fontsize',handles.fonts.graphtitlesize)

if strcmp(handles.graphinfo.(type).class,'control');
    facecolor=[0 0.5 1];
elseif strcmp(handles.graphinfo.(type).class,'annotation');
    facecolor=[0.7 0 1];
elseif strcmp(handles.graphinfo.(type).class,'table');
    facecolor=[0.7 0.9 0.1];    
else
    facecolor=[1 0.5 0];
end

edgecolor=facecolor/2;

%make movegraph grabbers:
grab1=line(position(1)+2,position(2)+3,100,'hittest','on','marker','s','markersize',5,'markerfacecolor',facecolor,'markeredgecolor',edgecolor,...
    'tag','SW','parent',handles.mainaxes,'buttondownfcn',@ravemovegraph,'linesmoothing','off');
grab2=line(position(1)+2,position(2)+position(4)-2,100,'marker','s','markersize',5,'markerfacecolor',facecolor,'markeredgecolor',edgecolor,...
    'tag','NW','parent',handles.mainaxes,'buttondownfcn',@ravemovegraph,'linesmoothing','off');
grab3=line(position(1)+position(3)-3,position(2)+position(4)-2,100,'marker','s','markersize',5,'markerfacecolor',facecolor,'markeredgecolor',edgecolor,...
    'tag','NE','parent',handles.mainaxes,'buttondownfcn',@ravemovegraph,'linesmoothing','off');
grab4=line(position(1)+position(3)-3,position(2)+3,100,'marker','s','markersize',5,'markerfacecolor',facecolor,'markeredgecolor',edgecolor,...
    'tag','SE','parent',handles.mainaxes,'buttondownfcn',@ravemovegraph,'linesmoothing','off');

if strcmpi(handles.graphinfo.(type).axesformat,'fixedsize')
   set([grab2 grab3 grab4],'visible','off'); 
end


%adjust margins to encompass tick mark labels
inset=get(newaxes,'tightinset');

axpos=axpos+[inset(1)+4 inset(2)+2 -inset(1)-inset(3)-4 -inset(2)-inset(4)];
try
    set(newaxes,'position',axpos)
    set(fakeraxes,'position',axpos)
    set(fakeaxes,'position',axpos)
catch %#ok<CTCH> %enforce minimum graph size:
%     position=get(handles.temp.putgraphhere,'position');
    axpos(3:4)=[198 198];
    set([handles.allblockers(ai) handles.expblockers(ai)],'position',axpos)
    
    set(grab1,'xdata',axpos(1)+2,'ydata',axpos(2)+3)
    set(grab2,'xdata',axpos(1)+2,'ydata',axpos(2)+axpos(4)-2)
    set(grab3,'xdata',axpos(1)+axpos(3)-3,'ydata',axpos(2)+axpos(4)-2)
    set(grab4,'xdata',axpos(1)+axpos(3)-3,'ydata',axpos(2)+3)
    axpos=axpos+[inset(1)+4 inset(2)+2 -inset(1)-inset(3)-4 -inset(2)-inset(4)];
    set(newaxes,'position',axpos)
    set(fakeraxes,'position',get(newaxes,'position'))
    set(fakeaxes,'position',get(newaxes,'position'))
    
end

%reorder to bring axes to top:
uistack(fakeraxes,'top')
uistack(fakeaxes,'top')
uistack(newaxes,'top')

%recolor position rectangles
set(handles.allblockers(ai),'facecolor',handles.colors.wsbg,'edgecolor',handles.colors.wsgrid,'linewidth',1,'hittest','on',...
    'buttondownfcn',@raveselectgraph);
set(handles.expblockers(ai),'facecolor',[.7 .7 .7],'linewidth',1,'hittest','off');
uistack(handles.currentview,'top');


%store handles of axes
handles.allgraphs(ai)=newaxes;
handles.fakeax(ai)=fakeaxes;
handles.fakerax(ai)=fakeraxes;
handles.grabbers{ai}=[grab1 grab2 grab3 grab4];
handles.subax{ai}=[];
handles.subfakeax{ai}=[];
handles.subfakerax{ai}=[];
handles.rasters{ai}=[];
handles.graphanalysis(ai)=1;
handles.graphdataset(ai)=1;
handles.sheet(ai)=sheet;
handles.graphnames{ai}='';
handles.graphtags{ai}='';

%clear the infobar
set(handles.infobar,'string','');

%undo creation function
set(handles.mainaxes,'buttondownfcn',@ravedragview);
set(handles.explorer,'buttondownfcn',@ravemoveview);
set(handles.fig,'windowkeypressfcn',@ravedefaultkeypressfcn)

%update guidata
guidata(handles.fig,handles);

%make this graph active:
raveselectgraph(newaxes,[])

%create the initial graph:
initfcn=str2func(['raveinitialize',type]);
if ~isstruct(udata)
    initfcn(handles.fig,[]);
else %If udata was supplied, create graph using this udata structure:
    initfcn(handles.fig,[],udata)
end

%refresh guidata
handles=guidata(handles.fig);






