function ravemakemodelgui(src,ev)
%gui to create a new surrogate model.
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

%refresh model info:
raveupdatemodeltypes(src,ev);

handles=guidata(src);


origin=get(handles.fig,'position');

if handles.height>=800
    issmall=false;
    pos=[origin(1)+origin(3)/2-600 origin(2)+origin(4)/2-400 1200 800];
else
    issmall=true;
    pos=[origin(1)+origin(3)/2-600 origin(2)+origin(4)/2-350 1200 700];
end

hhh=figure('menubar','none','units','pixels','position',pos,'color',handles.colors.frame,...
    'name','Create New Model','numbertitle','off','resize','off','windowstyle','modal','renderer','painters',...
    'windowkeypressfcn',@keycap);

%get context menu:
handles.temp.contextmenu=copyobj(handles.contextmenus.modelqueuetable,hhh);
if ~issmall
    titleaxes=axes('units','pixels','position',[20 750 400 40],'parent',hhh,'visible','off');
    text(0,.4,'Fit a model to your data','parent',titleaxes,'fontname',handles.fonts.title,'fontsize',handles.fonts.titlesize,...
        'color',handles.colors.title,'horizontalalignment','left','verticalalignment','baseline');
end
makehelpblob(hhh,380,765,'helproot?title=Create_New_Model_GUI',handles.colors.frame);

if ~issmall
    upos=[50 340 1100 410];
else
    upos=[50 280 1100 410];
end
u=uipanel('units','pixels','position',upos,'title','','bordertype','etchedin','backgroundcolor',[1 1 1]);

uicontrol('style','text','parent',u,'units','pixels','position',[10 376 200 22],'string','Select a data set:','backgroundcolor',get(u,'backgroundcolor'),...
    'fontsize',10,'fontname','arial','fontweight','bold','horizontalalign','left')

uicontrol('style','popup','parent',u,'units','pixels','position',[134 382 200 20],'string',handles.datasetnames,...
    'backgroundcolor',[1 1 1],'tag','datasetmenu','callback',@datasetmenu_callback)

h1axes=axes('parent',u,'units','pixels','position',[10 350 200 35],'visible','off');
text(0,0,'Variables to Model','parent',h1axes,'fontname',handles.fonts.tab,'fontsize',handles.fonts.tabsize,'color',handles.colors.text,...
    'hor','left','ver','baseline')
uuu=uicontrol('style','listbox','parent',u,'units','pixels','position',[20 10 200 330],'string',handles.allnames{1},...
    'backgroundcolor',[1 1 1],'max',100,'tag','responselist');
uicontrol('style','text','parent',u,'units','pixels','position',[20 10 200 330],'string','The selected model type does not model an individual variable.',...
    'backgroundcolor',[1 1 1],'max',100,'tag','noresponsetext','visible','off')

h2axes=axes('parent',u,'units','pixels','position',[260 350 200 35],'visible','off');
text(0,0,'As Functions of','parent',h2axes,'fontname',handles.fonts.tab,'fontsize',handles.fonts.tabsize,'color',handles.colors.text,...
    'hor','left','ver','baseline')
uicontrol('style','listbox','parent',u,'units','pixels','position',[270 10 200 330],'string',handles.allnames{1},...
    'backgroundcolor',[1 1 1],'max',100,'tag','inputlist')

h3axes=axes('parent',u,'units','pixels','position',[260+250 380 200 35],'visible','off');
text(0,0,'Model Type & Settings','parent',h3axes,'fontname',handles.fonts.tab,'fontsize',handles.fonts.tabsize,'color',handles.colors.text,...
    'hor','left','ver','baseline')

methodnames=fieldnames(handles.modelinfo);
longnames={};
for g=1:numel(methodnames)
    longnames{end+1}=handles.modelinfo.(methodnames{g}).longname; %#ok<AGROW>
end

uicontrol('style','popup','parent',u,'units','pixels','position',[512 348 280 22],'string',longnames,...
    'backgroundcolor',[1 1 1],'callback',@makemethodcontrols,'value',1,'tag','modelmethodmenu')

h4axes=axes('parent',u,'units','pixels','position',[825 380 200 35],'visible','off');
text(0,0,'Fit & Validation Data','parent',h4axes,'fontname',handles.fonts.tab,'fontsize',handles.fonts.tabsize,'color',handles.colors.text,...
    'hor','left','ver','baseline')

uicontrol('style','text','Position',[840 354 200 20],'string','Choose Fit-Data Column:','fontsize',10,'fontweight','bold',...
    'hor','left','backgroundcolor',get(u,'backgroundcolor'),'parent',u);
uicontrol('style','popup','parent',u,'units','pixels','position',[855 336 200 20],'string',[{'<Fit model to all rows>'},handles.allnames{1}],...
    'backgroundcolor',get(u,'backgroundcolor'),'tag','fitcolmenu','callback',@fitcolmenu_callback)

makehelpblob(u,1065,277+30+30,'helproot?title=Fit,_Validation,_and_Test_Data',get(u,'backgroundcolor'));

uicontrol('style','text','Position',[855 312 200 20],'string','Fit model to rows whose value is:','fontsize',10,'fontweight','normal',...
    'hor','left','backgroundcolor',[1 1 1],'parent',u);
uicontrol('style','edit','Position',[870 294 150 20],'string','0',...
    'hor','center','backgroundcolor',[1 1 1],'tag','fitrowsedit','parent',u,'enable','off');
uicontrol('style','text','Position',[855 268 240 20],'string','Validate model with rows whose value is:','fontsize',10,'fontweight','normal',...
    'hor','left','backgroundcolor',[1 1 1],'parent',u,'tag','validaterowstext');
uicontrol('style','edit','Position',[870 250 150 20],'string','1',...
    'hor','center','backgroundcolor',[1 1 1],'tag','validaterowsedit','parent',u,'enable','off');
uicontrol('style','text','Position',[855 224 240 20],'string','Test model with rows whose value is:','fontsize',10,'fontweight','normal',...
    'hor','left','backgroundcolor',[1 1 1],'parent',u);
uicontrol('style','edit','Position',[870 206 150 20],'string','2',...
    'hor','center','backgroundcolor',[1 1 1],'tag','testrowsedit','parent',u,'enable','off');

uicontrol('style','text','Position',[840 180 200 20],'string','Or...','fontsize',10,'fontweight','bold',...
    'hor','left','backgroundcolor',[1 1 1],'parent',u);
uicontrol('style','pushbutton','parent',u,'units','pixels','position',[880-10 140 210 40],'string','Make New Fit-Data Column',...
    'fontsize',10,'enable','on','callback',@makenewfitcolumn)

uicontrol('style','text','Position',[840 55 200 20],'string','All settings ready?','fontsize',12,'fontweight','bold',...
    'hor','left','backgroundcolor',[1 1 1],'parent',u);
uicontrol('style','pushbutton','parent',u,'units','pixels','position',[870 10 210 40],'string','Add Model(s) to Queue',...
    'fontsize',12,'enable','on','foregroundcolor',[0 .7 0],'callback',@addmodeltoqueue,'fontweight','bold')

a=axes('units','pixels','parent',hhh,'position',[15 264 1170 10],'visible','off');
line([0 1],[.5 .5],'parent',a,'color',[0 0 0],'linewidth',1);

uicontrol('style','pushbutton','parent',hhh,'position',[1090 170 90 90],'string','<html>Create<br>Models</html>',...
    'fontsize',0.66*handles.fonts.tabsize,'enable','on','foregroundcolor',[0 .7 0],...
    'callback',@ravemakesurrogatemodels,'fontweight','bold','tag','createbutton');

uicontrol('style','text','parent',hhh,'position',[1090 70 90 90],'string','Right click entries to reorder or delete models.',...
    'foregroundcolor',[0 0 0],'hor','left',...
    'backgroundcolor',handles.colors.frame);

uicontrol('style','togglebutton','parent',hhh,'position',[1090 170 90 90],'string','<html>Pause</html>',...
    'fontsize',0.66*handles.fonts.tabsize,'enable','on','foregroundcolor',[0 0 0],...
    'callback',[],'fontweight','bold','tag','pausebutton','visible','off');

uicontrol('style','pushbutton','parent',hhh,'position',[1090 10 90 28],'string','Select >>',...
    'fontsize',14,'enable','on',...
    'callback',@donebutton_callback,'fontweight','normal');

handles.temp.methodcontrols=[];

dmod=makequeuetablemodel(handles.fig,[]);
javaObjectEDT(dmod);
jtable=javax.swing.JTable(dmod);
javaObjectEDT(jtable);
j=javax.swing.JScrollPane(jtable);
jtable.setFillsViewportHeight(true);
[jscrollpane,mscrollpane]=javacomponent(j,[25 10 1050 250],hhh);

% Set column widths:
jtable.getTableHeader().setReorderingAllowed(false) %turn off draggable columns
jtable.setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_OFF)%turn off auto resizing
colwidths=[200 200 550 100]; %some column widths
for c=1:4 %set the widths:
    jtable.getColumnModel().getColumn(c-1).setPreferredWidth(colwidths(c))
end

% renderer:
rend=RaveRowStatusRenderer();
javaObjectEDT(rend);
obj=java.lang.Object;
class=obj.getClass();
jtable.setDefaultRenderer(class,rend);

handles.temp.jscroll=jscrollpane;
handles.temp.mscroll=mscrollpane;

set(handle(jtable,'callbackproperties'),'MousePressedCallback',{@queueTableCallback,hhh})

guidata(hhh,handles);

makemethodcontrols(hhh,[]); %load first method

uicontrol(uuu);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function datasetmenu_callback(src,ev)
handles=guidata(src);
tfig=ancestor(src,'figure');
responselist=findobj(tfig,'tag','responselist');
inputlist=findobj(tfig,'tag','inputlist');
fitcolmenu=findobj(gcf,'tag','fitcolmenu');

d=get(src,'value');

set(responselist,'string',handles.allnames{d},'value',1);
set(inputlist,'string',handles.allnames{d},'value',1);
set(fitcolmenu,'string',[{'Select a column...'},handles.allnames{d}],'value',1);

%reload method controls:
makemethodcontrols(tfig,[]); %load first method


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function makemethodcontrols(src,ev)
handles=guidata(src);
delete(handles.temp.methodcontrols);

methodmenu=findobj(gcf,'tag','modelmethodmenu');
modelkeywords=fieldnames(handles.modelinfo);
thiskeyword=modelkeywords{get(methodmenu,'value')};

%load default settings:
settings=struct();
eval(['modelsettings',thiskeyword]);
handles.temp.settings=settings;

tfig=ancestor(src,'figure');
vt=findobj(tfig,'tag','validaterowstext');
ve=findobj(tfig,'tag','validaterowsedit');
te=findobj(tfig,'tag','testrowsedit');

%show/hide controls depending on model's settings:
if settings.usesvalidation
    set([vt ve],'visible','on');
    set(ve,'string','1')
    set(te,'string','2')
else
    set([vt ve],'visible','off');
    set(te,'string','1, 2');
end

rm=findobj(tfig,'tag','responselist');
nr=findobj(tfig,'tag','noresponsetext');
if settings.usesresponse
    set(rm,'visible','on')
    set(nr,'visible','off')
else
    set(rm,'visible','off')
    set(nr,'visible','on')
end




%create controls:
eval(['ui=makemodeluicontrols',thiskeyword,'(src,[]);']);
handles.temp.methodcontrols=ui;

guidata(src,handles);


function addmodeltoqueue(src,ev)
handles=guidata(src);
datasets=raveguidata(handles.fig,'datasets');

tfig=ancestor(src,'figure');
datasetmenu=findobj(tfig,'tag','datasetmenu');
responselist=findobj(tfig,'tag','responselist');
inputlist=findobj(tfig,'tag','inputlist');
methodmenu=findobj(gcf,'tag','modelmethodmenu');
fitcolmenu=findobj(gcf,'tag','fitcolmenu');
fitrowsedit=findobj(gcf,'tag','fitrowsedit');
validaterowsedit=findobj(gcf,'tag','validaterowsedit');
testrowsedit=findobj(gcf,'tag','testrowsedit');


d=get(datasetmenu,'value');
y=get(responselist,'value');
x=get(inputlist,'value');


fitcol=get(fitcolmenu,'value')-1;
if fitcol==0 %fit to all data
    fitrows=true(size(handles.isselected{d}{1},1),1);
    validaterows=false(size(handles.isselected{d}{1},1),1);
    testrows=false(size(handles.isselected{d}{1},1),1);
else
    
    try
        fitrows=str2num(get(fitrowsedit,'string')); %#ok<ST2NM>
    catch %#ok<CTCH>
        errordlg('You must enter a list of numbers in the fit-rows edit box. Separate values with a space or comma.')
        return
    end
    
    try
        validaterows=str2num(get(validaterowsedit,'string')); %#ok<ST2NM>
    catch %#ok<CTCH>
        errordlg('You must enter a list of numbers in the validation-rows edit box. Separate values with a space or comma.')
        return
    end
    
    try
        testrows=str2num(get(testrowsedit,'string')); %#ok<ST2NM>
    catch %#ok<CTCH>
        errordlg('You must enter a list of numbers in the test-rows edit box. Separate values with a space or comma.')
        return
    end
    
    %combine fit/test/validation data into index matrix:
    if isempty(fitrows)
        errordlg('No rows match the fit-data criteria. Change the fit-data column and/or fit-data row values');
        return;
    end
    
    %turn fit/test/validate rows into indices:
    fitdata=datasets.alldata{d}(:,fitcol);
    
    fitrows=ismember(fitdata,fitrows);
    validaterows=ismember(fitdata,validaterows);
    testrows=ismember(fitdata,testrows);
end

%Check for constant variables, which break everything:
fitdata=datasets.alldata{d}(fitrows,x);
if any(all(bsxfun(@eq,fitdata(1,:),fitdata),1))
   badvars = x(all(bsxfun(@eq,fitdata(1,:),fitdata),1));
   
   str= 'Model cannot be created using the selected settings because the following variables have identical values for every point in the "Fit model to" rows. Remove these variables from your model and try again.';
   for i = 1:numel(badvars)
      str=[str,' ',handles.allnames{d}{badvars(i)},',']; 
   end
   
   
   errordlg(str(1:end-1),'Constant Variables')
   return;
end

if ~any(fitrows)
    errordlg('The selected settings would cause the model to be fit to an empty data set. Change the "Fit-data column" and/or "Fit model to rows..." and try again.','Empty fit data')
    return
end

settings=handles.temp.settings; %load previously created settings.

%convert validation to test for models that don't use validation:
if ~settings.usesvalidation
    % testrows=testrows | validaterows;
    validaterows(:)=false;
end

settings.dataset=d; %data set index.
settings.datasetpath=[]; %path to this data set's initial file.
settings.inputs=x; %column indices of variables that are independent variables to this model.
settings.output=[]; %column indices of variables to fit models to. (set in loop below);
settings.fitrows=fitrows; %vector of 0,1,2 indicating how to use each row.
settings.validaterows=validaterows;
settings.testrows=testrows;
settings.fitcolumn=fitcol; %index of the column being used to fit data
settings.status='Waiting';
settings.r2=[]; % correlation coefficient
settings.rmse=[]; %root mean square error
settings.meanerror=[]; %mean error
settings.sigmaerror=[]; %standard deviation of error
settings.notes='';
methodstring=get(methodmenu,'string');
settings.longname=methodstring{get(methodmenu,'value')};
settings.modelstring={''};
settings.name='';
settings.userdata=[];
settings.actualvalues=[]; %need to store the actual values in case the data set changes after this model was made.
settings.predictedvalues=[];
settings.modeledcolumn=[]; %a list of columns in the data set that are based on this model.

ui=handles.temp.methodcontrols; %#ok<NASGU>

%add model-specific settings:
modelkeywords=fieldnames(handles.modelinfo);
thiskeyword=modelkeywords{get(methodmenu,'value')};
settings.modeltype=thiskeyword;
eval(['addtoqueue',thiskeyword]);

%Did the above script return an error?
if strcmpi(settings.status,'Error')
    return
end


%add to the queue:
if settings.usesresponse
    for j=1:numel(y);
        settings.output=y(j);
        settings.actualvalues=datasets.alldata{d}(:,y(j));
        handles.modelqueue{end+1}=settings;
    end
else
    settings.output=0;
    settings.actualvalues=[];
    handles.modelqueue{end+1}=settings;
end

guidata(gcf,handles);
guidata(handles.fig,handles);

%update queue table:
jscroll=handles.temp.jscroll;
jtable=jscroll.getComponent(0).getComponent(0);
dmod=makequeuetablemodel(src,ev);
jtable.setModel(dmod);

% Set column widths:
jtable.getTableHeader().setReorderingAllowed(false) %turn off draggable columns
jtable.setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_OFF)%turn off auto resizing
colwidths=[200 200 550 100]; %some column widths
for c=1:4 %set the widths:
    jtable.getColumnModel().getColumn(c-1).setPreferredWidth(colwidths(c))
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function makenewfitcolumn(src,ev)
handles=guidata(src);
ttt=handles.temp;

h=ravemakenewfitcolumn(src,ev);
uiwait(h);

handles=guidata(handles.fig); %get updated handles...
handles.temp=ttt;
guidata(gcf,handles); %update makemodelgui's handles

%update variable lists:
datasetmenu=findobj(gcf,'tag','datasetmenu');
responselist=findobj(gcf,'tag','responselist');
inputlist=findobj(gcf,'tag','inputlist');
fitcolmenu=findobj(gcf,'tag','fitcolmenu');

d=get(datasetmenu,'value');
set(responselist,'string',handles.allnames{d});
set(inputlist,'string',handles.allnames{d});
set(fitcolmenu,'string',[{'<Fit model to all rows>'},handles.allnames{d}],'value',numel(handles.allnames{d})+1);



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function queueTableCallback(jsrc,jev,src) %#ok<INUSL>
handles=guidata(src);

if isempty(handles.modelqueue);return;end

MouseEvent=jev;

if MouseEvent.getClickCount == 2 %double click
    return
elseif MouseEvent.getButton > 1    %right click
    handles=guidata(src);
    
    %determine where to put context menu:
    xpos1=get(handles.temp.mscroll,'position');
    xpos2=MouseEvent.getX;
    
    ypos1=get(handles.temp.mscroll,'position');
    ypos3=MouseEvent.getY;
    ypos4=handles.temp.jscroll.getComponent(0).getComponent(0).getTableHeader.getHeight();
    
    finalpos=[xpos1(1)+xpos2(1) ypos1(2)+ypos1(4)-ypos3-ypos4];
    set(handles.temp.contextmenu,'position',finalpos,'visible','on');
    set(0,'userdata',MouseEvent);
    
else %presumably single left click
    return
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function dmod=makequeuetablemodel(src,ev)
%NOTE, THIS FUNCTION IS DUPLICATED IN ravereordermodelqueue (because i am lazy). Make sure to
%synchronize changes between these two files.

handles=guidata(src);

nmodels=numel(handles.modelqueue);

if nmodels<1
    javarray = {'','','',''};
    rowstatus=[];
else
    javarray=javaArray('java.lang.String',nmodels,4);
    rowstatus=javaArray('java.lang.Integer',nmodels);
    for r=1:nmodels
        d=handles.modelqueue{r}.dataset;
        javarray(r,1)=java.lang.String(handles.modelqueue{r}.longname);
        if handles.modelqueue{r}.output>0
            javarray(r,2)=java.lang.String(handles.allnames{d}{handles.modelqueue{r}.output});
        else
            javarray(r,2)=java.lang.String('<No Response Variable>');
        end
        
        javarray(r,3)=java.lang.String(handles.modelqueue{r}.notes);
        javarray(r,4)=java.lang.String(handles.modelqueue{r}.status);
        
        %row status colors:
        %0 = white
        %1 = red
        %2 = yellow
        %3 = green
        %4 = blue
        %5 = gray
        switch handles.modelqueue{r}.status
            case 'Waiting'
                rowstatus(r)=java.lang.Integer(4);
            case 'Creating...'
                rowstatus(r)=java.lang.Integer(2);
            case 'Error'
                rowstatus(r)=java.lang.Integer(1);
            case 'Finished'
                rowstatus(r)=java.lang.Integer(3);
            case 'Disabled'
                rowstatus(r)=java.lang.Integer(5);
            otherwise
                rowstatus(r)=java.lang.Integer(0);
        end
    end
end
cnames = {'Model Type','Variable','Notes','Status'};

%dmod=javax.swing.table.DefaultTableModel(javarray,cnames);
dmod=RaveDataTypeTableModel(javarray,cnames);
javaObjectEDT(dmod);
if ~isempty(rowstatus)
    dmod.setRowStatus(rowstatus);
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function donebutton_callback(src,ev)
handles=guidata(src);
close(gcf);
raveselectmodelgui(handles.fig,[]);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function fitcolmenu_callback(src,ev)

fr=findobj(gcf,'tag','fitrowsedit');
tr=findobj(gcf,'tag','testrowsedit');
vr=findobj(gcf,'tag','validaterowsedit');

if get(src,'value')==1
    set([fr tr vr],'enable','off')
    
else
    set([fr tr vr],'enable','on')
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function keycap(src,ev)
try %try because i'm not sure if older versions use same format for eventdata:
    if strcmp(ev.Key,'escape')
        close(gcbf);
    end
end
