function [c,cdata]=ravemakerowcdata(handles,datasets,udata,ccol,cmap,index)
%Make cdata that indexes into figure colormap for column ccol for colormap index cmap.
%Outputs:     c = indicies into colormap (n x 1) 
%         cdata = truecolor (n x 3)
%Udata structure may also contain udata.format.reversecolors and
%udata.format.ncolor (noisy colors)
%
% ccol==0 means use row colors, else colors by the specified column

%Optional 5th input, index, only returns cdata for the rows listed in index
%(may be either logical or row numbers)
%

d=udata.view.dataset;
a=udata.view.analysis;

if nargin<6
    index=true(size(datasets.alldata{d},1),1);
end

allcolorz=get(handles.fig,'colormap');
if ccol~=0 %Color by column
    c=datasets.alldata{d}(index,ccol);
    %normalize cdata to desired colormap:
    c=(c-min(c))/(max(c)-min(c))*127+128*(cmap-1)+1;
    if isfield(udata.format,'reversecolors')
        if udata.format.reversecolors
            c=(1-(c-min(c))/(max(c)-min(c)))*127+128*(cmap-1)+1;
        end
    end
    c(isnan(c))=128*(cmap-1)+1;
    cdata=allcolorz(floor(c),:);
    
else %USE ROW COLORS
    c=1920+(handles.rowcolors{d}{a}(index))'; %non-noisy colors cdata=0.5
    cdata=allcolorz(c,:);
    if isfield(udata.format,'ncolor')
        if udata.format.ncolor %do noisy colors
            noise=0.3; %noise multiplier
            cdata=cdata+(rand(size(cdata))-.5)*noise;
            cdata(cdata<0)=0;
            cdata(cdata>1)=1;
        end
    end
end

c=double(c(:)); %important to prevent a dumb matlab bug when c has exactly 3 elements. Also must be double.