function [xdim,ydim,zdim]=ravemovecustomticks(src,ev,alwaysrotate)
%updates the location of custom ticks to reflect a new graph size
% src is the handle of the graph whose ticks should be moved.
% also updates the location of xlabel/ylabel regardless of whether
% the graph uses custom ticks.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2012 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

%only reference to handles is in a preference. just clear this function
%after setting the pref.

xdim=0;
ydim=0;
zdim=0;

persistent handles
if isempty(handles)
    handles=guidata(src);
end

if nargin<3
    alwaysrotate=false;
end

xlim=get(src,'xlim');
ylim=get(src,'ylim');
zlim=get(src,'zlim');
axpos=getpixelposition(src);

%significantly faster than using findall:
state=get(0,'ShowHiddenHandles');
set(0,'ShowHiddenHandles','on');
xticks=ravefindobjinternal(src,'tag','xticklabel');
yticks=ravefindobjinternal(src,'tag','yticklabel');
zticks=ravefindobjinternal(src,'tag','zticklabel');
set(0,'ShowHiddenHandle',state);

v=get(src,'view');
az=v(1);
el=v(2);

xunitsperpix=(xlim(2)-xlim(1))./axpos(3); %*sin(el*pi/180)*cos(az*pi/180);

if az==0 && el==90
    yunitsperpix=(ylim(2)-ylim(1))./axpos(4); % *sin(el*pi/180)*sin(az*pi/180);
else
    yunitsperpix=(ylim(2)-ylim(1))./axpos(3); % *sin(el*pi/180)*sin(az*pi/180);
end

zunitsperpix=(zlim(2)-zlim(1))./axpos(4); %*cos(el*pi/180);
w=axpos(3);
h=axpos(4);


%% move x ticks:
if ~isempty(xticks)
    %estimate 4 pixel gap:
    if az==0 && el==90  %2D VIEW
        ypos=ylim(1)-4*yunitsperpix;
    else
        if az>-90 && az<=90
            ypos=ylim(1)-6*yunitsperpix;
        else
            ypos=ylim(2)+6*yunitsperpix;
        end
    end
    
    zpos=zlim(1)-2*zunitsperpix;
    
    %determine how many pixels are available for each label:
    
    ticks=get(src,'xtick');
    if numel(ticks)~=numel(xticks)
        warning('error in xtick count');
        return
    end
    tickpix=interp1(xlim,[0 w],ticks,'linear','extrap'); %extrap is important to make sure it doesnt return NaN if ticks are outside of axes limits. 
    if numel(tickpix)>1
        divs=tickpix(1:end-1)+diff(tickpix)/2;
        overshoot=20; %number of pixels beyond width of axes that tick labels are allowed to extend.
        maxtickwidth=diff([-overshoot divs w+overshoot]);
        maxtickwidth(1)=maxtickwidth(2);
        maxtickwidth(end)=maxtickwidth(end-1);
    else %if only one tick label:
        maxtickwidth=w;
    end
    
    if alwaysrotate
        status=false(1,numel(xticks));
    else
        status=true(1,numel(xticks));
    end
    
    if alwaysrotate;status(:)=0;end
    set(0,'ShowHiddenHandles','on');
    xticks=ravefindobjinternal(src,'tag','xticklabel');
    set(0,'ShowHiddenHandle',state);

    
    str=get(xticks,'string');
    if ~iscell(str);str={str};end
    for s=numel(xticks):-1:1;
        
        if iscell(str{s})
            str{s}=cell2mat(str{s}(:)');
        end
        if all(status) %if no ticks need to be rotated, keep wrapping (if any are rotated, no need to wrap)
            %[wrapped{s},status(s),nrows(s)]=superiorlinewrap(str{s},maxtickwidth(s),handles.fonts.tick,handles.fonts.ticksize,'plain',handles.axiscanvas);
            [wrapped{s},status(s),nrows(s)]=superiorlinewrapfast(str{s},maxtickwidth(s),handles.fontmetrics.tick);
            
            set(xticks(s),'units','data','string',wrapped{s},'position',[ticks(s),ypos,zpos],'rotation',0,'hor','center','vert','top','visible','on');
        end
    end

    if any(status==0) %rotate ticks
        set(xticks,'rotation',90,'hor','right','ver','middle');
        for s=numel(xticks):-1:1
            set(xticks(s),'string',str{s},'units','data','position',[ticks(s),ypos],'visible','on')
        end
        [xdims,tickheight]=ravecalculatestringlengthfast(str,handles.fontmetrics.tick);
        xdim=max(xdims);
        
        %check if there is still not enough room to clearly show the
        %ticks. If not, turn some of their visibilities off:
        avgwidth=mean(maxtickwidth);
        skip=ceil((tickheight-3)/avgwidth); %tick height is a little generous, so subtract 3 pixels...
        idx=1:skip:numel(xticks);
        if all(~isnan(idx)) %because this happens sometimes...
        set(xticks,'visible','off');
        set(xticks(idx),'visible','on');
        end
    else
        [junk,tickheight]=ravecalculatestringlengthfast('M',handles.fontmetrics.tick); %presumably M is the tallest letter.
        xdim=tickheight*max(nrows);
    end
    
    %hide ticks outside of xlim bounds:
    isout=ticks<xlim(1)-0.01*(xlim(2)-xlim(1)) | ticks>xlim(2)+0.01*(xlim(2)-xlim(1));
    set(xticks(isout),'visible','off');
end

if az==0 && el==90 %2D VIEW
    %Align according to code above
else %3D VIEW
    if (az>-20 && az<=20) || az<-160 || az>160
        set(xticks,'hor','center')
    elseif (az>20 && az<=90) || az<-90
        set(xticks,'hor','right')
    elseif (az<-20 && az>=-90) || az>90
        set(xticks,'hor','left')
    end
end

%% move xlabel:
if ~isempty(xticks)
    %determine where to move xlabel:
    xlab=get(src,'xlabel');
    if az==0 && el==90
        labelypos=ylim(1)-(5+xdim)*yunitsperpix; %additonal 5 pixel buffer
        pos=[mean(xlim) labelypos zlim(1)];
        set(xlab,'units','data','position',pos);
    else
        if az<=-178 || az>=178
            set(xlab,'hor','center','rotation',0)
            ypos=ylim(2)+20*yunitsperpix;
        elseif az>-178 && az<-93
            set(xlab,'hor','right','rotation',0)
            ypos=ylim(2)+20*yunitsperpix;
        elseif az>=-93 && az<-90
            set(xlab,'hor','center','rotation',90)
            ypos=ylim(2)+20*yunitsperpix;
        elseif az>=-90 && az<-85
            set(xlab,'hor','center','rotation',90)
            ypos=ylim(1)-30*yunitsperpix;
        elseif az>=-85 && az<-2
            set(xlab,'hor','left','rotation',0)
            ypos=ylim(1)-20*yunitsperpix;
        elseif az>-2 && az<=2
            set(xlab,'hor','center','rotation',0)
            ypos=ylim(1)-10*yunitsperpix;
        elseif az>2 && az<=85
            set(xlab,'hor','right','rotation',0)
            ypos=ylim(1)-20*yunitsperpix;
        elseif az>85 && az<=90
            set(xlab,'hor','center','rotation',90)
            ypos=ylim(1)-40*yunitsperpix;
        elseif az>90 && az<=93
            set(xlab,'hor','center','rotation',90)
            ypos=ylim(2)+30*yunitsperpix;
        elseif az>93 && az<178
            set(xlab,'hor','left','rotation',0)
            ypos=ylim(2)+20*yunitsperpix;
        end
        pos=[mean(xlim) ypos zlim(1)-20*zunitsperpix];
        set(xlab,'units','data','position',pos);
    end
else
    %guess height of matlab ticks:
    [junk,tickheight]=ravecalculatestringlengthfast('M',handles.fontmetrics.tick); %presumably M is the tallest letter.

    %convert pixels to data units:
    ylim=get(src,'ylim');
    yr=ylim(2)-ylim(1);
    offset=yr/axpos(4)*tickheight;
    
    %ylim=get(src,'ylim');
    pos=[mean(xlim) ylim(1)-offset zlim(1)];
    xlab=get(src,'xlabel');
    set(xlab,'units','data','position',pos);
end

%% move y ticks:
if ~isempty(yticks)
    %estimate 3 pixel gap:
    zpos=zlim(1)-2*zunitsperpix;
    if az==0 && el==90  %2D VIEW
        xpos=xlim(1)-4*xunitsperpix;
    else
        if az<=0
            xpos=xlim(1)-6*xunitsperpix;
        else
            xpos=xlim(2)+6*xunitsperpix;
        end
    end
    
    ystr=get(yticks,'string');
    [ydim,tickheight]=ravecalculatestringlengthfast(ystr,handles.fontmetrics.tick);
    
    
    ticks=get(src,'ytick');
    %determine how many pixels are available for each label:
    set(0,'ShowHiddenHandles','on');
    yticks=ravefindobjinternal(src,'tag','yticklabel');
    set(0,'ShowHiddenHandle',state);

    for s=1:min(numel(yticks),numel(ticks));
        set(yticks(s),'units','data','position',[xpos,ticks(s),zpos],'rotation',0,'hor','right','vert','middle','visible','on');
    end
    
    %check if there is still not enough room to clearly show the
    %ticks. If not, turn some of their visibilities off:
    
    %determine how many pixels are available for each label:
    h=axpos(4);
    if numel(ticks)~=numel(yticks);
        warning('error in ytick count');
        return;
    end
    tickpix=interp1(ylim,[0 h],ticks,'linear','extrap');
    if numel(tickpix)>1
        divs=tickpix(1:end-1)+diff(tickpix)/2;
        overshoot=20; %number of pixels beyond width of axes that tick labels are allowed to extend.
        maxtickwidth=diff([-overshoot divs w+overshoot]);
        maxtickwidth(1)=maxtickwidth(2);
        maxtickwidth(end)=maxtickwidth(end-1);
    else %if only one tick label:
        maxtickwidth=h;
    end
    avgwidth=mean(maxtickwidth);
    skip=ceil((tickheight-3)/avgwidth); %tick height is a little generous, so subtract 3 pixels...
    idx=1:skip:numel(yticks);
    if all(~isnan(idx)) %because this happens sometimes...
        set(yticks,'visible','off');
        set(yticks(idx),'visible','on');
    end

    %hide ticks outside of xlim bounds:
    isout=ticks<ylim(1) | ticks>ylim(2);
    set(yticks(isout),'visible','off');
    
end

if az==0 && el==90 %2D VIEW
    %Align according to code above
else %3D VIEW
    if (az>70 && az<110) || (az<-70 && az>-110)
        set(yticks,'hor','center','vert','top')
    elseif az<-110 || (az>0 && az<=70)
        set(yticks,'hor','left','vert','top')
    else
        set(yticks,'hor','right','vert','top')
    end
end

%% move y label:
if ~isempty(yticks)
    ylab=get(src,'ylabel');
    if az==0 && el==90
        labelxpos=xlim(1)-(5+max(ydim))*xunitsperpix;
        pos=[labelxpos mean(ylim) zlim(1)];
        set(ylab,'units','data','position',pos);
    else
        if az<-175
            set(ylab,'hor','center','rotation',90)
            xpos=xlim(1)-40*xunitsperpix;
        elseif (az<-92 && az>=-175)
            set(ylab,'hor','left','rotation',0)
            xpos=xlim(1)-20*xunitsperpix;
        elseif (az<-88 && az>=-92)
            set(ylab,'hor','center','rotation',0)
            xpos=xlim(1)-20*xunitsperpix;
        elseif (az>-88 && az<=-3)
            set(ylab,'hor','right','rotation',0)
            xpos=xlim(1)-20*xunitsperpix;
        elseif az>-3 && az<=0
            set(ylab,'hor','center','rotation',90)
            xpos=xlim(1)-40*xunitsperpix;
        elseif az<=87 && az>=0
            set(ylab,'hor','left','rotation',0)
            xpos=xlim(2)+20*xunitsperpix;
        elseif az<93 && az>87
            set(ylab,'hor','center','rotation',0)
            xpos=xlim(2)+20*xunitsperpix;
        elseif az>93 && az<175
            set(ylab,'hor','right','rotation',0)
            xpos=xlim(2)+20*xunitsperpix;
        elseif az>175
            set(ylab,'hor','center','rotation',90)
            xpos=xlim(2)+40*xunitsperpix;
        end
        
        pos=[xpos mean(ylim) zlim(1)-20*zunitsperpix];
        set(ylab,'units','data','position',single(pos));
    end

else
    %determine width of matlab ticks:
    labz=get(src,'yticklabel');
    if ~isempty(labz)
        w=ravecalculatestringlengthfast(labz,handles.fontmetrics.tick);
        
        maxw=max(w)+10;
        %convert pixels to data units:
        xr=xlim(2)-xlim(1);
        offset=xr/axpos(3)*maxw;
        
        pos=[xlim(1)-offset mean(ylim) zlim(1)];
        ylab=get(src,'ylabel');
        set(ylab,'units','data','position',pos);
    end
end

%% move z ticks:
if ~isempty(zticks)
    %estimate 3 pixel gap:
    if az>0 && az<=90
        xpos=xlim(1)-3*xunitsperpix;
        ypos=ylim(1)-3*yunitsperpix;
    elseif az>90
        xpos=xlim(2)+3*xunitsperpix;
        ypos=ylim(1)-3*yunitsperpix;
    elseif az<-90
        xpos=xlim(2)+3*xunitsperpix;
        ypos=ylim(2)+3*yunitsperpix;
    else
        xpos=xlim(1)-3*xunitsperpix;
        ypos=ylim(2)+3*yunitsperpix;
    end
    
    ticks=get(src,'ztick');
    %determine how many pixels are available for each label:
    set(0,'ShowHiddenHandles','on');
    zticks=ravefindobjinternal(src,'tag','zticklabel');
    set(0,'ShowHiddenHandle',state);
    
    for s=1:min(numel(zticks),numel(ticks));
        set(zticks(s),'units','data','position',[xpos,ypos,ticks(s)],'rotation',0,'hor','right','vert','middle','visible','on');
    end

    %hide ticks outside of xlim bounds:
    isout=ticks<zlim(1) | ticks>zlim(2);
    set(zticks(isout),'visible','off');
end

%% move z label:
zlab=get(src,'zlabel');
if ~isempty(zticks)
    
    zstrings=get(zticks,'string');
    w=ravecalculatestringlengthfast(zstrings,handles.fontmetrics.tick);
    maxw=(max(w)+10)*0.85; %slightly less than max(w) since we apply this offset in two dimensions, diagonal distance will be maxw*sqrt(2)

    if az>0 && az<=90
        xpos=xlim(1)-maxw*xunitsperpix;
        ypos=ylim(1)-maxw*yunitsperpix;
    elseif az>90
        xpos=xlim(2)+maxw*xunitsperpix;
        ypos=ylim(1)-maxw*yunitsperpix;
    elseif az<-90
        xpos=xlim(2)+maxw*xunitsperpix;
        ypos=ylim(2)+maxw*yunitsperpix;
    else
        xpos=xlim(1)-maxw*xunitsperpix;
        ypos=ylim(2)+maxw*yunitsperpix;
    end

    pos=[xpos ypos mean(zlim)];
    set(zlab,'units','data','position',pos,'rotation',90,'hor','center','ver','bottom');
else
    %determine width of matlab ticks:
    labz=get(src,'zticklabel');
    
    if ~isempty(labz)
        w=ravecalculatestringlengthfast(labz,handles.fontmetrics.tick);
        maxw=max(w)+10;
    else
        maxw=10;
    end
    
    %convert pixels to data units:
    xr=xlim(2)-xlim(1);
    offset=xr/axpos(3)*maxw;
    
    pos=[xlim(1)-offset mean(ylim) zlim(1)];
    zlab=get(src,'zlabel');
    set(zlab,'units','data','position',pos,'ver','bottom');
end

