function [xdim,ydim,zdim]=ravemovecustomticksrow(src,ev,alwaysrotate)
% A variation of ravemovecustomticks to account for the fact that axis
% labels on matrix plots should all line up.
%NOTE: this function only handles 2D axes. If a 3D matrix plot exists in
%the future it will need a custom arrangeobjects function.

xdim=0;
ydim=0;
zdim=0;

handles=guidata(src);

if nargin<3
    alwaysrotate=false;
end

%significantly faster than using findall:
state=get(0,'ShowHiddenHandles');
set(0,'ShowHiddenHandles','on');

bax=handles.subax{handles.activeindex}(end,:);
xdim=zeros(size(bax));
Xlim=cell(size(bax));
Ylim=cell(size(bax));
Xticks=cell(size(bax));
Axpos=cell(size(bax));

for c=1:numel(bax)
    src=bax(c);
    
    xlim=get(src,'xlim');
    ylim=get(src,'ylim');
    xticks=ravefindobjinternal(src,'tag','xticklabel');
    axpos=getpixelposition(src);
    
    Xlim{c}=xlim;
    Ylim{c}=ylim;
    Xticks{c}=xticks;
    Axpos{c}=axpos;

    yunitsperpix=(ylim(2)-ylim(1))./axpos(4);
    w=axpos(3);
    
    %% move x ticks:
    if ~isempty(xticks)
        %estimate 4 pixel gap:
        ypos=ylim(1)-4*yunitsperpix;
        
        %determine how many pixels are available for each label:
        
        ticks=get(src,'xtick');
        if numel(ticks)~=numel(xticks);return;end
        tickpix=interp1(xlim,[0 w],ticks);
        if numel(tickpix)>1
            divs=tickpix(1:end-1)+diff(tickpix)/2;
            overshoot=20; %number of pixels beyond width of axes that tick labels are allowed to extend.
            maxtickwidth=diff([-overshoot divs w+overshoot]);
            maxtickwidth(1)=maxtickwidth(2);
            maxtickwidth(end)=maxtickwidth(end-1);
        else %if only one tick label:
            maxtickwidth=w;
        end
        
        if alwaysrotate
            status=false(1,numel(xticks));
        else
            status=true(1,numel(xticks));
        end
        
        if alwaysrotate;status(:)=0;end
        xticks=ravefindobjinternal(src,'tag','xticklabel');

        str=get(xticks,'string');
        for s=numel(xticks):-1:1;
            
            if iscell(str{s})
                str{s}=cell2mat(str{s}(:)');
            end
            if all(status) %if no ticks need to be rotated, keep wrapping (if any are rotated, no need to wrap)
                %[wrapped{s},status(s),nrows(s)]=superiorlinewrap(str{s},maxtickwidth(s),handles.fonts.tick,handles.fonts.ticksize,'plain',handles.axiscanvas);
                [wrapped{s},status(s),nrows(s)]=superiorlinewrapfast(str{s},maxtickwidth(s),handles.fontmetrics.tick);
                set(xticks(s),'units','data','string',wrapped{s},'position',[ticks(s),ypos],'rotation',0,'hor','center','vert','top','visible','on');
            end
        end
        
        if any(status==0) %rotate ticks
            set(xticks,'rotation',90,'hor','right','ver','middle');
            for s=numel(xticks):-1:1
                set(xticks(s),'string',str{s},'units','data','position',[ticks(s),ypos],'visible','on')
            end
            [xdims,tickheight]=ravecalculatestringlengthfast(str,handles.fontmetrics.tick);
            xdim(c)=max(xdims);
            
            %check if there is still not enough room to clearly show the
            %ticks. If not, turn some of their visibilities off:
            avgwidth=mean(maxtickwidth);
            skip=ceil((tickheight-3)/avgwidth); %tick height is a little generous, so subtract 3 pixels...
            idx=1:skip:numel(xticks);
            if all(~isnan(idx)) %because this happens sometimes...
                set(xticks,'visible','off');
                set(xticks(idx),'visible','on');
            end
        else
            [junk,tickheight]=ravecalculatestringlengthfast('M',handles.fontmetrics.tick); %presumably M is the tallest letter.
            xdim(c)=tickheight*max(nrows);
        end
        
        %hide ticks outside of xlim bounds:
        isout=ticks<xlim(1)-0.01*(xlim(2)-xlim(1)) | ticks>xlim(2)+0.01*(xlim(2)-xlim(1));
        set(xticks(isout),'visible','off');
    end
end

xdim=max(xdim);

for c=1:numel(bax)
    src=bax(c);
    xlim=Xlim{c};
    ylim=Ylim{c};
    xticks=Xticks{c};
    axpos=Axpos{c};
    
    xunitsperpix=(xlim(2)-xlim(1))./axpos(3);
    yunitsperpix=(ylim(2)-ylim(1))./axpos(4);
    w=axpos(3);
    
    xlab=get(src,'xlabel');
    
    %% move xlabels:
    if ~isempty(xticks)
        %determine where to move xlabel:
        labelypos=ylim(1)-(5+xdim)*yunitsperpix; %additonal 3 pixel buffer
        pos=[mean(xlim) labelypos];
        set(xlab,'units','data','position',pos);
    else
        %guess height of matlab ticks:
        [junk,tickheight]=ravecalculatestringlengthfast('M',handles.fontmetrics.tick); %presumably M is the tallest letter.
        %convert pixels to data units:
        yr=ylim(2)-ylim(1);
        offset=yr/axpos(4)*tickheight;
        pos=[mean(xlim) ylim(1)-offset 1];
        set(xlab,'units','data','position',pos);
    end
end

%% move y ticks:

lax=handles.subax{handles.activeindex}(:,1);
ydim=zeros(size(bax));
Xlim=cell(size(bax));
Ylim=cell(size(bax));
Yticks=cell(size(bax));
Axpos=cell(size(bax));

for r=1:numel(lax)
    src=lax(r);

    xlim=get(src,'xlim');
    ylim=get(src,'ylim');
    yticks=ravefindobjinternal(src,'tag','yticklabel');
    axpos=getpixelposition(src);
    
    Xlim{r}=xlim;
    Ylim{r}=ylim;
    Yticks{r}=yticks;
    Axpos{r}=axpos;
    
    xunitsperpix=(xlim(2)-xlim(1))./axpos(3);
    yunitsperpix=(ylim(2)-ylim(1))./axpos(4);
    w=axpos(3);
    
    if ~isempty(yticks)
        %estimate 3 pixel gap:
        xpos=xlim(1)-4*xunitsperpix;
        ystr=get(yticks,'string');
        [ydims,tickheight]=ravecalculatestringlengthfast(ystr,handles.fontmetrics.tick);
        ydim(r)=max(ydims);
        ticks=get(src,'ytick');
        %determine how many pixels are available for each label:
        yticks=ravefindobjinternal(src,'tag','yticklabel');
        
        for s=1:min(numel(yticks),numel(ticks));
            set(yticks(s),'units','data','position',[xpos,ticks(s)],'rotation',0,'hor','right','vert','middle','visible','on');
        end
        
        %check if there is still not enough room to clearly show the
        %ticks. If not, turn some of their visibilities off:
        
        %determine how many pixels are available for each label:
        h=axpos(4);
        if numel(ticks)~=numel(yticks);return;end
        tickpix=interp1(ylim,[0 h],ticks);
        if numel(tickpix)>1
            divs=tickpix(1:end-1)+diff(tickpix)/2;
            overshoot=20; %number of pixels beyond width of axes that tick labels are allowed to extend.
            maxtickwidth=diff([-overshoot divs w+overshoot]);
            maxtickwidth(1)=maxtickwidth(2);
            maxtickwidth(end)=maxtickwidth(end-1);
        else %if only one tick label:
            maxtickwidth=h;
        end
        avgwidth=mean(maxtickwidth);
        skip=ceil((tickheight-3)/avgwidth); %tick height is a little generous, so subtract 3 pixels...
        idx=1:skip:numel(yticks);
        if all(~isnan(idx)) %because this happens sometimes...
            set(yticks,'visible','off');
            set(yticks(idx),'visible','on');
        end
        
        %hide ticks outside of xlim bounds:
        isout=ticks<ylim(1) | ticks>ylim(2);
        set(yticks(isout),'visible','off');
    end
end

ydim=max(ydim);

for r=1:numel(lax)
    src=lax(r);
    xlim=Xlim{r};
    ylim=Ylim{r};
    yticks=Yticks{r};
    axpos=Axpos{r};
    xunitsperpix=(xlim(2)-xlim(1))./axpos(3);
    
    %% move y label:
    if ~isempty(yticks)
        ylab=get(src,'ylabel');
        
        labelxpos=xlim(1)-(5+ydim)*xunitsperpix;
        pos=[labelxpos mean(ylim)];
        set(ylab,'units','data','position',pos);
    else
        %determine width of matlab ticks:
        labz=get(src,'yticklabel');
        if ~isempty(labz)
            w=ravecalculatestringlengthfast(labz,handles.fontmetrics.tick);
            maxw=max(w)+5;
            %convert pixels to data units:
            xr=xlim(2)-xlim(1);
            offset=xr/axpos(3)*maxw;
            
            pos=[xlim(1)-offset mean(ylim)];
            ylab=get(src,'ylabel');
            set(ylab,'units','data','position',pos);
        end
    end
end

set(0,'ShowHiddenHandles',state);
