function ravenamegraphgui(src,ev,ai)
%name and tag the current graph
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);

if nargin<3
    ai=handles.activeindex;
end

if isempty(ai);return;end

set(0,'userdata',[]);

screensize=get(handles.fig,'position');

if isempty(handles.graphnames{ai})
    name=ravenamegraph(handles.fig,[],ai);
else
    name=handles.graphnames{ai};
end

if isempty(handles.graphtags{ai})
    tag='';
else
    tag=handles.graphtags{ai};
end

askbox=figure('units','pixels','position',[screensize(1)+screensize(3)/2-250 screensize(2)+screensize(4)/2-125 500 250],...
    'numbertitle','off','menubar','none','name','Name Selected Graph','color',handles.colors.frame,...
    'resize','off','windowstyle','modal');
uicontrol('style','text','backgroundcolor',handles.colors.frame,'units','pixels',...
    'position',[20 160 460 90],...
    'string','You don''t need to name every workspace object; in fact, you will hardly ever need to name them. Rave uses objects'' names when it needs to identify a particular object to you. The name should be descriptive enough to uniquely identify the object. Tags are not used by Rave''s built-in functions, but you can use them to identify specific workspace objects in functions that you write for use within Rave. Tags should be short unique identifiers that you can easily refer to in your functions.',...
    'horizontalalignment','left')
uicontrol('style','text','backgroundcolor',handles.colors.frame,'units','pixels',...
    'position',[20 130 225 20],...
    'string','Enter a name for the selected graph: ','horizontalalignment','left')
uicontrol('style','edit','string',name,'backgroundcolor',[1 1 1],'units','pixels',...
    'position',[30 110 440 22],'tag','namebox','horizontalalignment','left')
uicontrol('style','text','backgroundcolor',handles.colors.frame,'units','pixels',...
    'position',[20 80 225 20],...
    'string','Enter a tag for the selected graph: ','horizontalalignment','left')
uicontrol('style','edit','string',tag,'backgroundcolor',[1 1 1],'units','pixels',...
    'position',[30 60 440 22],'tag','tagbox','horizontalalignment','left')

uicontrol('style','pushbutton','string','Done','parent',askbox,'units','pixels',...
    'position',[270 10 100 30],'callback',@putawaypopup_callback,'fontsize',10);
uicontrol('style','pushbutton','string','Cancel','parent',askbox,'units','pixels',...
    'position',[380 10 100 30],'callback',@putawaypopup_callback,'fontsize',10);

uiwait(askbox)

data=get(0,'userdata');

if isempty(data)
    return %user hit cancel
end

handles.graphnames{ai}=data.name;
handles.graphtags{ai}=data.tag;
guidata(handles.fig,handles);


function putawaypopup_callback(src,ev)

popup=get(src,'parent');
if strcmp(get(src,'string'),'Cancel');
    delete(popup);
    return
end

namebox=findobj(popup,'tag','namebox');
tagbox=findobj(popup,'tag','tagbox');
graphname=get(namebox,'string');
graphtag=get(tagbox,'string');
data.name=graphname;
data.tag=graphtag;
set(0,'userdata',data);
delete(popup);






