function ravenewgraph(src,eventdata,newgraphtype)  %#ok<INUSL>
%optional 3rd argument is string of graph type to create (overrides gui)
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

raveupdategraphtypes(src,[]);
handles=guidata(src);

clear raveplacefixedgraph %clear persistents

if handles.tablemode;return;end %cant make graphs while in table mode.

if isempty(handles.datasetnames);
    set(handles.infobar,'string','You must load a data set before making workspace objects. Click the "Manage Data Sets" button.')
    return
end

if nargin==2
    set(0,'userdata',get(handles.fig,'position'));
    handy=ravenewgraphgui(handles.fig);
    uiwait(handy);
    %get plot type:
    handles.temp.newgraphtype=get(0,'userdata');
    set(0,'userdata',[]); 
else
    handles.temp.newgraphtype=newgraphtype;
    set(0,'userdata',[]); 
end

figure(handles.fig); %to activate keypressfcn
uicontrol(handles.dummyobj);

%prompt for placement:
if strcmp(handles.temp.newgraphtype,'cancel');
    %do nothing
elseif strcmp(handles.graphinfo.(handles.temp.newgraphtype).axesformat,'fixedsize')
    %make nonresizable graph:
    buttons=findobj(handles.fig,'type','uicontrol','enable','on');
    set(buttons,'enable','inactive'); 
    set(handles.infobar,'string','Click to place this object; Hit ESC to cancel.')
    set(handles.fig,'windowkeypressfcn',{@ravecancelgraph,buttons})
    set(handles.fig,'windowbuttonmotionfcn',{@raveplacefixedgraph,handles,buttons})
    set(handles.mainaxes,'buttondownfcn',@ravesizegraphdone)
    set(handles.explorer,'buttondownfcn',@ravesizegraphdone)

else
    buttons=findobj(handles.fig,'type','uicontrol','enable','on');
    set(buttons,'enable','inactive');
    set(handles.infobar,'string','Click and drag to place this object; Hit ESC to cancel.')
    set(handles.fig,'windowkeypressfcn',{@ravecancelgraph,buttons})
    set(handles.mainaxes,'buttondownfcn',{@raveplacegraph,handles,buttons})
    %set(handles.explorer,'buttondownfcn',{@raveplacegraph,handles})
end

guidata(src,handles)

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function raveplacefixedgraph(src,ev,handles,buttons)
handles=guidata(src);

persistent putgraphhere
if isempty(putgraphhere) || ~isfield(handles.temp,'putgraphhere') %not sure why but the isempty check doesnt always work here...
    
    if strcmp(handles.graphinfo.(handles.temp.newgraphtype).class,'control');
        edgecolor=[0 0.5 1];
    elseif strcmp(handles.graphinfo.(handles.temp.newgraphtype).class,'annotation');
        edgecolor=[0.7 0 1];
    elseif strcmp(handles.graphinfo.(handles.temp.newgraphtype).class,'table');
        edgecolor=[0.7 0.9 0.1];
    else
        edgecolor=[1 0.5 0];
    end
    
    putgraphhere(1)=rectangle('position',[0 0 1 1],'linewidth',2,'edgecolor',edgecolor,...
        'parent',handles.mainaxes,'hittest','off','buttondownfcn','','facecolor',handles.colors.wsbg);
    putgraphhere(2)=rectangle('position',[0 0 1 1],'linewidth',1,'edgecolor',edgecolor,...
        'parent',handles.explorer,'hittest','off','buttondownfcn','');
    
    if handles.ishg2
       set(putgraphhere,'alignvertexcenters','on') 
    end
    
    handles.temp.putgraphhere=putgraphhere;
    guidata(src,handles);
end

if isempty(handles.temp.newgraphtype);return;end

set(handles.fig,'windowbuttondownfcn',[]);
set(handles.fig,'windowkeypressfcn',@ravedefaultkeypressfcn);

buttons=findobj(handles.fig,'type','uicontrol');
set(buttons,'enable','on');

%determine if cursor is in mainaxes or exploreraxes

cax=hittest(handles.fig);

%check that they clicked on exploreraxes or mainaxes:
if ~any([cax==handles.explorer,cax==handles.mainaxes])
    return
end

clicked=get(cax,'currentpoint');

xclick=clicked(1,1,1);
yclick=clicked(1,2,1);

set(handles.infobar,'string',['x=',num2str(xclick),' y=',num2str(yclick)]);

%round down to nearest grid box:
xclick=handles.snapsize*floor(xclick/handles.snapsize);
yclick=handles.snapsize*floor(yclick/handles.snapsize);

handles.temp.startx=xclick;
handles.temp.starty=yclick;

%deterimine graph dims:
xdim=handles.graphinfo.(handles.temp.newgraphtype).minwidth;
ydim=handles.graphinfo.(handles.temp.newgraphtype).minheight;

set(handles.temp.putgraphhere,'position',[xclick yclick xdim ydim]);

guidata(src,handles);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function raveplacegraph(src,eventdata,handles,buttons) 
if isempty(handles.temp.newgraphtype);return;end

set(handles.allblockers(ishandle(handles.allblockers)),'hittest','off');

set(handles.fig,'windowbuttondownfcn',[]);
set(handles.fig,'windowkeypressfcn',@ravedefaultkeypressfcn);

set(buttons,'enable','on');

%check that they clicked on exploreraxes or mainaxes:
if ~any([gco==handles.explorer,gco==handles.mainaxes])
    return
end

clicked=get(gco,'currentpoint');

xclick=clicked(1,1,1);
yclick=clicked(1,2,1);
handles.temp.startx=xclick;
handles.temp.starty=yclick;

xclick=handles.snapsize*floor(xclick/handles.snapsize);
yclick=handles.snapsize*floor(yclick/handles.snapsize);

if strcmp(handles.graphinfo.(handles.temp.newgraphtype).class,'control');
    edgecolor=[0 0.5 1];
elseif strcmp(handles.graphinfo.(handles.temp.newgraphtype).class,'annotation');
    edgecolor=[0.7 0 1];
elseif strcmp(handles.graphinfo.(handles.temp.newgraphtype).class,'table');
    edgecolor=[0.7 0.9 0.1];
else
    edgecolor=[1 0.5 0];
end

handles.temp.putgraphhere(1)=rectangle('position',[xclick yclick handles.prefs.gridsize handles.prefs.gridsize],'linewidth',2,'edgecolor',edgecolor,...
    'parent',handles.mainaxes,'hittest','on','buttondownfcn','','facecolor',handles.colors.wsbg);
handles.temp.putgraphhere(2)=rectangle('position',[xclick yclick handles.prefs.gridsize handles.prefs.gridsize],'linewidth',1,'edgecolor',edgecolor,...
    'parent',handles.explorer,'hittest','on','buttondownfcn','');

if handles.ishg2
   set(handles.temp.putgraphhere,'alignvertexcenters','on') 
end

%allow dragging
set(handles.fig,'WindowButtonMotionFcn',@ravesizegraph);
set(handles.fig,'WindowButtonUpFcn',@ravesizegraphdone);
set(handles.fig,'windowbuttondownfcn',[]);

guidata(src,handles);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ravesizegraph(src,eventdata) %#ok<INUSD>
handles=guidata(src);

finish=get(gco,'currentpoint');
starty=handles.temp.starty;
startx=handles.temp.startx;
endx=finish(1,1,1);
endy=finish(1,2,1);

pos1=handles.snapsize*floor(min(startx,endx)/handles.snapsize);
pos3=handles.snapsize*ceil(max(startx,endx)/handles.snapsize)-pos1;
pos2=handles.snapsize*floor(min(starty,endy)/handles.snapsize);
pos4=handles.snapsize*ceil(max(starty,endy)/handles.snapsize)-pos2;

widthinches=sprintf('%6.2f',round(pos3/96*4)/4);
heightinches=sprintf('%6.2f',round(pos4/96*4)/4);

set(handles.temp.putgraphhere,'position',[pos1 pos2 pos3 pos4]);

set(handles.infobar,'string',['Width=',widthinches,'" Height=',heightinches,'"  at 96 dpi'])

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ravesizegraphdone(src,eventdata,buttons) %#ok<INUSD>
handles=guidata(src);
%stop dragging
set(handles.fig,'WindowButtonMotionFcn',@ravedefaultwindowbuttonmotionfcn);
set(handles.fig,'WindowButtonUpFcn',[]);
set(handles.allblockers(ishandle(handles.allblockers)),'hittest','on');

position=get(handles.temp.putgraphhere(1),'position');
ai = numel(handles.allgraphs)+1;
type=handles.temp.newgraphtype;

handles.allblockers(ai)=handles.temp.putgraphhere(1);
handles.expblockers(ai)=handles.temp.putgraphhere(2);

%clear the temp stuff
handles.temp=[];
guidata(handles.fig,handles);

%Make the graph:

handles=ravemakegraph(src,ai,type,handles.activesheet,position);

%make this graph active:
% try %gives an error if the graph that was created is somehow invalid, so that newaxes doesnt exist. 
%     raveselectgraph(newaxes,[])
% end

guidata(handles.fig,handles);
%display view tab:
if ~isempty(handles.activegraph) %avoid error if graph didn't create successfully
    viewtab(handles.fig,[]);
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function ravecancelgraph(src,ev,varargin)
if ~strcmpi(ev.Key,'escape');return;end

handles=guidata(src);

buttons=findobj(handles.fig,'type','uicontrol','enable','inactive');
    set(buttons,'enable','on');

set(0,'userdata','cancel') %in case anyone needs to be notified of the cancellation
set(handles.infobar,'string','')
set(handles.mainaxes,'buttondownfcn',@ravedragview)
set(handles.explorer,'buttondownfcn',@ravemoveview)
set(handles.fig,'windowkeypressfcn',@ravedefaultkeypressfcn)

if strcmp(get(handles.fig,'waitstatus'),'waiting')
    uiresume(handles.fig)  
end





