function f=ravenewgraphgui(varargin)
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(varargin{1});

%make figure:
f=figure('units','pixels','position',[0 0 900 700],'name',...
    'Create New Workspace Object','numbertitle','off','menubar','none','resize','off','windowstyle','normal',...
    'windowkeypressfcn',@keycap);

handles.gui=f;
origin=get(0,'userdata');
set(0,'userdata','cancel');
thispos=get(f,'position');
thispos(1)=origin(1)+origin(3)/2-thispos(3)/2;
thispos(2)=thispos(2)+origin(2)+origin(4)/2-thispos(4)/2;
set(f,'position',thispos);

%make tabs:

%get theme
% load handles.prefs
% handles.prefs=handles.prefs;
set(f,'color',handles.prefs.colors.frame)

%make initial uicontrols:
handles.background=axes('units','pixels','position',[58 55 250 600],'visible','off','xlimmode','manual','ylimmode','manual');
handles.tabaxes=axes('units','pixels','position',[28 55 30 600],'parent',handles.gui,'visible','off','tag','tabaxes');  
handles.titleaxes=axes('units','pixels','position',[11 660 485 41],'parent',handles.gui,'visible','off','tag','titleaxes');

handles.previewpane=uipanel('units','pixels','position',[330 55 540 600],'backgroundcolor',handles.prefs.colors.tabbg);
handles.previewname=axes('parent',handles.previewpane,'units','pixels','position',[20 555 500 40],'visible','off');
handles.nametext=text(0,0.5,'Line Plot','parent',handles.previewname,'fontname',handles.fonts.tab,'fontsize',handles.fonts.tabsize,'color',handles.prefs.colors.text);
 handles.previewtext=uicontrol('parent',handles.previewpane,'style','text',...
 'string','Discrete graphs are based on the data in the data set. They can be created for any data set.',...
     'position',[20 455 500 105],'backgroundcolor',handles.prefs.colors.tabbg,'fontsize',10,'hor','left');

% handles.previewtextaxes=axes('parent',handles.previewpane,'units','pixels','position',[20 425 500 133],'visible','off');
% handles.previewtext=text(0,0.5,'','parent',handles.previewtextaxes,...
%     'backgroundcolor',handles.prefs.colors.tabbg,'fontsize',10,'hor','left',...
%     'fontsmoothing','off');

handles.sampleaxes=axes('parent',handles.previewpane,'units','pixels','position',[5 5 525 450],'visible','off','ydir','reverse','color',[1 0 1]);
handles.nosample=text(.5,.5,{'No Sample Image';'Available'},'parent',handles.sampleaxes,...
    'fontname',handles.fonts.tab,'fontsize',handles.fonts.tabsize,'color',handles.prefs.colors.text,...
    'hor','center','ver','middle','visible','off');


[x,y,v]=calculatetabprofile([1 1.2 .75 .75 1.3 .9],.1,1,1); 
handles.tabbg=patch(x,y,handles.prefs.colors.tabbg,'parent',handles.tabaxes,'edgecolor','none');
handles.bg=patch([0 1 1 0],[0 0 1 1],handles.prefs.colors.tabbg,'parent',handles.background,'edgecolor','none');

handles.tablines=line(x,y,'parent',handles.tabaxes,'color',handles.prefs.colors.tabfg);

handles.discrete=text(.2,v(1),' Discrete','parent',handles.tabaxes,'rotation',90,'fontname',handles.prefs.fonts.tab,'fontsize',0.8*handles.prefs.fonts.tabsize,...
    'color',handles.prefs.colors.text,'vert','cap','buttondownfcn',@discrete_callback);
handles.continuous=text(.2,v(2),' Continuous','parent',handles.tabaxes,'rotation',90,'fontname',handles.prefs.fonts.tab,'fontsize',0.8*handles.prefs.fonts.tabsize,...
    'color',handles.prefs.colors.text,'vert','cap','buttondownfcn',@continuous_callback);
handles.special=text(.2,v(3),' Special','parent',handles.tabaxes,'rotation',90,'fontname',handles.prefs.fonts.tab,'fontsize',0.8*handles.prefs.fonts.tabsize,...
    'color',handles.prefs.colors.text,'vert','cap','buttondownfcn',@special_callback);
handles.tables=text(.2,v(4),' Tables','parent',handles.tabaxes,'rotation',90,'fontname',handles.prefs.fonts.tab,'fontsize',0.8*handles.prefs.fonts.tabsize,...
    'color',handles.prefs.colors.text,'vert','cap','buttondownfcn',@tables_callback);
handles.annotations=text(.2,v(5),' Annotations','parent',handles.tabaxes,'rotation',90,'fontname',handles.prefs.fonts.tab,'fontsize',0.8*handles.prefs.fonts.tabsize,...
    'color',handles.prefs.colors.text,'vert','cap','buttondownfcn',@annotations_callback);
handles.controls=text(.2,v(6),' Controls','parent',handles.tabaxes,'rotation',90,'fontname',handles.prefs.fonts.tab,'fontsize',0.8*handles.prefs.fonts.tabsize,...
    'color',handles.prefs.colors.text,'vert','cap','buttondownfcn',@controls_callback);

handles.tabs=[handles.discrete handles.continuous handles.special handles.tables, handles.annotations, handles.controls];

line([0 1 1 0],[0 0 1 1],'color',handles.prefs.colors.tabfg,'parent',handles.background);


handles.classtext=uicontrol('parent',f,'style','text','string','Discrete graphs are based on the data in the data set. They can be created for any data set.',...
    'position',[64 580 240 70],'backgroundcolor',[1 1 1],'fontsize',10,'hor','left');

handles.selecttext=uicontrol('parent',f,'style','text','string','Select a Graph Type:',...
    'position',[64 550 240 20],'backgroundcolor',[1 1 1],'fontsize',10,'hor','left','fontweight','bold');

handles.graphmenu=uicontrol('parent',f','style','listbox','string','Loading...',...
    'position',[74 250 220 290],'callback',@graphmenu_callback,'fontsize',10,'backgroundcolor',[1 1 1]);


handles.donebutton=uicontrol('style','pushbutton','units','pixels','position',[265 11 156 35],'tag','donebutton',...
    'fontsize',14,'string','Create','callback',@donebutton_callback);
handles.cancelbutton=uicontrol('style','pushbutton','units','pixels','position',[438 11 156 35],'tag','donebutton',...
        'fontsize',14,'string','Cancel','callback',@cancelbutton_callback);
    
%make titles
set(handles.titleaxes,'color','none','xcolor',handles.prefs.colors.frame,...
    'ycolor',handles.prefs.colors.frame,'zcolor',handles.prefs.colors.frame);

text(0,.2,'Create a new workspace object','parent',handles.titleaxes,'fontname',handles.fonts.title,'fontsize',handles.fonts.titlesize,...
   'color',handles.prefs.colors.title,'horizontalalignment','left','verticalalignment','baseline');

handles.makethistype='';
handles.tabobjects=[];

anyfun=0;
for d=1:numel(handles.isfunction)
if any(handles.isfunction{d});anyfun=1;end
end
if ~anyfun;set(handles.continuous,'color',handles.colors.tabfg);end

% Update handles structure
guidata(f, handles);

%load discrete tab:
discrete_callback(f,[])

function discrete_callback(src,ev)
%display uicontrols for defining column roles (input or response)
handles=guidata(src);
set(handles.classtext,'string','Discrete graphs are based on the data in the data set. They can be created for any data set.');
set(handles.donebutton,'enable','on')

getgraphtypes(src,'discrete');
handles=guidata(src); %refresh handles.

graphmenu_callback(src,ev)
 
[x,y,v]=calculatetabprofile([1 1.2 .75 .75 1.3 .9],.1,1,1); 
set(handles.tablines,'xdata',x,'ydata',y);

function continuous_callback(src,ev)
%display uicontrols for defining column roles (input or response)
handles=guidata(src);


anyfun=0;
for d=1:numel(handles.isfunction)
if any(handles.isfunction{d});anyfun=1;end
end
if ~anyfun;set(handles.donebutton,'enable','off');end


set(handles.classtext,'string','Continuous graphs show slices of the data space around a point of interest. They require a data set with one or more functional models.');

getgraphtypes(src,'continuous');
handles=guidata(src); %refresh handles.

graphmenu_callback(src,ev)

[x,y,v]=calculatetabprofile([1 1.2 .75 .75 1.3 .9],.1,2,1); 
set(handles.tablines,'xdata',x,'ydata',y);

function special_callback(src,ev)
%display uicontrols for defining column roles (input or response)
handles=guidata(src);
set(handles.classtext,'string','Special graphs are assorted graphs that fulfill a specific purpose. They may require certain types of data.')
set(handles.donebutton,'enable','on')

getgraphtypes(src,'special');
handles=guidata(src); %refresh handles.

graphmenu_callback(src,ev)
[x,y,v]=calculatetabprofile([1 1.2 .75 .75 1.3 .9],.1,3,1); 
set(handles.tablines,'xdata',x,'ydata',y);


function tables_callback(src,ev)
%display uicontrols for defining column roles (input or response)
handles=guidata(src);
set(handles.classtext,'string','Tables display data related to one or more points in the data set.')
set(handles.donebutton,'enable','on')

getgraphtypes(src,'table');
handles=guidata(src); %refresh handles.

graphmenu_callback(src,ev)
[x,y,v]=calculatetabprofile([1 1.2 .75 .75 1.3 .9],.1,4,1); 
set(handles.tablines,'xdata',x,'ydata',y);

function annotations_callback(src,ev)
%display uicontrols for defining column roles (input or response)
handles=guidata(src);
set(handles.classtext,'string','Annotations are used to display additional information on the workspace.')
set(handles.donebutton,'enable','on')

getgraphtypes(src,'annotation');
handles=guidata(src); %refresh handles.

graphmenu_callback(src,ev)

[x,y,v]=calculatetabprofile([1 1.2 .75 .75 1.3 .9],.1,5,1); 
set(handles.tablines,'xdata',x,'ydata',y);

function controls_callback(src,ev)
%display uicontrols for defining column roles (input or response)
handles=guidata(src);
set(handles.classtext,'string','Controls can be placed on the workspace to create interactive displays.')
set(handles.donebutton,'enable','on')

getgraphtypes(src,'control');
handles=guidata(src); %refresh handles.

graphmenu_callback(src,ev)
[x,y,v]=calculatetabprofile([1 1.2 .75 .75 1.3 .9],.1,6,1); 
set(handles.tablines,'xdata',x,'ydata',y);



function cancelbutton_callback(src,ev)
delete(gcbf);



function graphmenu_callback(src,ev)
handles=guidata(src);
src=handles.graphmenu; %because sometimes it isnt...

idx=get(src,'value');
set(handles.nametext,'string',handles.longnames{idx});
set(handles.previewtext,'string',handles.graphinfo{idx});
delete(findobj(handles.sampleaxes,'type','image'));
if ~isempty(handles.samplefile{idx})
    
    set(handles.nosample,'visible','off');
    thisimg=imread(handles.samplefile{idx});
    
    axpos = get(handles.sampleaxes,'position');
    dims=size(thisimg);
    
    xmargin = floor((axpos(3)-dims(2))/2);
    ymargin = floor((axpos(4) - dims(1))/2);
    
    xpos = [xmargin xmargin+dims(2)];
    ypos = [ymargin ymargin+dims(1)];
    
    image('cdata',thisimg,'parent',handles.sampleaxes,'xdata',xpos,'ydata',ypos)

    set(handles.sampleaxes,'xlim',[-0.5 axpos(3)+0.5],'ylim',[-0.5 axpos(4)+0.5]);
else
   
    set(handles.sampleaxes,'xlim',[0 1],'ylim',[0 1]);
    set(handles.nosample,'visible','on');
end

handles.makethistype=handles.keywords{idx};
guidata(src,handles);

function donebutton_callback(src,ev)
handles=guidata(src);
set(0,'userdata',handles.makethistype);
close(gcbf);

function getgraphtypes(src,types)
%types = 'discrete','continuous',...
handles=guidata(src);

raveroot=which('rave');
raveroot=raveroot(1:end-6);
maindir=fullfile(raveroot,'graphics',types);
graphtypes=dir(maindir);

longnames=cell(0);
for g=3:numel(graphtypes)
    h=g-2;
    if graphtypes(g).isdir && graphtypes(g).name(1)~='X'
        thisname=graphtypes(g).name;
        %remove leading numbers:
        firstletter=find(isstrprop(thisname,'alpha'),1);
        keywords{h}=thisname(firstletter:end);
        %get graphinfo:
        fid=fopen(fullfile(maindir,thisname,'graphinfo.txt'),'r');  
%         try
        longnames{h}=fgetl(fid);
%         catch
%             continue
%         end
        longinfo='';
        while 1
            tline=fgetl(fid);
            if ~ischar(tline);break;end
            longinfo = [longinfo,' ',tline];
        end
        fclose(fid);
        graphinfo{h}=longinfo;
        
        if exist(fullfile(maindir,thisname,'sample.png'),'file');
           samplefile{h}=fullfile(maindir,thisname,'sample.png');
        else
            samplefile{h}='';
        end
    end
end


if isempty(longnames)
    errordlg('No valid graphs of this type were found in your RAVE directory.');
    return
end
handles.keywords=keywords;
handles.longnames=longnames;
handles.graphinfo=graphinfo;
handles.samplefile=samplefile;

set(handles.graphmenu,'string',longnames,'value',1)

guidata(src,handles);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function keycap(src,ev)
try %try because i'm not sure if older versions use same format for eventdata:
    if strcmp(ev.Key,'escape')
        close(gcbf);
    end
end


