function [c,ceq]=ravenonlcon(x,handles,settings,index,mins,ranges)
%A generic function for evaluting nonlinear constraints in optimizers.
%Follows the required format for Optimization Toolbox functions.

% x = point at which to evaluate constraints
% src = a handle to use with guidata
% settings = optimization settings structure
% index = index of columns of x into columns of datasets.alldata
% mins = minimum bound of x variables
% maxs = maximum bound of x variables

%are there any constraints?
if numel(settings.activeconstraints)==1 && settings.activeconstraints==0
    c=0;
    ceq=0;
    return
end

%denormalize x:
x=bsxfun(@times,x,ranges);
x=bsxfun(@plus,x,mins);

%deal columns of x to correct data set columns:
% xdata=zeros(1,size(datasets.alldata{settings.d},2));
% xdata(settings.funvars(1:end-1))=x; %remove the t from the end


%Assess constraints:
iqs=settings.activeconstraints(ismember(handles.constrainttype{settings.d}{settings.a}(settings.activeconstraints),'inequality'));
eqs=settings.activeconstraints(ismember(handles.constrainttype{settings.d}{settings.a}(settings.activeconstraints),'equality'));

niq=numel(iqs);
neq=numel(eqs);

c=zeros(niq,1);
ceq=zeros(neq,1);

X=zeros(1,size(handles.funnames{settings.d},2));
X(index)=x;

for k=1:niq
    c(k)=raveevaluateconstraint(handles,settings.d,settings.a,iqs(k),X,false(1,size(X,2)));
end

for k=1:neq
    ceq(k)=raveevaluateconstraint(handles,settings.d,settings.a,eqs(k),X,false(1,size(X,2)));
end


