function raveopensavedfig(varargin)
%This function is used to open saved rave fig files. It is very similar
%to raveopenstate, but leaves out parts that deal with the rave interface.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu



%add rave java stuff to path:
if isempty(which('RaveTableSorter'))
    javaaddpath(which('RaveJavaClasses.jar'));
end

if nargin==1
    %only one input, was double clicked in windows.
    fullname=varargin{1}; 
else
    %two inputs
    src=varargin{1};
    eventdata=varargin{2};
    [name,path,index]=uigetfile({'*.rve','RAVE Project (*.rve)';'*.*','All Files (*.*)'},'Choose project to load:');
    if name==0;return;end
    fullname=fullfile(path,name);
end

[pathstr, name, ext] = fileparts(fullname); 
copyfile(fullname,fullfile(pathstr,['RAVETEMP' name '.fig']),'f')

%warning('off','MATLAB:dispatcher:UnresolvedFunctionHandle');

handy=openfig(fullfile(pathstr,['RAVETEMP' name '.fig']),'new','invisible');
%  
% 
%  handy=openfig(fullfile(pathstr,['RAVETEMP' name '.fig']));
%  set(handy,'name','Histograms')
%  saveas(handy,'tempfig.fig');
%  return

set(handy,'doublebuffer','on') %not sure why but this sometimes turns off.

warning('on','MATLAB:dispatcher:UnresolvedFunctionHandle');
delete(fullfile(pathstr,['RAVETEMP' name '.fig']))
handles=guidata(handy);


%replace timer:
handles.mousetimer=timer('TimerFcn',[],'startdelay',handles.prefs.mouseoverdelay,'executionmode','singleshot');

%fix text units, which for some reason get set to normalized:
ttt=findall(handles.fig,'type','text');
set(ttt,'units','data');

guidata(handles.fig,handles);

msgstate=warning('query','MATLAB:hg:uicontrol:ParameterValuesMustBeValid');
warning('off','MATLAB:hg:uicontrol:ParameterValuesMustBeValid');


%replace axeslisteners... evidently they are not serializable:
% for g=1:numel(handles.axeslisteners)
%     handles.axeslisteners{g}=[];
% end
guidata(handles.fig,handles);

warning(msgstate.state,'MATLAB:hg:uicontrol:ParameterValuesMustBeValid');

%drawnow;
%re-javacomponent the popupmenus on any graphaxes controls. Not sure why
%this needs to be done, but for some reason saving/opening the figure
%resets their parent to []. Javacomponent makes a new
%com.mathworks.hg.HPPanel for their parent:

ag=handles.activegraph;
ai=handles.activeindex;
at=handles.activetype;


for g=1:numel(handles.allgraphs)
    if ~ishandle(handles.allgraphs(g));continue;end
    if strcmp(get(handles.allgraphs(g),'userdata'),'graphaxes')
        %udata=get(handles.fakeax(g),'userdata');
        handles.activegraph=handles.allgraphs(g);
        handles.activeindex=g;
        handles.activetype='graphaxes';
        
        guidata(handles.fig,handles);        
        ravecreategraphaxes(handles.fig,[]);
        handles=guidata(handles.fig);
    end  
end


for g=1:numel(handles.allgraphs)
    if ~ishandle(handles.allgraphs(g));continue;end
        %reposition axis labels, which for some reason reset to their default
    %positions when opening the figure:
    handles.activegraph=handles.allgraphs(g);
    handles.activeindex=g;
    handles.activetype=get(handles.activegraph,'userdata');
    guidata(handles.fig,handles); 
    if strcmp(handles.activetype,'histogram')
        ravearrangeobjects(handles.activegraph,[]);
        set(handles.subax{g},'units','normalized')
        set(handles.subfakeax{g},'units','normalized')
        set(handles.subfakerax{g},'units','normalized')
    elseif ismember(handles.activetype,{'scatter','contour','carpet2d','planeprofiler'})
        ravemovecustomticks(handles.allgraphs(g),[],0);
    end
   
    
end


handles.activegraph=ag;
handles.activeindex=ai;
handles.activetype=at;

guidata(handles.fig,handles);

set(handy,'visible','on');






