function raveopenstate(varargin)
%
%This function is used to open saved rave projects when they are
%doubleclicked in windows explorer.
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

%clear any persistent stuff that may be hanging around when it shouldnt be:
clear functions


handles=struct();
datainfo=struct();
graphinfo=struct();

%update rave path in case new folders were added since this file was saved:
raveroot=which('rave');
raveroot=raveroot(1:end-6);
addpath(genpath(raveroot))

%add rave java stuff to path:
if isempty(which('RaveTableSorter'))
    javaaddpath(which('RaveJavaClasses.jar'));
    javaaddpath(which('jlatexmath-1.0.2.jar'));
end

if nargin==1
    %only one input, file was double clicked in operating system.
    fullname=varargin{1};
else
    %two inputs, file was opened from rave load button
    src=varargin{1};
    eventdata=varargin{2};
    [name,path,index]=uigetfile({'*.rve','RAVE Project (*.rve)';'*.*','All Files (*.*)'},'Choose project to load:');
    if name==0;return;end
    fullname=fullfile(path,name);
end

[pathstr, name, ext] = fileparts(fullname);
copyfile(fullname,fullfile(pwd,['RAVETEMP' name '.m']),'f')
rehash;

%Create a new Rave session:
fig = rave('-nosplash');
set(fig,'windowbuttonmotionfcn','')
%Import saved handles into rave session:
handles = guidata(fig);




set(handles.infobar,'string','\color[rgb]{1 0 0}Opening saved state... please wait...');
drawnow;

%Load saved state (this automatically merges with the new state's handles
%structure:
eval(['RAVETEMP' name]);
delete(fullfile(pwd,['RAVETEMP' name '.m']));

%Update handles:
guidata(fig,handles);
src=fig;

%Merge saved preferences with local preferences:
load raveprefs
handles.prefs.defaultdir = raveprefs.defaultdir;


% plzwait=helpdlg('Loaing saved Rave session, please wait...','Loading...');


%Delete mouseover handles, they should not have been saved in the first place (bug in 0.9.11)
handles.mouseoverhandles=[];
guidata(handles.fig,handles);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Make data sets from datainfo, or if datainfo does not exist, make them by
%parsing files in handles.datasetfiles

if ~isempty(datainfo)
    for D=1:numel(datainfo) %for each data set found in the file:
        %make data set
        datasetname=handles.datasetnames{D};
        datasource=handles.datasettype{D};
        infile=handles.datasetfiles{D};
        varnames=handles.allnames{D};
        data=datainfo(D).alldata;
        discretevalues=datainfo(D).alltextvalues;
        datatype=datainfo(D).datatype;
        
        %Note: Can't call ravemakedataset here because we don't want to reinitialize all the fields in handles. Use the "basic" version that just
        %recreates the "raveguidata" variables.
        ravemakedatasetBASIC(handles.fig,D,infile,datasetname,datasource,varnames,data,discretevalues,datatype);
        
        handles=guidata(handles.fig);
        
        pause(0.1); %attempt to avoid java error
        guidata(src,handles);
        guidata(handles.fig,handles)
        
        %create tableModel:
        [model,rhmodel]=ravecreatemaintablemodel(handles.fig,[],D,1);
        handles.maintable.models{D}=model;
        handles.maintable.rhmodels{D}=rhmodel;
        handles.maintable.d=D;
        handles.maintable.a=1;
        handles.maintable.x{D}=1:numel(handles.allnames{D});
        drawnow;
        guidata(src,handles);
    end
else %parse file list in handles.datafiles
    %Parse for rave data:
    infile=get(sourcefiletext,'string');
    [varnames,data,textdata,discretevalues,datatype,datasetnames,datasource]=raveparse2dataset(handles,infile,get(newnametext,'string'),true);
end



%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Recreate constraint annonymous functions:
handles.constrainthandles = cell(0);

%store this function annonymously in handles.
for d = 1:numel(handles.datasetnames)
    handles.constrainthandles{d} = cell(0);
    for a = 1:numel(handles.analysisnames)
        handles.constrainthandles{d}{a} = cell(0);
        for i = 1:numel(handles.constrainttext{d}{a})
            %Parse constrainttext to get string, equality, and value:
            tokens = regexp(handles.constrainttext{d}{a}{i},'([^<>=]+)([<>=]+)([^<>=]+)','tokens');
            constraintstring = strtrim(tokens{1}{1});
            thisequality = strtrim(tokens{1}{2});
            constraintvalue = str2double(tokens{1}{3});
            
            [constraintfunctionstring,constrainttype,constraintvars]=raveprocessconstraint(handles,d,constraintstring,thisequality,constraintvalue);
            
            handles.constrainthandles{d}{a}{i}=eval(['@(data) ',constraintfunctionstring,';']);
            
            
            
        end
    end
end



try
    newhandle=eval(['@(data) ',constraintfunctionstring,';']);
catch %#ok<CTCH>
    errordlg('Something went wrong while creating a constraint. Check to see if you spelled everything right and used correct MATLAB syntax.')
end




%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%Recreate all workspace objects:
if ~isempty(fieldnames(graphinfo))
    for i = 1:numel(graphinfo)
        type=graphinfo(i).graphtype;
        udata=graphinfo(i).udata;
        page=handles.sheet(i);
        ai=graphinfo(i).ai;
        position=graphinfo(i).position;
        
        %Was this graph deleted? if so go to the next one.
        if isempty(graphinfo(i).ai)
            continue
        end
        
        %Does this object type exist in this version of Rave?
        if isempty(which(['raveinitialize',type,'.m']))
            warndlg('The workspace object type "',type,'" could not be found in your Rave installation. This may be because you are using an outdated version of Rave, or the indicated object type may not be part of the standard Rave distribution. Objects of this type will be missing in the loaded project state. WARNING: If you overwrite this .rve file by saving the loaded project state, the object will be permantently deleted.');
            continue
        end
        
        thissheet=handles.sheet(ai);
        
        %validate udata:
        handles.activeindex=ai;
        guidata(handles.fig,handles);
        udata=ravevalidateudata(handles.fig,udata,type);
        
        %Make graph: (need to re-set handles.activeindex so raveselectgraph
        %doesnt think this graph already exists:
        
        if ai>1
            handles.activeindex=ai-1;
        else
            handles.activeindex=[];
        end
        guidata(handles.fig,handles);
        
        ravemakegraph(handles.fig,ai,type,page,position,udata);
        
        %If this object is on a different page, send it to the boneyard:
        if thissheet~=1
            ravesendgraphtoboneyard(src,ai)
        end
        
        handles=guidata(handles.fig);
        
        %restore to appropriate sheet:
        %         handles.sheet(ai)=thissheet;
        
    end
end

%Turn on hittests of tabs, which are off by default in a new Rave session (normally they get turned on after data is loaded:
set(handles.alltabs,'hittest','on');

%Do this last:
set(handles.fig,'windowbuttonmotionfcn',@ravedefaultwindowbuttonmotionfcn)
set(handles.infobar,'string','Project loaded successfully.');

%refresh managetab so all uicontrols become active:
managetab(handles.fig,[]);


function ravemakedatasetBASIC(src,D,infile,datasetname,datasettype,varnames,data,discretevalues,datatype)
%Similar to ravemakedataset but doesn't reset all the things in the imported handles structure.

handles=guidata(src);

%store backupdata:
backupdata=getappdata(handles.fig,'backupdata');
backupdata{D}=data;
setappdata(handles.fig,'backupdata',backupdata);

%store data:
datasets=raveguidata(handles.fig,'datasets');
datasets.datatype{D} = datatype;
datasets.alldata{D} = data;
datasets.isvalid{D}=true(size(data));
datasets.isvirtual{D} = false(1,size(data,2));
datasets.alltextvalues{D} = discretevalues;
raveguidata(handles.fig,'datasets',datasets);

vectordata=raveguidata(handles.fig,'vectordata');
vectordata.vectordata{D}=cell(size(data));
raveguidata(handles.fig,'vectordata',vectordata);



function ravesendgraphtoboneyard(src,ai)
%Moves a workspace object to the boneyard
%(Copied from ravedisplaypage.m)

%% hide last page:
handles=guidata(src);

stackorder=get(handles.boneyard,'children');


udata=get(handles.fakeax(ai),'userdata'); %#ok this line may be used by the eval statements!
%hide main axes:
allparents=[handles.allgraphs(ai);handles.fakeax(ai);handles.fakerax(ai);...
    handles.subax{ai}(:);handles.subfakeax{ai}(:);handles.subfakerax{ai}(:);...
    ];

%hide legends:
legdata=get(handles.fakerax(ai),'userdata');
if isempty(legdata)
    legs=[];
else
    legs=findobj(legdata(ishandle(legdata)),'parent',handles.wspanel);
end

%move other graph-specific objects:
graphtype=get(handles.allgraphs(ai),'userdata');

if isfield(handles.graphinfo.(graphtype),'deletehandles')
    deletehandles=handles.graphinfo.(graphtype).deletehandles;
    for f=1:numel(deletehandles)
        thesehandles=eval([deletehandles{f},';']);
        allparents=[allparents;thesehandles(:)];  %#ok<AGROW>
    end
end

allparents=allparents(ishandle(allparents)); %yes, this happens

%for items whose parent is mainaxes, just set visibility off:
allmainaxes=findobj(allparents,'parent',handles.mainaxes,'-depth',1);
set(allmainaxes,'visible','off');

%java objects also need visibility turned off, since uipanels only
%affect positioning of java objects:
alljava=findobj(allparents,'type','hgjavacomponent');
set(alljava,'visible','off')

%Handles to move to the boneyard:
allparents=findobj(allparents,'parent',handles.wspanel,'-not','type','hgjavacomponent','-depth',1);

stackorder=[legs(:);stackorder;allparents];
set(handles.allblockers(ai),'visible','off')
set(handles.expblockers(ai),'visible','off')
set(handles.grabbers{ai},'visible','off')

%outkidz=get(handles.boneyard,'children');
set(stackorder,'parent',handles.boneyard);
set(handles.boneyard,'children',stackorder);
