function [varnames,data,textdata,discretevalues,datatype,datasetnames,datasource]=raveparse2dataset(handles,infile,newname,dofast)
%Determines the type of data set represented by "infile" and calls the
%appropriate parsing function.
%newname is the suggested name for the data set, which may be
%overwritten/appended/ignored by the individual parsing functions


if isempty(infile)
    infile='-pastespecial';
end

%check if this is a variable name:
if ~any(infile==filesep) %it is a variable name or database name
    if ~isempty(which('getdatasources'))
        sources=getdatasources;
        if ismember(infile,sources)
            datasource='database';
        else
            datasource='variable';
        end
    else
        datasource='variable';
    end
else
    [path,name,ext]=fileparts(infile);
    switch lower(ext)
        case {'.txt','.csv'}
            datasource='plaintext';
        case {'.xls','.xlsx'}
            datasource='excel';
        case {'.bif','.oobn','.xml','.dsc'} %read as bayesian belief network
            datasource='bbn';
        otherwise
            datasource='plaintext';
    end
end

%If not creating a data set from a file, do whatever preprocessing is
%needed to make the "in" structure:
switch datasource
    case 'plaintext'
        [varnames,data,textdata,discretevalues,datatype]=raveparsetxt2dataset(handles,infile,dofast);
        datasetnames={newname};
    case 'excel';
        [varnames,data,textdata,discretevalues,datatype,datasetnames]=raveparseexcel2dataset(handles,infile,dofast);
        namestr=newname;
        if ~isempty(namestr)
            for g=1:numel(datasetnames)
                datasetnames{g}=[namestr,'.',datasetnames{g}];
            end
        end
    case 'bbn'
        [varnames,data,textdata,discretevalues,datatype]=raveparsebbn2dataset(handles,infile,dofast);
        datasetnames={newname};
    case 'variable'
        [varnames,data,textdata,discretevalues,datatype]=raveparsevariables2dataset(handles,infile,dofast);
        datasetnames={newname};
    case 'database'
        [varnames,data,textdata,discretevalues,datatype,datasetnames]=raveparsedatabase2dataset(handles,infile,dofast);
end