function [varnames,data,textdata,discretevalues,datatype,datasetnames]=raveparseexcel2dataset(handles,infile,dofast)
%parse the data file "infile" (a string) and return variable names and
%data. Infile can also be a string name of a variable in the base
%workspace. "in" is a structure with fields columnheaders and textdata,
%both of which are cell arrays of strings. If infile is not a file, you must manually
%create this structure and include it as an input to this function
%Infobar is any matlab object with a string property that can be used to
%display status updates

infobar=handles.infobar;
set(infobar,'string','Preparing to import data...');drawnow;

%load data from each sheet:
try %Attempt to avoid xlsread, which is unnecessarily slow but works on computers without excel
    set(infobar,'string','Opening Excel workbook...');drawnow;
    excel=actxserver('excel.application');
    excelworkbook = excel.workbooks.Open(infile,0,true);
    worksheets = excelworkbook.sheets;
    sheetnames=cell(1,worksheets.Count);
    Rawdata=cell(1,numel(sheetnames));
    for d=1:worksheets.Count
        thissheet=get(worksheets,'item',d);
        sheetnames{d}=thissheet.Name;
        set(infobar,'string',['Importing worksheet ',sheetnames{d},' (',num2str(d),'/',num2str(worksheets.Count),')']);drawnow;
        set(thissheet,'Visible','xlSheetVisible');
        Activate(thissheet);
        Activate(Range(excel,'A1'));
        datarange = excel.ActiveSheet.UsedRange;
        Rawdata{d} = datarange.Value;
    end
    excelworkbook.Close(false);
    excel.Quit;
catch %#ok<CTCH>
    set(infobar,'string','Getting Excel workbook info...');drawnow;
    [fileType, openAction, loadAction, sheetnames] = finfo(infile);
    for d=1:numel(sheetnames)
        set(infobar,'string',['Importing worksheet ',sheetnames{d},' (',num2str(d),'/',num2str(numel(sheetnames)),')']);drawnow;
        [num,txt,Rawdata{d}]=xlsread(infile,sheetnames{d});
    end
end

set(infobar,'string','Preprocessing Excel data...');drawnow;
for d=1:numel(sheetnames)
    rawdata=Rawdata{d};
    
    if ~iscell(rawdata);continue;end %ignore empty worksheets
    
    colheaders=rawdata(1,:);
    rawdata=rawdata(2:end,:);
    
    %make sure column headers are non-empty:
    isnans=cellfun(@cellisnan,colheaders);
    if any(isnans)
        idx=find(isnans);
        names=makecolumnnames(max(idx));
        colheaders(isnans)=names(idx);
    end
    
    %maker sure column headers are text:
    isnumbers=cellfun(@isnumeric,colheaders);
    colheaders(isnumbers)=cellfun(@num2str,colheaders(isnumbers),'uniformoutput',false);
    
    %remove empty columns:
    empties=cellfun(@cellisnan,rawdata); %anything empty should be NaN
    if all(empties(:));continue;end %entire sheet is blank
    emptycols=all(empties,1);
    colheaders=colheaders(~emptycols);
    data=rawdata(:,~emptycols);
    
    if isempty(data);continue;end
    
    if numel(colheaders)~=size(data,2)
        errordlg(['The selected Excel file could not be loaded because the number of column headers on sheet "',sheetnames{d},'" does not match the number of columns of data.']);
        return
    end
    
    if size(data,1)<2
        errordlg(['The selected Excel file could not be loaded because sheet "',sheetnames{d},'" contains fewer than 2 rows of data.']);
        return
    end
    
    in(d).colheaders=colheaders;
    in(d).textdata=data;
end

%Parse structure:
datasetnames=sheetnames;
[varnames,data,textdata,discretevalues,datatype]=raveparsedatastructure(handles,in);

if isempty(in)
    errordlg('The selected file does not contain a valid data set.');
    return
end


function is=cellisnan(c) %determine if anything in a cell isnan
is=isnumeric(c) && isnan(c);


function names=makecolumnnames(n)
names=cell(1,n);
for i=1:n
    names{i}=['Column ',num2str(i)];
end
