function [varnames,data,textdata,discretevalues,datatype,datasetnames]=raveparsetxt2dataset(handles,infile,dofast)
%parse the data file "infile" (a string) and return variable names and
%data. Infile can also be a string name of a variable in the base
%workspace. "in" is a structure with fields columnheaders and textdata,
%both of which are cell arrays of strings. If infile is not a file, you must manually
%create this structure and include it as an input to this function
%Infobar is any matlab object with a string property that can be used to
%display status updates

infobar=handles.infobar;

datasetnames={};

set(infobar,'string','Preparing to import data...');drawnow;

fid=fopen(infile,'r');
hline=fgetl(fid); %headers
dline=fgetl(fid); %first line of data
fclose all;

%look for delimiters:
if any(dline==char(9))
    delim=char(9);
elseif any(dline==',')
    delim=',';
else
    delim=' ';
end

numhdelim=nnz(hline==delim);
numddelim=nnz(dline==delim);

if dline(end)==delim
    numddelim=numddelim-1;
end

if hline(end)==delim
    numhdelim=numhdelim-1;
end

if numhdelim~=numddelim
    errordlg('The selected data set could not be loaded because the number of column headers does not match the number of columns of data.');
end

if dofast
    
    set(handles.infobar,'string','Loading data...');
    drawnow;
    %scan first line of file:
    formatstring=strtrim(repmat('%s ',1,numddelim+1));
    
    varnames = textscan(hline,formatstring,'delimiter',delim,'EmptyValue',NaN,'MultipleDelimsAsOne',true);
    varnames=[varnames{:}];
    
    alldata=textscan(dline,formatstring,'delimiter',delim,'EmptyValue',NaN,'MultipleDelimsAsOne',true);
    
    %See which of these are numbers:
    isnumber = ~isnan(str2double([alldata{:}]));
    numbercols = find(isnumber);
    formatstring(3*numbercols-1)='f';
    
    fid=fopen(infile,'r');
    alldata=textscan(fid,formatstring,'delimiter',delim,'EmptyValue',NaN,'headerlines',1,'MultipleDelimsAsOne',true);
    fclose all;
    
    data = zeros(size(alldata{1},1),numel(alldata));
    if ~isempty(numbercols)
        data(:,numbercols)=[alldata{numbercols}];
    end
    
    discretevalues=cell(1,numel(alldata));
    datatype=cell(1,numel(alldata));
    
    for i = 1:numel(isnumber)
        if ~isnumber(i)
            [discretevalues{i},ia,data(:,i)] = unique(alldata{i});
            datatype{i}='string';
        else
            discretevalues{i}=[];
            datatype{i}='continuous';
        end
    end
    data={data};
    textdata={[]}; 
    datatype={datatype};
    varnames={varnames};
    discretevalues={discretevalues};

else
    %make textscan string:
    formatstring=strtrim(repmat('%s ',1,numddelim+1));
    fid=fopen(infile,'r');
    alldata=textscan(fid,formatstring,'delimiter',delim,'EmptyValue',NaN);
    fclose all;
    
    alldata=[alldata{:}];
    
    if size(alldata,1)<3
        errordlg('The selected data set could not be loaded because it contains fewer than 2 rows of data.');
    end
    
    in.colheaders=alldata(1,:);
    in.textdata=alldata(2:end,:);
    %Parse structure:
    [varnames,data,textdata,discretevalues,datatype]=raveparsedatastructure(handles,in);
end

