function savewindow=ravepopoutgraph(src,position,ai,includelegends,inset)
%Copies the graph in ai to a new figure, suitable for saving as an image.
%If includelegends=false, only the graph itself is copied (things within
%the extent of allblockers)
% inset is a 4-element inset vector of margins to add around each side of
% the figure.
%
% The figure's visibility is "off" initially.
%
% If ai is a vector, exports all the indicated graphs into a single figure.
%
% returns the handle of the figure created.

handles=guidata(src);

color=get(handles.mainaxes,'color');

if ishandle(handles.savewindow) %reuse existing figure if it exists... this saves time when rasterizing.
    savewindow=handles.savewindow;
    delete(get(savewindow,'children'));
    set(savewindow,'position',[1 1 position(3) position(4)],'color',color','colormap',get(handles.fig,'colormap'));
else %make a new savewindow.
    savewindow=figure('name','Saving... DO NOT CLOSE THIS WINDOW','units','pixels','position',[1 50 position(3) position(4)],...
        'color',color,'colormap',get(handles.fig,'colormap'),'visible','off',...
        'inverthardcopy','off','numbertitle','off','integerhandle','off','handlevisibility','off');
end

wspos=get(handles.wspanel,'position');

panel=uipanel('units','pixels','parent',savewindow,'position',[1 1 wspos(3) wspos(4)],'backgroundcolor',color,'bordertype','none');


graphpos=get(handles.allblockers(ai),'position');
if numel(ai)==1
    graphpos={graphpos}; %make into a cell if it's not already.
end

offset=cell(numel(ai),1);
for g=1:numel(ai)
    offset{g}=graphpos{g}-position;
end

%reset graph vectors for newhandles:
newhandles=handles;
newhandles.activeindex=1;
newhandles.activegraph=[];
newhandles.allgraphs=[];
newhandles.fakeax=[];
newhandles.fakerax=[];
newhandles.subax={};
newhandles.subfakeax={};
newhandles.subfakerax={};
newhandles.allblockers=[];
newhandles.expblockers=[];

allorigax=[];
allnewax=[];

for f=1:numel(ai)
    udata=get(handles.fakeax(ai(f)),'userdata');
    %compare figure size to the size you told it to be. for large graphs
    %figure may be smaller.
    
    %     ratio=get(savewindow,'position')./position;
    %     ratio=min(ratio(3:4));
    
    %copy graphs to new window. Only copy objects whose parent is wspanel. Other objects presumably have a parent that is also in this list,
    %so we don't want to double copy them:
    
    origax=[];
    newax=[];
    
    
    if get(handles.fakerax(ai(f)),'parent')==handles.wspanel
        SfakerAX=copyobj(handles.fakerax(ai(f)),panel);
        origax = [origax;handles.fakerax(ai(f))];
        newax = [newax;SfakerAX(:)];
    end
    
    if get(handles.fakeax(ai(f)),'parent')==handles.wspanel
        SfakeAX=copyobj(handles.fakeax(ai(f)),panel);
        origax = [origax;handles.fakeax(ai(f))];
        newax = [newax;SfakeAX(:)];
    end
    
    if get(handles.allgraphs(ai(f)),'parent')==handles.wspanel
        SAX=copyobj(handles.allgraphs(ai(f)),panel);
        origax = [origax;handles.allgraphs(ai(f))];
        newax = [newax;SAX(:)];
    end
    
    SsubfakerAX=copyobj(handles.subfakerax{ai(f)},panel); %NOTE: need to make ALL graphs expand these arrays or this will give error for reference greater than index
    origax = [origax;handles.subfakerax{ai(f)}(:)];
    newax = [newax;SsubfakerAX(:)];
    
    SsubfakeAX=copyobj(handles.subfakeax{ai(f)},panel);
    origax = [origax;handles.subfakeax{ai(f)}(:)];
    newax = [newax;SsubfakeAX(:)];
    
    
    SsubAX=copyobj(handles.subax{ai(f)},panel);
    origax = [origax;handles.subax{ai(f)}(:)];
    newax = [newax;SsubAX(:)];
    
    if includelegends
        legdata=get(handles.fakerax(ai(f)),'userdata');
        legdata=legdata(ishandle(legdata));
        Slegends=copyobj(legdata,panel);
    else
        legdata=[];
        Slegends=[];
    end
    
    %copy other graph-specific objects:
    type=get(handles.allgraphs(ai(f)),'userdata');
    if isfield(handles.graphinfo.(type),'deletehandles')
        deletehandles=handles.graphinfo.(type).deletehandles;
        for g=1:numel(deletehandles)
            thesehandles=eval(deletehandles{g});
            pares = get(thesehandles,'parent');
            if iscell(pares);pares=[pares{:}];end
            thesehandles = thesehandles(ishandle(thesehandles) & pares==handles.wspanel) ;
            copyobj(thesehandles,panel);
        end
    end
    
    %copy explore methods objects:
    if isfield(udata,'explore');
        exps=fieldnames(udata.explore);
        exps=setdiff(exps,'method');
        for i=1:numel(exps)
            if isfield(udata.explore.(exps{i}),'deletehandles')
                thesehandles=udata.explore.(exps{i}).deletehandles;
                copyobj(thesehandles(ishandle(thesehandles)),panel);
            end
        end
    end
    
    %fix axis label positioning: (for some reason copyobj resets axis label
    %positions)
    %     origax=[handles.fakerax(ai(f));handles.fakeax(ai(f));handles.allgraphs(ai(f));handles.subfakerax{ai(f)}(:);handles.subfakeax{ai(f)}(:);handles.subax{ai(f)}(:);legdata(:)];
    %     newax=[SfakerAX;SfakeAX;SAX;SsubfakerAX(:);SsubfakeAX(:);SsubAX(:);Slegends(:)];
    
    %exclude any of these that are not axes (e.g. they could be uipanels)
    typez=get(newax,'type');
    idx=strcmp(typez,'axes');
    
    if any(idx)
    origxlabelz=get(origax(idx),'xlabel');
    origylabelz=get(origax(idx),'ylabel');
    origzlabelz=get(origax(idx),'zlabel');
    
    newxlabelz=get(newax(idx),'xlabel');
    newylabelz=get(newax(idx),'ylabel');
    newzlabelz=get(newax(idx),'zlabel');
    
    
    origposses=get([origxlabelz{:},origylabelz{:},origzlabelz{:}],'position');
    set([newxlabelz{:},newylabelz{:},newzlabelz{:}],{'position'},origposses)
    end
    
    %save handles for use below...
    allorigax=[allorigax;origax]; %#ok<AGROW>
    allnewax=[allnewax;newax]; %#ok<AGROW>
end

% origin=get(handles.allblockers(ai),'position');
% if iscell(origin);origin=cell2mat(origin);end
% origin=min(origin,[],1);


%if necessary, enlarge window to encompass legends:
% if enlargetofit

%positions of all axes being exported:
axposses=get(allnewax,'position');
if iscell(axposses);axposses=cell2mat(axposses);end

%positions of all blockers of graphs being exported;
blockerposses=get(handles.allblockers(ai),'position');
if iscell(blockerposses);blockerposses=cell2mat(blockerposses);end

cvpos = get(handles.currentview,'position');
blockerposses = bsxfun(@minus,blockerposses,[cvpos(1) cvpos(2) 0 0]);


posses=[axposses;blockerposses];

%check bottom left corner:
leftmost=min(posses(:,1));
bottommost=min(posses(:,2));

%check upper right corner:
rightmost=max(posses(:,1)+posses(:,3));
topmost=max(posses(:,2)+posses(:,4));

%Move all children so their origin is shifted by leftmost, bottommost:
childs = get(panel,'children');
for i = 1:numel(childs)
    pos=get(childs(i),'position');
    set(childs(i),'position',pos+[-leftmost+inset(1) -bottommost+inset(2) 0 0]);
end

set([panel savewindow],'position',[1 1 rightmost-leftmost+inset(1)+inset(3) topmost-bottommost+inset(2)+inset(4)]);

% end

%determine stack order: some graphs may put fakeax in front of real ax
%(e.g. parallelcoords does this). Check how the axes are stacked inthe
%original Rave window and restack them that way in the new window:
allkidz=get(handles.wspanel,'children');
idx=ismember(allkidz,allorigax);
subkidz=allkidz(idx); %the handles from the main rave window, in the correct stack order


%Delete copied objects whose original parent wasnt handles.wspanel. These objects parent is presumably also in the list of copied objects, so we don't
%want 2 copies of them:
% pares = get(allorigax,'parent');
% keeperz = [pares{:}]==handles.wspanel;
% delete(allnewax(~keeperz));
% allnewax = allnewax(keeperz);
%


%make newidx vector so that allorigax(newidx)==subkidz
newidx=zeros(size(subkidz));
for q=1:numel(subkidz); %im sure there's a non-loop way to do this, but why bother...
    newidx(q)=find(allorigax==subkidz(q));
end

%now apply newidx order to allnewax:
allnewax=allnewax(newidx);
allkidz=get(panel,'children');
idx=ismember(allkidz,allnewax);
allkidz(idx)=allnewax;
set(panel,'children',allkidz);







