function raveputaway(src,ev)
%put away the current graph.
%See also: raveputawaynoprompt
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);

set(0,'userdata',[]);

if isempty(handles.activeindex);return;end

screensize=get(handles.fig,'position');

askbox=figure('units','pixels','position',[screensize(1)+screensize(3)/2-250 screensize(2)+screensize(4)/2-75 500 150],...
    'numbertitle','off','menubar','none','name','Put Away Selected Graph','color',handles.colors.frame,...
    'resize','off','windowstyle','modal');
uicontrol('style','text','backgroundcolor',handles.colors.frame,'units','pixels',...
    'position',[20 80 225 40],...
    'string','Enter a name for the selected graph: ','horizontalalignment','left')
uicontrol('style','edit','string',ravenamegraph(handles.fig,[]),'backgroundcolor',[1 1 1],'units','pixels',...
    'position',[30 70 440 22],'tag','namebox','horizontalalignment','left')
uicontrol('style','pushbutton','string','Put Away','parent',askbox,'units','pixels',...
    'position',[270 10 100 30],'callback',@putawaypopup_callback,'fontsize',10);
uicontrol('style','pushbutton','string','Cancel','parent',askbox,'units','pixels',...
    'position',[380 10 100 30],'callback',@putawaypopup_callback,'fontsize',10);

uiwait(askbox)

name=get(0,'userdata');

if isempty(name)
    return %user hit cancel
end

%else put away this graph:
ai=handles.activeindex;

udata=get(handles.fakeax(ai),'userdata'); %#ok this line may be used by the eval statements!
%hide main axes:
allparents=[handles.allgraphs(ai);handles.fakeax(ai);handles.fakerax(ai);...
    handles.subax{ai}(:);handles.subfakeax{ai}(:);handles.subfakerax{ai}(:)];

%hide legends:
legdata=get(handles.fakerax(ai),'userdata');
if isempty(legdata)
    legs=[];
else
    legs=findobj(legdata(ishandle(legdata)),'parent',handles.wspanel);
end

%move other graph-specific objects:
graphtype=get(handles.allgraphs(ai),'userdata');
if isfield(handles.graphinfo.(graphtype),'deletehandles')
    deletehandles=handles.graphinfo.(graphtype).deletehandles;
    for f=1:numel(deletehandles)
        thesehandles=eval([deletehandles{f},';']);
        allparents=[allparents;thesehandles(:)];  %#ok<AGROW>
    end
end

allparents=findobj(allparents,'parent',handles.wspanel,'-depth',1);

lastkids=get(handles.boneyard,'children');
set(allparents,'parent',handles.boneyard);
set(legs,'parent',handles.boneyard);
set(handles.boneyard,'children',[lastkids;legs;allparents]);
set(handles.allblockers(ai),'visible','off')
set(handles.expblockers(ai),'visible','off')
set(handles.grabbers{ai},'visible','off')

handles.graphnames{handles.activeindex}=name;
handles.putaway=union(handles.putaway,handles.activeindex);

%make another graph the active graph:
newgraph=[];
for s=1:length(handles.allgraphs)
    if ishandle(handles.allgraphs(s)) && ~ismember(s,handles.putaway) %this graph hasnt been deleted and isnt put away
        newgraph=handles.allgraphs(s);
        break
    end
end

handles.activeindex=[];
handles.activegraph=[];
guidata(src,handles);

if ~isempty(newgraph) %graphs exist
    raveselectgraph(newgraph,[]) %make this graph current.
end


%Enable restore button:
restorebutton=findobj(handles.tabobjects,'tag','restorebutton');
if ~isempty(restorebutton)
    jb=get(restorebutton,'JavaPeer');
    jb.setEnabled(true);
end

function putawaypopup_callback(src,ev)

popup=get(src,'parent');
if strcmp(get(src,'string'),'Cancel');
    delete(popup);
    return
end

namebox=findobj(popup,'tag','namebox');
graphname=get(namebox,'string');
set(0,'userdata',graphname);
delete(popup);






