function ravequitfunction(varargin)
%this is the closerequestfunction for rave. It runs when you try to close
%rave.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu


handles=guidata(varargin{1});

%set preferred position to current position:
prefpath=which('raveprefs.mat');
load raveprefs
raveprefs.preferredposition=get(handles.fig,'position');
jf=findjobj(handles.fig,'class','FigureFrame');
if ~isempty(jf)
    raveprefs.ismaximized=jf.getExtendedState;
end
save(prefpath,'raveprefs');

if ~isempty(handles.datasetnames) %dont bother prompting if it's an empty rave session...
    
    %set(0,'userdata','cancel');
    origin=get(handles.fig,'position');
    
    askbox=figure('units','pixels','position',[origin(1)+origin(3)/2-350/2 origin(2)+origin(4)/2-150/2 350 150],...
        'numbertitle','off','menubar','none','name','Close RAVE','color',handles.colors.frame,...
        'resize','off','windowstyle','modal','windowbuttonmotionfcn',@quitmotion);
    
    uicontrol('style','text','backgroundcolor',handles.colors.frame,'units','pixels',...
        'position',[0 50 350 40],'string','Are you sure you want to quit RAVE?','horizontalalignment','center','fontsize',10)
    
    uicontrol('style','text','backgroundcolor',handles.colors.frame,'units','pixels',...
        'position',[00 125 350 20],'string',{'Please send comments, bugs, and suggestions to:'},'hor','center','fontsize',8)
    
    
    uicontrol('style','text','units','pixels',...
        'position',[120 105 120 16],'string',{'mattdaskil@gatech.edu'},'hor','center','fontsize',8,...
        'foregroundcolor',[0 0 1],'buttondownfcn',@mailmatt,'backgroundcolor',handles.colors.frame,...
        'hittest','on','enable','inactive','tag','emailbutton')
    
    uicontrol('style','pushbutton','string','Quit','parent',askbox,'units','pixels',...
        'position',[130 10 100 30],'callback',@ravequitpopup_callback,'fontsize',10);
    u=uicontrol('style','pushbutton','string','Cancel','parent',askbox,'units','pixels',...
        'position',[240 10 100 30],'callback',@ravequitpopup_callback,'fontsize',10);
    
    uiwait(askbox)
    
    if ishandle(askbox)
        answer=get(get(askbox,'currentobject'),'string');
        delete(askbox);
    else
        answer='';
    end
    
    % cancel if they pick no or cancel
    if ~strcmp(answer,'Quit')
        return
    end
end

%if quitting:
%check for unsaved regressions, prompt to save or delete these.

%shut it down... shut it down.
shh= get(0,'showhiddenhandles');
set(0,'showhiddenhandles','on');
try
    pause(0.05) %need a pause or a drawnow or something or it'll crash when it tries to delete trayicon.
    
    sysTray = java.awt.SystemTray.getSystemTray;
    if (sysTray.isSupported)
        java.awt.SystemTray.getSystemTray.remove(handles.trayIcon);
    end
    
    %delete javacomponents first since they tend to have deletefunctions:
    f=findobj(handles.fig,'type','hgjavacomponent');
    delete(f);
    drawnow;
    delete(handles.fig);
catch
    %shut it down... shut it down.
    delete(handles.fig);
end

set(0,'showhiddenhandles',shh);

try
    stop(handles.mousetimer);
end

try
    delete(handles.mousetimer);
end

function ravequitpopup_callback(src,ev) %#ok<INUSD>

% set(0,'userdata',get(src,'string'));
% close(gcbf);

uiresume(ancestor(src,'figure'));

function mailmatt(src,ev)
web(['mailto:mattdaskil@gatech.edu?subject=RAVE_ver',ravever])

function quitmotion(src,ev)

obj=hittest(src);

if strcmp(get(obj,'tag'),'emailbutton')
    set(src,'pointer','hand')
else
    set(src,'pointer','default')
end





