function ravereordermodelqueue(src,ev,order)
%reorder or delete rows from the modelqueue.

handles=guidata(src);
jtable=handles.temp.jscroll.getComponent(0).getComponent(0);
selection=jtable.getSelectedRows+1;

if isempty(selection);return;end

switch order
    case 'top'
        nummod=numel(handles.modelqueue);
        idx=[selection' setdiff(1:nummod,selection)]; 
        handles.modelqueue=handles.modelqueue(idx);
        
        
    case 'bottom'
        nummod=numel(handles.modelqueue);
        idx=[setdiff(1:nummod,selection) selection']; 
        handles.modelqueue=handles.modelqueue(idx);
        
        
    case 'delete'
        answer=questdlg('Are you sure you want to delete the selected models from the queue?','Delete models');
        if ~strcmpi(answer,'yes');return;end
        handles.modelqueue(selection)=[];
        
    case 'disable'
        for s=1:numel(selection)
            if strcmp(handles.modelqueue{selection(s)}.status,'Waiting')
                handles.modelqueue{selection(s)}.status='Disabled';
            end
        end
    case 'enable'
        for s=1:numel(selection)
            if strcmp(handles.modelqueue{selection(s)}.status,'Disabled')
                handles.modelqueue{selection(s)}.status='Waiting';
            end
        end

end


guidata(gcf,handles);
guidata(handles.fig,handles);

%update queue table:
%drawnow; %All of these drawnows are to avoid a severe java crash whose cause I dont know. 
dmod=makequeuetablemodel(src,ev);
drawnow;
jtable.setModel(dmod);
drawnow;

% Set column widths:
jtable.getTableHeader().setReorderingAllowed(false) %turn off draggable columns
jtable.setAutoResizeMode(javax.swing.JTable.AUTO_RESIZE_OFF)%turn off auto resizing
colwidths=[200 200 550 100]; %some column widths
for c=1:4 %set the widths:
    jtable.getColumnModel().getColumn(c-1).setPreferredWidth(colwidths(c))
end
drawnow;

function dmod=makequeuetablemodel(src,ev)
%NOTE, THIS FUNCTION IS DUPLICATED FROM ravemakemodelgui. Make sure to
%synchronize changes between these two files.

handles=guidata(src);

nmodels=numel(handles.modelqueue);

if nmodels<1
    javarray = {'','','',''};
    rowstatus=[];
else
    javarray=javaArray('java.lang.String',nmodels,4);
    rowstatus=javaArray('java.lang.Integer',nmodels);
    for r=1:nmodels
        d=handles.modelqueue{r}.dataset;
        javarray(r,1)=java.lang.String(handles.modelqueue{r}.longname);
        javarray(r,2)=java.lang.String(handles.allnames{d}{handles.modelqueue{r}.output});
        javarray(r,3)=java.lang.String(handles.modelqueue{r}.notes);
        javarray(r,4)=java.lang.String(handles.modelqueue{r}.status);
        switch handles.modelqueue{r}.status
            case 'Waiting'
                rowstatus(r)=java.lang.Integer(4); 
            case 'Creating...'
                rowstatus(r)=java.lang.Integer(2); 
            case 'Error'
                rowstatus(r)=java.lang.Integer(1);  
            case 'Finished'      
                rowstatus(r)=java.lang.Integer(3);
            case 'Disabled'
                rowstatus(r)=java.lang.Integer(5);
            otherwise
                rowstatus(r)=java.lang.Integer(0);
        end
    end
end
cnames = {'Model Type','Variable','Notes','Status'};

%dmod=javax.swing.table.DefaultTableModel(javarray,cnames);
dmod=RaveDataTypeTableModel(javarray,cnames);
javaObjectEDT(dmod);
if ~isempty(rowstatus)
    dmod.setRowStatus(rowstatus);
end


