function raveresizeaxes(src,ev,targetindex)
%RAVE: resize the current target axes to fit inside their bounding box,
%including axes titles/labels.
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu


handles=guidata(src);
udata=get(handles.fakeax(targetindex),'userdata');
targetaxes=handles.allgraphs(targetindex);
targettype=get(handles.allgraphs(targetindex),'userdata');
targetpos=get(handles.allblockers(targetindex),'position');

oldxtick = get(targetaxes,'xtick');
oldytick = get(targetaxes,'ytick');
oldztick = get(targetaxes,'ztick');


%Shift targetpos by cvpos:
cvpos = get(handles.currentview,'position');
targetpos = targetpos - [cvpos(1) cvpos(2) 0 0];


%for non-resizable graphs, simply move axes to new position:
if strcmp(handles.graphinfo.(targettype).axesformat,'fixedsize')
    inset=handles.graphinfo.(targettype).inset;
    targetpos=targetpos+[inset(1) inset(2) -(inset(1)+inset(3)) -(inset(2)+inset(4))];
    set(targetaxes,'position',targetpos);
    set(handles.fakeax(targetindex),'position',targetpos)
    set(handles.fakerax(targetindex),'position',targetpos)
    ravearrangeobjects(targetaxes,[]);
    raveupdateexploremethods(handles.fig,handles,targetindex,'resize');
    return
end

%If this graph is an image or has a custom raveresizeaxes function, do that instead.
if ~ischar(handles.graphinfo.(targettype).inset)
    inset=get(targetaxes,'tightinset')+handles.graphinfo.(targettype).inset;
elseif strcmp(handles.graphinfo.(targettype).inset,'image')
    if udata.view.invalid
        inset=[10 10 10 10];
    else
        imdata=udata.view.imdata;
        
        %determine how to resize axes:
        maxsize=get(handles.allblockers(targetindex),'position');
        maxsize=maxsize(3:4)-[24 20];
        maxratio=maxsize(1)/maxsize(2);
        
        ratio=size(imdata,2)/size(imdata,1);
        
        if ratio>maxratio %width constrained
            
            newheight=maxsize(1)/ratio;
            
            if newheight>size(imdata,1)
                newheight=size(imdata,1);
            end
            
            heightdiff=maxsize(2)-newheight;
            oldpos=get(handles.allblockers(targetindex),'position')+[10 10 -24 -20];
            oldpos = oldpos - [cvpos(1) cvpos(2) 0 0];
            
            if oldpos(3)>size(imdata,2)
                widthdiff=oldpos(3)-size(imdata,2);
                oldpos(3)=size(imdata,2);
            else
                widthdiff=0;
            end
            
            set(targetaxes,'position',[oldpos(1)+widthdiff/2 oldpos(2)+heightdiff/2 oldpos(3) newheight])
            
            if ~udata.view.fastresize && ~isempty(which('imresize')) && size(imdata,1)>newheight
                imdata=imresize(imdata,[newheight oldpos(3)]);
                img=findobj(targetaxes,'type','image');
                set(img,'cdata',imdata);
                
            end
            set(handles.activegraph,'xlim',[0.5 size(imdata,2)+0.5],'ylim',[0.5 size(imdata,1)+0.5])
        else %height constrained
            
            newwidth=maxsize(2)*ratio;
            
            if newwidth>size(imdata,2)
                newwidth=size(imdata,2);
            end
            
            widthdiff=maxsize(1)-newwidth;
            oldpos=get(handles.allblockers(targetindex),'position')+[10 10 -24 -20];
            oldpos = oldpos - [cvpos(1) cvpos(2) 0 0];
            
            if oldpos(4)>size(imdata,1)
                heightdiff=oldpos(4)-size(imdata,1);
                oldpos(4)=size(imdata,1);
            else
                heightdiff=0;
            end
            
            set(targetaxes,'position',[oldpos(1)+widthdiff/2 oldpos(2)+heightdiff/2 newwidth oldpos(4)])
            
            if ~udata.view.fastresize && ~isempty(which('imresize')) && size(imdata,1)>oldpos(4)
                imdata=imresize(imdata,[oldpos(4) newwidth]);
                img=findobj(targetaxes,'type','image');
                set(img,'cdata',imdata);
                
            end
            set(handles.activegraph,'xlim',[0.5 size(imdata,2)+0.5],'ylim',[0.5 size(imdata,1)+0.5])
        end
        return
    end
else
    if isempty(which(['raveresizeaxes',targettype]))
        errordlg(['Function raveresizeaxes',targettype,'.m not found. Add this function to your path or remove ''custom'' option from graphproperties',targettype,'.m file.','File Not Found']);
        return
    end
    
    eval(['raveresizeaxes',targettype,'(handles.fig,[],targetindex)']);
    return
    
end

%% Here comes the insanity:
axesformat=handles.graphinfo.(targettype).axesformat;

%1) Begin with default graph inset:
if isnumeric(handles.graphinfo.(targettype).inset);
    inset=handles.graphinfo.(targettype).inset;
else
    switch axesformat
        case {'single','row','matrix'}
            inset=[10 10 15 10];
        case {'3d','3D'}
            inset=[25 20 50 10];
    end
end
targetpos=targetpos+[inset(1) inset(2) -(inset(1)+inset(3)) -(inset(2)+inset(4))];

%2) %Apply extra margins for any customizations:
if isfield(udata.explore,'extrainset')
    marg=udata.explore.extrainset;
    targetpos=targetpos+[marg(1) marg(2) -(marg(1)+marg(3)) -(marg(2)+marg(4))];
end


%2.5) %If graph has a colorbar, give it 100 pixels:
if handles.graphinfo.(targettype).hascolorbar;
    if udata.format.colorbar && udata.view.c;
        targetpos(3)=targetpos(3)-100;
    end
end

% 3) If graph has a title, allow for it based on font size:
htitle=get(targetaxes,'title');
titlestr=get(htitle,'str');
if ~isempty(titlestr)
    ti=get(targetaxes,'tightinset');
    targetpos=targetpos+[0 0 0 -ti(4)];
end

% 4) Space for xlabel and ylabel if they are present:

%determine which axes will have labels/ticks that affect placement:
%prefixes: l = left, b = bottom
switch axesformat
    case {'single','3d','3D','image'}
        lax=targetaxes;
        bax=targetaxes;
    case {'matrix','row'}
        lax=handles.subax{targetindex}(:,1);
        bax=handles.subax{targetindex}(end,:);
end

xlab=get(bax,'xlabel');
ylab=get(lax,'ylabel');
zlab=get(lax,'zlabel');

% if iscell(xlab);xlab=cell2mat(xlab);end
% if iscell(ylab);ylab=cell2mat(ylab);end
% if iscell(zlab);zlab=cell2mat(zlab);end

if iscell(xlab);xlab=[xlab{:}];end
if iscell(ylab);ylab=[ylab{:}];end
if iscell(zlab);zlab=[zlab{:}];end



%get strings:
xstr=get(xlab,'string');
ystr=get(ylab,'string');
zstr=get(zlab,'string');

if ~iscell(xstr);xstr={xstr};end
if ~iscell(ystr);ystr={ystr};end
if ~iscell(zstr);zstr={zstr};end

%Determine how many lines each label has:
for h=numel(xstr):-1:1
    if iscell(xstr{h})
        xlines(h)=numel(xstr{h});
    elseif ~isempty(xstr{h})
        xlines(h)=1;
    else
        xlines(h)=0;
    end
end

for h=numel(ystr):-1:1
    if iscell(ystr{h})
        ylines(h)=numel(ystr{h});
    elseif ~isempty(ystr{h})
        ylines(h)=1;
    else
        ylines(h)=0;
    end
end

for h=numel(zstr):-1:1
    if iscell(zstr{h})
        zlines(h)=numel(zstr{h});
    elseif ~isempty(zstr{h})
        zlines(h)=1;
    else
        zlines(h)=0;
    end
end

%interpolate to guess how many pixels are needed based on number of lines
%and fontsize:
[junk,labelheight]=ravecalculatestringlengthfast('M',handles.fontmetrics.graph);

xaxisallowance=max(xlines)*labelheight;
yaxisallowance=max(ylines)*labelheight;
zaxisallowance=max(zlines)*labelheight;

%add 5 extra pixels for padding between label and tick labels:
xaxisallowance=xaxisallowance+5;
yaxisallowance=yaxisallowance+5;
zaxisallowance=zaxisallowance+5;

switch axesformat
    case {'3d','3D'}
        xaxisallowance=max(xaxisallowance,yaxisallowance);
        targetpos=targetpos+[zaxisallowance xaxisallowance -zaxisallowance -xaxisallowance];
    otherwise
        targetpos=targetpos+[yaxisallowance xaxisallowance -yaxisallowance -xaxisallowance];
end

%5) Calculate space for the tick marks on the left side of the graph
%(yticks for 2D graphs, zticks for 3D graphs)
switch axesformat
    case {'3D','3d'}
        lticks=findall(lax,'type','text','tag','zticklabel');
        if ~isempty(lticks) %get labels from axis:
            lstrings=get(lticks,'string');
        else
            lstrings=get(lax,'zticklabel');
        end
    otherwise
        lticks=findall(lax,'type','text','tag','yticklabel');
        if ~isempty(lticks) %get strings from custom ticks:
            lstrings=get(lticks,'string');
        else %get strings from axes:
            lstrings=get(lax,'yticklabel');
        end
end

if ~iscell(lstrings)
    lstrings=mat2cell(lstrings,ones(size(lstrings,1),1),size(lstrings,2));
end

pixels=ravecalculatestringlengthfast(lstrings,handles.fontmetrics.tick);

maxpixels=max(pixels);
%allow for 5 pixels for space between ticks and axes: (added by ravemovecustomticks)
maxpixels=maxpixels+5;
targetpos=targetpos+[maxpixels 0 -maxpixels 0];
if targetpos(3)<20;targetpos(3)=20;end %enforce a minimum graph size


%6) If any ylim (zlim for 3d) has order of magnitude >10^4,
%   add 15 pixels at the top of the graph for the
%   exponential notation matlab puts there: (if there is a title,
%   don't bother since there's already room up there.)
switch axesformat
    case {'3D','3d'}
        llim=get(lax,'zlim');
    otherwise
        llim=get(lax,'ylim');
end

if iscell(llim);llim=cell2mat(llim);end
if isempty(titlestr) && floor(log10(max(abs(llim(:)))))>3
    targetpos=targetpos+[0 0 0 -15];
end

if strcmpi(axesformat,'3D')
    targetpos=targetpos+[0 0 -50 0]; %subtract 50 pixels from width if it's a 3d graph to allow room for the tick marks on the far right side of the graph.
    %eventually replace this with something smarter.
end

%7) witdh is now fully calculated. apply positioning to get accurate xaxis scales:
newpos=floor(targetpos);
if newpos(3)<10;newpos(3)=10;end
if newpos(4)<10;newpos(4)=10;end

set(targetaxes,'position',single(newpos))
set(handles.fakeax(targetindex),'position',single(newpos))
set(handles.fakerax(targetindex),'position',single(newpos))


% do layout:
% ravearrangeobjects(targetaxes,[]);


%8) Determine what to do with x tick labels:

[oldxtick,oldytick,oldztick]=synchticks(handles,targetaxes,axesformat,oldxtick,oldytick,oldztick);

%% check if ticks need to be updated:
%{
switch axesformat
    case {'matrix','row'}
        %Do nothing, ticks are handled by ravearrangeobjects
    otherwise
        
        newxtick = get(targetaxes,'xtick');
        newytick = get(targetaxes,'ytick');
        newztick = get(targetaxes,'ztick');
        
        changedticks='';
        newticks={};
        
        if ~isequal(oldxtick,newxtick)
           changedticks(end+1)='x';
           newticks{1}=newxtick;
        end
        if ~isequal(oldytick,newytick)
            changedticks(end+1)='y';
            newticks{2}=newytick;
        end
        if ~isequal(oldztick,newztick)
            changedticks(end+1)='z';
            newticks{3}=newztick;
        end
        
        if ~isempty(changedticks)
            newaxeschangelistener(targetaxes,[],changedticks,newticks,handles);
        end
        
        oldxtick=newxtick;
        oldytick=newytick;
        oldztick=newztick;
end
%}


%get x tick label strings:
for b=1:numel(bax)
    
    xticks=findall(bax(b),'type','text','tag','xticklabel');
    if ~isempty(xticks) %get labels from axis:
        xstrings=get(xticks,'string');
    else
        xstrings=get(bax,'xticklabel');
    end
    
    if ~iscell(xstrings)
        xstrings=mat2cell(xstrings,ones(size(xstrings,1),1),size(xstrings,2));
    end
    
    %if xstrings are cells, make them just be strings (not sure what causes
    %them to be cells, but it breaks betterlinewrap:
    for s=1:numel(xstrings)
        if iscell(xstrings{s})
            xstrings{s}=[xstrings{s}{:}];
        end
    end
    
    %determine number of rows in tick labels:
    if ~isempty(xstrings)
        xlim=get(bax(b),'xlim');
        thispos=get(bax(b),'position');
        
        %determine how many pixels are available for each label:
        %axpos=getpixelposition(src);
        w=thispos(3);
        ticks=get(bax(b),'xtick');
        tickpix=interp1(xlim,[0 w],ticks);
        
        %subtract half of last ticklabel width from axes to allow room
        %for rightmost tick label, but only if this tick is within 10%
        %of max xlim
        if numel(tickpix)>1 && max(ticks)>(0.9*(xlim(2)-xlim(1))+xlim(1));
            lastgap=tickpix(end)-tickpix(end-1);
            wid=ravecalculatestringlengthfast(xstrings{end},handles.fontmetrics.graph);
            thispos=thispos+[0 0 -min([lastgap*0.4 wid 30]) 0];
            w=thispos(3);
            ticks=get(bax(b),'xtick');
            tickpix=interp1(xlim,[0 w],ticks);
        end
        
        if numel(tickpix)>1
            divs=tickpix(1:end-1)+diff(tickpix)/2;
            overshoot=floor(min(20,w/6)); %number of pixels beyond width of axes that tick labels are allowed to extend.
            maxtickwidth=diff([-overshoot divs w+overshoot]);
            maxtickwidth(1)=maxtickwidth(2);
            maxtickwidth(end)=maxtickwidth(end-1);
        else %if only one tick label:
            maxtickwidth=w;
        end
        
        %determine wrapping based on maxtickwidth:
        nrows=1;
        if ismember('alwaysrotatexticks',udata.attributes)
            status=false(1,numel(xticks));
        else
            status=true(1,numel(xticks));
        end
        
        for s=numel(xticks):-1:1;
            if all(status) %if no ticks need to be rotated, keep wrapping (if any are rotated, no need to wrap)
                %[wrapped,status(s)]=superiorlinewrap(xstrings{s},maxtickwidth(s),handles.fonts.tick,handles.fonts.ticksize,'plain',handles.axiscanvas);
                [wrapped,status(s)]=superiorlinewrapfast(xstrings{s},maxtickwidth(s),handles.fontmetrics.tick);
                if iscell(wrapped) && numel(wrapped)>nrows;nrows=numel(wrapped);end
                set(xticks(s),'string',wrapped,'rotation',0,'hor','center','vert','top');
            end
        end
        
        if any(status==0) %if any are rotated
            set(xticks,'rotation',90,'hor','right','ver','middle');
            for s=1:numel(xticks)
                set(xticks(s),'string',xstrings{s})
            end
        end
        
    else %no ticks
        status=1; %unrotated
        nrows=0;
    end
    
    %pixels=zeros(1,numel(xstrings));
    %determine pixels for height:
    if any(status==0) %tick labels are rotated, so base height on string length:
        for g=1:numel(xstrings)
            if iscell(xstrings{g});xstrings{g}=xstrings{g}{:};end
        end
        pixels=ravecalculatestringlengthfast(xstrings,handles.fontmetrics.tick);
        
        maxpixels(b)=max(pixels);
    else %base height on number of rows of text:
        [junk,tickheight]=ravecalculatestringlengthfast('M',handles.fontmetrics.tick); %M is as good a letter as anything...
        if isempty(xticks)
            maxpixels(b)=nrows*tickheight;
        else
            maxpixels(b)=nrows*tickheight;
        end
    end
end

%5 pixels extra for space between axes on tick labels:
maxpixels=maxpixels+5;
maxpixels=max(maxpixels);
targetpos=targetpos+[0 maxpixels 0 -maxpixels];
if targetpos(4)<20;targetpos(4)=20;end

%% Resize graph:
ticklength=handles.prefs.ticklength./max(targetpos(3),targetpos(4));
newpos=floor(targetpos);
if newpos(3)<10;newpos(3)=10;end
if newpos(4)<10;newpos(4)=10;end

set([targetaxes handles.fakeax(targetindex) handles.fakerax(targetindex)],'position',single(newpos));

isax=findobj([targetaxes handles.fakeax(targetindex) handles.fakerax(targetindex)],'flat','type','axes');
set(isax,'ticklength',[ticklength ticklength]);

[oldxtick,oldytick,oldztick]=synchticks(handles,targetaxes,axesformat,oldxtick,oldytick,oldztick);



%% do layout:
ravearrangeobjects(targetaxes,[]);

%% notify explore methods of resize:
raveupdateexploremethods(handles.fig,handles,targetindex,'resize');



