function raverotate3d(src,eventdata)

%src must be the axes to be rotated 

% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
% 
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
% 
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
% 
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

handles=guidata(src);
ai=find(handles.allgraphs==src);
[az,el]=view(src);
cp=get(0,'PointerLocation');
set(handles.fig,'windowbuttonmotionfcn',{@rotate_motion,src,ai,cp,az,el});
set(handles.fig,'windowbuttonupfcn',{@rotate_done,ai});
handles.temp.infobarstring=get(handles.infobar,'string');
set(handles.infobar,'string',['Az: ',sprintf('%3.0f',az),'  El: ',sprintf('%3.0f',el)])
guidata(handles.fig,handles);

function rotate_motion(src,ev,ax,ai,cp,az,el)
newpt=get(0,'PointerLocation');


dy=newpt(2)-cp(2);
dx=newpt(1)-cp(1);

newel=el-dy/3;
newaz=az-dx/3;

if newel<0;newel=0;end
if newel>90;newel=90;end

%"unwrap" angle to keep az between -180 to 180
if newaz>180
   newaz=-180+rem(newaz,180);
elseif newaz<-180
    newaz=180+rem(newaz,180);
end

oldxtick = get(ax,'xtick');
oldytick = get(ax,'ytick');
oldztick = get(ax,'ztick');

view(ax,newaz,newel)

if ~isempty(ai)
    handles=guidata(src);
    set(handles.infobar,'string',['Az: ',sprintf('%3.0f',newaz),'  El: ',sprintf('%3.0f',newel)])
    view(handles.fakeax(ai),newaz,newel);
    view(handles.fakerax(ai),newaz,newel);
end

%update ticks:
[newxtick,newytick,newztick]=synchticks(handles,ax,'3D',oldxtick,oldytick,oldztick);

%make sure tick labels are correctly located:
ravemovecustomticks(ax,[],false);



function rotate_done(src,ev,ai)
handles=guidata(src);
set(handles.fig,'windowbuttonmotionfcn',@ravedefaultwindowbuttonmotionfcn);
set(handles.fig,'windowbuttonupfcn',[]);

if ~isempty(ai)
    [az,el]=view(handles.allgraphs(ai));
    udata=get(handles.fakeax(ai),'userdata');
    udata.view.az=az;
    udata.view.el=el;
    set(handles.fakeax(ai),'userdata',udata);
end

try %this will give an error if you are trying to rotate a graph while Rave is busy, for example while it is running an optimizer. 
    set(handles.infobar,'string',handles.temp.infobarstring);
end






