function y = ravesamplefromdistribution(src,method,d,a,x,n,i)
%Samples n points from the current active distributions of variables x
%for dataset d.
%Optional 7th input is index of distributions to sample from (instead of
%using current active distributions)
%
%Output y has numel(x) columns, and is in the same order as x.

y=zeros(n,numel(x));

handles=guidata(src);
if nargin<7
    i=handles.activedistribution{d}(x);
end

%Deal with constants:
consts=i==0;
if any(consts)
    y(:,consts)=repmat(handles.xdials{d}{a}(x(consts)),n,1);
end

%Group according to joint distributions:
serials=zeros(numel(x),1);
for c=1:numel(x)
    if i(c)>0
        serials(c)=handles.distributions{d}{x(c)}{i(c)}.serialnumber;
    end
end

[uniqueserials]=unique(serials);
uniqueserials=setdiff(uniqueserials,0);

for u=1:numel(uniqueserials);
    thisc=find(serials==uniqueserials(u));
    
    settings=handles.distributions{d}{x(thisc(1))}{i(thisc(1))}; %For a joint distribution, settings will be identical for all variables involved, so just use the first one
    
    %Evaluate any strings in settings:
    settings=raveprocessdistribution(handles,settings,d,a);
    
    disttype=settings.type;
    sampler=str2func(['distributionsample',disttype]);
    
    data=makerands(method,n,numel(thisc)); %Generate random or pseudorandom points to use as a basis
    thesepoints=sampler(n,settings,data);
    
    if numel(thisc)==1
        y(:,thisc)=thesepoints;
    else %For a joint distribution, only put columns into y for variables that appear in x.
        thesepoints=thesepoints(:,ismember(settings.jointwith,x));        
        thesecees=ismember(x,settings.jointwith);
        y(:,thesecees)=thesepoints;
    end
end

function data=makerands(method,r,c)
%generate uniform random or pseudo-random numbers according to a particular
%technique

switch method.name
    case 'rand'
        data=rand(r,c);
    otherwise
        error('ravesamplefromdistribution was called with an unknown sampling method for its 2nd input')
end









