function f=ravesamplinggui(src,ev,mode)
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu

%% 
handles=guidata(src);
handles.temp=[];
guidata(handles.fig,handles);

%make figure:
f=figure('units','pixels','position',[0 0 651 700],'name',...
    'Sample New Data','numbertitle','off','menubar','none','resize','off',...
    'color',handles.colors.frame,'windowstyle','modal');
origin=get(handles.fig,'position');
thispos=get(f,'position');
thispos(1)=origin(1)+origin(3)/2-thispos(3)/2;
thispos(2)=thispos(2)+origin(2)+origin(4)/2-thispos(4)/2;
set(f,'position',thispos);

%make tabs:
titleaxes=axes('units','pixels','position',[11 699 485 41],'parent',f,'visible','off','tag','titleaxes');

handles.cancelbutton=uicontrol('style','pushbutton','units','pixels','position',[465 10 180 30],'tag','cancelbutton',...
    'fontsize',16,'string','Close','callback',@cancelbutton_Callback);

u=uipanel('parent',f,'units','pixels','position',[50 50 550 640],'backgroundcolor',handles.colors.tabbg);


a1=axes('parent',u,'units','pixels','position',[5 580+24 400 40],'visible','off');
text(0,0.5,'1. Choose Method','parent',a1,'fontname',handles.fonts.tab,...
    'color',handles.colors.text,'fontsize',0.8*handles.fonts.tabsize);

uicontrol('parent',u,'position',[35 558+26 200 22],'style','text',...
    'string','Data Set:','backgroundcolor',[1 1 1],'fontweight','bold','fontsize',10,'hor','left');
uicontrol('parent',u,'position',[35 528+26 200 22],'style','text',...
    'string','Sample by:','backgroundcolor',[1 1 1],'fontweight','bold','fontsize',10,'hor','left');

uicontrol('parent',u,'position',[115 560+26 200 22],'style','popupmenu',...
    'string',handles.datasetnames,'backgroundcolor',[1 1 1],'tag','datasetmenu','callback',@datasetmenu_Callback);
uicontrol('parent',u,'position',[325 560+26 200 22],'style','popupmenu',...
    'string',handles.analysisnames,'backgroundcolor',[1 1 1],'tag','analysismenu','callback',@analysismenu_Callback);
methodmenu=uicontrol('parent',u,'position',[115 530+26 200 22],'style','popupmenu',...
    'string','','backgroundcolor',[1 1 1],'tag','methodmenu','callback',@methodmenu_Callback);

a2=axes('parent',u,'units','pixels','position',[5 435+26 400 40],'visible','off');
text(0,0.5,'2. Choose Settings','parent',a2,'fontname',handles.fonts.tab,...
    'color',handles.colors.text,'fontsize',0.8*handles.fonts.tabsize);
uu=uipanel('parent',f,'units','pixels','position',[56 220 540 300],'backgroundcolor',handles.colors.tabbg,'bordertype','none','tag','methodspanel');


a3=axes('parent',u,'units','pixels','position',[5 105+46-20 400 40],'visible','off');
text(0,0.5,'3. Choose Destination','parent',a3,'fontname',handles.fonts.tab,...
    'color',handles.colors.text,'fontsize',0.8*handles.fonts.tabsize);
destmenu=uicontrol('parent',u,'position',[35 85+50-20 250 22],'style','popupmenu',...
    'string',{'Create as a new data set';'Export to a file';'Append to current data set';'Replace current data set'},...
    'tag','destinationmenu','backgroundcolor',[1 1 1],'callback',@destmenu_Callback);

handles.donebutton=uicontrol('style','pushbutton','units','pixels','position',[200 10 180 30],'tag','donebutton',...
    'fontsize',16,'string','Sample Data','callback',@donebutton_Callback,'tag','donebutton','parent',u);


handles.temp.tabobjects=[];
handles.temp.destobjects=[];
handles.temp.u=u;
handles.temp.f=f;
handles.temp.i=handles.activedistribution{1};
handles.temp.mode=mode;

guidata(f,handles);
guidata(handles.fig,handles);

%Load data for methods:
raveroot=which('rave');
raveroot=raveroot(1:end-6);
handles.modelinfo=struct;

maindir=fullfile(raveroot,'sampling',mode);
sampletypes=dir(maindir);
for g=3:numel(sampletypes)
    if sampletypes(g).isdir
        thisname=sampletypes(g).name;
        %remove leading numbers:
        firstletter=find(isstrprop(thisname,'alpha'),1);
        keywords{g-2}=thisname(firstletter:end);
        fid=fopen(fullfile(maindir,thisname,'samplinginfo.txt'),'r');
        longnames{g-2}=fgetl(fid);
        fclose(fid);
        
        %check if directory is on path:
        if isempty(which(fullfile(maindir,thisname,'samplinginfo.txt')))
            addpath(fullfile(maindir,thisname))
        end
    end
end

set(methodmenu,'string',longnames,'userdata',keywords,'value',1);


%load first method:
methodmenu_Callback(methodmenu,[]);
destmenu_Callback(destmenu,[]);

%% %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%

function datasetmenu_Callback(src,ev)
handles=guidata(src);
methodmenu=findobj(handles.temp.f,'tag','methodmenu');
methodmenu_Callback(methodmenu,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function analysismenu_Callback(src,ev)
handles=guidata(src);
methodmenu=findobj(handles.temp.f,'tag','methodmenu');
methodmenu_Callback(methodmenu,[]);


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function methodmenu_Callback(src,ev)
handles=guidata(src);

delete(handles.temp.tabobjects)
handles.temp.tabobjects=[];
guidata(handles.temp.f,handles);

%Note: Must be called with self as src
val=get(src,'value');
keywords=get(src,'userdata');
keyword=keywords{val};
%Create ui controls:
fun=str2func(['ravesamplemakeui',keyword]);
fun(handles.temp.f,[]);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function cancelbutton_Callback(src,ev)
close(gcf);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function donebutton_Callback(src,ev)
handles=guidata(src);
methodmenu=findobj(handles.temp.f,'tag','methodmenu');

%check that destination names are valid...
result=processdest(src,ev);
if ~result;return;end

%Note: Must be called with self as src
val=get(methodmenu,'value');
keywords=get(methodmenu,'userdata');
keyword=keywords{val};
%Create ui controls:
fun=str2func(['ravesample',keyword]);
points=fun(handles.temp.f,[]);

process_Sampling(handles.temp.f,[],points);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function destmenu_Callback(src,ev)
handles=guidata(src);
delete(handles.temp.destobjects);
handles.temp.destobjects=[];
u=handles.temp.u;

val=get(src,'value');

if val==1 % new data set
    txt=uicontrol('position',[35 60+50-20 400 22],'parent',handles.temp.u,...
        'style','text','backgroundcolor',[1 1 1],'string','Data set name:',...
        'hor','left','fontsize',10,'fontweight','bold');
    editbox=uicontrol('position',[50 40+50-20 200 22],'parent',handles.temp.u,...
        'style','edit','backgroundcolor',[1 1 1],'hor','left','tag','namebox');
    andfunctions=uicontrol('position',[50 14+50-20 400 22],'parent',handles.temp.u,...
        'style','checkbox','string','New data set inherits function variables from current data set',...
        'tag','andfunctions','backgroundcolor',[1 1 1],'value',1);
    handles.temp.destobjects=[txt editbox andfunctions];
    
elseif val==2 % export to file
    txt=uicontrol('position',[35 60+50-20 400 22],'parent',handles.temp.u,...
        'style','text','backgroundcolor',[1 1 1],'string','Destination file path: (*.txt)',...
        'hor','left','fontsize',10,'fontweight','bold');
    editbox=uicontrol('position',[50 40+50-20 400 22],'parent',handles.temp.u,...
        'style','edit','backgroundcolor',[1 1 1],'tag','browsebox','hor','left');
    browsebutton=uicontrol('position',[460 40+50-20 80 22],'parent',handles.temp.u,...
        'style','push','callback',@browsebutton_callback,'string','Browse...');
    andfunctions=uicontrol('position',[50 14+50-20 400 22],'parent',handles.temp.u,...
        'style','checkbox','string','Evaluate function variables and include them as columns in exported file',...
        'tag','andfunctions','backgroundcolor',[1 1 1]);
    
    handles.temp.destobjects=[txt editbox browsebutton andfunctions];
    
else %append or replace
    colortestbutton=zeros(1,10);
    for c=1:10
        colortestbutton(c)=uicontrol('position',[35+22*c 35+50-20 18 18],'parent',u,...
            'style','togglebutton','string','','value',0,'backgroundcolor',handles.prefs.rowcolors(c,:),...
            'fontname','arial narrow','fontsize',6,'horizontalalignment','left',...
            'tag','rowcolorblob','userdata',c,'callback',@colorbutton_callback);
    end
    set(colortestbutton(handles.prefs.defaultrowcolor),'value',1);
    colortext=uicontrol('units','pixel','position',[35 55+50-20 300 22],'parent',u,'style','text',...
        'string','Row color of newly added rows:','backgroundcolor',handles.colors.tabbg,'fontsize',10,'fontweight','bold','hor','left');
    selectexisting=uicontrol('units','pixel','position',[35 10+50-20 200 22],'parent',u,'style','checkbox',...
        'string','Select new rows','backgroundcolor',handles.colors.tabbg,'fontsize',10,'tag','selectexisting');
    handles.temp.destobjects=[colortestbutton colortext selectexisting];
    
end

guidata(handles.fig,handles)
guidata(handles.temp.f,handles)


%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function colorbutton_callback(src,ev)
handles=guidata(src);

cbuttons=findobj(handles.temp.f,'tag','rowcolorblob');
set(cbuttons,'value',0);
set(src,'value',1);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function browsebutton_callback(src,ev)
handles=guidata(src);

[name,path]=uiputfile('*.txt','Save file name');
if ~ischar(name);return;end
name=ravemakevalidfilename(name);
fullpath=fullfile(path,name);
browsebox=findobj(handles.temp.u,'tag','browsebox');
set(browsebox,'string',fullpath);

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function result=processdest(src,ev)
result=true;
handles=guidata(src);
destmenu=findobj(handles.temp.f,'tag','destinationmenu');

if get(destmenu,'value')==1
    namebox=findobj('tag','namebox');
    namestr=get(namebox,'string');
    if isempty(namestr)
        result=false;
        errordlg('You must enter a name for the new data set.');
    end
elseif get(destmenu,'value')==2
    browsebox=findobj('tag','browsebox');
    browsestr=get(browsebox,'string');
    if isempty(browsestr)
        result=false;
        errordlg('You must enter a file name.');
    end
end

%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
function process_Sampling(src,ev,points)
%sends the DoE defined by "points" to the proper destination:

handles=guidata(src);
datasets=raveguidata(handles.fig,'datasets');
% textdata=raveguidata(handles.fig,'textdata');
vectordata=raveguidata(handles.fig,'vectordata');

destmenu=findobj(handles.temp.f,'tag','destinationmenu');
datasetmenu=findobj(handles.temp.f,'tag','datasetmenu');
analysismenu=findobj(handles.temp.f,'tag','analysismenu');

val=get(destmenu,'value');

d=get(datasetmenu,'value');
a=get(analysismenu,'value');

allnames=handles.allnames{d};
indeps=~handles.isfunction{d};

nrows=size(points,1);

newdata=zeros(nrows,length(indeps));
newdata(:,indeps)=points;

if val==1; %data set
    andfunctions=findobj(handles.temp.f,'tag','andfunctions');
    andfunctions=get(andfunctions,'value');
    
    namebox=findobj(handles.temp.f,'tag','namebox');
    dest=get(namebox,'string');
    
    %first, copy existing data set:
    raveduplicatedataset(handles.fig,[],d,dest)
    handles=guidata(handles.fig);
    datasets=raveguidata(handles.fig,'datasets');
%     textdata=raveguidata(handles.fig,'textdata');
    vectordata=raveguidata(handles.fig,'vectordata');

    handles.ispointofinterest{end}=false(nrows,1);
    
    for a=1:numel(handles.analysisnames)
        handles.rowcolors{end}{a}=handles.prefs.defaultrowcolor*ones(nrows,1,'uint16');
        handles.isvisible{end}{a}=true(nrows,1);
        handles.isselected{end}{a}=false(nrows,1);
        handles.isfiltered{end}{a}=false(nrows,1);
    end
    
    guidata(handles.fig,handles);
    
    newd=numel(handles.datasetnames);
    
    if ~andfunctions
        %now remove extra columns if not keeping functions:
        ravedeletevars(handles.fig,[],newd,handles.isfunction{newd});
        handles=guidata(handles.fig);
        newdata=points;
    else
        %evaluate functions
        index=find(handles.isfunction{d});
        outs=raveevaluatefunction(handles.fig,d,index,newdata,handles);
        newdata(:,index)=outs;
    end
    
    %now replace data with DoE and call raveupdatedataset.
    datasets.alldata{end}=newdata;
%     textdata.alltext{end}=cell(size(newdata));
    vectordata.vectordata{end}=cell(size(newdata));
    datasets.isvalid{end}=true(size(newdata));

    raveguidata(handles.fig,'datasets',datasets);
%     raveguidata(handles.fig,textdata);
    raveguidata(handles.fig,'vectordata',vectordata);
    raveupdatedataset(handles.fig,handles,'replaceall',[],numel(handles.datasetnames));
    
elseif val==2; %file
    andfunctions=findobj(handles.temp.f,'tag','andfunctions');
    andfunctions=get(andfunctions,'value');
    
    if ~andfunctions;
        allnames=allnames(indeps);
        newdata=points;
    else
        %evaluate functions
        index=find(handles.isfunction{d});
        outs=raveevaluatefunction(handles.fig,d,index,newdata,handles);
        newdata(:,index)=outs;
    end
    
    browsebox=findobj(handles.temp.f,'tag','browsebox');
    dest=get(browsebox,'string');
    
    fid=fopen(dest,'wt');
    for t=1:numel(allnames)-1
        fprintf(fid,'%s\t',allnames{t});
    end
    fprintf(fid,'%s\n',allnames{end});
    fclose(fid);
    dlmwrite(dest,newdata,'-append','delimiter','\t','newline','pc','precision',8);
    
elseif val==3; %append
    %evaluate functions
    index=find(handles.isfunction{d});
    outs=raveevaluatefunction(handles.fig,d,index,newdata,handles);
    newdata(:,index)=outs;
    
    cblob=findobj(handles.temp.f,'tag','rowcolorblob','value',1);
    rowcolor=get(cblob,'userdata');
    
    selectexisting=findobj(handles.temp.f,'tag','selectexisting');
    doselect=get(selectexisting,'value');
    
    raveappendrows(handles.fig,d,newdata,[],doselect,true,rowcolor,false,1);

elseif val==4; %replace
    %evaluate functions
    index=find(handles.isfunction{d});
    outs=raveevaluatefunction(handles.fig,d,index,newdata,handles);
    newdata(:,index)=outs;
    
    cblob=findobj(handles.temp.f,'tag','rowcolorblob','value',1);
    rowcolor=get(cblob,'userdata');
    
    selectexisting=findobj(handles.temp.f,'tag','selectexisting');
    doselect=get(selectexisting,'value');

    datasets.alldata{d}=newdata;
%     textdata.alltext{d}=cell(size(newdata));
    vectordata.vectordata{d}=cell(size(newdata));
    datasets.isvalid{d}=true(size(newdata));
    
    handles.ispointofinterest{d}=false(nrows,1);
    
    for a=1:numel(handles.analysisnames)
        handles.rowcolors{d}{a}=rowcolor*ones(nrows,1);
        handles.isvisible{d}{a}=true(nrows,1);
        handles.isfiltered{d}{a}=false(nrows,1);

        if doselect
            newselect=true(size(newdata,1),1);
        else
            newselect=false(size(newdata,1),1);
        end
        
        handles.isselected{d}{a}=[newselect];
    end

    raveguidata(handles.fig,'datasets',datasets);
    raveguidata(handles.fig,'vectordata',vectordata);
    raveupdatedataset(handles.fig,handles,'replaceall',[],numel(handles.datasetnames));
end

ravefinish
set(handles.infobar,'string','Sampling created successfully.');

