function result=ravetxt2m(src,eventdata,responsename,responsestring,dataset) %#ok<INUSL>
%function to convert TXT or JMP function to rave function.
%INPUTS: src, eventdata, name of variable, string containing txt function
%text, index of data set to add this function to.
%
% RAVE Rapid Analysis and Visualization Environment
% Copyright (C) 2010 Georgia Tech Research Corporation
% ALL RIGHTS RESERVED
%
% This program is free software; you can redistribute it and/or
% modify it under the terms of the GNU General Public License
% as published by the Free Software Foundation; version 2
% of the License.
%
% This program is distributed in the hope that it will be useful,
% but WITHOUT ANY WARRANTY; without even the implied warranty of
% MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
% GNU General Public License for more details.
%
% Written by: Matthew Daskilewicz mdaskilewicz@asdl.gatech.edu


handles=guidata(src);
datasets=raveguidata(handles.fig,'datasets');
% textdata=raveguidata(handles.fig,'textdata');
vectordata=raveguidata(handles.fig,'vectordata');

result = 1; %success

indata=responsestring;
VarName=responsename;
d=dataset;


%make into 1-D string:
if iscell(indata)
    indata=indata';
    indata=cell2mat(indata);
else
    indata=reshape(indata',1,size(indata,1)*size(indata,2));
end

indata=indata(indata~=32);
indata=indata(indata~=9);

%brainn puts Name(" ") around variable names... find these and delete them.
startindx=strfind(indata,'Name("');
if ~isempty(startindx)
    startindx=repmat(startindx,[6 1]);
    startindx=bsxfun(@plus,startindx,(0:5)');
    startindx=startindx(:)';
    startindx=setdiff(1:length(indata),startindx);
    indata=indata(startindx);
end

startindx=strfind(indata,'")');
if ~isempty(startindx)
    startindx=repmat(startindx,[2 1]);
    startindx=bsxfun(@plus,startindx,[0:1]');
    startindx=startindx(:)';
    startindx=setdiff(1:length(indata),startindx);
    indata=indata(startindx);
end

%remove empty spaces and colons before variable names.
indata=indata(indata~=char(58) & indata~=char(32) & indata~=char(9));

%replace "+-" with just "-"
indata=strrep(indata,'+-','-');

%remove tabs:
%indata=strrep(indata,char(9),'');

%replace vector multiply/divide with scalar:
indata=strrep(indata,'*','.*');
indata=strrep(indata,'/','./');
indata=strrep(indata,'^','.^');
indata=strrep(indata,'..','.');

%convert funtion names to matlabese:
indata=strrep(indata,'Exp','exp');
indata=strrep(indata,'Sine','sin');
indata=strrep(indata,'Squish','logsig');
indata=strrep(indata,'Cosine','cos');
indata=strrep(indata,'Tangent','tan');
indata=strrep(indata,'Log','log');
indata=strrep(indata,'Log','log');
indata=strrep(indata,'Abs','abs');
indata=strrep(indata,'Round','round');
indata=strrep(indata,'Floor','floor');
indata=strrep(indata,'Ceiling','ceil');

indata=strrep(indata,'Name','');
indata=strrep(indata,'"','');

%extract variable names from indata:
varnames=symvar(indata);
nvars=length(varnames);

%check that all function variables match up with variable names in data
%set.
importnames=varnames;
existingnames=strrep(handles.basenames{d},char(32),'');

if ~any(indata=='=') && sum(ismember(importnames,existingnames))<length(importnames)
    %some names arent in data set
    missingvars=importnames(~ismember(importnames,existingnames));
    for m=1:numel(missingvars)-1
        missingvars{m}=[missingvars{m},', '];
    end
    
    
    warndlg(['I can''t find matches for all of this function''s independent variables in the data set you selected. Fix the variable names and then try reloading this function. ',...
        'The variables that you referenced that do not exist in your data set are: ',missingvars{:}],'Variable Not Found');
    result=-1;

    return
end

%find which column of data set holds each input variable:
varorder=zeros(1,nvars);
for i=1:nvars
    if ~isempty(find(strcmp(existingnames,importnames(i)),1))
        varorder(i)=find(strcmp(existingnames,importnames(i)),1);
    else
        varorder(i)=0; %a non-dataset variable
    end
end

%% replace special variables with appropriate functions:
usesidealbest=false(size(varnames));
usesidealworst=false(size(varnames));
usespreference=false(size(varnames));
usesnorm=false(size(varnames));
isobjective=false;

for g=1:numel(varnames);
    thisvar=varnames{g};
    if ~isempty(strfind(indata,['$+#' thisvar]))
        indata=strrep(indata,['$+#' thisvar],['(best(' num2str(varorder(g)) ')./ssq(' num2str(varorder(g)) '))']);
        usesidealbest(g)=true;
        usesnorm(g)=true;
        isobjective=true;
    end
    
    if ~isempty(strfind(indata,['$-#' thisvar]))
        indata=strrep(indata,['$-#' thisvar],['(worst(' num2str(varorder(g)) ')./ssq(' num2str(varorder(g)) '))']);
        usesidealworst(g)=true;
        usesnorm(g)=true;
        isobjective=true;
    end
    
    if ~isempty(strfind(indata,['$+' thisvar]))
        indata=strrep(indata,['$+' thisvar],['(best(' num2str(varorder(g)) '))']);
        usesidealbest(g)=true;
        isobjective=true;
    end
    
    if ~isempty(strfind(indata,['$-' thisvar]))
        indata=strrep(indata,['$-' thisvar],['(worst(' num2str(varorder(g)) '))']);
        usesidealworst(g)=true;
        isobjective=true;
    end
    
    if ~isempty(strfind(indata,['#' thisvar]))
        indata=strrep(indata,['#' thisvar],['(' thisvar './ssq(' num2str(varorder(g)) '))']);
        usesnorm(g)=true;
    end
    
    if ~isempty(strfind(indata,['%' thisvar]))
        indata=strrep(indata,['%' thisvar],['(prefs(' num2str(varorder(g)) '))']);
        usespreference(g)=true;
        isobjective=true;
    end
end

%% reformat to multiple lines if necessary
if strcmp(indata(1,1:2),'If') %check for piecewiseness
    %find the parenthesis that ends the if statement, treat if statement as
    %a chunk, and then treat anything after the if statement separately.
    parens=parenthesesmatch(indata);
    %the if statement will have the first parenthesis, so the closer for it
    %will be inposition parens(1,2);
    endif=parens(1,2);
    
    theifstatement=indata(1:endif-1);   %exclude trailing parenthesis
    %find the commas:
    splits=find(theifstatement==char(44));
    npieces=length(splits);
    %append an extra split at the end of the indata:
    splits(end+1)=length(theifstatement)+1;
    %extract conditional statement:
    condition=theifstatement(4:splits(1)-1);
    equation=cell(npieces,1);
    for n=1:npieces
        equation{n}=theifstatement(splits(n)+1:splits(n+1)-1);
    end
    
    %now tack on whatever was left over after the if statement:
    leftovers=indata(endif+1:end);
    yeahlogical=true;
    
elseif any(indata==';') %check for multiple statements
    splits=find(indata==';');
    npieces=length(splits);
    %append an extra split at the end of the indata:
    splits(end+1)=length(indata)+1;
    splits=[0 splits];
    %extract conditional statement:
    equation=cell(npieces,1);
    for n=1:npieces
        equation{n}=indata(splits(n)+1:splits(n+1)-1);
    end
    leftovers=[];
    yeahlogical=false;
else
    equation=indata;
    condition=[];
    leftovers=[];
end

if isempty(leftovers)
    leftovers='';
end

%% get file name/make file:
PathName=handles.prefs.defaultdir;
VarNameFormatted=VarName(isstrprop(VarName,'alphanum')); %remove spaces/punctuation to make it a valid function name.
FileName=[VarNameFormatted,'FUN','.m'];
fid=fopen(fullfile(PathName,'ravetempfun.m'),'w+'); %note, this line gives an error on new matlab installs, maybe because old defaultdir isnt on new path.
functionname=FileName(1:end-2);


%% make the function m file:

%print a help file for this function:

% if ~any(usespreference) && ~any(usesidealbest | usesidealworst)
%     fprintf(fid,'function response=%s(inputs)\n',functionname); %error here on new install, see fid=fopen... line above.
% elseif any(usespreference) && ~any(usesidealbest | usesidealworst)
%     fprintf(fid,'function response=%s(inputs,prefs)\n',functionname); %error here on new install, see fid=fopen... line above.
% elseif ~any(usespreference) && any(usesidealbest | usesidealworst)
%     fprintf(fid,'function response=%s(inputs,targets)\n',functionname); %error here on new install, see fid=fopen... line above.
% else
fprintf(fid,'function %s=%s(inputs,prefs,targets)\n',VarNameFormatted,functionname); %error here on new install, see fid=fopen... line above.
%end

fprintf(fid,'%%This is a function for the variable: %s.\n',functionname);
fprintf(fid,'%%This file was auto-generated by the "Import txt Function" button in RAVE.\n');
fprintf(fid,'%%This function is not meant to be called from the command line.\n');
fprintf(fid,'%%I don''t even know why you''re reading this help file.\n');
fprintf(fid,'%%This function was created with RAVETXT2M version 1.0\n');
fprintf(fid,'\n\n');
%Print the function:
fprintf(fid,'%%expand input array out to its component vectors:\n');
for jj=1:nvars
    if varorder(jj)~=0
        fprintf(fid,'%s=inputs(:,%1.0f);\n',varnames{jj},varorder(jj));
    end
end
fprintf(fid,'\n');
fprintf(fid,'nruns=size(inputs,1);\n');

if any(usesidealbest) || any(usesidealworst);
    fprintf(fid,'best=zeros(size(inputs,2),1);\n');
    fprintf(fid,'worst=zeros(size(inputs,2),1);\n');
    fprintf(fid,['bestidx=' mat2str(varorder(usesidealbest|usesidealworst)) ';\n']);
    fprintf(fid,'for g=1:numel(bestidx)\n');
    fprintf(fid,'    idx=bestidx(g);\n');
    fprintf(fid,'    if targets(idx)==inf\n');
    fprintf(fid,'        best(idx)=max(inputs(:,idx));\n');
    fprintf(fid,'        worst(idx)=min(inputs(:,idx));\n');
    fprintf(fid,'    elseif targets(idx)==-inf\n');
    fprintf(fid,'        best(idx)=min(inputs(:,idx));\n');
    fprintf(fid,'        worst(idx)=max(inputs(:,idx));\n');
    fprintf(fid,'    else\n');
    fprintf(fid,'        best(idx)=min(abs(inputs(:,idx)-targets(idx)));\n');
    fprintf(fid,'        worst(idx)=max(abs(inputs(:,idx)-targets(idx)));\n');
    fprintf(fid,'    end\n');
    fprintf(fid,'end;\n');
    fprintf(fid,'\n');
end

if any(usesnorm);
    fprintf(fid,'ssq=ones(size(inputs,2),1);\n');
    fprintf(fid,['ssqidx=' mat2str(varorder(usesnorm)) ';\n']);
    fprintf(fid,'for g=1:numel(ssqidx)\n');
    fprintf(fid,'    idx=bestidx(g);\n');
    fprintf(fid,'    ssq(idx)=sqrt(sum(inputs(:,idx).^2));\n');
    fprintf(fid,'end;\n');
    fprintf(fid,'\n');
end

if ~iscell(equation)
    %for nonpiecewise function, simple evaluation:
    fprintf(fid,'%%evaluate function:\n');
    fprintf(fid,'logsig=@(n) 1./(1 + exp(-n));\n');
    if ~any(equation=='=')
        fprintf(fid,'%s=%s;',VarNameFormatted,indata);
    else
        fprintf(fid,'%s;',indata);
    end
    
elseif yeahlogical
    %for piecewise function, construct logical statement:
    fprintf(fid,'%%evaluate function:\n');
    fprintf(fid,'logsig=@(n) 1./(1 + exp(-n));\n');
    fprintf(fid,'%s=zeros(nruns,1);\n',VarNameFormatted);
    fprintf(fid,'isIf=%s;\n',condition);
    fprintf(fid,'response1=(%s) %s;\n',equation{1},leftovers);
    fprintf(fid,'response2=(%s) %s;\n',equation{end},leftovers);
    fprintf(fid,'isif=find(isIf);\n'); %FOR GPGPU
    fprintf(fid,'isntif=find(~isIf);\n');    %FOR GPGPU
    fprintf(fid,'%s(isif)=response1(isif);\n',VarNameFormatted);
    fprintf(fid,'%s(isntif)=response2(isntif);\n',VarNameFormatted);
else %non-piecewise multiline function:
    for n=1:npieces
        fprintf(fid,'%s;\n\n',equation{n});
    end
end

fclose(fid);



%{
%check if another file on the path already has this name:
origname=FileName;
w=which(FileName);
q=2;
newfun=true;

while ~isempty(w)
    %check if these two files are identical:
    fid=fopen(fullfile(PathName,'ravetempfun.m'));
    fid2=fopen(w);
    A=fread(fid);
    B=fread(fid2);
    fclose all;
    
    if numel(A)==numel(B) && all(A==B); %files are identical
        %Delete new file and just use old file:
        delete(fullfile(PathName,'ravetempfun.m'))
        %need to recheck filename because it may have different letter
        %cases:
        [p,n,e]=fileparts(w);
        FileName=[n,e];
        newfun=false;
        break
    else
        %need to make new filename
        FileName=[origname(1:end-2),'_',int2str(q),'.m'];
        w=which(FileName);
        q=q+1;
    end
end

%save final function name:
functionname=FileName(1:end-2);

if newfun
    movefile(fullfile(PathName,'ravetempfun.m'),fullfile(PathName,FileName));
    %refresh path to include this function
    rehash
end
%}

destination=ravemakefile(fullfile(PathName,'ravetempfun.m'),fullfile(PathName,FileName));
[junk,functionname,ext]=fileparts(destination);

%evaluate function:
funhandle=str2func(functionname);
prefs=handles.preferences{d};
targets=handles.targetvalues{d};
newvalues=funhandle(datasets.alldata{dataset},prefs,targets);
%if error here, check that defaultdir is on the path.

%update fields:
datasets.datatype{d}{end+1}='continuous';
idx = numel(datasets.datatype{d});
datasets.alldata{d}(:,idx)=newvalues;
datasets.isvalid{d}(:,idx)=true;
% textdata.alltext{d}=[textdata.alltext{d},cell(length(newvalues),1)];
vectordata.vectordata{d}=[vectordata.vectordata{d},cell(length(newvalues),1)];
handles.basenames{d}{end+1}=VarName;
% handles.datasetlinks{d}{end+1}=[];
% handles.datasetlinkinfo{d}{end+1}={};
handles.variablecolors{d}(end+1,:)=rand(1,3);
handles.allunits{d}{end+1}=''; %havent implemented this yet
handles.isincluded{d}(end+1)=1;
handles.isconstant{d}(end+1)=min(newvalues)==max(newvalues);
handles.isfunction{d}(end+1)=1;
handles.funclass{d}{end+1}='matlab_threeinput';
handles.funattributes{d}{end+1}={''};
handles.funnames{d}{end+1}=functionname;
invars=sort(varorder(varorder~=0)); %remove non data set variables); %used to be = varnames
handles.funvars{d}{end+1}=invars;
handles.depvars{d}{end+1}=[];
for i=1:numel(invars)
    handles.depvars{d}{invars(i)}(end+1)=numel(handles.isfunction{d});
end
handles.nargout{d}(end+1)=1;
handles.datatype{d}{end+1}='continuous';
handles.isnoise{d}(end+1)=false;
handles.preferences{d}(end+1)=0;
handles.targetvalues{d}(end+1)=-inf;
handles.isobjective{d}(end+1)=isobjective;
handles.maintable.colwidths{d}(end+1)=handles.prefs.colwidth;
handles.distributions{d}{end+1}{1}=struct();
handles.aggregatingstatistic{d}{end+1}=handles.prefs.defaultstatistic;
handles.activedistribution{d}(end+1)=0;

for a=1:numel(handles.analysisnames)
    handles.xdials{d}{a}(end+1)=0;
    handles.minlimit{d}{a}(end+1)=min(newvalues);
    handles.maxlimit{d}{a}(end+1)=max(newvalues);
    handles.minfilter{d}{a}(end+1)=min(newvalues);
    handles.maxfilter{d}{a}(end+1)=max(newvalues);
    handles.discretevalues{d}{a}{end+1}=[];
end

handles.allnames{d}(end+1)=ravemakevariablename(handles,d,numel(handles.basenames{d}));  

raveguidata(src,'datasets',datasets);
raveguidata(src,'vectordata',vectordata);
raveupdatedataset(handles.fig,handles,'addcolumns',[],d);


